@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -74,7 +74,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 		public void run() {
 			saveContent(getInput());
 		}
-	};
+	}
 	
 	/**
 	 * Property names.
@@ -162,6 +162,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 			fDirection= dir;
 			fControl= c;
 			fLiveResize= !(fControl instanceof Sash);
+			updateCursor(c, dir);
 			fControl.addMouseListener(this);
 			fControl.addMouseMoveListener(this);
 			fControl.addDisposeListener(
@@ -172,7 +173,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 				}
 			);
 		}
-		
+				
 		public void mouseDoubleClick(MouseEvent e) {
 			if ((fDirection & HORIZONTAL) != 0)
 				fHSplit= HSPLIT;
@@ -232,7 +233,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 			fComposite.layout(true);
 			fControl.getDisplay().update();
 		}
-	};
+	}
 
 	/** Style bits for top level composite */
 	private int fStyles;
@@ -271,12 +272,18 @@ public abstract class ContentMergeViewer extends ContentViewer
 	private Image fRightArrow;
 	private Image fLeftArrow;
 	private Image fBothArrow;
+	Cursor fNormalCursor;
+	private Cursor fHSashCursor;
+	private Cursor fVSashCursor;
+	private Cursor fHVSashCursor;
+
 	//---- end
 	
 	/**
 	 * Creates a new content merge viewer and initializes with a resource bundle and a
 	 * configuration.
-	 *
+	 * 
+	 * @param style SWT style bits
 	 * @param bundle the resource bundle
 	 * @param cc the configuration object
 	 */
@@ -370,6 +377,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 	 * @param toolBarManager the toolbar manager to contribute to
 	 */
 	protected void createToolItems(ToolBarManager toolBarManager) {
+		// empty implementation
 	}
 
 	/**
@@ -425,6 +433,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 	 * The <code>ContentMergeViewer</code> implementation of this 
 	 * <code>ContentViewer</code> method
 	 * checks to ensure that the content provider is an <code>IMergeViewerContentProvider</code>.
+	 * @param contentProvider the content provider to set. Must implement IMergeViewerContentProvider. 
 	 */
 	public void setContentProvider(IContentProvider contentProvider) {
 		Assert.isTrue(contentProvider instanceof IMergeViewerContentProvider);
@@ -438,6 +447,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 	/**
 	 * The <code>ContentMergeViewer</code> implementation of this 
 	 * <code>Viewer</code> method returns the empty selection. Subclasses may override.
+	 * @return empty selection.
 	 */
 	public ISelection getSelection() {
 		return new ISelection() {
@@ -447,11 +457,12 @@ public abstract class ContentMergeViewer extends ContentViewer
 		};
 	}
 	
-	/**
+	/*
 	 * The <code>ContentMergeViewer</code> implementation of this 
 	 * <code>Viewer</code> method does nothing. Subclasses may reimplement.
 	 */
-	public void setSelection(ISelection s, boolean reveal) {
+	public void setSelection(ISelection selection, boolean reveal) {
+		// empty implementation
 	}
 
 	/* package */ void propertyChange(PropertyChangeEvent event) {
@@ -461,10 +472,47 @@ public abstract class ContentMergeViewer extends ContentViewer
 		if (key.equals(ANCESTOR_ENABLED)) {
 			fAncestorEnabled= Utilities.getBoolean(getCompareConfiguration(), ANCESTOR_ENABLED, fAncestorEnabled);
 			fComposite.layout(true);
+			
+			updateCursor(fLeftLabel, VERTICAL);
+			updateCursor(fDirectionLabel, HORIZONTAL | VERTICAL);
+			updateCursor(fRightLabel, VERTICAL);
+			
 			return;
 		}
 	}
 	
+	void updateCursor(Control c, int dir) {
+		if (!(c instanceof Sash)) {
+			Cursor cursor= null;
+			switch (dir) {
+			case VERTICAL:
+				if (fAncestorEnabled) {
+					if (fVSashCursor == null) fVSashCursor= new Cursor(c.getDisplay(), SWT.CURSOR_SIZENS);
+					cursor= fVSashCursor;
+				} else {
+					if (fNormalCursor == null) fNormalCursor= new Cursor(c.getDisplay(), SWT.CURSOR_ARROW);
+					cursor= fNormalCursor;
+				}
+				break;
+			case HORIZONTAL:
+				if (fHSashCursor == null) fHSashCursor= new Cursor(c.getDisplay(), SWT.CURSOR_SIZEWE);
+				cursor= fHSashCursor;
+				break;
+			case VERTICAL + HORIZONTAL:
+				if (fAncestorEnabled) {
+					if (fHVSashCursor == null) fHVSashCursor= new Cursor(c.getDisplay(), SWT.CURSOR_SIZEALL);
+					cursor= fHVSashCursor;
+				} else {
+					if (fHSashCursor == null) fHSashCursor= new Cursor(c.getDisplay(), SWT.CURSOR_SIZEWE);
+					cursor= fHSashCursor;
+				}
+				break;
+			}
+			if (cursor != null)
+				c.setCursor(cursor);
+		}
+	}
+
 	void setAncestorVisibility(boolean visible, boolean enabled) {
 		if (fAncestorItem != null) {
 			Action action= (Action) fAncestorItem.getAction();
@@ -604,7 +652,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 			if (fAncestorEnabled && oldFlag != fShowAncestor)
 				fComposite.layout(true);
 			
-			ToolBarManager tbm= CompareViewerSwitchingPane.getToolBarManager(fComposite.getParent());
+			ToolBarManager tbm= CompareViewerPane.getToolBarManager(fComposite.getParent());
 			if (tbm != null) {
 				updateToolItems();
 				tbm.update(true);
@@ -634,7 +682,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 	 * @return the new control
 	 */
 	protected final Control buildControl(Composite parent) {
-									
+											
 		fComposite= new Composite(parent, fStyles) {
 			public boolean setFocus() {
 				return internalSetFocus();
@@ -667,7 +715,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 		IWorkbenchPartSite ps= Utilities.findSite(fComposite);
 		fKeyBindingService= ps != null ? ps.getKeyBindingService() : null;
 						
-		ToolBarManager tbm= CompareViewerSwitchingPane.getToolBarManager(parent);
+		ToolBarManager tbm= CompareViewerPane.getToolBarManager(parent);
 		if (tbm != null) {
 			tbm.removeAll();
 			
@@ -746,7 +794,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 		return fComposite;
 	}
 	
-	/**
+	/*
 	 * Called on the viewer disposal.
 	 * Unregisters from the compare configuration.
 	 * Clients may extend if they have to do additional cleanup.
@@ -788,6 +836,23 @@ public abstract class ContentMergeViewer extends ContentViewer
 			fBothArrow.dispose();
 			fBothArrow= null;
 		}
+
+		if (fNormalCursor != null) {
+			fNormalCursor.dispose();
+			fNormalCursor= null;
+		}
+		if (fHSashCursor != null) {
+			fHSashCursor.dispose();
+			fHSashCursor= null;
+		}
+		if (fVSashCursor != null) {
+			fVSashCursor.dispose();
+			fVSashCursor= null;
+		}
+		if (fHVSashCursor != null) {
+			fHVSashCursor.dispose();
+			fHVSashCursor= null;
+		}
 		
 		super.handleDispose(event);
   	}
@@ -861,7 +926,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 //		return null;
 //	}
 		
-	/**
+	/*
 	 * Calculates the height of the header.
 	 */
 	/* package */ int getHeaderHeight() {
@@ -872,7 +937,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 
 	//---- merge direction
 	
-	/**
+	/*
 	 * Returns true if both sides are editable.
 	 */
 	/* package */ boolean canToggleMergeDirection() {
@@ -939,7 +1004,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 		}
 	}
 	
-	/**
+	/*
 	 * Save the viewers's content.
 	 * Note: this method is for internal use only. Clients should not call this method. 
 	 * @since 2.0
@@ -948,7 +1013,7 @@ public abstract class ContentMergeViewer extends ContentViewer
 		saveContent(getInput());
 	}
 	
-	/**
+	/*
 	 * Save modified content back to input elements via the content provider.
 	 */
 	/* package */ void saveContent(Object oldInput) {
