@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -14,12 +14,10 @@ package org.eclipse.compare.rangedifferencer;
 
 	static final int INSERT= 0;
 	static final int DELETE= 1;
-	static final int CHANGE= 2;
-	static final int ERROR= 3;
 
 	LinkedRangeDifference fNext;
 
-	/**
+	/*
 	 * Creates a LinkedRangeDifference an initializes it to the error state
 	 */
 	LinkedRangeDifference() {
@@ -27,7 +25,7 @@ package org.eclipse.compare.rangedifferencer;
 		fNext= null;
 	}
 
-	/**
+	/*
 	 * Constructs and links a LinkeRangeDifference to another LinkedRangeDifference
 	 */
 	LinkedRangeDifference(LinkedRangeDifference next, int operation) {
@@ -35,7 +33,7 @@ package org.eclipse.compare.rangedifferencer;
 		fNext= next;
 	}
 
-	/**
+	/*
 	 * Follows the next link
 	 */
 	LinkedRangeDifference getNext() {
@@ -50,7 +48,7 @@ package org.eclipse.compare.rangedifferencer;
 		return kind() == INSERT;
 	}
 
-	/**
+	/*
 	 * Sets the next link of this LinkedRangeDifference
 	 */
 	void setNext(LinkedRangeDifference next) {
