@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -21,8 +21,11 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-import org.apache.xerces.dom.DocumentImpl;
+import javax.xml.parsers.ParserConfigurationException;
+import javax.xml.transform.TransformerException;
+
 import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Status;
 import org.eclipse.debug.core.DebugException;
 import org.eclipse.debug.core.DebugPlugin;
@@ -102,7 +105,7 @@ public class LaunchConfigurationInfo {
 	 * 
 	 * @return attribute specified by given key or the defaultValue
 	 *  if undefined
-	 * @exception if the attribute with the given key exists
+	 * @throws CoreException if the attribute with the given key exists
 	 *  but is not a <code>String</code>
 	 */
 	protected String getStringAttribute(String key, String defaultValue) throws CoreException {
@@ -110,14 +113,13 @@ public class LaunchConfigurationInfo {
 		if (attr != null) {
 			if (attr instanceof String) {
 				return (String)attr;
-			} else {
-				throw new DebugException(
-					new Status(
-					 Status.ERROR, DebugPlugin.getUniqueIdentifier(),
-					 DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.getString("LaunchConfigurationInfo.Attribute_{0}_is_not_of_type_java.lang.String._1"), new String[] {key}), null //$NON-NLS-1$
-					)
-				);
-			}
+			} 
+			throw new DebugException(
+				new Status(
+				 IStatus.ERROR, DebugPlugin.getUniqueIdentifier(),
+				 DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.LaunchConfigurationInfo_Attribute__0__is_not_of_type_java_lang_String__1, new String[] {key}), null //$NON-NLS-1$
+				)
+			);
 		}
 		return defaultValue;
 	}
@@ -128,7 +130,7 @@ public class LaunchConfigurationInfo {
 	 * 
 	 * @return attribute specified by given key or the defaultValue
 	 *  if undefined
-	 * @exception if the attribute with the given key exists
+	 * @throws CoreException if the attribute with the given key exists
 	 *  but is not an <code>int</code>
 	 */
 	protected int getIntAttribute(String key, int defaultValue) throws CoreException {
@@ -136,14 +138,13 @@ public class LaunchConfigurationInfo {
 		if (attr != null) {
 			if (attr instanceof Integer) {
 				return ((Integer)attr).intValue();
-			} else {
-				throw new DebugException(
-					new Status(
-					 Status.ERROR, DebugPlugin.getUniqueIdentifier(),
-					 DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.getString("LaunchConfigurationInfo.Attribute_{0}_is_not_of_type_int._2"), new String[] {key}), null //$NON-NLS-1$
-					)
-				);
-			}
+			} 
+			throw new DebugException(
+				new Status(
+				 IStatus.ERROR, DebugPlugin.getUniqueIdentifier(),
+				 DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.LaunchConfigurationInfo_Attribute__0__is_not_of_type_int__2, new String[] {key}), null //$NON-NLS-1$
+				)
+			);
 		}
 		return defaultValue;
 	}
@@ -154,7 +155,7 @@ public class LaunchConfigurationInfo {
 	 * 
 	 * @return attribute specified by given key or the defaultValue
 	 *  if undefined
-	 * @exception if the attribute with the given key exists
+	 * @throws CoreException if the attribute with the given key exists
 	 *  but is not a <code>boolean</code>
 	 */
 	protected boolean getBooleanAttribute(String key, boolean defaultValue) throws CoreException {
@@ -162,14 +163,13 @@ public class LaunchConfigurationInfo {
 		if (attr != null) {
 			if (attr instanceof Boolean) {
 				return ((Boolean)attr).booleanValue();
-			} else {
-				throw new DebugException(
-					new Status(
-					 Status.ERROR, DebugPlugin.getUniqueIdentifier(),
-					 DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.getString("LaunchConfigurationInfo.Attribute_{0}_is_not_of_type_boolean._3"), new String[] {key}), null //$NON-NLS-1$
-					)
-				);
-			}
+			} 
+			throw new DebugException(
+				new Status(
+				 IStatus.ERROR, DebugPlugin.getUniqueIdentifier(),
+				 DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.LaunchConfigurationInfo_Attribute__0__is_not_of_type_boolean__3, new String[] {key}), null //$NON-NLS-1$
+				)
+			);
 		}
 		return defaultValue;
 	}
@@ -180,7 +180,7 @@ public class LaunchConfigurationInfo {
 	 * 
 	 * @return attribute specified by given key or the defaultValue
 	 *  if undefined
-	 * @exception if the attribute with the given key exists
+	 * @throws CoreException if the attribute with the given key exists
 	 *  but is not a <code>java.util.List</code>
 	 */
 	protected List getListAttribute(String key, List defaultValue) throws CoreException {
@@ -188,14 +188,13 @@ public class LaunchConfigurationInfo {
 		if (attr != null) {
 			if (attr instanceof List) {
 				return (List)attr;
-			} else {
-				throw new DebugException(
-					new Status(
-					 Status.ERROR, DebugPlugin.getUniqueIdentifier(),
-					 DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.getString("LaunchConfigurationInfo.Attribute_{0}_is_not_of_type_java.util.List._1"), new String[] {key}), null //$NON-NLS-1$
-					)
-				);
-			}
+			} 
+			throw new DebugException(
+				new Status(
+				 IStatus.ERROR, DebugPlugin.getUniqueIdentifier(),
+				 DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.LaunchConfigurationInfo_Attribute__0__is_not_of_type_java_util_List__1, new String[] {key}), null //$NON-NLS-1$
+				)
+			);
 		}
 		return defaultValue;
 	}
@@ -206,7 +205,7 @@ public class LaunchConfigurationInfo {
 	 * 
 	 * @return attribute specified by given key or the defaultValue
 	 *  if undefined
-	 * @exception if the attribute with the given key exists
+	 * @throws CoreException if the attribute with the given key exists
 	 *  but is not a <code>java.util.Map</code>
 	 */
 	protected Map getMapAttribute(String key, Map defaultValue) throws CoreException {
@@ -214,14 +213,13 @@ public class LaunchConfigurationInfo {
 		if (attr != null) {
 			if (attr instanceof Map) {
 				return (Map)attr;
-			} else {
-				throw new DebugException(
-					new Status(
-					 Status.ERROR, DebugPlugin.getUniqueIdentifier(),
-					 DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.getString("LaunchConfigurationInfo.Attribute_{0}_is_not_of_type_java.util.Map._1"), new String[] {key}), null //$NON-NLS-1$
-					)
-				);
-			}
+			} 
+			throw new DebugException(
+				new Status(
+				 IStatus.ERROR, DebugPlugin.getUniqueIdentifier(),
+				 DebugException.REQUEST_FAILED, MessageFormat.format(DebugCoreMessages.LaunchConfigurationInfo_Attribute__0__is_not_of_type_java_util_Map__1, new String[] {key}), null //$NON-NLS-1$
+				)
+			);
 		}
 		return defaultValue;
 	}
@@ -258,7 +256,7 @@ public class LaunchConfigurationInfo {
 	}
 	
 	/**
-	 * Returns a copy of this info's attriubte map.
+	 * Returns a copy of this info's attribute map.
 	 * 
 	 * @return a copy of this info's attribute map
 	 */
@@ -271,7 +269,7 @@ public class LaunchConfigurationInfo {
 	 * working copy's should use this API.
 	 * 
 	 * @param key attribute key
-	 * @param value attribuet value
+	 * @param value attribute value
 	 */
 	protected void setAttribute(String key, Object value) {
 		if (value == null) {
@@ -285,11 +283,14 @@ public class LaunchConfigurationInfo {
 	 * Returns the content of this info as XML
 	 * 
 	 * @return the content of this info as XML
-	 * @exception IOException if an exception occurs creating the XML
+	 * @throws CoreException if a attribute has been set with a null key
+	 * @throws IOException if an exception occurs creating the XML
+	 * @throws ParserConfigurationException if an exception occurs creating the XML
+	 * @throws TransformerException if an exception occurs creating the XML
 	 */
-	protected String getAsXML() throws IOException {
+	protected String getAsXML() throws CoreException, IOException, ParserConfigurationException, TransformerException {
 
-		Document doc = new DocumentImpl();
+		Document doc = LaunchManager.getDocument();
 		Element configRootElement = doc.createElement("launchConfiguration"); //$NON-NLS-1$
 		doc.appendChild(configRootElement);
 		
@@ -298,6 +299,14 @@ public class LaunchConfigurationInfo {
 		Iterator keys = getAttributeTable().keySet().iterator();
 		while (keys.hasNext()) {
 			String key = (String)keys.next();
+			if (key == null) {
+				throw new DebugException(
+					new Status(
+						IStatus.ERROR, DebugPlugin.getUniqueIdentifier(),
+						DebugException.REQUEST_FAILED, DebugCoreMessages.LaunchConfigurationInfo_36, null //$NON-NLS-1$
+					)
+				);
+			}
 			Object value = getAttributeTable().get(key);
 			if (value == null) {
 				continue;
@@ -372,25 +381,25 @@ public class LaunchConfigurationInfo {
 		String id = root.getAttribute("type"); //$NON-NLS-1$
 		if (id == null) {
 			throw getInvalidFormatDebugException();
-		} else {
-			ILaunchConfigurationType type = DebugPlugin.getDefault().getLaunchManager().getLaunchConfigurationType(id);
-			if (type == null) {
-				String message= MessageFormat.format(DebugCoreMessages.getString("LaunchConfigurationInfo.Launch_configuration_type_id___{0}___does_not_exist._nPossible_causes__n_tMissing_specification_of_a_launch_type_(missing_plugin)_n_tIncorrect_launch_configuration_XML_1"), new Object[]{id}); //$NON-NLS-1$
-				throw new DebugException(
-						new Status(
-						 Status.ERROR, DebugPlugin.getUniqueIdentifier(),
-						 DebugException.REQUEST_FAILED, message, null)
-					);
-			}
-			setType(type);
+		} 
+		
+		ILaunchConfigurationType type = DebugPlugin.getDefault().getLaunchManager().getLaunchConfigurationType(id);
+		if (type == null) {
+			String message= MessageFormat.format(DebugCoreMessages.LaunchConfigurationInfo_missing_type, new Object[]{id}); //$NON-NLS-1$
+			throw new DebugException(
+					new Status(
+					 IStatus.ERROR, DebugPlugin.getUniqueIdentifier(),
+					 DebugException.MISSING_LAUNCH_CONFIGURATION_TYPE, message, null)
+				);
 		}
+		setType(type);
 		
 		NodeList list = root.getChildNodes();
 		int length = list.getLength();
 		for (int i = 0; i < length; ++i) {
 			Node node = list.item(i);
-			short type = node.getNodeType();
-			if (type == Node.ELEMENT_NODE) {
+			short nodeType = node.getNodeType();
+			if (nodeType == Node.ELEMENT_NODE) {
 				Element element = (Element) node;
 				String nodeName = element.getNodeName();
 				
@@ -490,8 +499,8 @@ public class LaunchConfigurationInfo {
 		return 
 			new DebugException(
 				new Status(
-				 Status.ERROR, DebugPlugin.getUniqueIdentifier(),
-				 DebugException.REQUEST_FAILED, DebugCoreMessages.getString("LaunchConfigurationInfo.Invalid_launch_configuration_XML._10"), null //$NON-NLS-1$
+				 IStatus.ERROR, DebugPlugin.getUniqueIdentifier(),
+				 DebugException.REQUEST_FAILED, DebugCoreMessages.LaunchConfigurationInfo_Invalid_launch_configuration_XML__10, null //$NON-NLS-1$
 				)
 			);
 	}
