@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -11,9 +11,8 @@
 package org.eclipse.debug.internal.ui.actions;
 
  
-import org.eclipse.debug.core.model.IDebugElement;
 import org.eclipse.debug.ui.IDebugView;
-import org.eclipse.jface.viewers.TreeViewer;
+import org.eclipse.swt.widgets.TreeItem;
 
 /**
  * Used to copy the values of variables to the clipboard from
@@ -22,17 +21,10 @@ import org.eclipse.jface.viewers.TreeViewer;
 public class CopyVariablesToClipboardActionDelegate extends CopyToClipboardActionDelegate {
 
 	/**
-	 * Only append children that are visible in the tree viewer
+	 * Only append children that are expanded in the tree viewer
 	 */
-	protected boolean shouldAppendChildren(Object e) {
-		return((TreeViewer)getViewer()).getExpandedState(e);
-	}
-	
-	/**
-	 * @see AbstractDebugActionDelegate#isEnabledFor(Object)
-	 */
-	protected boolean isEnabledFor(Object element) {
-		return element instanceof IDebugElement;
+	protected boolean shouldAppendChildren(TreeItem item) {
+		return item.getExpanded();
 	}
 	
 	protected String getActionId() {
