@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -42,6 +42,7 @@ import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.graphics.Font;
 import org.eclipse.swt.graphics.Image;
+import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.Button;
@@ -89,6 +90,11 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 	 */
 	private Composite fVisibleArea;
 	
+	/**
+	 * Name label widget
+	 */
+	private Label fNameLabel;
+	
 	/**
 	 * Name text widget
 	 */
@@ -121,7 +127,7 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 	private int fCurrentTabIndex = -1;
 	
 	/**
-	 * Apply & Rever buttons
+	 * Apply & Revert buttons
 	 */
 	private Button fApplyButton;
 	private Button fRevertButton;
@@ -132,6 +138,17 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 	private boolean fDisposingTabs = false;
 	private boolean fInitializingTabs = false;
 
+	/**
+	 * Controls when the redraw flag is set on the visible area
+	 */
+	private boolean fRedraw = true;
+
+	/**
+	 * The description of the currently selected launch configuration or
+	 * launch configuration type or <code>null</code> if none.
+	 */
+	private String fDescription = null;
+
 	/**
 	 * Constructs a viewer in the given composite, contained by the given
 	 * launch configuration dialog.
@@ -191,11 +208,11 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 		container.setLayoutData(gd);
 		setVisibleArea(container);
 
-		Label nameLabel = new Label(container, SWT.HORIZONTAL | SWT.LEFT);
-		nameLabel.setText(LaunchConfigurationsMessages.getString("LaunchConfigurationDialog.&Name__16")); //$NON-NLS-1$
+		fNameLabel = new Label(container, SWT.HORIZONTAL | SWT.LEFT);
+		fNameLabel.setText(LaunchConfigurationsMessages.LaunchConfigurationDialog__Name__16); //$NON-NLS-1$
 		gd = new GridData(GridData.BEGINNING);
-		nameLabel.setLayoutData(gd);
-		nameLabel.setFont(font);
+		fNameLabel.setLayoutData(gd);
+		fNameLabel.setFont(font);
 		
 		Text nameText = new Text(container, SWT.SINGLE | SWT.BORDER);
 		gd = new GridData(GridData.FILL_HORIZONTAL);
@@ -225,18 +242,7 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 		gd.horizontalSpan = 2;
 		fTabComposite.setLayoutData(gd);
 
-		TabFolder tabFolder = new TabFolder(fTabComposite, SWT.NONE);
-		setTabFolder(tabFolder);
-		gd = new GridData(GridData.FILL_BOTH);
-		tabFolder.setLayoutData(gd);
-		tabFolder.setFont(font);
-		getTabFolder().addSelectionListener(new SelectionAdapter() {
-			public void widgetSelected(SelectionEvent event) {
-				if (!isInitializingTabs()) {
-					handleTabSelected();
-				}
-			}
-		});
+		createTabFolder(fTabComposite);
 
 		Composite buttonComp = new Composite(container, SWT.NONE);
 		GridLayout buttonCompLayout = new GridLayout();
@@ -247,7 +253,7 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 		buttonComp.setLayoutData(gd);
 
 		setApplyButton(new Button(buttonComp, SWT.PUSH));
-		getApplyButton().setText(LaunchConfigurationsMessages.getString("LaunchConfigurationDialog.&Apply_17")); //$NON-NLS-1$
+		getApplyButton().setText(LaunchConfigurationsMessages.LaunchConfigurationDialog__Apply_17); //$NON-NLS-1$
 		gd = new GridData(GridData.HORIZONTAL_ALIGN_END);
 		getApplyButton().setLayoutData(gd);
 		getApplyButton().setFont(font);
@@ -259,7 +265,7 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 		});
 
 		setRevertButton(new Button(buttonComp, SWT.PUSH));
-		getRevertButton().setText(LaunchConfigurationsMessages.getString("LaunchConfigurationDialog.Revert_2"));   //$NON-NLS-1$
+		getRevertButton().setText(LaunchConfigurationsMessages.LaunchConfigurationDialog_Revert_2);   //$NON-NLS-1$
 		gd = new GridData(GridData.HORIZONTAL_ALIGN_END);
 		getRevertButton().setLayoutData(gd);
 		getRevertButton().setFont(font);
@@ -272,6 +278,28 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 
 	}
 	
+	private void createTabFolder(Composite parent) {
+		Point size = null;
+		if (fTabFolder != null) {
+			size = fTabFolder.getSize();
+			fTabFolder.dispose();
+		}
+		fTabFolder = new TabFolder(parent, SWT.NONE);
+		GridData gd = new GridData(GridData.FILL_BOTH);
+		fTabFolder.setLayoutData(gd);
+		fTabFolder.setFont(parent.getFont());
+		if (size != null) {
+			fTabFolder.setSize(size);
+		}
+		getTabFolder().addSelectionListener(new SelectionAdapter() {
+			public void widgetSelected(SelectionEvent event) {
+				if (!isInitializingTabs()) {
+					handleTabSelected();
+				}
+			}
+		});		
+	}
+	
 	/**
 	 * Sets the apply button
 	 */
@@ -300,13 +328,6 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 		return fRevertButton;
 	}	
 	
-	/**
-	 * Sets the tab folder
-	 */
-	private void setTabFolder(TabFolder tabFolder) {
-		fTabFolder = tabFolder;
-	}
-	
 	/**
 	 * Sets the tab folder
 	 */
@@ -368,15 +389,18 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 	public ISelection getSelection() {
 		if (getActiveTab() == null) {
 			return new StructuredSelection();
-		} else {
-			return new StructuredSelection(getActiveTab());
-		}
+		} 
+		return new StructuredSelection(getActiveTab());
 	}
 
 	/**
 	 * @see org.eclipse.jface.viewers.Viewer#refresh()
 	 */
 	public void refresh() {
+		if (isInitializingTabs()) {
+			return;
+		}
+		
 		ILaunchConfigurationTab[] tabs = getTabs();
 		if (!isInitializingTabs() && tabs != null) {
 			// update the working copy from the active tab
@@ -386,6 +410,7 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 			TabFolder folder = getTabFolder();
 			for (int i = 0; i < tabs.length; i++) {
 				ILaunchConfigurationTab tab = tabs[i];
+				tab.isValid(getWorkingCopy());
 				boolean error = tab.getErrorMessage() != null;
 				TabItem item = folder.getItem(i);
 				setTabIcon(item, error, tab);
@@ -420,9 +445,8 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 		if (input == null) {
 			if (fInput == null) {
 				return;
-			} else {
-				inputChanged(input);
 			}
+			inputChanged(input);
 		} else {
 			if (!input.equals(fInput)) {
 				inputChanged(input);
@@ -437,47 +461,78 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 	 */
 	protected void inputChanged(Object input) {
 		fInput = input;
-		if (input instanceof ILaunchConfiguration) {
-			ILaunchConfiguration configuration = (ILaunchConfiguration)input;
-			setOriginal(configuration);
-			try {
-				setWorkingCopy(configuration.getWorkingCopy());
-			} catch (CoreException e) {
-				errorDialog(e);
+		Runnable r = new Runnable() {
+			public void run() {
+				try {
+					if (fInput instanceof ILaunchConfiguration) {
+						ILaunchConfiguration configuration = (ILaunchConfiguration)fInput;
+						setOriginal(configuration);
+						setWorkingCopy(configuration.getWorkingCopy());
+						displayInstanceTabs();
+					} else if (fInput instanceof ILaunchConfigurationType) {
+						ILaunchConfiguration configuration = LaunchConfigurationManager.getSharedTypeConfig((ILaunchConfigurationType)fInput);
+						setOriginal(configuration);
+						setWorkingCopy(configuration.getWorkingCopy());
+						displaySharedTabs();
+					} else {
+						setNoInput();
+					}
+					setRedraw(true);
+				} catch (CoreException ce) {
+					errorDialog(ce);
+					setNoInput();
+					setRedraw(true);
+				}
 			}
-			displayTabs();
-		} else {
-			setOriginal(null);
-			setWorkingCopy(null);
-			getVisibleArea().setVisible(false);
-			disposeExistingTabs();
-		}
+		};
+		BusyIndicator.showWhile(getShell().getDisplay(), r);
 	}
 	
+	private void setNoInput() {
+		setOriginal(null);
+		setWorkingCopy(null);
+		getVisibleArea().setVisible(false);
+		disposeExistingTabs();				
+	}
+	
+    protected void setFocusOnName() {
+        fNameWidget.setFocus();
+        
+    }
+    
+	private void setRedraw(boolean b) {
+		if (fRedraw != b) {
+			fRedraw = b;
+			getVisibleArea().setRedraw(fRedraw);
+		}	
+	}	
 	/**
 	 * Displays tabs for the current working copy
 	 */
-	protected void displayTabs() {
+	protected void displayInstanceTabs() {
 		// Turn on initializing flag to ignore message updates
 		setInitializingTabs(true);
 
 		ILaunchConfigurationType type = null;
 		try {
 			type = getWorkingCopy().getType();
-			showTabsFor(type);
+			showInstanceTabsFor(type);
 		} catch (CoreException e) {
 			errorDialog(e);
 			setInitializingTabs(false);
 			return;
 		}
 
+		// show the name area
+		fNameLabel.setVisible(true);
+		fNameWidget.setVisible(true);
 		// Update the name field before to avoid verify error
 		getNameWidget().setText(getWorkingCopy().getName());
 
 		// Retrieve the current tab group.  If there is none, clean up and leave
 		ILaunchConfigurationTabGroup tabGroup = getTabGroup();
 		if (tabGroup == null) {
-			IStatus status = new Status(IStatus.ERROR, DebugUIPlugin.getUniqueIdentifier(), 0, MessageFormat.format(LaunchConfigurationsMessages.getString("LaunchConfigurationTabGroupViewer.No_tabs_defined_for_launch_configuration_type_{0}_1"), new String[]{type.getName()}), null); //$NON-NLS-1$
+			IStatus status = new Status(IStatus.ERROR, DebugUIPlugin.getUniqueIdentifier(), 0, MessageFormat.format(LaunchConfigurationsMessages.LaunchConfigurationTabGroupViewer_No_tabs_defined_for_launch_configuration_type__0__1, new String[]{type.getName()}), null); //$NON-NLS-1$
 			CoreException e = new CoreException(status);
 			errorDialog(e);
 			setInitializingTabs(false);
@@ -498,44 +553,166 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 		if (!getVisibleArea().isVisible()) {
 			getVisibleArea().setVisible(true);
 		}
-
+		
 		refreshStatus();		
 	}
 	
+	/**
+	 * Displays tabs for the current config type
+	 */
+	protected void displaySharedTabs() {
+		// Turn on initializing flag to ignore message updates
+		setInitializingTabs(true);
+
+		ILaunchConfigurationType type = null;
+		try {
+			type = getWorkingCopy().getType();
+			showSharedTabsFor(type);
+		} catch (CoreException e) {
+			errorDialog(e);
+			setInitializingTabs(false);
+			return;
+		}
+
+		// hide the name area
+		fNameLabel.setVisible(false);
+		fNameWidget.setVisible(false);
+		// Update the name field before to avoid verify error
+		getNameWidget().setText(getWorkingCopy().getName());
+
+		// Retrieve the current tab group.  If there is none, clean up and leave
+		ILaunchConfigurationTabGroup tabGroup = getTabGroup();
+		if (tabGroup == null) {
+			IStatus status = new Status(IStatus.ERROR, DebugUIPlugin.getUniqueIdentifier(), 0, MessageFormat.format(LaunchConfigurationsMessages.LaunchConfigurationTabGroupViewer_No_tabs_defined_for_launch_configuration_type__0__1, new String[]{type.getName()}), null); //$NON-NLS-1$
+			CoreException e = new CoreException(status);
+			errorDialog(e);
+			setInitializingTabs(false);
+			return;
+		}
+
+		// Update the tabs with the new working copy
+		tabGroup.initializeFrom(getWorkingCopy());
+
+		// Update the name field after in case client changed it
+		getNameWidget().setText(getWorkingCopy().getName());
+		
+		fCurrentTabIndex = getTabFolder().getSelectionIndex();
+
+		// Turn off initializing flag to update message
+		setInitializingTabs(false);
+		
+		if (!getVisibleArea().isVisible()) {
+			getVisibleArea().setVisible(true);
+		}
+		
+		fDescription = getDescription(null);
+		
+		refreshStatus();		
+	}	
+	
 	/**
 	 * Populate the tabs in the configuration edit area to be appropriate to the current
 	 * launch configuration type.
 	 */
-	private void showTabsFor(ILaunchConfigurationType configType) {
+	private void showInstanceTabsFor(ILaunchConfigurationType configType) {
 
 		// Don't do any work if the current tabs are for the current config type
-		if (getTabType() != null && getTabType().equals(configType)) {
+		if (getTabType() != null && getTabType().equals(configType) && !(getTabGroup() instanceof PerspectiveTabGroup)) {
 			return;
 		}
-
-		// Avoid flicker
-		getControl().setRedraw(false);
-
-		// Dispose the current tabs
-		disposeExistingTabs();
-
+		
+		// try to keep on same tab
+		Class tabKind = null;
+		if (getActiveTab() != null) {
+			tabKind = getActiveTab().getClass();
+		}
+		
 		// Build the new tabs
 		ILaunchConfigurationTabGroup group = null;
 		try {
 			group = createGroup(configType);
-			setTabGroup(group);
 		} catch (CoreException ce) {
-			DebugUIPlugin.errorDialog(getShell(), LaunchConfigurationsMessages.getString("LaunchConfigurationDialog.Error_19"), LaunchConfigurationsMessages.getString("LaunchConfigurationDialog.Exception_occurred_creating_launch_configuration_tabs_27"),ce); //$NON-NLS-1$ //$NON-NLS-2$
+			DebugUIPlugin.errorDialog(getShell(), LaunchConfigurationsMessages.LaunchConfigurationDialog_Error_19, LaunchConfigurationsMessages.LaunchConfigurationDialog_Exception_occurred_creating_launch_configuration_tabs_27,ce); //$NON-NLS-1$ //$NON-NLS-2$
 			return;
 		}
 
-		// Create the Control for each tab
+		showTabsFor(group);
+		setTabGroup(group);
+		setTabType(configType);
+		
+		// select same tab as before, if possible
+		ILaunchConfigurationTab[] tabs = getTabs();
+		for (int i = 0; i < tabs.length; i++) {
+			ILaunchConfigurationTab tab = tabs[i];
+			if (tab.getClass().equals(tabKind)) {
+				setActiveTab(tab);
+				break;
+			}
+		}
+		
+		fDescription = getDescription(configType);
+	}	
+
+	/**
+	 * Returns the description of the given configuration type
+	 * in the current mode or <code>null</code> if none.
+	 * 
+	 * @param configType the config type
+	 * @return the description of the given configuration type or <code>null</code>
+	 */
+	private String getDescription(ILaunchConfigurationType configType) {
+		String description = null;
+		if(configType != null) {
+			String mode = fDialog.getMode();
+			description = LaunchConfigurationPresentationManager.getDefault().getDescription(configType, mode);
+		}	
+		if (description == null)
+			description = ""; //$NON-NLS-1$
+		return description;
+	}
+	
+	/**
+	 * Populate the tabs in the configuration edit area for the shared info
+	 * for the given launch config type.
+	 */
+	private void showSharedTabsFor(ILaunchConfigurationType configType) {
+
+		// Don't do any work if the current tabs are for the current config type
+		if (getTabType() != null && getTabType().equals(configType) && (getTabGroup() instanceof PerspectiveTabGroup)) {
+			return;
+		}		
+		
+		// Build the new tabs
+		ILaunchConfigurationTabGroup group = new PerspectiveTabGroup(configType);
+		group.createTabs(getLaunchConfigurationDialog(), getLaunchConfigurationDialog().getMode());
 		ILaunchConfigurationTab[] tabs = group.getTabs();
+		for (int i = 0; i < tabs.length; i++) {
+			tabs[i].setLaunchConfigurationDialog(getLaunchConfigurationDialog());
+		}
+				
+		showTabsFor(group);
+		setTabType(configType);
+		setTabGroup(group);		
+	}		
+
+	/**
+	 * Create the tabs in the configuration edit area for the given tab group.
+	 */
+	private void showTabsFor(ILaunchConfigurationTabGroup tabGroup) {
+		// turn off redraw
+		setRedraw(false);
+		// Dispose the current tabs
+		disposeExistingTabs();
+
+		setTabGroup(tabGroup);
+
+		// Create the Control for each tab
+		ILaunchConfigurationTab[] tabs = tabGroup.getTabs();
 		for (int i = 0; i < tabs.length; i++) {
 			TabItem tab = new TabItem(getTabFolder(), SWT.NONE);
 			String name = tabs[i].getName();
 			if (name == null) {
-				name = LaunchConfigurationsMessages.getString("LaunchConfigurationDialog.unspecified_28"); //$NON-NLS-1$
+				name = LaunchConfigurationsMessages.LaunchConfigurationDialog_unspecified_28; //$NON-NLS-1$
 			}
 			tab.setText(name);
 			Image image = tabs[i].getImage();
@@ -546,10 +723,7 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 				tab.setControl(control);
 			}
 		}
-		
-		setTabGroup(group);
-		setTabType(configType);
-		getControl().setRedraw(true);
+
 	}	
 	
 	/**
@@ -566,7 +740,7 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 			public void run() {
 				ILaunchConfigurationTabGroup tabGroup = null;
 				try {
-					tabGroup = LaunchConfigurationPresentationManager.getDefault().getTabGroup(configType);
+					tabGroup = LaunchConfigurationPresentationManager.getDefault().getTabGroup(configType, getLaunchConfigurationDialog().getMode());
 					finalArray[0] = tabGroup;
 				} catch (CoreException ce) {
 					finalArray[1] = ce;
@@ -673,8 +847,10 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 	 * launch config dialog.
 	 */
 	protected void refreshStatus() {
-		getLaunchConfigurationDialog().updateButtons();
-		getLaunchConfigurationDialog().updateMessage();
+		if (!isInitializingTabs()) {
+			getLaunchConfigurationDialog().updateButtons();
+			getLaunchConfigurationDialog().updateMessage();
+		}
 	}	
 	
 	/**
@@ -728,6 +904,9 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 	 * </p>
 	 */
 	public boolean canSave() {
+		if (isInitializingTabs()) {
+			return false;
+		}
 		// First make sure that name doesn't prevent saving the config
 		try {
 			verifyName();
@@ -752,6 +931,14 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 	 * @see ILaunchConfigurationDialog#canLaunch()
 	 */
 	public boolean canLaunch() {
+		if(isInitializingTabs()) {
+			return false;
+		}
+		
+		if (getActiveTab() instanceof PerspectivesTab) {
+			return false;
+		}
+		
 		if (getWorkingCopy() == null) {
 			return false;
 		}
@@ -777,6 +964,10 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 	 * Returns the current error message or <code>null</code> if none.
 	 */
 	public String getErrorMesssage() {
+		if (isInitializingTabs()) {
+			return null;
+		}
+		
 		if (getWorkingCopy() == null) {
 			return null;
 		}
@@ -790,10 +981,8 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 		ILaunchConfigurationTab activeTab = getActiveTab();
 		if (activeTab == null) {
 			return null;
-		} else {
-			activeTab.isValid(getWorkingCopy());
-			message = activeTab.getErrorMessage();
-		}
+		} 
+		message = activeTab.getErrorMessage();
 		if (message != null) {
 			return message;
 		}
@@ -804,7 +993,6 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 			if (tab == activeTab) {
 				continue;
 			}
-			tab.isValid(getWorkingCopy());
 			message = tab.getErrorMessage();
 			if (message != null) {
 				StringBuffer temp= new StringBuffer();
@@ -833,50 +1021,80 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 	
 	/**
 	 * Returns the current message or <code>null</code> if none.
+	 * @return Returns an appropriate message for display to user. The message returned will be:
+	 * The message defined by the visible tab,
+	 * or The tab group description for the particular launch mode,
+	 * or The generic tab group description,
+	 * or <code>null</code> if no message is defined 
 	 */
-	public String getMesssage() {
-		ILaunchConfigurationTab tab = getActiveTab();
-		if (tab == null) {
+	public String getMessage() {
+		if (isInitializingTabs()) {
 			return null;
-		} else {
-			return tab.getMessage();
 		}
+		
+		String message = fDescription;
+		
+		ILaunchConfigurationTab tab = getActiveTab();
+		if (tab != null) {
+			String tabMessage = tab.getMessage();
+			if (tabMessage != null) {
+				message = tabMessage;
+			}
+		}
+		
+		return message;
 	}	
 		
 	/**
 	 * Verify that the launch configuration name is valid.
 	 */
 	protected void verifyName() throws CoreException {
-		String currentName = getNameWidget().getText().trim();
-
-		// If there is no name, complain
-		if (currentName.length() < 1) {
-			throw new CoreException(new Status(IStatus.ERROR,
-												 DebugUIPlugin.getUniqueIdentifier(),
-												 0,
-												 LaunchConfigurationsMessages.getString("LaunchConfigurationDialog.Name_required_for_launch_configuration_11"), //$NON-NLS-1$
-												 null));
-		}
-
-		// See if name contains any 'illegal' characters
-		IStatus status = ResourcesPlugin.getWorkspace().validateName(currentName, IResource.FILE);
-		if (status.getCode() != IStatus.OK) {
-			throw new CoreException(new Status(IStatus.ERROR,
-												 DebugUIPlugin.getDefault().getDescriptor().getUniqueIdentifier(),
-												 0,
-												 status.getMessage(),
-												 null));
-		}
-
-		// Otherwise, if there's already a config with the same name, complain
-		if (!getOriginal().getName().equals(currentName)) {
-			if (getLaunchManager().isExistingLaunchConfigurationName(currentName)) {
+		if (fNameWidget.isVisible()) {
+			String currentName = getNameWidget().getText().trim();
+	
+			// If there is no name, complain
+			if (currentName.length() < 1) {
 				throw new CoreException(new Status(IStatus.ERROR,
-													 DebugUIPlugin.getDefault().getDescriptor().getUniqueIdentifier(),
+													 DebugUIPlugin.getUniqueIdentifier(),
 													 0,
-													 LaunchConfigurationsMessages.getString("LaunchConfigurationDialog.Launch_configuration_already_exists_with_this_name_12"), //$NON-NLS-1$
+													 LaunchConfigurationsMessages.LaunchConfigurationDialog_Name_required_for_launch_configuration_11, //$NON-NLS-1$
 													 null));
 			}
+	
+			// See if name contains any 'illegal' characters
+			IStatus status = ResourcesPlugin.getWorkspace().validateName(currentName, IResource.FILE);
+			if (status.getCode() != IStatus.OK) {
+				throw new CoreException(new Status(IStatus.ERROR,
+													 DebugUIPlugin.getUniqueIdentifier(),
+													 0,
+													 status.getMessage(),
+													 null));
+			}
+			
+			// See if name contains any characters that we deem illegal.
+			// '@' and '&' are disallowed because they corrupt menu items.
+			char[] disallowedChars = new char[] { '@', '&' };
+			for (int i = 0; i < disallowedChars.length; i++) {
+				char c = disallowedChars[i];
+				if (currentName.indexOf(c) > -1) {
+					throw new CoreException(new Status(IStatus.ERROR,
+														DebugUIPlugin.getUniqueIdentifier(),
+														0,
+														MessageFormat.format(LaunchConfigurationsMessages.LaunchConfigurationTabGroupViewer_0, new String[] { new String(new char[] {c}), currentName }), //$NON-NLS-1$
+														null));
+				}
+			}
+	
+			// Otherwise, if there's already a config with the same name, complain
+			if (!getOriginal().getName().equals(currentName)) {
+				if (getLaunchManager().isExistingLaunchConfigurationName(currentName)) {
+					throw new CoreException(new Status(IStatus.ERROR,
+														 DebugUIPlugin.getUniqueIdentifier(),
+														 0,
+														 LaunchConfigurationsMessages.LaunchConfigurationDialog_Launch_configuration_already_exists_with_this_name_12, //$NON-NLS-1$
+														 null));
+				}
+			}
 		}
 	}
 	
@@ -902,6 +1120,7 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 		for (int i = 0; i < oldTabs.length; i++) {
 			oldTabs[i].dispose();
 		}
+		createTabFolder(fTabComposite);
 		disposeTabGroup();
 		setDisposingTabs(false);
 	}	
@@ -956,7 +1175,7 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 	 * the tab being entered.
 	 */
 	protected void handleTabSelected() {
-		if (isDisposingTabs()) {
+		if (isDisposingTabs() || isInitializingTabs()) {
 			return;
 		}
 		ILaunchConfigurationTab[] tabs = getTabs();
@@ -967,12 +1186,8 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 			ILaunchConfigurationTab tab = tabs[fCurrentTabIndex];
 			ILaunchConfigurationWorkingCopy wc = getWorkingCopy();
 			if (wc != null) {
-				setInitializingTabs(true);
-				// apply changes when leaving a tab
-				tab.performApply(wc);
-				// re-initialize a tab when entering it
-				getActiveTab().initializeFrom(wc);
-				setInitializingTabs(false);
+				tab.deactivated(wc);
+				getActiveTab().activated(wc);
 			}
 		}
 		fCurrentTabIndex = getTabFolder().getSelectionIndex();
@@ -1013,7 +1228,7 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 			}
 			updateButtons();
 		} catch (CoreException e) {
-			DebugUIPlugin.errorDialog(getShell(), LaunchConfigurationsMessages.getString("LaunchConfigurationDialog.Launch_Configuration_Error_46"), LaunchConfigurationsMessages.getString("LaunchConfigurationDialog.Exception_occurred_while_saving_launch_configuration_47"), e); //$NON-NLS-1$ //$NON-NLS-2$
+			DebugUIPlugin.errorDialog(getShell(), LaunchConfigurationsMessages.LaunchConfigurationDialog_Launch_Configuration_Error_46, LaunchConfigurationsMessages.LaunchConfigurationDialog_Exception_occurred_while_saving_launch_configuration_47, e); //$NON-NLS-1$ //$NON-NLS-2$
 			return;
 		}
 	}
@@ -1022,7 +1237,11 @@ public class LaunchConfigurationTabGroupViewer extends Viewer {
 	 * Notification that the 'Revert' button has been pressed
 	 */
 	protected void handleRevertPressed() {
-		inputChanged(getOriginal());
+		if (getActiveTab() instanceof PerspectivesTab) {
+			inputChanged(getTabType());	
+		} else {
+			inputChanged(getOriginal());
+		}
 	}	
 	
 	/**
