@@ -17,8 +17,9 @@ class MethodVerifier15 extends MethodVerifier {
 MethodVerifier15(LookupEnvironment environment) {
 	super(environment);
 }
-boolean areMethodsEqual(MethodBinding one, MethodBinding substituteTwo) {
-	return areParametersEqual(one, substituteTwo) && !doTypeVariablesClash(one, substituteTwo);
+boolean areMethodsEqual(MethodBinding one, MethodBinding two) {
+	MethodBinding sub = computeSubstituteMethod(two, one);
+	return sub != null && doesSubstituteMethodOverride(one, sub) && areReturnTypesEqual(one, sub);
 }
 boolean areParametersEqual(MethodBinding one, MethodBinding two) {
 	TypeBinding[] oneArgs = one.parameters;
@@ -79,17 +80,6 @@ boolean areTypesEqual(TypeBinding one, TypeBinding two) {
 	//		return ((UnresolvedReferenceBinding) two).resolvedType == one;
 	return false; // all other type bindings are identical
 }
-boolean areTypeVariablesInterchangeable(MethodBinding one, MethodBinding two) {
-	TypeVariableBinding[] vars = one.typeVariables;
-	TypeVariableBinding[] vars2 = two.typeVariables;
-	if (vars.length != vars2.length)
-		return false;
-	
-	for (int v = vars.length; --v >= 0;)
-		if (!vars[v].isInterchangeableWith(this.environment, vars2[v]))
-			return false;
-	return true;
-}
 boolean canSkipInheritedMethods() {
 	if (this.type.superclass() != null)
 		if (this.type.superclass().isAbstract() || this.type.superclass().isParameterizedType())
@@ -126,7 +116,7 @@ void checkConcreteInheritedMethod(MethodBinding concreteMethod, MethodBinding[]
 		this.type.addSyntheticBridgeMethod(originalInherited, concreteMethod.original());
 	}
 }
-void checkForBridgeMethod(MethodBinding currentMethod, MethodBinding inheritedMethod, MethodBinding[] otherInheritedMethods) {
+void checkForBridgeMethod(MethodBinding currentMethod, MethodBinding inheritedMethod, MethodBinding[] allInheritedMethods) {
 	if (currentMethod.isVarargs() != inheritedMethod.isVarargs())
 		problemReporter(currentMethod).varargsConflict(currentMethod, inheritedMethod, this.type);
 
@@ -148,25 +138,20 @@ void checkForBridgeMethod(MethodBinding currentMethod, MethodBinding inheritedMe
 	}
 
 	if (this.type.addSyntheticBridgeMethod(originalInherited, currentMethod.original()) != null) {
-		for (int i = 0, l = otherInheritedMethods.length; i < l; i++) {
-			if (otherInheritedMethods[i] != null) {
-				MethodBinding otherOriginal = otherInheritedMethods[i].original();
-				if (otherOriginal != otherInheritedMethods[i] && detectInheritedMethodClash(originalInherited, otherOriginal))
-					return;
-			}
-		}
+		for (int i = 0, l = allInheritedMethods.length; i < l; i++) {
+			MethodBinding otherInheritedMethod = allInheritedMethods[i];
+			MethodBinding otherOriginal = otherInheritedMethod.original();
+			if (otherOriginal == originalInherited || otherOriginal == otherInheritedMethod) continue;
 
-		// there is an ordering issue with the comparison in checkMethods
-		// its possible that compareTo(X) is walked first & removes Comparable.compareTo(T) from the inherited list before we can compare it to compareTo(Object)
-		// its only a problem when the matching inherited method creates a bridge method which collides with an unwalked current method
-		//		class X implements Comparable<X> {
-		//			public int compareTo(Object o) { return 0; }
-		//			public int compareTo(X o) { return 1; }
-		//		}
-		MethodBinding[] toCheck = (MethodBinding[]) this.currentMethods.get(currentMethod.selector);
-		for (int i = 0, l = toCheck.length; i < l; i++)
-			if (currentMethod != toCheck[i] && detectNameClash(toCheck[i], inheritedMethod))
+			MethodBinding compareMethod = inheritedMethod instanceof ParameterizedGenericMethodBinding
+				? ((ParameterizedGenericMethodBinding) inheritedMethod).originalMethod
+				: inheritedMethod;
+			MethodBinding substitute = computeSubstituteMethod(otherInheritedMethod, compareMethod);
+			if (substitute == null || doesSubstituteMethodOverride(compareMethod, substitute))
+				continue;
+			if (detectInheritedMethodClash(originalInherited, otherOriginal))
 				return;
+		}
 	}
 }
 void checkForInheritedNameClash(MethodBinding inheritedMethod, MethodBinding otherInheritedMethod) {
@@ -185,7 +170,6 @@ void checkForInheritedNameClash(MethodBinding inheritedMethod, MethodBinding oth
 	if (!inheritedMethod.declaringClass.isInterface())
 		detectInheritedMethodClash(inheritedMethod, otherInheritedMethod);
 }
-
 void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMethod) {
 	// sent from checkMethods() to compare a current method and an inherited method that are not 'equal'
 
@@ -193,17 +177,16 @@ void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMetho
 	//		abstract class AA<E extends Comparable> { abstract void test(E element); }
 	//		class A extends AA<Integer> { public void test(Integer i) {} }
 	//		public class B extends A { public void test(Comparable i) {} }
-	// AND
 	//		interface I<E extends Comparable> { void test(E element); }
 	//		class A implements I<Integer> { public void test(Integer i) {} }
 	//		public class B extends A { public void test(Comparable i) {} }
-	// AND
+
 	//		abstract class Y implements EqualityComparable<Integer>, Equivalent<String> {
 	//			public boolean equalTo(Integer other) { return true; }
 	//		}
 	//		interface Equivalent<T> { boolean equalTo(T other); }
 	//		interface EqualityComparable<T> { boolean equalTo(T other); }
-	// AND
+
 	//		class Y implements EqualityComparable, Equivalent<String>{
 	//			public boolean equalTo(String other) { return true; }
 	//			public boolean equalTo(Object other) { return true; }
@@ -211,6 +194,15 @@ void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMetho
 	//		interface Equivalent<T> { boolean equalTo(T other); }
 	//		interface EqualityComparable { boolean equalTo(Object other); }
 
+	//		class A<T extends Number> { void m(T t) {} }
+	//		class B<S extends Integer> extends A<S> { void m(S t) {}}
+	//		class D extends B<Integer> { void m(Number t) {}    void m(Integer t) {} }
+
+	//		inheritedMethods does not include I.test since A has a valid implementation
+	//		interface I<E extends Comparable<E>> { void test(E element); }
+	//		class A implements I<Integer> { public void test(Integer i) {} }
+	//		class B extends A { public void test(Comparable i) {} }
+
 	if (currentMethod.declaringClass.isInterface()) return;
 
 	if (!detectNameClash(currentMethod, inheritedMethod)) { // check up the hierarchy for skipped inherited methods
@@ -227,12 +219,20 @@ void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMetho
 		ReferenceBinding[][] interfacesToVisit = new ReferenceBinding[3][];
 		int lastPosition = -1;
 		ReferenceBinding[] itsInterfaces = null;
-		ReferenceBinding superType = this.type.superclass;
+		ReferenceBinding superType = inheritedMethod.declaringClass;
+		if ((itsInterfaces = superType.superInterfaces()) != NoSuperInterfaces) {
+			if (++lastPosition == interfacesToVisit.length)
+				System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[lastPosition * 2][], 0, lastPosition);
+			interfacesToVisit[lastPosition] = itsInterfaces;
+		}
+		superType = superType.superclass(); // now start with its superclass
 		while (superType != null && superType.isValidBinding()) {
 			MethodBinding[] methods = superType.getMethods(currentMethod.selector);
-			for (int m = 0, n = methods.length; m < n; m++)
-				if (!areMethodsEqual(currentMethod, methods[m]) && detectNameClash(currentMethod, methods[m]))
+			for (int m = 0, n = methods.length; m < n; m++) {
+				MethodBinding substitute = computeSubstituteMethod(methods[m], currentMethod);
+				if (substitute != null && !doesSubstituteMethodOverride(currentMethod, substitute) && detectNameClash(currentMethod, substitute))
 					return;
+			}
 			if ((itsInterfaces = superType.superInterfaces()) != NoSuperInterfaces) {
 				if (++lastPosition == interfacesToVisit.length)
 					System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[lastPosition * 2][], 0, lastPosition);
@@ -247,9 +247,11 @@ void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMetho
 				superType = interfaces[j];
 				if (superType.isValidBinding()) {
 					MethodBinding[] methods = superType.getMethods(currentMethod.selector);
-					for (int m = 0, n = methods.length; m < n; m++)
-						if (!areMethodsEqual(currentMethod, methods[m]) && detectNameClash(currentMethod, methods[m]))
+					for (int m = 0, n = methods.length; m < n; m++){
+						MethodBinding substitute = computeSubstituteMethod(methods[m], currentMethod);
+						if (substitute != null && !doesSubstituteMethodOverride(currentMethod, substitute) && detectNameClash(currentMethod, substitute))
 							return;
+					}
 					if ((itsInterfaces = superType.superInterfaces()) != NoSuperInterfaces) {
 						if (++lastPosition == interfacesToVisit.length)
 							System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[lastPosition * 2][], 0, lastPosition);
@@ -265,7 +267,7 @@ void checkInheritedMethods(MethodBinding[] methods, int length) {
 	nextMethod : for (int i = 0, l = length - 1; i < l;) {
 		MethodBinding method = methods[i++];
 		for (int j = i; j <= l; j++) {
-			if (method.declaringClass == methods[j].declaringClass && doesMethodOverride(method, methods[j])) {
+			if (method.declaringClass == methods[j].declaringClass && areMethodsEqual(method, methods[j])) {
 				// found an inherited ParameterizedType that defines duplicate methods
 				problemReporter().duplicateInheritedMethods(this.type, method, methods[j]);
 				count--;
@@ -286,6 +288,77 @@ void checkInheritedMethods(MethodBinding[] methods, int length) {
 
 	super.checkInheritedMethods(methods, length);
 }
+void checkMethods() {
+	boolean mustImplementAbstractMethods = mustImplementAbstractMethods();
+	boolean skipInheritedMethods = mustImplementAbstractMethods && canSkipInheritedMethods(); // have a single concrete superclass so only check overridden methods
+	char[][] methodSelectors = this.inheritedMethods.keyTable;
+	nextSelector : for (int s = methodSelectors.length; --s >= 0;) {
+		if (methodSelectors[s] == null) continue nextSelector;
+
+		MethodBinding[] current = (MethodBinding[]) this.currentMethods.get(methodSelectors[s]);
+		if (current == null && skipInheritedMethods)
+			continue nextSelector;
+
+		MethodBinding[] inherited = (MethodBinding[]) this.inheritedMethods.valueTable[s];
+		if (inherited.length == 1 && current == null) { // handle the common case
+			if (mustImplementAbstractMethods && inherited[0].isAbstract())
+				checkAbstractMethod(inherited[0]);
+			continue nextSelector;
+		}
+
+		int index = -1;
+		MethodBinding[] matchingInherited = new MethodBinding[inherited.length];
+		byte[] foundMatch = new byte[inherited.length];
+		if (current != null) {
+			for (int i = 0, length1 = current.length; i < length1; i++) {
+				MethodBinding currentMethod = current[i];
+				for (int j = 0, length2 = inherited.length; j < length2; j++) {
+					MethodBinding inheritedMethod = computeSubstituteMethod(inherited[j], currentMethod);
+					if (inheritedMethod != null) {
+						if (foundMatch[j] == 0 && doesSubstituteMethodOverride(currentMethod, inheritedMethod)) {
+							matchingInherited[++index] = inheritedMethod;
+							foundMatch[j] = 1; // cannot null out inherited methods
+						} else {
+							checkForNameClash(currentMethod, inheritedMethod);
+						}
+					}
+				}
+				if (index >= 0) {
+					checkAgainstInheritedMethods(currentMethod, matchingInherited, index + 1, inherited); // pass in the length of matching
+					while (index >= 0) matchingInherited[index--] = null; // clear the contents of the matching methods
+				}
+			}
+		}
+
+		for (int i = 0, length = inherited.length; i < length; i++) {
+			if (foundMatch[i] == 1) continue;
+
+			MethodBinding inheritedMethod = inherited[i];
+			matchingInherited[++index] = inheritedMethod;
+			for (int j = i + 1; j < length; j++) {
+				MethodBinding otherInheritedMethod = inherited[j];
+				if (foundMatch[j] == 1 || canSkipInheritedMethods(inheritedMethod, otherInheritedMethod))
+					continue;
+				otherInheritedMethod = computeSubstituteMethod(otherInheritedMethod, inheritedMethod);
+				if (otherInheritedMethod != null) {
+					if (doesSubstituteMethodOverride(inheritedMethod, otherInheritedMethod)) {
+						matchingInherited[++index] = otherInheritedMethod;
+						foundMatch[j] = 1; // cannot null out inherited methods
+					} else {
+						checkForInheritedNameClash(inheritedMethod, otherInheritedMethod);
+					}
+				}
+			}
+			if (index == -1) continue;
+
+			if (index > 0)
+				checkInheritedMethods(matchingInherited, index + 1); // pass in the length of matching
+			else if (mustImplementAbstractMethods && index == 0 && matchingInherited[0].isAbstract())
+				checkAbstractMethod(matchingInherited[0]);
+			while (index >= 0) matchingInherited[index--] = null; // clear the previous contents of the matching methods
+		}
+	}
+}
 void checkTypeVariableMethods() {
 	char[][] methodSelectors = this.inheritedMethods.keyTable;
 	nextSelector : for (int s = methodSelectors.length; --s >= 0;) {
@@ -305,14 +378,21 @@ void checkTypeVariableMethods() {
 					if (canSkipInheritedMethods(inheritedMethod, otherInheritedMethod))
 						continue;
 					otherInheritedMethod = computeSubstituteMethod(otherInheritedMethod, inheritedMethod);
-					if (otherInheritedMethod != null && areMethodsEqual(inheritedMethod, otherInheritedMethod)) {
+					if (otherInheritedMethod != null && doesSubstituteMethodOverride(inheritedMethod, otherInheritedMethod)) {
 						matchingInherited[++index] = otherInheritedMethod;
 						inherited[j] = null; // do not want to find it again
 					}
 				}
 			}
-			if (index > 0)
-				checkInheritedMethods(matchingInherited, index + 1); // pass in the length of matching
+			if (index > 0) {
+				MethodBinding first = matchingInherited[0];
+				int count = index + 1;
+				while (--count > 0 && areReturnTypesEqual(first, matchingInherited[count])){/*empty*/}
+				if (count > 0) {  // All inherited methods do NOT have the same vmSignature
+					problemReporter().inheritedMethodsHaveIncompatibleReturnTypes(this.type, matchingInherited, index + 1);
+					continue nextSelector;
+				}
+			}
 		}
 	}
 }
@@ -331,6 +411,7 @@ MethodBinding computeSubstituteMethod(MethodBinding inheritedMethod, MethodBindi
 	int inheritedLength = inheritedTypeVariables.length;
 	TypeVariableBinding[] typeVariables = currentMethod.typeVariables;
 	int length = typeVariables.length;
+	if (length > 0 && inheritedLength != length) return inheritedMethod;
 	TypeBinding[] arguments = new TypeBinding[inheritedLength];
 	if (inheritedLength <= length) {
 		System.arraycopy(typeVariables, 0, arguments, 0, inheritedLength);
@@ -341,9 +422,32 @@ MethodBinding computeSubstituteMethod(MethodBinding inheritedMethod, MethodBindi
 	}
 	ParameterizedGenericMethodBinding substitute =
 		new ParameterizedGenericMethodBinding(inheritedMethod, arguments, this.environment);
-	for (int i = 0; i < inheritedLength; i++)
-	    if (inheritedTypeVariables[i].boundCheck(substitute, arguments[i]) != TypeConstants.OK)
-	    	return inheritedMethod; // incompatible due to bound check
+
+	// interface I { <T> void foo(T t); }
+	// class X implements I { public <T extends I> void foo(T t) {} }
+	// for the above case, we do not want to answer the substitute method since its not a match
+	for (int i = 0; i < inheritedLength; i++) {
+		TypeVariableBinding inheritedTypeVariable = inheritedTypeVariables[i];
+		TypeBinding argument = arguments[i];
+		if (argument instanceof TypeVariableBinding) {
+			TypeVariableBinding typeVariable = (TypeVariableBinding) argument;
+			if (Scope.substitute(substitute, inheritedTypeVariable.superclass) != typeVariable.superclass)  
+				return inheritedMethod; // not a match
+			int interfaceLength = inheritedTypeVariable.superInterfaces.length;
+			ReferenceBinding[] interfaces = typeVariable.superInterfaces;
+			if (interfaceLength != interfaces.length)
+				return inheritedMethod; // not a match
+			next : for (int j = 0; j < interfaceLength; j++) {
+				TypeBinding superType = Scope.substitute(substitute, inheritedTypeVariable.superInterfaces[j]);
+				for (int k = 0; k < interfaceLength; k++)
+					if (superType == interfaces[k])
+						continue next;
+				return inheritedMethod; // not a match
+			}
+		} else if (inheritedTypeVariables[i].boundCheck(substitute, argument) != TypeConstants.OK) {
+	    	return inheritedMethod;
+		}
+	}
    return substitute;
 }
 boolean detectInheritedMethodClash(MethodBinding inherited, MethodBinding otherInherited) {
@@ -363,9 +467,31 @@ boolean detectNameClash(MethodBinding current, MethodBinding inherited) {
 	}
 	return false;
 }
-public boolean doesMethodOverride(MethodBinding one, MethodBinding two) {
-	MethodBinding sub = computeSubstituteMethod(two, one);
-	return sub != null && super.doesMethodOverride(one, sub);
+public boolean doesMethodOverride(MethodBinding method, MethodBinding inheritedMethod) {
+	MethodBinding substitute = computeSubstituteMethod(inheritedMethod, method);
+	return substitute != null && doesSubstituteMethodOverride(method, substitute);
+}
+boolean doesSubstituteMethodOverride(MethodBinding method, MethodBinding substituteMethod) {
+	if (doTypeVariablesClash(method, substituteMethod)) return false;
+	if (areParametersEqual(method, substituteMethod)) return true;
+	if (method.declaringClass == substituteMethod.declaringClass) return false;
+
+	TypeBinding[] params = method.parameters;
+	TypeBinding[] inheritedParams = substituteMethod.parameters;
+	int length = params.length;
+	if (length != inheritedParams.length)
+		return false;
+
+	// also allow a method such as Number foo(Number) to override <U> T foo(T) where T extends Number
+	if (method.typeVariables != NoTypeVariables || !substituteMethod.hasSubstitutedParameters())
+		return false;
+	if (method.declaringClass.findSuperTypeWithSameErasure(substituteMethod.declaringClass) == null)
+		return false;
+
+	for (int i = 0; i < length; i++)
+		if (params[i] != inheritedParams[i].erasure())
+			return false;
+	return true;
 }
 boolean doParametersClash(MethodBinding one, MethodBinding substituteTwo) {
 	// must check each parameter pair to see if parameterized types are compatible
@@ -390,11 +516,8 @@ boolean doParametersClash(MethodBinding one, MethodBinding substituteTwo) {
 	return false;
 }
 boolean doTypeVariablesClash(MethodBinding one, MethodBinding substituteTwo) {
-	if (one.typeVariables == NoTypeVariables) return false;
-	MethodBinding subTwo = substituteTwo instanceof ParameterizedGenericMethodBinding
-		? ((ParameterizedGenericMethodBinding) substituteTwo).originalMethod
-		: substituteTwo;
-	return !one.areTypeVariableErasuresEqual(subTwo);
+	// one has type variables and substituteTwo did not pass bounds check in computeSubstituteMethod()
+	return one.typeVariables != NoTypeVariables && !(substituteTwo instanceof ParameterizedGenericMethodBinding);
 }
 boolean isInterfaceMethodImplemented(MethodBinding inheritedMethod, MethodBinding existingMethod, ReferenceBinding superType) {
 	if (inheritedMethod.original() != inheritedMethod && existingMethod.declaringClass.isInterface())
