@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -65,6 +65,35 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 	private Button fParametersAddButton;
 	private Button fParametersRemoveButton;
 	private Button fParametersEditButton;
+	
+	private class AppletTabListener extends SelectionAdapter implements ModifyListener {
+
+		/* (non-Javadoc)
+		 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
+		 */
+		public void modifyText(ModifyEvent e) {
+			updateLaunchConfigurationDialog();
+		}
+		
+		/* (non-Javadoc)
+		 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
+		 */
+		public void widgetSelected(SelectionEvent e) {
+			Object source= e.getSource();
+			if (source == fParametersTable) {
+				setParametersButtonsEnableState();
+			} else if (source == fParametersAddButton) {
+				handleParametersAddButtonSelected();
+			} else if (source == fParametersEditButton) {
+				handleParametersEditButtonSelected();
+			} else if (source == fParametersRemoveButton) {
+				handleParametersRemoveButtonSelected();
+			}
+		}
+
+	}
+	
+	private AppletTabListener fListener= new AppletTabListener();
 
 	private static final String EMPTY_STRING = "";	 //$NON-NLS-1$
 	
@@ -90,8 +119,6 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 		comp.setLayout(topLayout);		
 		GridData gd;
 		
-		createVerticalSpacer(comp);
-		
 		Composite widthHeightNameComp = new Composite(comp, SWT.NONE);
 		gd = new GridData(GridData.FILL_HORIZONTAL);
 		widthHeightNameComp.setLayoutData(gd);
@@ -102,51 +129,39 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 		widthHeightNameComp.setLayout(widthHeightNameLayout);
 		
 		fWidthLabel= new Label(widthHeightNameComp, SWT.NONE);
-		fWidthLabel.setText(LauncherMessages.getString("appletlauncher.argumenttab.widthlabel.text")); //$NON-NLS-1$
+		fWidthLabel.setText(LauncherMessages.appletlauncher_argumenttab_widthlabel_text); //$NON-NLS-1$
 		fWidthLabel.setFont(font);
 		
 		fWidthText = new Text(widthHeightNameComp, SWT.SINGLE | SWT.BORDER);
 		gd = new GridData(GridData.FILL_HORIZONTAL);
 		fWidthText.setLayoutData(gd);
 		fWidthText.setFont(font);
-		fWidthText.addModifyListener(new ModifyListener() {
-			public void modifyText(ModifyEvent evt) {
-				updateLaunchConfigurationDialog();
-			}
-		});
+		fWidthText.addModifyListener(fListener);
 
 		fNameLabel = new Label(widthHeightNameComp, SWT.NONE);
-		fNameLabel.setText(LauncherMessages.getString("appletlauncher.argumenttab.namelabel.text")); //$NON-NLS-1$
+		fNameLabel.setText(LauncherMessages.appletlauncher_argumenttab_namelabel_text); //$NON-NLS-1$
 		fNameLabel.setFont(font);
 		
 		fNameText = new Text(widthHeightNameComp, SWT.SINGLE | SWT.BORDER);
 		gd = new GridData(GridData.FILL_HORIZONTAL);
 		fNameText.setLayoutData(gd);
 		fNameText.setFont(font);
-		fNameText.addModifyListener(new ModifyListener() {
-			public void modifyText(ModifyEvent evt) {
-				updateLaunchConfigurationDialog();
-			}
-		});	
+		fNameText.addModifyListener(fListener);	
 
 		fHeightLabel= new Label(widthHeightNameComp, SWT.NONE);
-		fHeightLabel.setText(LauncherMessages.getString("appletlauncher.argumenttab.heightlabel.text")); //$NON-NLS-1$
+		fHeightLabel.setText(LauncherMessages.appletlauncher_argumenttab_heightlabel_text); //$NON-NLS-1$
 		fHeightLabel.setFont(font);
 		
 		fHeightText = new Text(widthHeightNameComp, SWT.SINGLE | SWT.BORDER);
 		gd = new GridData(GridData.FILL_HORIZONTAL);
 		fHeightText.setLayoutData(gd);
 		fHeightText.setFont(font);
-		fHeightText.addModifyListener(new ModifyListener() {
-			public void modifyText(ModifyEvent evt) {
-				updateLaunchConfigurationDialog();
-			}
-		});
+		fHeightText.addModifyListener(fListener);
 		
 		Label blank = new Label(widthHeightNameComp, SWT.NONE);
 		blank.setText(EMPTY_STRING);
 		Label hint = new Label(widthHeightNameComp, SWT.NONE);
-		hint.setText(LauncherMessages.getString("AppletParametersTab.(optional_applet_instance_name)_1")); //$NON-NLS-1$
+		hint.setText(LauncherMessages.AppletParametersTab__optional_applet_instance_name__1); //$NON-NLS-1$
 		gd = new GridData(GridData.HORIZONTAL_ALIGN_CENTER);
 		hint.setLayoutData(gd);
 		hint.setFont(font);
@@ -164,7 +179,7 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 		parametersComp.setFont(font);
 		
 		Label parameterLabel = new Label(parametersComp, SWT.NONE);
-		parameterLabel.setText(LauncherMessages.getString("appletlauncher.argumenttab.parameterslabel.text")); //$NON-NLS-1$
+		parameterLabel.setText(LauncherMessages.appletlauncher_argumenttab_parameterslabel_text); //$NON-NLS-1$
 		gd = new GridData();
 		gd.horizontalSpan = 2;
 		parameterLabel.setLayoutData(gd);
@@ -178,18 +193,14 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 		gd = new GridData(GridData.FILL_BOTH);
 		fParametersTable.setLayoutData(gd);
 		TableColumn column1 = new TableColumn(this.fParametersTable, SWT.NONE);
-		column1.setText(LauncherMessages.getString("appletlauncher.argumenttab.parameterscolumn.name.text")); //$NON-NLS-1$
+		column1.setText(LauncherMessages.appletlauncher_argumenttab_parameterscolumn_name_text); //$NON-NLS-1$
 		TableColumn column2 = new TableColumn(this.fParametersTable, SWT.NONE);
-		column2.setText(LauncherMessages.getString("appletlauncher.argumenttab.parameterscolumn.value.text"));		 //$NON-NLS-1$
+		column2.setText(LauncherMessages.appletlauncher_argumenttab_parameterscolumn_value_text);		 //$NON-NLS-1$
 		tableLayout.addColumnData(new ColumnWeightData(100));
 		tableLayout.addColumnData(new ColumnWeightData(100));
 		fParametersTable.setHeaderVisible(true);
 		fParametersTable.setLinesVisible(true);
-		fParametersTable.addSelectionListener(new SelectionAdapter() {
-			public void widgetSelected(SelectionEvent evt) {
-				setParametersButtonsEnableState();
-			}
-		});
+		fParametersTable.addSelectionListener(fListener);
 		fParametersTable.addMouseListener(new MouseAdapter() {
 			public void mouseDoubleClick(MouseEvent e) {
 				setParametersButtonsEnableState();
@@ -208,26 +219,14 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 		envButtonComp.setLayoutData(gd);
 		envButtonComp.setFont(font);
 		
-		fParametersAddButton = createPushButton(envButtonComp ,LauncherMessages.getString("appletlauncher.argumenttab.parameters.button.add.text"), null); //$NON-NLS-1$
-		fParametersAddButton.addSelectionListener(new SelectionAdapter() {
-			public void widgetSelected(SelectionEvent evt) {
-				handleParametersAddButtonSelected();
-			}
-		});
+		fParametersAddButton = createPushButton(envButtonComp ,LauncherMessages.appletlauncher_argumenttab_parameters_button_add_text, null); //$NON-NLS-1$
+		fParametersAddButton.addSelectionListener(fListener);
 		
-		fParametersEditButton = createPushButton(envButtonComp, LauncherMessages.getString("appletlauncher.argumenttab.parameters.button.edit.text"), null); //$NON-NLS-1$
-		fParametersEditButton.addSelectionListener(new SelectionAdapter() {
-			public void widgetSelected(SelectionEvent evt) {
-				handleParametersEditButtonSelected();
-			}
-		});
+		fParametersEditButton = createPushButton(envButtonComp, LauncherMessages.appletlauncher_argumenttab_parameters_button_edit_text, null); //$NON-NLS-1$
+		fParametersEditButton.addSelectionListener(fListener);
 		
-		fParametersRemoveButton = createPushButton(envButtonComp, LauncherMessages.getString("appletlauncher.argumenttab.parameters.button.remove.text"), null); //$NON-NLS-1$
-		fParametersRemoveButton.addSelectionListener(new SelectionAdapter() {
-			public void widgetSelected(SelectionEvent evt) {
-				handleParametersRemoveButtonSelected();
-			}
-		});
+		fParametersRemoveButton = createPushButton(envButtonComp, LauncherMessages.appletlauncher_argumenttab_parameters_button_remove_text, null); //$NON-NLS-1$
+		fParametersRemoveButton.addSelectionListener(fListener);
 	}
 
 		
@@ -237,15 +236,15 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 	public boolean isValid(ILaunchConfiguration launchConfig) {
 		setErrorMessage(null);
 		try {
-			Integer.parseInt(fWidthText.getText().trim());
+			Integer.parseInt(getWidthText());
 		} catch(NumberFormatException nfe) {
-			setErrorMessage(LauncherMessages.getString("appletlauncher.argumenttab.width.error.notaninteger")); //$NON-NLS-1$
+			setErrorMessage(LauncherMessages.appletlauncher_argumenttab_width_error_notaninteger); //$NON-NLS-1$
 			return false;
 		}
 		try {
-			Integer.parseInt(fHeightText.getText().trim());
+			Integer.parseInt(getHeightText());
 		} catch(NumberFormatException nfe) {
-			setErrorMessage(LauncherMessages.getString("appletlauncher.argumenttab.height.error.notaninteger")); //$NON-NLS-1$
+			setErrorMessage(LauncherMessages.appletlauncher_argumenttab_height_error_notaninteger); //$NON-NLS-1$
 			return false;
 		}
 		return true;
@@ -254,8 +253,8 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 	private void handleParametersAddButtonSelected() {
 		NameValuePairDialog dialog = 
 			new NameValuePairDialog(getShell(), 
-				LauncherMessages.getString("appletlauncher.argumenttab.parameters.dialog.add.title"),  //$NON-NLS-1$
-				new String[] {LauncherMessages.getString("appletlauncher.argumenttab.parameters.dialog.add.name.text"), LauncherMessages.getString("appletlauncher.argumenttab.parameters.dialog.add.value.text")},  //$NON-NLS-1$ //$NON-NLS-2$
+				LauncherMessages.appletlauncher_argumenttab_parameters_dialog_add_title,  //$NON-NLS-1$
+				new String[] {LauncherMessages.appletlauncher_argumenttab_parameters_dialog_add_name_text, LauncherMessages.appletlauncher_argumenttab_parameters_dialog_add_value_text},  //$NON-NLS-1$ //$NON-NLS-2$
 				new String[] {EMPTY_STRING, EMPTY_STRING}); 
 		openNewParameterDialog(dialog, null);
 		setParametersButtonsEnableState();
@@ -267,8 +266,8 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 		String value = selectedItem.getText(1);
 		NameValuePairDialog dialog =
 			new NameValuePairDialog(getShell(), 
-				LauncherMessages.getString("appletlauncher.argumenttab.parameters.dialog.edit.title"),  //$NON-NLS-1$
-				new String[] {LauncherMessages.getString("appletlauncher.argumenttab.parameters.dialog.edit.name.text"), LauncherMessages.getString("appletlauncher.argumenttab.parameters.dialog.edit.value.text")},  //$NON-NLS-1$ //$NON-NLS-2$
+				LauncherMessages.appletlauncher_argumenttab_parameters_dialog_edit_title,  //$NON-NLS-1$
+				new String[] {LauncherMessages.appletlauncher_argumenttab_parameters_dialog_edit_name_text, LauncherMessages.appletlauncher_argumenttab_parameters_dialog_edit_value_text},  //$NON-NLS-1$ //$NON-NLS-2$
 				new String[] {name, value});
 		openNewParameterDialog(dialog, selectedItem);		
 	}
@@ -277,6 +276,7 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 		int[] selectedIndices = this.fParametersTable.getSelectionIndices();
 		this.fParametersTable.remove(selectedIndices);
 		setParametersButtonsEnableState();
+		updateLaunchConfigurationDialog();
 	}
 
 	/**
@@ -341,12 +341,34 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(ILaunchConfigurationWorkingCopy)
 	 */
 	public void performApply(ILaunchConfigurationWorkingCopy configuration) {
-		configuration.setAttribute(IJavaLaunchConfigurationConstants.ATTR_APPLET_WIDTH, Integer.parseInt(fWidthText.getText()));
-		configuration.setAttribute(IJavaLaunchConfigurationConstants.ATTR_APPLET_HEIGHT, Integer.parseInt(fHeightText.getText()));
-		configuration.setAttribute(IJavaLaunchConfigurationConstants.ATTR_APPLET_NAME, (String)fNameText.getText());
+		try {
+			configuration.setAttribute(IJavaLaunchConfigurationConstants.ATTR_APPLET_WIDTH, Integer.parseInt(getWidthText()));
+		} catch (NumberFormatException e) {
+		}
+		try {
+			configuration.setAttribute(IJavaLaunchConfigurationConstants.ATTR_APPLET_HEIGHT, Integer.parseInt(getHeightText()));
+		} catch (NumberFormatException e) {
+		}
+		configuration.setAttribute(IJavaLaunchConfigurationConstants.ATTR_APPLET_NAME, fNameText.getText());
 		configuration.setAttribute(IJavaLaunchConfigurationConstants.ATTR_APPLET_PARAMETERS, getMapFromParametersTable());
 	}
 	
+	/**
+	 * Returns the current width specified by the user
+	 * @return the width specified by the user
+	 */
+	private String getWidthText() {
+		return fWidthText.getText().trim();
+	}
+	
+	/**
+	 * Returns the current height specified by the user
+	 * @return the height specified by the user
+	 */
+	private String getHeightText() {
+		return fHeightText.getText().trim();
+	}
+	
 	private Map getMapFromParametersTable() {
 		TableItem[] items = fParametersTable.getItems();
 		if (items.length == 0) {
@@ -410,9 +432,9 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 			fHeightText.setText(Integer.toString(DEFAULT_APPLET_HEIGHT)); //$NON-NLS-1$
 		}
 		try {
-			fNameText.setText(config.getAttribute(IJavaLaunchConfigurationConstants.ATTR_APPLET_NAME, LauncherMessages.getString("appletlauncher.argumenttab.name.defaultvalue"))); //$NON-NLS-1$
+			fNameText.setText(config.getAttribute(IJavaLaunchConfigurationConstants.ATTR_APPLET_NAME, LauncherMessages.appletlauncher_argumenttab_name_defaultvalue)); //$NON-NLS-1$
 		} catch(CoreException ce) {
-			fNameText.setText(LauncherMessages.getString("appletlauncher.argumenttab.name.defaultvalue")); //$NON-NLS-1$
+			fNameText.setText(LauncherMessages.appletlauncher_argumenttab_name_defaultvalue); //$NON-NLS-1$
 		}
 		updateParametersFromConfig(config);
 	}
@@ -428,7 +450,7 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
 	 */
 	public String getName() {
-		return LauncherMessages.getString("appletlauncher.argumenttab.name"); //$NON-NLS-1$
+		return LauncherMessages.appletlauncher_argumenttab_name; //$NON-NLS-1$
 	}	
 	
 	/**
@@ -437,5 +459,19 @@ public class AppletParametersTab extends JavaLaunchConfigurationTab {
 	public Image getImage() {
 		return JavaDebugImages.get(JavaDebugImages.IMG_VIEW_ARGUMENTS_TAB);
 	}	
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#activated(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
+	 */
+	public void activated(ILaunchConfigurationWorkingCopy workingCopy) {
+		// do nothing when activated
+	}
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#deactivated(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
+	 */
+	public void deactivated(ILaunchConfigurationWorkingCopy workingCopy) {
+		// do nothing when deactivated
+	}	
 }
 
