@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -13,35 +13,33 @@ package org.eclipse.jdt.internal.debug.ui;
  
 import java.text.MessageFormat;
 import java.util.ArrayList;
-import java.util.HashMap;
 import java.util.List;
 import java.util.StringTokenizer;
 
 import org.eclipse.core.resources.IMarker;
 import org.eclipse.core.resources.IMarkerDelta;
 import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.IResourceChangeEvent;
-import org.eclipse.core.resources.IResourceChangeListener;
-import org.eclipse.core.resources.IResourceDelta;
 import org.eclipse.core.resources.IWorkspaceRunnable;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
+import org.eclipse.core.runtime.MultiStatus;
 import org.eclipse.core.runtime.Status;
+import org.eclipse.core.runtime.jobs.Job;
 import org.eclipse.debug.core.DebugEvent;
 import org.eclipse.debug.core.DebugException;
 import org.eclipse.debug.core.DebugPlugin;
 import org.eclipse.debug.core.IBreakpointsListener;
 import org.eclipse.debug.core.IDebugEventSetListener;
 import org.eclipse.debug.core.ILaunch;
+import org.eclipse.debug.core.ILaunchConfiguration;
 import org.eclipse.debug.core.ILaunchListener;
 import org.eclipse.debug.core.model.IBreakpoint;
 import org.eclipse.debug.core.model.IDebugTarget;
 import org.eclipse.debug.ui.DebugUITools;
-import org.eclipse.jdt.core.ICompilationUnit;
-import org.eclipse.jdt.core.IJavaElement;
-import org.eclipse.jdt.core.JavaCore;
+import org.eclipse.debug.ui.sourcelookup.ISourceLookupResult;
 import org.eclipse.jdt.core.dom.Message;
 import org.eclipse.jdt.debug.core.IJavaBreakpoint;
 import org.eclipse.jdt.debug.core.IJavaBreakpointListener;
@@ -55,12 +53,17 @@ import org.eclipse.jdt.debug.core.IJavaThread;
 import org.eclipse.jdt.debug.core.IJavaType;
 import org.eclipse.jdt.debug.core.IJavaWatchpoint;
 import org.eclipse.jdt.debug.core.JDIDebugModel;
-import org.eclipse.jdt.internal.debug.ui.actions.JavaBreakpointPropertiesDialog;
-import org.eclipse.jface.dialogs.Dialog;
+import org.eclipse.jdt.debug.ui.IJavaDebugUIConstants;
+import org.eclipse.jdt.internal.debug.core.logicalstructures.IJavaStructuresListener;
+import org.eclipse.jdt.internal.debug.core.logicalstructures.JavaLogicalStructures;
+import org.eclipse.jdt.internal.debug.ui.actions.JavaBreakpointPropertiesAction;
+import org.eclipse.jdt.internal.debug.ui.snippeteditor.ScrapbookLauncher;
 import org.eclipse.jface.preference.IPreferenceStore;
 import org.eclipse.jface.util.IPropertyChangeListener;
 import org.eclipse.jface.util.PropertyChangeEvent;
 import org.eclipse.jface.viewers.ILabelProvider;
+import org.eclipse.jface.viewers.StructuredSelection;
+import org.eclipse.jface.window.Window;
 import org.eclipse.swt.widgets.Display;
 import org.eclipse.swt.widgets.Shell;
 
@@ -77,25 +80,13 @@ import com.sun.jdi.ObjectReference;
  * debug action visibility.
  * </ul>
  */
-public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugEventSetListener, IPropertyChangeListener, IJavaBreakpointListener, ILaunchListener, IBreakpointsListener {
+public class JavaDebugOptionsManager implements IDebugEventSetListener, IPropertyChangeListener, IJavaBreakpointListener, ILaunchListener, IBreakpointsListener, IJavaStructuresListener {
 	
 	/**
 	 * Singleton options manager
 	 */
 	private static JavaDebugOptionsManager fgOptionsManager = null;
 	
-	/**
-	 * Map of problems to locations
-	 * (<code>IMarker</code> -> <code>Location</code>)
-	 */
-	private HashMap fProblemMap = new HashMap(10);
-	
-	/**
-	 * Map of locations to problems.
-	 * (<code>Location</code> -> <code>IMarker</code>)
-	 */
-	private HashMap fLocationMap = new HashMap(10);
-	
 	/**
 	 * Breakpoint used to suspend on uncaught exceptions
 	 */
@@ -125,61 +116,47 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 	private String[] fActiveStepFilters = null;
 	
 	/**
-	 * Helper class that describes a location in a stack
-	 * frame. A location consists of a package name, source
-	 * file name, and a line number.
+	 * Whether the manager has been activated
 	 */
-	class Location {
-		private String fPackageName;
-		private String fSourceName;
-		private int fLineNumber;
+	private boolean fActivated = false;
+	
+	class InitJob extends Job {
 		
-		public Location(String packageName, String sourceName, int lineNumber) {
-			fPackageName = packageName;
-			fSourceName = sourceName;
-			fLineNumber = lineNumber;
+		public InitJob() {
+			super(DebugUIMessages.JavaDebugOptionsManager_0); //$NON-NLS-1$
 		}
 		
-		public boolean equals(Object o) {
-			if (o instanceof Location) {
-				Location l = (Location)o;
-				return l.fPackageName.equals(fPackageName) && l.fSourceName.equals(fSourceName) && l.fLineNumber == fLineNumber;
-				
+		protected IStatus run(IProgressMonitor monitor) {
+			MultiStatus status = new MultiStatus(JDIDebugUIPlugin.getUniqueIdentifier(), IJavaDebugUIConstants.INTERNAL_ERROR, DebugUIMessages.JavaDebugOptionsManager_1, null); //$NON-NLS-1$
+			// compilation error breakpoint 
+			try {
+				IJavaExceptionBreakpoint bp = JDIDebugModel.createExceptionBreakpoint(ResourcesPlugin.getWorkspace().getRoot(),"java.lang.Error", true, true, false, false, null); //$NON-NLS-1$
+				bp.setPersisted(false);
+				setSuspendOnCompilationErrorsBreakpoint(bp);
+			} catch (CoreException e) {
+				status.add(e.getStatus());
 			}
-			return false;
-		}
-		
-		public int hashCode() {
-			return fPackageName.hashCode() + fSourceName.hashCode() + fLineNumber;
-		}
-	}
-
-	/**
-	 * Update cache of problems as they are added/removed.
-	 * 
-	 * @see IResourceChangeListener#resourceChanged(IResourceChangeEvent)
-	 */
-	public void resourceChanged(IResourceChangeEvent event) {
-		IMarkerDelta[] deltas = event.findMarkerDeltas("org.eclipse.jdt.core.problem", true); //$NON-NLS-1$
-		if (deltas != null) {
-			for (int i = 0; i < deltas.length; i++) {
-				IMarkerDelta delta = deltas[i];
-				switch (delta.getKind()) {
-					case IResourceDelta.ADDED:
-						problemAdded(delta.getMarker());
-						break;
-					case IResourceDelta.REMOVED:
-						problemRemoved(delta.getMarker());
-						break;
-				}
+			
+			// uncaught exception breakpoint
+			try {
+				IJavaExceptionBreakpoint bp = JDIDebugModel.createExceptionBreakpoint(ResourcesPlugin.getWorkspace().getRoot(),"java.lang.Throwable", false, true, false, false, null); //$NON-NLS-1$
+				bp.setPersisted(false);
+				setSuspendOnUncaughtExceptionBreakpoint(bp);
+			} catch (CoreException e) {
+				status.add(e.getStatus());
 			}
+			
+			if (status.getChildren().length == 0) {
+				return Status.OK_STATUS;
+			}
+			return status;
 		}
 	}
 	
 	/**
 	 * Not to be instantiated
 	 * 
-	 * @see JavaDebugOptionsManager.getDefault();
+	 * @see JavaDebugOptionsManager#getDefault();
 	 */
 	private JavaDebugOptionsManager() {
 	}
@@ -195,9 +172,9 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 	}
 	
 	/**
-	 * Called at startup by the java debug ui plug-in
+	 * Called at startup by the Java debug ui plug-in
 	 */
-	public void startup() throws CoreException {
+	public void startup() {
 		// lazy initialization will occur on the first launch
 		DebugPlugin debugPlugin = DebugPlugin.getDefault();
 		debugPlugin.getLaunchManager().addLaunchListener(this);
@@ -208,8 +185,7 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 	/**
 	 * Called at shutdown by the Java debug ui plug-in
 	 */
-	public void shutdown() throws CoreException {
-		ResourcesPlugin.getWorkspace().removeResourceChangeListener(this);
+	public void shutdown() {
 		DebugPlugin debugPlugin = DebugPlugin.getDefault();
 		debugPlugin.removeDebugEventListener(this);
 		debugPlugin.getLaunchManager().removeLaunchListener(this);
@@ -219,8 +195,7 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 			JDIDebugUIPlugin.getDefault().getPreferenceStore().removePropertyChangeListener(this);
 		}
 		JDIDebugModel.removeJavaBreakpointListener(this);
-		fProblemMap.clear();
-		fLocationMap.clear();
+        JavaLogicalStructures.removeStructuresListener(this);
 		System.getProperties().remove(JDIDebugUIPlugin.getUniqueIdentifier() + ".debuggerActive"); //$NON-NLS-1$
 	}	
 
@@ -229,90 +204,11 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 	 * on uncaught exceptions.
 	 */
 	protected void initializeProblemHandling() {
-		IWorkspaceRunnable wr = new IWorkspaceRunnable() {
-			public void run(IProgressMonitor monitor) throws CoreException {
-				// compilation error breakpoint
-				IJavaExceptionBreakpoint bp = JDIDebugModel.createExceptionBreakpoint(ResourcesPlugin.getWorkspace().getRoot(),"java.lang.Error", true, true, false, false, null); //$NON-NLS-1$
-				bp.setPersisted(false);
-				bp.setRegistered(false);
-				// disabled until there are errors
-				bp.setEnabled(false);
-				setSuspendOnCompilationErrorsBreakpoint(bp);
-				
-				// uncaught exception breakpoint
-				bp = JDIDebugModel.createExceptionBreakpoint(ResourcesPlugin.getWorkspace().getRoot(),"java.lang.Throwable", false, true, false, false, null); //$NON-NLS-1$
-				bp.setPersisted(false);
-				bp.setRegistered(false);
-				bp.setEnabled(isSuspendOnUncaughtExceptions());
-				setSuspendOnUncaughtExceptionBreakpoint(bp);
-				
-				// note existing compilation errors
-				IMarker[] problems = ResourcesPlugin.getWorkspace().getRoot().findMarkers("org.eclipse.jdt.core.problem", true, IResource.DEPTH_INFINITE); //$NON-NLS-1$
-				if (problems != null) {
-					for (int i = 0; i < problems.length; i++) {
-						problemAdded(problems[i]);
-					}
-				}				
-			}
-		};
-		
-		try {
-			ResourcesPlugin.getWorkspace().run(wr, null);
-		} catch (CoreException e) {
-			JDIDebugUIPlugin.log(e);
-		}
+		InitJob job = new InitJob();
+		job.setSystem(true);
+		job.schedule();
 	}
-		
-	/**
-	 * The given problem has been added. Cross
-	 * reference the problem with its location.
-	 * Enable the error breakpoint if the suspend
-	 * option is on, and this is the first problem
-	 * being added.
-	 */
-	protected void problemAdded(IMarker problem) {
-		if (problem.getAttribute(IMarker.SEVERITY, IMarker.SEVERITY_INFO) == IMarker.SEVERITY_ERROR) {
-			IResource res = problem.getResource();
-			IJavaElement cu = JavaCore.create(res);
-			if (cu != null && cu instanceof ICompilationUnit) {
-				// auto-enable the exception breakpoint if this is the first problem added
-				// and the preference is turned on.
-				boolean autoEnable = fProblemMap.isEmpty();
-				int line = problem.getAttribute(IMarker.LINE_NUMBER, -1);
-				String name = cu.getElementName();
-				Location l = new Location(cu.getParent().getElementName(), name, line);
-				fLocationMap.put(l, problem);
-				fProblemMap.put(problem, l);
-				if (autoEnable) {
-					try {
-						getSuspendOnCompilationErrorBreakpoint().setEnabled(isSuspendOnCompilationErrors());
-					} catch (CoreException e) {
-						JDIDebugUIPlugin.log(e);
-					}
-				}
-			}
-		}
-	}
-	
-	/**
-	 * The given problem has been removed. Remove
-	 * cross reference of problem and location.
-	 * Disable the breakpoint if there are no errors.
-	 */
-	protected void problemRemoved(IMarker problem) {
-		Object location = fProblemMap.remove(problem);
-		if (location != null) {
-			fLocationMap.remove(location);
-		}
-		if (fProblemMap.isEmpty()) {
-			try {
-				getSuspendOnCompilationErrorBreakpoint().setEnabled(false);
-			} catch (CoreException e) {
-				JDIDebugUIPlugin.log(e);
-			}
-		}
-	}
-				
+						
 	/**
 	 * Notifies java debug targets of the given breakpoint
 	 * addition or removal.
@@ -388,12 +284,20 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 		if (event.getProperty().equals(IJDIPreferencesConstants.PREF_SUSPEND_ON_COMPILATION_ERRORS)) {
 			IBreakpoint breakpoint = getSuspendOnCompilationErrorBreakpoint();
 			if (breakpoint != null) {
-				setEnabled(breakpoint, JDIDebugUIPlugin.getDefault().getPreferenceStore().getBoolean(IJDIPreferencesConstants.PREF_SUSPEND_ON_COMPILATION_ERRORS));
+				int kind = REMOVED;
+				if (isSuspendOnCompilationErrors()) {
+					kind = ADDED;
+				}
+				notifyTargets(breakpoint, kind);
 			}
 		} else if (event.getProperty().equals(IJDIPreferencesConstants.PREF_SUSPEND_ON_UNCAUGHT_EXCEPTIONS)) {
 			IBreakpoint breakpoint = getSuspendOnUncaughtExceptionBreakpoint();
 			if (breakpoint != null) {
-				setEnabled(breakpoint, JDIDebugUIPlugin.getDefault().getPreferenceStore().getBoolean(IJDIPreferencesConstants.PREF_SUSPEND_ON_UNCAUGHT_EXCEPTIONS));
+				int kind = REMOVED;
+				if (isSuspendOnUncaughtExceptions()) {
+					kind = ADDED;
+				}			
+				notifyTargets(breakpoint, kind);
 			}
 		} else if (isUseFilterProperty(event.getProperty())) {
 			notifyTargetsOfFilters();
@@ -573,11 +477,25 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 				if (source instanceof IJavaDebugTarget) {
 					IJavaDebugTarget javaTarget = (IJavaDebugTarget)source;
 					
-					// compilation breakpoints				
-					notifyTarget(javaTarget, getSuspendOnCompilationErrorBreakpoint(), ADDED);
+					// compilation breakpoints	
+					if (isSuspendOnCompilationErrors()) {
+						notifyTarget(javaTarget, getSuspendOnCompilationErrorBreakpoint(), ADDED);
+					}
 					
 					// uncaught exception breakpoint
-					notifyTarget(javaTarget, getSuspendOnUncaughtExceptionBreakpoint(), ADDED);
+					if (isSuspendOnUncaughtExceptions()) {
+						ILaunchConfiguration launchConfiguration = javaTarget.getLaunch().getLaunchConfiguration();
+						boolean isSnippetEditor = false;
+						
+						try {
+							isSnippetEditor = (launchConfiguration.getAttribute(ScrapbookLauncher.SCRAPBOOK_LAUNCH, (String)null) != null);
+						} catch (CoreException e) {
+						}
+						
+						if (!isSnippetEditor) { 
+							notifyTarget(javaTarget, getSuspendOnUncaughtExceptionBreakpoint(), ADDED);
+						}
+					}
 					
 					// step filters
 					notifyTargetOfFilters(javaTarget);
@@ -595,16 +513,22 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 	/**
 	 * @see IJavaBreakpointListener#installingBreakpoint(IJavaDebugTarget, IJavaBreakpoint, IJavaType)
 	 */
-	public boolean installingBreakpoint(IJavaDebugTarget target, IJavaBreakpoint breakpoint, IJavaType type) {
-		return true;
+	public int installingBreakpoint(IJavaDebugTarget target, IJavaBreakpoint breakpoint, IJavaType type) {
+		return DONT_CARE;
 	}
 	
 	/**
 	 * @see IJavaBreakpointListener#breakpointHit(IJavaThread, IJavaBreakpoint)
 	 */
-	public boolean breakpointHit(IJavaThread thread, IJavaBreakpoint breakpoint) {
+	public int breakpointHit(IJavaThread thread, IJavaBreakpoint breakpoint) {
 		if (breakpoint == getSuspendOnCompilationErrorBreakpoint()) {
-			return  getProblem(thread) != null;
+		    try {
+		        return getProblem(thread) != null ? SUSPEND : DONT_SUSPEND;
+		    } catch (DebugException e) {
+		        JDIDebugUIPlugin.log(e);
+		        // don't suspend if we can't determine if there is a problem
+		        return DONT_SUSPEND;
+		    }
 		}
 		if (breakpoint == getSuspendOnUncaughtExceptionBreakpoint()) {
 			// the "uncaught" exceptions breakpoint subsumes the "compilation error" breakpoint
@@ -612,22 +536,33 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 			// error here, but the option to suspend on compilation errors is off, we should
 			// resume (i.e. do not suspend)
 			if (!isSuspendOnCompilationErrors()) {
-				// only suspend if there is no compilation error
-				return getProblem(thread) == null;
+			    try {
+			        if (getProblem(thread) != null) {
+			            return DONT_SUSPEND;
+			        }
+			    } catch (DebugException e) {
+			        JDIDebugUIPlugin.log(e);
+			        // unable to determine if there was a compilation problem, so fall thru and suspend
+			    }
 			}
+			return SUSPEND;
 		}
-		return true;
+		return DONT_CARE;
 	}
 	
-	private IMarker getProblem(IJavaThread thread) {
-		try {
-			IJavaStackFrame frame = (IJavaStackFrame)thread.getTopStackFrame();
-			if (frame != null) {
-				return  getProblem(frame);
-			}
-		} catch (DebugException e) {
-			JDIDebugUIPlugin.log(e);
-		}	
+	/**
+	 * Returns any problem for the top stack frame in the given thread, or
+	 * <code>null</code> if none.
+	 * 
+	 * @param thread thread to look for a compilation problem in
+	 * @return problem or <code>null</code>
+	 * @throws DebugException if an exception occurrs retrieveing the problem
+	 */
+	private IMarker getProblem(IJavaThread thread) throws DebugException {
+		IJavaStackFrame frame = (IJavaStackFrame)thread.getTopStackFrame();
+		if (frame != null) {
+			return  getProblem(frame);
+		}
 		return null;	
 	}
 
@@ -649,45 +584,33 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 	 * 
 	 * @param frame stack frame
 	 * @return marker representing compilation problem, or <code>null</code>
+	 * @throws DebugException if an exception occurrs retrieveing the problem
 	 */
 	protected IMarker getProblem(IJavaStackFrame frame) {
-		try {
-			String name = frame.getSourceName();
-			String packageName = frame.getDeclaringTypeName();
-			int index = packageName.lastIndexOf('.');
-			if (index == -1) {
-				if (name == null) {
-					// guess at source name if no debug attribute
-					name = packageName;
-					int dollar = name.indexOf('$');
-					if (dollar >= 0) {
-						name = name.substring(0, dollar);
+		ILaunch launch = frame.getLaunch();
+		if (launch != null) {
+			ISourceLookupResult result = DebugUITools.lookupSource(frame, null);
+			Object sourceElement = result.getSourceElement();
+			if (sourceElement instanceof IResource) {
+				try {
+					IResource resource = (IResource) sourceElement;
+					IMarker[] markers = resource.findMarkers("org.eclipse.jdt.core.problem", true, IResource.DEPTH_INFINITE); //$NON-NLS-1$
+					int line = frame.getLineNumber();
+					for (int i = 0; i < markers.length; i++) {
+						IMarker marker = markers[i];
+						if (marker.getAttribute(IMarker.LINE_NUMBER, -1) == line) {
+							return marker;
+						}
 					}
-					name+= ".java"; //$NON-NLS-1$
+				} catch (CoreException e) {
 				}
-				packageName = ""; //$NON-NLS-1$
-			} else {
-				if (name == null) {
-					name = packageName.substring(index + 1);
-					int dollar = name.indexOf('$');
-					if (dollar >= 0) {
-						name = name.substring(0, dollar);
-					}
-					name += ".java"; //$NON-NLS-1$
-				}
-				packageName = packageName.substring(0,index);
 			}
-			int line = frame.getLineNumber();
-			Location l = new Location(packageName, name, line);
-			return  (IMarker)fLocationMap.get(l);		
-		} catch (DebugException e) {
-			JDIDebugUIPlugin.log(e);
 		}
 		return null;
 	}	
 	
-	/**
-	 * @see IJavaConditionalBreakpointListener#breakpointHasRuntimeException(IJavaLineBreakpoint, Throwable)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpointListener#breakpointHasRuntimeException(org.eclipse.jdt.debug.core.IJavaLineBreakpoint, org.eclipse.debug.core.DebugException)
 	 */
 	public void breakpointHasRuntimeException(final IJavaLineBreakpoint breakpoint, final DebugException exception) {
 		IStatus status;
@@ -699,11 +622,11 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 		} else {
 			status= exception.getStatus();
 		}
-		openConditionErrorDialog(breakpoint, DebugUIMessages.getString("JavaDebugOptionsManager.Conditional_breakpoint_encountered_runtime_exception._1"), status); //$NON-NLS-1$
+		openConditionErrorDialog(breakpoint, DebugUIMessages.JavaDebugOptionsManager_Conditional_breakpoint_encountered_runtime_exception__1, status); //$NON-NLS-1$
 	}
 
-	/**
-	 * @see IJavaConditionalBreakpointListener#breakpointHasCompilationErrors(IJavaLineBreakpoint, Message[])
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpointListener#breakpointHasCompilationErrors(org.eclipse.jdt.debug.core.IJavaLineBreakpoint, org.eclipse.jdt.core.dom.Message[])
 	 */
 	public void breakpointHasCompilationErrors(final IJavaLineBreakpoint breakpoint, final Message[] errors) {
 		StringBuffer message= new StringBuffer();
@@ -714,7 +637,7 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 			message.append("\n "); //$NON-NLS-1$
 		}
 		IStatus status= new Status(IStatus.ERROR, JDIDebugUIPlugin.getUniqueIdentifier(), IStatus.ERROR, message.toString(), null);
-		openConditionErrorDialog(breakpoint, DebugUIMessages.getString("JavaDebugOptionsManager.Conditional_breakpoint_has_compilation_error(s)._2"), status); //$NON-NLS-1$
+		openConditionErrorDialog(breakpoint, DebugUIMessages.JavaDebugOptionsManager_Conditional_breakpoint_has_compilation_error_s___2, status); //$NON-NLS-1$
 	}
 	
 	private void openConditionErrorDialog(final IJavaLineBreakpoint breakpoint, final String errorMessage, final IStatus status) {
@@ -731,9 +654,10 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 				Shell shell= JDIDebugUIPlugin.getActiveWorkbenchShell();
 				ConditionalBreakpointErrorDialog dialog= new ConditionalBreakpointErrorDialog(shell, message, status);
 				int result = dialog.open();
-				if (result == Dialog.OK) {
-					JavaBreakpointPropertiesDialog breakpointPropertiesDialog = new JavaBreakpointPropertiesDialog(shell, breakpoint);
-					breakpointPropertiesDialog.open();
+				if (result == Window.OK) {
+					JavaBreakpointPropertiesAction action= new JavaBreakpointPropertiesAction();
+					action.selectionChanged(null, new StructuredSelection(breakpoint));
+					action.run(null);
 				}
 			}
 		});
@@ -748,11 +672,15 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 	 * there is a running debug target.
 	 */
 	private void activate() {
+		if (fActivated) {
+			return;
+		}
+		fActivated = true;
 		initializeProblemHandling();
 		notifyTargetsOfFilters();
 		DebugPlugin.getDefault().addDebugEventListener(this);
-		ResourcesPlugin.getWorkspace().addResourceChangeListener(this, IResourceChangeEvent.POST_AUTO_BUILD);
 		JDIDebugModel.addJavaBreakpointListener(this);
+        JavaLogicalStructures.addStructuresListener(this);
 	}	
 
 	/**
@@ -812,13 +740,13 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 					IBreakpoint breakpoint = breakpoints[i];
 					if (breakpoint instanceof IJavaBreakpoint) {
 						String info = fLabelProvider.getText(breakpoint);
-						String type = DebugUIMessages.getString("JavaDebugOptionsManager.Breakpoint___1"); //$NON-NLS-1$
+						String type = DebugUIMessages.JavaDebugOptionsManager_Breakpoint___1; //$NON-NLS-1$
 						if (breakpoint instanceof IJavaMethodBreakpoint || breakpoint instanceof IJavaMethodEntryBreakpoint) {
-							type = DebugUIMessages.getString("JavaDebugOptionsManager.Method_breakpoint___2"); //$NON-NLS-1$
+							type = DebugUIMessages.JavaDebugOptionsManager_Method_breakpoint___2; //$NON-NLS-1$
 						} else if (breakpoint instanceof IJavaWatchpoint) {
-							type = DebugUIMessages.getString("JavaDebugOptionsManager.Watchpoint___3");  //$NON-NLS-1$
+							type = DebugUIMessages.JavaDebugOptionsManager_Watchpoint___3;  //$NON-NLS-1$
 						} else if (breakpoint instanceof IJavaLineBreakpoint) {
-							type = DebugUIMessages.getString("JavaDebugOptionsManager.Line_breakpoint___4"); //$NON-NLS-1$
+							type = DebugUIMessages.JavaDebugOptionsManager_Line_breakpoint___4; //$NON-NLS-1$
 						}
 						breakpoint.getMarker().setAttribute(IMarker.MESSAGE, type + info);
 					}
@@ -826,7 +754,7 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 			}
 		};
 		try {
-			ResourcesPlugin.getWorkspace().run(runnable, null);
+			ResourcesPlugin.getWorkspace().run(runnable, null, 0, null);
 		} catch (CoreException e) {
 			JDIDebugUIPlugin.log(e);
 		}
@@ -852,4 +780,21 @@ public class JavaDebugOptionsManager implements IResourceChangeListener, IDebugE
 		IMarkerDelta[] deltas) {
 	}
 
+    /* (non-Javadoc)
+     * @see org.eclipse.jdt.internal.debug.core.logicalstructures.IJavaStructuresListener#logicalStructuresChanged()
+     */
+    public void logicalStructuresChanged() {
+        // If a Java stack frame is selected in the Debug view, fire a change event on
+        // it so the variables view will update for any structure changes.
+        IAdaptable selected = DebugUITools.getDebugContext();
+        if (selected != null) {
+            IJavaStackFrame frame= (IJavaStackFrame) selected.getAdapter(IJavaStackFrame.class);
+            if (frame != null) {
+                DebugPlugin.getDefault().fireDebugEventSet(new DebugEvent[] { 
+                        new DebugEvent(frame, DebugEvent.CHANGE)
+                });
+            }
+        }
+    }
+
 }
