@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -23,11 +23,11 @@ public class MethodActionFilter implements IActionFilter {
 	 */
 	public boolean testAttribute(Object target, String name, String value) {
 		if (name.equals("MethodActionFilter") //$NON-NLS-1$
-			&& value.equals("isNotAbstract")) { //$NON-NLS-1$
+			&& value.equals("isAbstract")) { //$NON-NLS-1$
 			if (target instanceof IMethod) {
 				IMethod method = (IMethod) target;
 				try {
-					return !Flags.isAbstract(method.getFlags());
+					return Flags.isAbstract(method.getFlags());
 				} catch (JavaModelException e) {
 					JDIDebugUIPlugin.log(e);
 				}
