@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -12,10 +12,13 @@ package org.eclipse.jdt.internal.debug.ui.actions;
 
 
 
+import java.lang.reflect.InvocationTargetException;
 import java.text.MessageFormat;
 import java.util.Iterator;
 
+import org.eclipse.core.resources.IResource;
 import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Status;
 import org.eclipse.debug.core.DebugEvent;
@@ -30,6 +33,7 @@ import org.eclipse.debug.ui.IDebugUIConstants;
 import org.eclipse.debug.ui.IDebugView;
 import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IJavaProject;
+import org.eclipse.jdt.core.JavaCore;
 import org.eclipse.jdt.debug.core.IJavaArray;
 import org.eclipse.jdt.debug.core.IJavaDebugTarget;
 import org.eclipse.jdt.debug.core.IJavaObject;
@@ -42,17 +46,22 @@ import org.eclipse.jdt.debug.eval.IEvaluationEngine;
 import org.eclipse.jdt.debug.eval.IEvaluationListener;
 import org.eclipse.jdt.debug.eval.IEvaluationResult;
 import org.eclipse.jdt.debug.ui.IJavaDebugUIConstants;
+import org.eclipse.jdt.internal.debug.core.JDIDebugPlugin;
 import org.eclipse.jdt.internal.debug.ui.EvaluationContextManager;
 import org.eclipse.jdt.internal.debug.ui.JDIDebugUIPlugin;
-import org.eclipse.jdt.internal.debug.ui.display.DataDisplay;
+import org.eclipse.jdt.internal.debug.ui.JavaWordFinder;
 import org.eclipse.jdt.internal.debug.ui.display.IDataDisplay;
 import org.eclipse.jdt.internal.debug.ui.display.JavaInspectExpression;
 import org.eclipse.jdt.internal.debug.ui.snippeteditor.ISnippetStateChangedListener;
 import org.eclipse.jdt.internal.debug.ui.snippeteditor.JavaSnippetEditor;
 import org.eclipse.jface.action.IAction;
 import org.eclipse.jface.dialogs.ErrorDialog;
+import org.eclipse.jface.operation.IRunnableWithProgress;
+import org.eclipse.jface.text.BadLocationException;
+import org.eclipse.jface.text.IDocument;
+import org.eclipse.jface.text.IRegion;
 import org.eclipse.jface.text.ITextSelection;
-import org.eclipse.jface.text.ITextViewer;
+import org.eclipse.jface.text.Region;
 import org.eclipse.jface.viewers.ISelection;
 import org.eclipse.jface.viewers.ISelectionProvider;
 import org.eclipse.jface.viewers.IStructuredSelection;
@@ -65,11 +74,13 @@ import org.eclipse.ui.IObjectActionDelegate;
 import org.eclipse.ui.IPartListener;
 import org.eclipse.ui.IViewActionDelegate;
 import org.eclipse.ui.IViewPart;
+import org.eclipse.ui.IWorkbench;
 import org.eclipse.ui.IWorkbenchPage;
 import org.eclipse.ui.IWorkbenchPart;
 import org.eclipse.ui.IWorkbenchWindow;
 import org.eclipse.ui.IWorkbenchWindowActionDelegate;
 import org.eclipse.ui.PartInitException;
+import org.eclipse.ui.texteditor.ITextEditor;
 
 import com.sun.jdi.InvocationException;
 import com.sun.jdi.ObjectReference;
@@ -86,6 +97,7 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 	private IWorkbenchPart fTargetPart;
 	private IWorkbenchWindow fWindow;
 	private Object fSelection;
+	private IRegion fRegion;
 	
 	/**
 	 * Is the action waiting for an evaluation.
@@ -186,19 +198,7 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 			if (display.isDisposed()) {
 				return;
 			}
-			if (result.hasErrors()) {
-				display.asyncExec(new Runnable() {
-					public void run() {
-						if (display.isDisposed()) {
-							return;
-						}
-						reportErrors(result);
-						evaluationCleanup();
-					}
-				});
-			} else if (value != null) {
-				displayResult(result);
-			}
+			displayResult(result);
 		}
 	}
 	
@@ -213,52 +213,74 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 	
 	protected void run() {		
 		// eval in context of object or stack frame
-		IJavaObject object = getObjectContext();		
-		IJavaStackFrame stackFrame= getStackFrameContext();
+		final IJavaObject object = getObjectContext();		
+		final IJavaStackFrame stackFrame= getStackFrameContext();
 		if (stackFrame == null) {
-			reportError(ActionMessages.getString("Evaluate.error.message.stack_frame_context")); //$NON-NLS-1$
+			reportError(ActionMessages.Evaluate_error_message_stack_frame_context); //$NON-NLS-1$
 			return;
 		}
 		
 		// check for nested evaluation
 		IJavaThread thread = (IJavaThread)stackFrame.getThread();
 		if (thread.isPerformingEvaluation()) {
-			reportError(ActionMessages.getString("EvaluateAction.Cannot_perform_nested_evaluations._1")); //$NON-NLS-1$
+			reportError(ActionMessages.EvaluateAction_Cannot_perform_nested_evaluations__1); //$NON-NLS-1$
 			return;
 		}
 		
 		setNewTargetPart(getTargetPart());
-		if (stackFrame.isSuspended()) {
-			IJavaElement javaElement= getJavaElement(stackFrame);
-			if (javaElement != null) {
-				IJavaProject project = javaElement.getJavaProject();
-				IEvaluationEngine engine = null;
-				try {
-					Object selection= getSelectedObject();
-					if (!(selection instanceof String)) {
-						return;
-					}
-					String expression= (String)selection;
-					
-					engine = JDIDebugUIPlugin.getDefault().getEvaluationEngine(project, (IJavaDebugTarget)stackFrame.getDebugTarget());
-					setEvaluating(true);
-					if (object == null) {
-						engine.evaluate(expression, stackFrame, this, DebugEvent.EVALUATION, true);
-					} else {
-						engine.evaluate(expression, object, (IJavaThread)stackFrame.getThread(), this, DebugEvent.EVALUATION, true);
-					}
-					return;
-				} catch (CoreException e) {
-					reportError(getExceptionMessage(e));
-				}
-			} else {
-				reportError(ActionMessages.getString("Evaluate.error.message.src_context")); //$NON-NLS-1$
-			}
-		} else {
-			// thread not suspended
-			reportError(ActionMessages.getString("EvaluateAction.Thread_not_suspended_-_unable_to_perform_evaluation._1")); //$NON-NLS-1$
-		}
-		evaluationCleanup();
+        
+        IRunnableWithProgress runnable = new IRunnableWithProgress() {
+            public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
+                if (stackFrame.isSuspended()) {
+                    IJavaElement javaElement= getJavaElement(stackFrame);
+                    if (javaElement != null) {
+                        IJavaProject project = javaElement.getJavaProject();
+                        IEvaluationEngine engine = null;
+                        try {
+                            Object selection= getSelectedObject();
+                            if (!(selection instanceof String)) {
+                                return;
+                            }
+                            String expression= (String)selection;
+                            
+                            engine = JDIDebugPlugin.getDefault().getEvaluationEngine(project, (IJavaDebugTarget)stackFrame.getDebugTarget());
+                            setEvaluating(true);
+                            boolean hitBreakpoints= JDIDebugModel.getPreferences().getBoolean(JDIDebugModel.PREF_SUSPEND_FOR_BREAKPOINTS_DURING_EVALUATION);
+                            if (object == null) {
+                                engine.evaluate(expression, stackFrame, EvaluateAction.this, DebugEvent.EVALUATION, hitBreakpoints);
+                            } else {
+                                engine.evaluate(expression, object, (IJavaThread)stackFrame.getThread(), EvaluateAction.this, DebugEvent.EVALUATION, hitBreakpoints);
+                            }
+                            return;
+                        } catch (CoreException e) {
+                            throw new InvocationTargetException(e, getExceptionMessage(e));
+                        }
+                    }
+                    throw new InvocationTargetException(null, ActionMessages.Evaluate_error_message_src_context); //$NON-NLS-1$
+                }
+                // thread not suspended
+                throw new InvocationTargetException(null, ActionMessages.EvaluateAction_Thread_not_suspended___unable_to_perform_evaluation__1); //$NON-NLS-1$
+            }
+        };
+        
+        IWorkbench workbench = JDIDebugUIPlugin.getDefault().getWorkbench();
+        try {
+            workbench.getProgressService().busyCursorWhile(runnable);
+        } catch (InvocationTargetException e) {
+        	evaluationCleanup();
+        	String message = e.getMessage();
+        	if (message == null) {
+        		message = e.getClass().getName();
+        		if (e.getCause() != null) {
+        			message = e.getCause().getClass().getName();
+        			if (e.getCause().getMessage() != null) {
+        				message = e.getCause().getMessage();
+        			}
+        		}
+        	}
+            reportError(message);
+        } catch (InterruptedException e) {
+        }
 	}
 		
 	protected IJavaElement getJavaElement(IStackFrame stackFrame) {
@@ -275,6 +297,11 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 		Object sourceElement = locator.getSourceElement(stackFrame);
 		if (sourceElement instanceof IJavaElement) {
 			return (IJavaElement) sourceElement;
+		} else if (sourceElement instanceof IResource) {
+			IJavaProject project = JavaCore.create(((IResource)sourceElement).getProject());
+			if (project.exists()) {
+				return project;
+			}
 		}			
 		return null;
 	}
@@ -296,22 +323,34 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 	 */
 	protected void resolveSelectedObject() {
 		Object selectedObject= null;
+		fRegion = null;
 		ISelection selection= getTargetSelection();
 		if (selection instanceof ITextSelection) {
-			String text= ((ITextSelection)selection).getText();
+			ITextSelection ts = (ITextSelection)selection;
+			String text= ts.getText();
 			if (textHasContent(text)) {
 				selectedObject= text;
+				fRegion = new Region(ts.getOffset(), ts.getLength());
+			} else if (getTargetPart() instanceof IEditorPart) {
+				IEditorPart editor= (IEditorPart)getTargetPart();
+				if (editor instanceof ITextEditor) {
+					selectedObject = resolveSelectedObjectUsingToken(selectedObject, ts, editor);
+				}
 			}
 		} else if (selection instanceof IStructuredSelection) {
 			if (!selection.isEmpty()) {
 				if (getTargetPart().getSite().getId().equals(IDebugUIConstants.ID_DEBUG_VIEW)) {
 					//work on the editor selection
-					setTargetPart(getTargetPart().getSite().getPage().getActiveEditor());
+					IEditorPart editor= getTargetPart().getSite().getPage().getActiveEditor();
+					setTargetPart(editor);
 					selection= getTargetSelection();
 					if (selection instanceof ITextSelection) {
-						String text= ((ITextSelection)selection).getText();
+						ITextSelection ts = (ITextSelection)selection;
+						String text= ts.getText();
 						if (textHasContent(text)) {
 							selectedObject= text;
+						} else if (editor instanceof ITextEditor) {
+							selectedObject= resolveSelectedObjectUsingToken(selectedObject, ts, editor);
 						}
 					}
 				} else {
@@ -330,6 +369,19 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 		setSelectedObject(selectedObject);
 	}
 	
+	private Object resolveSelectedObjectUsingToken(Object selectedObject, ITextSelection ts, IEditorPart editor) {
+		ITextEditor textEditor= (ITextEditor) editor;
+		IDocument doc= textEditor.getDocumentProvider().getDocument(editor.getEditorInput());
+		fRegion= JavaWordFinder.findWord(doc, ts.getOffset());
+		if (fRegion != null) {
+			try {
+				selectedObject= doc.get(fRegion.getOffset(), fRegion.getLength());
+			} catch (BadLocationException e) {
+			}
+		}
+		return selectedObject;
+	}
+
 	protected ISelection getTargetSelection() {
 		IWorkbenchPart part = getTargetPart();
 		if (part != null) {
@@ -369,9 +421,8 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 	protected Shell getShell() {
 		if (getTargetPart() != null) {
 			return getTargetPart().getSite().getShell();
-		} else {
-			return JDIDebugUIPlugin.getActiveWorkbenchShell();
 		}
+		return JDIDebugUIPlugin.getActiveWorkbenchShell();
 	}
 	
 	protected IDataDisplay getDataDisplay() {
@@ -383,12 +434,12 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 		if (page != null) {
 			IWorkbenchPart activePart= page.getActivePart();
 			if (activePart != null) {
-				IViewPart view = page.findView(IJavaDebugUIConstants.ID_DISPLAY_VIEW);;
+				IViewPart view = page.findView(IJavaDebugUIConstants.ID_DISPLAY_VIEW);
 				if (view == null) {
 					try {
 						view= page.showView(IJavaDebugUIConstants.ID_DISPLAY_VIEW);
 					} catch (PartInitException e) {
-						JDIDebugUIPlugin.errorDialog(ActionMessages.getString("EvaluateAction.Cannot_open_Display_view"), e); //$NON-NLS-1$
+						JDIDebugUIPlugin.errorDialog(ActionMessages.EvaluateAction_Cannot_open_Display_view, e); //$NON-NLS-1$
 					} finally {
 						page.activate(activePart);
 					}
@@ -407,12 +458,6 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 		IWorkbenchPart part= getTargetPart();
 		if (part != null) {
 			IDataDisplay display= (IDataDisplay)part.getAdapter(IDataDisplay.class);
-//			if (display == null) {
-//				ITextViewer viewer = (ITextViewer)part.getAdapter(ITextViewer.class);
-//				if (viewer != null) {
-//					display= new DataDisplay(viewer);
-//				}
-//			}
 			if (display != null) {
 				IWorkbenchPage page= JDIDebugUIPlugin.getActivePage();
 				if (page != null) {
@@ -434,14 +479,11 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 				if (display != null) {
 					return display;
 				}	
-				ITextViewer viewer = (ITextViewer)activePart.getAdapter(ITextViewer.class);
-				if (viewer != null) {
-					return new DataDisplay(viewer);
-				}
 			}
 		}
 		return null;
 	}
+	
 	protected boolean textHasContent(String text) {
 		if (text != null) {
 			int length= text.length();
@@ -468,13 +510,13 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 		IDataDisplay dataDisplay= getDirectDataDisplay();
 		if (dataDisplay != null) {
 			if (message.length() != 0) {
-				dataDisplay.displayExpressionValue(MessageFormat.format(ActionMessages.getString("EvaluateAction.(evaluation_failed)_Reason"), new String[] {format(message)})); //$NON-NLS-1$
+				dataDisplay.displayExpressionValue(MessageFormat.format(ActionMessages.EvaluateAction__evaluation_failed__Reason, new String[] {format(message)})); //$NON-NLS-1$
 			} else {
-				dataDisplay.displayExpressionValue(ActionMessages.getString("EvaluateAction.(evaluation_failed)_1")); //$NON-NLS-1$
+				dataDisplay.displayExpressionValue(ActionMessages.EvaluateAction__evaluation_failed__1); //$NON-NLS-1$
 			}
 		} else {
 			Status status= new Status(IStatus.ERROR, JDIDebugUIPlugin.getUniqueIdentifier(), IStatus.ERROR, message, null);
-			ErrorDialog.openError(getShell(), ActionMessages.getString("Evaluate.error.title.eval_problems"), null, status); //$NON-NLS-1$
+			ErrorDialog.openError(getShell(), ActionMessages.Evaluate_error_title_eval_problems, null, status); //$NON-NLS-1$
 		}
 	}
 	
@@ -490,7 +532,7 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 		return result.toString();
 	}
 	
-	protected String getExceptionMessage(Throwable exception) {
+	public static String getExceptionMessage(Throwable exception) {
 		if (exception instanceof CoreException) {
 			CoreException ce = (CoreException)exception;
 			Throwable throwable= ce.getStatus().getException();
@@ -502,9 +544,9 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 			}
 			return ce.getStatus().getMessage();
 		}
-		String message= MessageFormat.format(ActionMessages.getString("Evaluate.error.message.direct_exception"), new Object[] { exception.getClass() }); //$NON-NLS-1$
+		String message= MessageFormat.format(ActionMessages.Evaluate_error_message_direct_exception, new Object[] { exception.getClass() }); //$NON-NLS-1$
 		if (exception.getMessage() != null) {
-			message= MessageFormat.format(ActionMessages.getString("Evaluate.error.message.exception.pattern"), new Object[] { message, exception.getMessage() }); //$NON-NLS-1$
+			message= MessageFormat.format(ActionMessages.Evaluate_error_message_exception_pattern, new Object[] { message, exception.getMessage() }); //$NON-NLS-1$
 		}
 		return message;
 	}
@@ -512,19 +554,18 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 	/**
 	 * Returns a message for the exception wrapped in an invocation exception
 	 */
-	protected String getInvocationExceptionMessage(com.sun.jdi.InvocationException exception) {
-			InvocationException ie= (InvocationException) exception;
+	protected static String getInvocationExceptionMessage(com.sun.jdi.InvocationException exception) {
+			InvocationException ie= exception;
 			ObjectReference ref= ie.exception();
-			return MessageFormat.format(ActionMessages.getString("Evaluate.error.message.wrapped_exception"), new Object[] { ref.referenceType().name() }); //$NON-NLS-1$
+			return MessageFormat.format(ActionMessages.Evaluate_error_message_wrapped_exception, new Object[] { ref.referenceType().name() }); //$NON-NLS-1$
 	}
 	
 	protected String getErrorMessage(IEvaluationResult result) {
 		String[] errors= result.getErrorMessages();
 		if (errors.length == 0) {
 			return getExceptionMessage(result.getException());
-		} else {
-			return getErrorMessage(errors);
 		}
+		return getErrorMessage(errors);
 	}
 	
 	protected String getErrorMessage(String[] errors) {
@@ -534,22 +575,22 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 			if (i == 0) {
 				message= msg;
 			} else {
-				message= MessageFormat.format(ActionMessages.getString("Evaluate.error.problem_append_pattern"), new Object[] { message, msg }); //$NON-NLS-1$
+				message= MessageFormat.format(ActionMessages.Evaluate_error_problem_append_pattern, new Object[] { message, msg }); //$NON-NLS-1$
 			}
 		}
 		return message;
 	}
 	
-	/**
-	 * @see IActionDelegate#run(IAction)
+	/* (non-Javadoc)
+	 * @see org.eclipse.ui.IActionDelegate#run(IAction)
 	 */
 	public void run(IAction action) {
 		update();
 		run();
 	}
 
-	/**
-	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
+	/* (non-Javadoc)
+	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(IAction, ISelection)
 	 */
 	public void selectionChanged(IAction action, ISelection selection) { 
 		setAction(action);
@@ -711,7 +752,7 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 	 * @see ISnippetStateChangedListener#snippetStateChanged(JavaSnippetEditor)
 	 */
 	public void snippetStateChanged(JavaSnippetEditor editor) {
-		if (editor != null && !editor.isEvaluating()) {
+		if (editor != null && !editor.isEvaluating() && editor.getFile() != null) {
 			update();
 		} else {
 			getAction().setEnabled(false);
@@ -733,4 +774,13 @@ public abstract class EvaluateAction implements IEvaluationListener, IWorkbenchW
 	protected void setEvaluating(boolean evaluating) {
 		fEvaluating = evaluating;
 	}
+	
+	/**
+	 * Returns the selected text region, or <code>null</code> if none.
+	 * 
+	 * @return
+	 */
+	protected IRegion getRegion() {
+		return fRegion;
+	}
 }
