@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -21,7 +21,7 @@ public class OpenDeclaringTypeAction extends OpenStackFrameAction {
 		return ((IJavaStackFrame)frame).getDeclaringTypeName();
 	}
 	
-	protected void doAction(Object e) throws DebugException {
+	protected void doAction(Object e) {
 		Object sourceElement= getSourceElement(e);
 		if (sourceElement != null) {
 			openInEditor(sourceElement);
