@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -29,10 +29,12 @@ import java.util.Set;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
+import javax.xml.transform.TransformerException;
 
-import org.apache.xerces.dom.DocumentImpl;
+import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.jdt.launching.IVMInstall;
+import org.eclipse.jdt.launching.IVMInstall2;
 import org.eclipse.jdt.launching.IVMInstallType;
 import org.eclipse.jdt.launching.JavaRuntime;
 import org.eclipse.jdt.launching.LibraryLocation;
@@ -43,6 +45,7 @@ import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
+import org.xml.sax.helpers.DefaultHandler;
 
 /**
  * This is a container for VM definitions such as the VM definitions that are
@@ -68,16 +71,16 @@ public class VMDefinitionsContainer {
 	 */
 	private Map fVMTypeToVMMap;
 	
+	/**
+	 * Cached list of VMs in this container
+	 */
+	private List fVMList;
+	
 	/**
 	 * VMs managed by this container whose install locations don't actually exist.
 	 */
 	private List fInvalidVMList;
-		
-	/**
-	 * The number of VMs managed by this container.
-	 */
-	private int fVMCount = 0;
-	
+			
 	/**
 	 * The composite identifier of the default VM.  This consists of the install type ID
 	 * plus an ID for the VM.
@@ -94,7 +97,8 @@ public class VMDefinitionsContainer {
 	 */
 	public VMDefinitionsContainer() {
 		fVMTypeToVMMap = new HashMap(10);
-		fInvalidVMList = new ArrayList(5);			
+		fInvalidVMList = new ArrayList(10);	
+		fVMList = new ArrayList(10);		
 	}
 		
 	/**
@@ -107,19 +111,21 @@ public class VMDefinitionsContainer {
 	 * 
 	 * @param vm the VM to be added to this container
 	 */
-	public void addVM(IVMInstall vm) {	
-		IVMInstallType vmInstallType = vm.getVMInstallType();
-		List vmList = (List) fVMTypeToVMMap.get(vmInstallType);
-		if (vmList == null) {
-			vmList = new ArrayList(3);
-			fVMTypeToVMMap.put(vmInstallType, vmList);			
-		}		
-		vmList.add(vm);
-		File installLocation = vm.getInstallLocation();
-		if (installLocation == null || !vmInstallType.validateInstallLocation(installLocation).isOK()) {
-			fInvalidVMList.add(vm);
+	public void addVM(IVMInstall vm) {
+		if (!fVMList.contains(vm)) {	
+			IVMInstallType vmInstallType = vm.getVMInstallType();
+			List vmList = (List) fVMTypeToVMMap.get(vmInstallType);
+			if (vmList == null) {
+				vmList = new ArrayList(3);
+				fVMTypeToVMMap.put(vmInstallType, vmList);			
+			}
+			vmList.add(vm);
+			File installLocation = vm.getInstallLocation();
+			if (installLocation == null || !vmInstallType.validateInstallLocation(installLocation).isOK()) {
+				fInvalidVMList.add(vm);
+			}
+			fVMList.add(vm);
 		}
-		fVMCount++;
 	}
 	
 	/**
@@ -159,17 +165,7 @@ public class VMDefinitionsContainer {
 	 * @return List the data structure containing all VMs managed by this container
 	 */
 	public List getVMList() {
-		List resultList = new ArrayList(fVMCount);
-		
-		Set keySet = getVMTypeToVMMap().keySet();
-		Iterator keyIterator = keySet.iterator();
-		while (keyIterator.hasNext()) {
-			IVMInstallType vmInstallType = (IVMInstallType) keyIterator.next();
-			List vmList = (List) getVMTypeToVMMap().get(vmInstallType);
-			resultList.addAll(vmList);
-		}
-		
-		return resultList;
+		return fVMList;
 	}
 	
 	/**
@@ -236,11 +232,13 @@ public class VMDefinitionsContainer {
 	 * @throws IOException if this method fails. Reasons include:<ul>
 	 * <li>serialization of the XML document failed</li>
 	 * </ul>
+	 * @throws ParserConfigurationException if creation of the XML document failed
+	 * @throws TransformerException if serialization of the XML document failed
 	 */
-	public String getAsXML() throws IOException{
+	public String getAsXML() throws ParserConfigurationException, IOException, TransformerException {
 		
 		// Create the Document and the top-level node
-		Document doc = new DocumentImpl();
+		Document doc = LaunchingPlugin.getDocument();
 		Element config = doc.createElement("vmSettings");    //$NON-NLS-1$
 		doc.appendChild(config);
 		
@@ -319,6 +317,22 @@ public class VMDefinitionsContainer {
 			element.setAttribute("javadocURL", url.toExternalForm()); //$NON-NLS-1$
 		}
 		
+		if (vm instanceof IVMInstall2) {
+			String vmArgs = ((IVMInstall2)vm).getVMArgs();
+			if (vmArgs != null && vmArgs.length() > 0) {
+				element.setAttribute("vmargs", vmArgs); //$NON-NLS-1$
+			}
+		} else {
+			String[] vmArgs = vm.getVMArguments();
+			if (vmArgs != null && vmArgs.length > 0) {
+				StringBuffer buffer = new StringBuffer();
+				for (int i = 0; i < vmArgs.length; i++) {
+					buffer.append(vmArgs[i] + " "); //$NON-NLS-1$
+				}
+				element.setAttribute("vmargs", buffer.toString()); //$NON-NLS-1$
+			}
+		}
+		
 		return element;
 	}
 	
@@ -332,7 +346,16 @@ public class VMDefinitionsContainer {
 			Element element = doc.createElement("libraryLocation");  //$NON-NLS-1$
 			element.setAttribute("jreJar", locations[i].getSystemLibraryPath().toString()); //$NON-NLS-1$
 			element.setAttribute("jreSrc", locations[i].getSystemLibrarySourcePath().toString()); //$NON-NLS-1$
-			element.setAttribute("pkgRoot", locations[i].getPackageRootPath().toString()); //$NON-NLS-1$
+
+			IPath packageRootPath = locations[i].getPackageRootPath();
+            if (packageRootPath != null) {
+                element.setAttribute("pkgRoot", packageRootPath.toString()); //$NON-NLS-1$
+            }
+            
+			URL url= locations[i].getJavadocLocation();
+			if (url != null) {
+				element.setAttribute("jreJavadoc", url.toExternalForm()); //$NON-NLS-1$
+			}
 			root.appendChild(element);
 		}
 		return root;
@@ -373,19 +396,20 @@ public class VMDefinitionsContainer {
 		Element config= null;		
 		try {
 			DocumentBuilder parser= DocumentBuilderFactory.newInstance().newDocumentBuilder();
+			parser.setErrorHandler(new DefaultHandler());
 			config = parser.parse(new InputSource(reader)).getDocumentElement();
 		} catch (SAXException e) {
-			throw new IOException(LaunchingMessages.getString("JavaRuntime.badFormat")); //$NON-NLS-1$
+			throw new IOException(LaunchingMessages.JavaRuntime_badFormat); //$NON-NLS-1$
 		} catch (ParserConfigurationException e) {
 			reader.close();
-			throw new IOException(LaunchingMessages.getString("JavaRuntime.badFormat")); //$NON-NLS-1$
+			throw new IOException(LaunchingMessages.JavaRuntime_badFormat); //$NON-NLS-1$
 		} finally {
 			reader.close();
 		}
 		
 		// If the top-level node wasn't what we expected, bail out
 		if (!config.getNodeName().equalsIgnoreCase("vmSettings")) { //$NON-NLS-1$
-			throw new IOException(LaunchingMessages.getString("JavaRuntime.badFormat")); //$NON-NLS-1$
+			throw new IOException(LaunchingMessages.JavaRuntime_badFormat); //$NON-NLS-1$
 		}
 		
 		// Populate the default VM-related fields
@@ -433,7 +457,7 @@ public class VMDefinitionsContainer {
 				}
 			}
 		} else {
-			LaunchingPlugin.log(LaunchingMessages.getString("JavaRuntime.VM_type_element_with_unknown_id_1")); //$NON-NLS-1$
+			LaunchingPlugin.log(LaunchingMessages.JavaRuntime_VM_type_element_with_unknown_id_1); //$NON-NLS-1$
 		}
 	}
 
@@ -488,8 +512,14 @@ public class VMDefinitionsContainer {
 					LaunchingPlugin.log(e);
 				}
 			}
+			
+			// vm Arguments
+			String vmArgs = vmElement.getAttribute("vmargs"); //$NON-NLS-1$
+			if (vmArgs != null && vmArgs.length() >0) {
+				vmStandin.setVMArgs(vmArgs);
+			}
 		} else {
-			LaunchingPlugin.log(LaunchingMessages.getString("JavaRuntime.VM_element_specified_with_no_id_attribute_2")); //$NON-NLS-1$
+			LaunchingPlugin.log(LaunchingMessages.JavaRuntime_VM_element_specified_with_no_id_attribute_2); //$NON-NLS-1$
 		}
 	}	
 	
@@ -501,11 +531,21 @@ public class VMDefinitionsContainer {
 		String jreJar= libLocationElement.getAttribute("jreJar"); //$NON-NLS-1$
 		String jreSrc= libLocationElement.getAttribute("jreSrc"); //$NON-NLS-1$
 		String pkgRoot= libLocationElement.getAttribute("pkgRoot"); //$NON-NLS-1$
-		if (jreJar != null && jreSrc != null && pkgRoot != null) {
-			return new LibraryLocation(new Path(jreJar), new Path(jreSrc), new Path(pkgRoot));
+		String jreJavadoc= libLocationElement.getAttribute("jreJavadoc"); //$NON-NLS-1$
+		URL javadocURL= null;
+		if (jreJavadoc.length() == 0) {
+			jreJavadoc= null;
 		} else {
-			LaunchingPlugin.log(LaunchingMessages.getString("JavaRuntime.Library_location_element_incorrectly_specified_3")); //$NON-NLS-1$
+			try {
+				javadocURL= new URL(jreJavadoc);
+			} catch (MalformedURLException e) {
+				LaunchingPlugin.log(LaunchingMessages.JavaRuntime_Library_location_element_incorrectly_specified_3); //$NON-NLS-1$
+			}
 		}
+		if (jreJar != null && jreSrc != null && pkgRoot != null) {
+			return new LibraryLocation(new Path(jreJar), new Path(jreSrc), new Path(pkgRoot), javadocURL);
+		}
+		LaunchingPlugin.log(LaunchingMessages.JavaRuntime_Library_location_element_incorrectly_specified_3); //$NON-NLS-1$
 		return null;
 	}
 	
