@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -22,10 +22,10 @@ import org.eclipse.jface.resource.ImageDescriptor;
 import org.eclipse.jface.util.Assert;
 
 import org.eclipse.ui.ISharedImages;
+import org.eclipse.ui.ide.IDE;
 import org.eclipse.ui.model.IWorkbenchAdapter;
 
 import org.eclipse.jdt.core.Flags;
-import org.eclipse.jdt.core.IField;
 import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IJavaProject;
 import org.eclipse.jdt.core.IMember;
@@ -70,12 +70,10 @@ public class JavaElementImageProvider {
 
 	private static ImageDescriptor DESC_OBJ_PROJECT_CLOSED;	
 	private static ImageDescriptor DESC_OBJ_PROJECT;	
-	private static ImageDescriptor DESC_OBJ_FOLDER;
 	{
 		ISharedImages images= JavaPlugin.getDefault().getWorkbench().getSharedImages(); 
-		DESC_OBJ_PROJECT_CLOSED= images.getImageDescriptor(ISharedImages.IMG_OBJ_PROJECT_CLOSED);
-		DESC_OBJ_PROJECT= 		 images.getImageDescriptor(ISharedImages.IMG_OBJ_PROJECT);
-		DESC_OBJ_FOLDER= 		 images.getImageDescriptor(ISharedImages.IMG_OBJ_FOLDER);
+		DESC_OBJ_PROJECT_CLOSED= images.getImageDescriptor(IDE.SharedImages.IMG_OBJ_PROJECT_CLOSED);
+		DESC_OBJ_PROJECT= 		 images.getImageDescriptor(IDE.SharedImages.IMG_OBJ_PROJECT);
 	}
 	
 	private ImageDescriptorRegistry fRegistry;
@@ -136,7 +134,7 @@ public class JavaElementImageProvider {
 	}	
 
 	/**
-	 * Returns an image descriptor for a compilatio unit not on the class path.
+	 * Returns an image descriptor for a compilation unit not on the class path.
 	 * The descriptor includes overlays, if specified.
 	 */
 	public ImageDescriptor getCUResourceImageDescriptor(IFile file, int flags) {
@@ -182,14 +180,22 @@ public class JavaElementImageProvider {
 			switch (element.getElementType()) {	
 				case IJavaElement.INITIALIZER:
 					return JavaPluginImages.DESC_MISC_PRIVATE; // 23479
-				case IJavaElement.METHOD:
+				case IJavaElement.METHOD: {
+					IMethod method= (IMethod) element;
+					IType declType= method.getDeclaringType();
+					int flags= method.getFlags();
+					if (declType.isEnum() && isDefaultFlag(flags) && method.isConstructor())
+						return JavaPluginImages.DESC_MISC_PRIVATE;
+					return getMethodImageDescriptor(JavaModelUtil.isInterfaceOrAnnotation(declType), flags);				
+				}
+				case IJavaElement.FIELD: {
 					IMember member= (IMember) element;
-					return getMethodImageDescriptor(member.getDeclaringType().isInterface(), member.getFlags());				
-				
-				case IJavaElement.FIELD:
-					IField field= (IField) element;
-					return getFieldImageDescriptor(field.getDeclaringType().isInterface(), field.getFlags());				
-					
+					IType declType= member.getDeclaringType();
+					return getFieldImageDescriptor(JavaModelUtil.isInterfaceOrAnnotation(declType), member.getFlags());
+				}
+				case IJavaElement.LOCAL_VARIABLE:
+					return JavaPluginImages.DESC_OBJS_LOCAL_VARIABLE;				
+
 				case IJavaElement.PACKAGE_DECLARATION:
 					return JavaPluginImages.DESC_OBJS_PACKDECL;
 				
@@ -201,13 +207,11 @@ public class JavaElementImageProvider {
 				
 				case IJavaElement.TYPE: {
 					IType type= (IType) element;
-					boolean isInterface= type.isInterface();
-					
-					if (useLightIcons(renderFlags)) {
-						return isInterface ? JavaPluginImages.DESC_OBJS_INTERFACEALT : JavaPluginImages.DESC_OBJS_CLASSALT;
-					}
-					boolean isInner= type.getDeclaringType() != null;
-					return getTypeImageDescriptor(isInterface, isInner, type.getFlags());
+
+					IType declType= type.getDeclaringType();
+					boolean isInner= declType != null;
+					boolean isInInterfaceOrAnnotation= isInner && JavaModelUtil.isInterfaceOrAnnotation(declType);
+					return getTypeImageDescriptor(isInner, isInInterfaceOrAnnotation, type.getFlags(), useLightIcons(renderFlags));
 				}
 
 				case IJavaElement.PACKAGE_FRAGMENT_ROOT: {
@@ -267,10 +271,13 @@ public class JavaElementImageProvider {
 					
 				case IJavaElement.JAVA_MODEL:
 					return JavaPluginImages.DESC_OBJS_JAVA_MODEL;
+
+				case IJavaElement.TYPE_PARAMETER:
+					return JavaPluginImages.DESC_OBJS_LOCAL_VARIABLE;
 			}
 			
-			Assert.isTrue(false, JavaUIMessages.getString("JavaImageLabelprovider.assert.wrongImage")); //$NON-NLS-1$
-			return null; //$NON-NLS-1$
+			Assert.isTrue(false, JavaUIMessages.JavaImageLabelprovider_assert_wrongImage); 
+			return JavaPluginImages.DESC_OBJS_GHOST;
 		
 		} catch (JavaModelException e) {
 			if (e.isDoesNotExist())
@@ -280,6 +287,10 @@ public class JavaElementImageProvider {
 		}
 	}
 	
+	private static boolean isDefaultFlag(int flags) {
+		return !Flags.isPublic(flags) && !Flags.isProtected(flags) && !Flags.isPrivate(flags);
+	}
+	
 	protected ImageDescriptor getPackageFragmentIcon(IJavaElement element, int renderFlags) throws JavaModelException {
 		IPackageFragment fragment= (IPackageFragment)element;
 		boolean containsJavaElements= false;
@@ -312,13 +323,16 @@ public class JavaElementImageProvider {
 				int modifiers= member.getFlags();
 				if (Flags.isAbstract(modifiers) && confirmAbstract(member))
 					flags |= JavaElementImageDescriptor.ABSTRACT;
-				if (Flags.isFinal(modifiers) || isInterfaceField(member))
+				if (Flags.isFinal(modifiers) || isInterfaceOrAnnotationField(member) || isEnumConstant(member, modifiers))
 					flags |= JavaElementImageDescriptor.FINAL;
 				if (Flags.isSynchronized(modifiers) && confirmSynchronized(member))
 					flags |= JavaElementImageDescriptor.SYNCHRONIZED;
-				if (Flags.isStatic(modifiers) || isInterfaceField(member))
+				if (Flags.isStatic(modifiers) || isInterfaceOrAnnotationFieldOrType(member) || isEnumConstant(member, modifiers))
 					flags |= JavaElementImageDescriptor.STATIC;
 				
+				if (Flags.isDeprecated(modifiers))
+					flags |= JavaElementImageDescriptor.DEPRECATED;
+				
 				if (member.getElementType() == IJavaElement.TYPE) {
 					if (JavaModelUtil.hasMainMethod((IType) member)) {
 						flags |= JavaElementImageDescriptor.RUNNABLE;
@@ -334,26 +348,44 @@ public class JavaElementImageProvider {
 	private static boolean confirmAbstract(IMember element) throws JavaModelException {
 		// never show the abstract symbol on interfaces or members in interfaces
 		if (element.getElementType() == IJavaElement.TYPE) {
-			return ((IType) element).isClass();
+			return ! JavaModelUtil.isInterfaceOrAnnotation((IType) element);
 		}
-		return element.getDeclaringType().isClass();
+		return ! JavaModelUtil.isInterfaceOrAnnotation(element.getDeclaringType());
 	}
 	
-	private static boolean isInterfaceField(IMember element) throws JavaModelException {
-		// always show the final && static symbol on interface fields
+	private static boolean isInterfaceOrAnnotationField(IMember element) throws JavaModelException {
+		// always show the final symbol on interface fields
 		if (element.getElementType() == IJavaElement.FIELD) {
-			return element.getDeclaringType().isInterface();
+			return JavaModelUtil.isInterfaceOrAnnotation(element.getDeclaringType());
 		}
 		return false;
 	}	
 	
+	private static boolean isInterfaceOrAnnotationFieldOrType(IMember element) throws JavaModelException {
+		// always show the static symbol on interface fields and types
+		if (element.getElementType() == IJavaElement.FIELD) {
+			return JavaModelUtil.isInterfaceOrAnnotation(element.getDeclaringType());
+		} else if (element.getElementType() == IJavaElement.TYPE && element.getDeclaringType() != null) {
+			return JavaModelUtil.isInterfaceOrAnnotation(element.getDeclaringType());
+		}
+		return false;
+	}	
+	
+	private static boolean isEnumConstant(IMember element, int modifiers) throws JavaModelException {
+		if (element.getElementType() == IJavaElement.FIELD) {
+			return Flags.isEnum(modifiers);
+		}
+		return false;
+	}
+
 	private static boolean confirmSynchronized(IJavaElement member) {
 		// Synchronized types are allowed but meaningless.
 		return member.getElementType() != IJavaElement.TYPE;
 	}
 	
-	public static ImageDescriptor getMethodImageDescriptor(boolean isInInterface, int flags) {
-		if (Flags.isPublic(flags) || isInInterface)
+	
+	public static ImageDescriptor getMethodImageDescriptor(boolean isInInterfaceOrAnnotation, int flags) {
+		if (Flags.isPublic(flags) || isInInterfaceOrAnnotation)
 			return JavaPluginImages.DESC_MISC_PUBLIC;
 		if (Flags.isProtected(flags))
 			return JavaPluginImages.DESC_MISC_PROTECTED;
@@ -363,8 +395,8 @@ public class JavaElementImageProvider {
 		return JavaPluginImages.DESC_MISC_DEFAULT;
 	}
 		
-	public static ImageDescriptor getFieldImageDescriptor(boolean isInInterface, int flags) {
-		if (Flags.isPublic(flags) || isInInterface)
+	public static ImageDescriptor getFieldImageDescriptor(boolean isInInterfaceOrAnnotation, int flags) {
+		if (Flags.isPublic(flags) || isInInterfaceOrAnnotation || Flags.isEnum(flags))
 			return JavaPluginImages.DESC_FIELD_PUBLIC;
 		if (Flags.isProtected(flags))
 			return JavaPluginImages.DESC_FIELD_PROTECTED;
@@ -374,21 +406,54 @@ public class JavaElementImageProvider {
 		return JavaPluginImages.DESC_FIELD_DEFAULT;
 	}		
 	
-	public static ImageDescriptor getTypeImageDescriptor(boolean isInterface, boolean isInner, int flags) {
-		if (isInner) {
-			if (isInterface) {
-				return getInnerInterfaceImageDescriptor(flags);
-			} else {
-				return getInnerClassImageDescriptor(flags);
+	/**
+	 * @deprecated
+	 */
+	public static ImageDescriptor getTypeImageDescriptor(boolean isInterface, boolean isInner, boolean isInInterface, int flags) {
+		return getTypeImageDescriptor(isInner, isInInterface, isInterface ? flags | Flags.AccInterface : flags, false);
+	}
+	
+	public static ImageDescriptor getTypeImageDescriptor(boolean isInner, boolean isInInterfaceOrAnnotation, int flags, boolean useLightIcons) {
+		if (Flags.isEnum(flags)) {
+			if (useLightIcons) {
+				return JavaPluginImages.DESC_OBJS_ENUM_ALT;
 			}
+			if (isInner) {
+				return getInnerEnumImageDescriptor(isInInterfaceOrAnnotation, flags);
+			}
+			return getEnumImageDescriptor(flags);
+		} else if (Flags.isAnnotation(flags)) {
+			if (useLightIcons) {
+				return JavaPluginImages.DESC_OBJS_ANNOTATION_ALT;
+			}
+			if (isInner) {
+				return getInnerAnnotationImageDescriptor(isInInterfaceOrAnnotation, flags);
+			}
+			return getAnnotationImageDescriptor(flags);
+		}  else if (Flags.isInterface(flags)) {
+			if (useLightIcons) {
+				return JavaPluginImages.DESC_OBJS_INTERFACEALT;
+			}
+			if (isInner) {
+				return getInnerInterfaceImageDescriptor(isInInterfaceOrAnnotation, flags);
+			}
+			return getInterfaceImageDescriptor(flags);
 		} else {
-			if (isInterface) {
-				return getInterfaceImageDescriptor(flags);
-			} else {
-				return getClassImageDescriptor(flags);
+			if (useLightIcons) {
+				return JavaPluginImages.DESC_OBJS_CLASSALT;
 			}
+			if (isInner) {
+				return getInnerClassImageDescriptor(isInInterfaceOrAnnotation, flags);
+			}
+			return getClassImageDescriptor(flags);
 		}
 	}
+	
+	
+	public static Image getDecoratedImage(ImageDescriptor baseImage, int adornments, Point size) {
+		return JavaPlugin.getImageDescriptorRegistry().get(new JavaElementImageDescriptor(baseImage, adornments, size));
+	}
+	
 
 	private static ImageDescriptor getClassImageDescriptor(int flags) {
 		if (Flags.isPublic(flags) || Flags.isProtected(flags) || Flags.isPrivate(flags))
@@ -397,8 +462,8 @@ public class JavaElementImageProvider {
 			return JavaPluginImages.DESC_OBJS_CLASS_DEFAULT;
 	}
 	
-	private static ImageDescriptor getInnerClassImageDescriptor(int flags) {
-		if (Flags.isPublic(flags))
+	private static ImageDescriptor getInnerClassImageDescriptor(boolean isInInterfaceOrAnnotation, int flags) {
+		if (Flags.isPublic(flags) || isInInterfaceOrAnnotation)
 			return JavaPluginImages.DESC_OBJS_INNER_CLASS_PUBLIC;
 		else if (Flags.isPrivate(flags))
 			return JavaPluginImages.DESC_OBJS_INNER_CLASS_PRIVATE;
@@ -408,6 +473,42 @@ public class JavaElementImageProvider {
 			return JavaPluginImages.DESC_OBJS_INNER_CLASS_DEFAULT;
 	}
 	
+	private static ImageDescriptor getEnumImageDescriptor(int flags) {
+		if (Flags.isPublic(flags) || Flags.isProtected(flags) || Flags.isPrivate(flags))
+			return JavaPluginImages.DESC_OBJS_ENUM;
+		else
+			return JavaPluginImages.DESC_OBJS_ENUM_DEFAULT;
+	}
+	
+	private static ImageDescriptor getInnerEnumImageDescriptor(boolean isInInterfaceOrAnnotation, int flags) {
+		if (Flags.isPublic(flags) || isInInterfaceOrAnnotation)
+			return JavaPluginImages.DESC_OBJS_ENUM;
+		else if (Flags.isPrivate(flags))
+			return JavaPluginImages.DESC_OBJS_ENUM_PRIVATE;
+		else if (Flags.isProtected(flags))
+			return JavaPluginImages.DESC_OBJS_ENUM_PROTECTED;
+		else
+			return JavaPluginImages.DESC_OBJS_ENUM_DEFAULT;
+	}
+	
+	private static ImageDescriptor getAnnotationImageDescriptor(int flags) {
+		if (Flags.isPublic(flags) || Flags.isProtected(flags) || Flags.isPrivate(flags))
+			return JavaPluginImages.DESC_OBJS_ANNOTATION;
+		else
+			return JavaPluginImages.DESC_OBJS_ANNOTATION_DEFAULT;
+	}
+	
+	private static ImageDescriptor getInnerAnnotationImageDescriptor(boolean isInInterfaceOrAnnotation, int flags) {
+		if (Flags.isPublic(flags) || isInInterfaceOrAnnotation)
+			return JavaPluginImages.DESC_OBJS_ANNOTATION;
+		else if (Flags.isPrivate(flags))
+			return JavaPluginImages.DESC_OBJS_ANNOTATION_PRIVATE;
+		else if (Flags.isProtected(flags))
+			return JavaPluginImages.DESC_OBJS_ANNOTATION_PROTECTED;
+		else
+			return JavaPluginImages.DESC_OBJS_ANNOTATION_DEFAULT;
+	}
+	
 	private static ImageDescriptor getInterfaceImageDescriptor(int flags) {
 		if (Flags.isPublic(flags) || Flags.isProtected(flags) || Flags.isPrivate(flags))
 			return JavaPluginImages.DESC_OBJS_INTERFACE;
@@ -415,8 +516,8 @@ public class JavaElementImageProvider {
 			return JavaPluginImages.DESC_OBJS_INTERFACE_DEFAULT;
 	}
 	
-	private static ImageDescriptor getInnerInterfaceImageDescriptor(int flags) {
-		if (Flags.isPublic(flags))
+	private static ImageDescriptor getInnerInterfaceImageDescriptor(boolean isInInterfaceOrAnnotation, int flags) {
+		if (Flags.isPublic(flags) || isInInterfaceOrAnnotation)
 			return JavaPluginImages.DESC_OBJS_INNER_INTERFACE_PUBLIC;
 		else if (Flags.isPrivate(flags))
 			return JavaPluginImages.DESC_OBJS_INNER_INTERFACE_PRIVATE;
@@ -424,5 +525,5 @@ public class JavaElementImageProvider {
 			return JavaPluginImages.DESC_OBJS_INNER_INTERFACE_PROTECTED;
 		else
 			return JavaPluginImages.DESC_OBJS_INTERFACE_DEFAULT;
-	}	
+	}
 }
