@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -12,19 +12,18 @@ package org.eclipse.jdt.internal.ui.wizards.buildpaths;
 
 import java.io.File;
 import java.lang.reflect.InvocationTargetException;
-import java.util.ArrayList;
+import java.util.List;
 
-import org.eclipse.core.resources.IFile;
-import org.eclipse.core.resources.IFolder;
-import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.IWorkspaceRoot;
-import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Path;
 
+import org.eclipse.core.resources.IResource;
+import org.eclipse.core.resources.IWorkspaceRoot;
+import org.eclipse.core.resources.ResourcesPlugin;
+
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.CLabel;
 import org.eclipse.swt.layout.GridData;
@@ -36,25 +35,24 @@ import org.eclipse.swt.widgets.FileDialog;
 import org.eclipse.swt.widgets.Label;
 import org.eclipse.swt.widgets.Shell;
 
-import org.eclipse.jface.dialogs.MessageDialog;
+import org.eclipse.jface.dialogs.Dialog;
 import org.eclipse.jface.operation.IRunnableWithProgress;
 import org.eclipse.jface.viewers.ILabelProvider;
 import org.eclipse.jface.viewers.ITreeContentProvider;
 import org.eclipse.jface.viewers.ViewerFilter;
+import org.eclipse.jface.window.Window;
 
-import org.eclipse.ui.dialogs.ElementTreeSelectionDialog;
-import org.eclipse.ui.help.WorkbenchHelp;
+import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.model.WorkbenchContentProvider;
 import org.eclipse.ui.model.WorkbenchLabelProvider;
 
-import org.eclipse.jdt.core.ClasspathContainerInitializer;
-import org.eclipse.jdt.core.IClasspathContainer;
 import org.eclipse.jdt.core.IClasspathEntry;
 import org.eclipse.jdt.core.IJavaProject;
 import org.eclipse.jdt.core.JavaCore;
-import org.eclipse.jdt.core.JavaModelException;
 
 import org.eclipse.jdt.internal.corext.Assert;
+import org.eclipse.jdt.internal.corext.util.Messages;
+
 import org.eclipse.jdt.internal.ui.IJavaHelpContextIds;
 import org.eclipse.jdt.internal.ui.JavaPlugin;
 import org.eclipse.jdt.internal.ui.dialogs.StatusInfo;
@@ -62,7 +60,6 @@ import org.eclipse.jdt.internal.ui.dialogs.StatusUtil;
 import org.eclipse.jdt.internal.ui.util.PixelConverter;
 import org.eclipse.jdt.internal.ui.wizards.IStatusChangeListener;
 import org.eclipse.jdt.internal.ui.wizards.NewWizardMessages;
-import org.eclipse.jdt.internal.ui.wizards.TypedElementSelectionValidator;
 import org.eclipse.jdt.internal.ui.wizards.dialogfields.DialogField;
 import org.eclipse.jdt.internal.ui.wizards.dialogfields.IDialogFieldListener;
 import org.eclipse.jdt.internal.ui.wizards.dialogfields.IStringButtonAdapter;
@@ -76,34 +73,7 @@ import org.eclipse.jdt.internal.ui.wizards.dialogfields.StringButtonDialogField;
  * variable entries and for normal (internal or external) jar.
  */
 public class SourceAttachmentBlock {
-	
-	private static class UpdatedClasspathContainer implements IClasspathContainer {
-
-		private IClasspathEntry[] fNewEntries;
-		private IClasspathContainer fOriginal;
-
-		public UpdatedClasspathContainer(IClasspathContainer original, IClasspathEntry[] newEntries) {
-			fNewEntries= newEntries;
-			fOriginal= original;
-		}
-
-		public IClasspathEntry[] getClasspathEntries() {
-			return fNewEntries;
-		}
-
-		public String getDescription() {
-			return fOriginal.getDescription();
-		}
-
-		public int getKind() {
-			return fOriginal.getKind();
-		}
-
-		public IPath getPath() {
-			return fOriginal.getPath();
-		}
-	}
-	
+		
 	private IStatusChangeListener fContext;
 	
 	private StringButtonDialogField fFileNameField;
@@ -112,12 +82,6 @@ public class SourceAttachmentBlock {
 	
 	private IStatus fNameStatus;
 	
-	/**
-	 * The file to which the archive path points to.
-	 * Only set when the file exists.
-	 */
-	private File fResolvedFile; 
-
 	/**
 	 * The path to which the archive variable points.
 	 * Null if invalid path or not resolvable. Must not exist.
@@ -133,30 +97,17 @@ public class SourceAttachmentBlock {
 	private IClasspathEntry fEntry;
 	private IPath fContainerPath;
 
-	/**
-	 * @deprecated 
-	 */
-	public SourceAttachmentBlock(IWorkspaceRoot root, IStatusChangeListener context, IClasspathEntry oldEntry) {
-		this(context, oldEntry, null, null);
-	}	
-	
+
 	/**
 	 * @param context listeners for status updates
-	 * @param entryToEdit The entry to edit
-	 * @param containerPath Path of the container that contains the given entry or
-	 * <code>null</code> if the entry does not belong to a container.
-	 * @param project Project to which the entry belongs. Can be
-	 * <code>null</code> if <code>getRunnable</code> is not run and the entry
-	 * does not belong to a container.
-	 *
+	 * @param entry The entry to edit
 	 */
-	public SourceAttachmentBlock(IStatusChangeListener context, IClasspathEntry entry, IPath containerPath, IJavaProject project) {
+	public SourceAttachmentBlock(IStatusChangeListener context, IClasspathEntry entry) {
 		Assert.isNotNull(entry);
 		
 		fContext= context;
 		fEntry= entry;
-		fContainerPath= containerPath;
-		fProject= project;
+
 		
 		int kind= entry.getEntryKind();
 		Assert.isTrue(kind == IClasspathEntry.CPE_LIBRARY || kind == IClasspathEntry.CPE_VARIABLE);
@@ -171,29 +122,38 @@ public class SourceAttachmentBlock {
 		if (isVariableEntry()) {
 			fFileNameField= new VariablePathDialogField(adapter);
 			fFileNameField.setDialogFieldListener(adapter);
-			fFileNameField.setLabelText(NewWizardMessages.getString("SourceAttachmentBlock.filename.varlabel")); //$NON-NLS-1$
-			fFileNameField.setButtonLabel(NewWizardMessages.getString("SourceAttachmentBlock.filename.external.varbutton")); //$NON-NLS-1$
-			((VariablePathDialogField)fFileNameField).setVariableButtonLabel(NewWizardMessages.getString("SourceAttachmentBlock.filename.variable.button")); //$NON-NLS-1$
+			fFileNameField.setLabelText(NewWizardMessages.SourceAttachmentBlock_filename_varlabel); 
+			fFileNameField.setButtonLabel(NewWizardMessages.SourceAttachmentBlock_filename_external_varbutton); 
+			((VariablePathDialogField)fFileNameField).setVariableButtonLabel(NewWizardMessages.SourceAttachmentBlock_filename_variable_button); 
 					
 		} else {
 			fFileNameField= new StringButtonDialogField(adapter);
 			fFileNameField.setDialogFieldListener(adapter);
-			fFileNameField.setLabelText(NewWizardMessages.getString("SourceAttachmentBlock.filename.label")); //$NON-NLS-1$
-			fFileNameField.setButtonLabel(NewWizardMessages.getString("SourceAttachmentBlock.filename.externalfile.button")); //$NON-NLS-1$
+			fFileNameField.setLabelText(NewWizardMessages.SourceAttachmentBlock_filename_label); 
+			fFileNameField.setButtonLabel(NewWizardMessages.SourceAttachmentBlock_filename_externalfile_button); 
 		
 			fWorkspaceButton= new SelectionButtonDialogField(SWT.PUSH);
 			fWorkspaceButton.setDialogFieldListener(adapter);
-			fWorkspaceButton.setLabelText(NewWizardMessages.getString("SourceAttachmentBlock.filename.internal.button")); //$NON-NLS-1$
+			fWorkspaceButton.setLabelText(NewWizardMessages.SourceAttachmentBlock_filename_internal_button); 
 
 			fExternalFolderButton= new SelectionButtonDialogField(SWT.PUSH);
 			fExternalFolderButton.setDialogFieldListener(adapter);
-			fExternalFolderButton.setLabelText(NewWizardMessages.getString("SourceAttachmentBlock.filename.externalfolder.button")); //$NON-NLS-1$
+			fExternalFolderButton.setLabelText(NewWizardMessages.SourceAttachmentBlock_filename_externalfolder_button); 
 		}	
 	
 		// set the old settings
 		setDefaults();
 	}
 	
+	/**
+	 * @deprecated Use API {@link org.eclipse.jdt.ui.wizards.BuildPathDialogAccess#configureSourceAttachment(Shell, IClasspathEntry)}
+	 */
+	public SourceAttachmentBlock(IStatusChangeListener context, IClasspathEntry entry, IPath containerPath, IJavaProject project) {
+		this(context, entry);
+		fContainerPath= containerPath;
+		fProject= project;
+	}
+	
 	public void setDefaults() {
 		if (fEntry.getSourceAttachmentPath() != null) {
 			fFileNameField.setText(fEntry.getSourceAttachmentPath().toString());
@@ -214,7 +174,7 @@ public class SourceAttachmentBlock {
 		if (fFileNameField.getText().length() == 0) {
 			return null;
 		}
-		return new Path(fFileNameField.getText());
+		return getFilePath();
 	}
 
 	/**
@@ -225,6 +185,11 @@ public class SourceAttachmentBlock {
 		return null;
 	}
 	
+	public IClasspathEntry getNewEntry() {
+		CPListElement elem= CPListElement.createFromExisting(fEntry, fProject);
+		elem.setAttribute(CPListElement.SOURCEATTACHMENT, getSourceAttachmentPath());
+		return elem.getClasspathEntry();
+	}
 		
 	/**
 	 * Creates the control
@@ -248,17 +213,18 @@ public class SourceAttachmentBlock {
 			
 			GridData gd= new GridData(GridData.HORIZONTAL_ALIGN_FILL);
 			gd.horizontalSpan= 4;
+			gd.widthHint= converter.convertWidthInCharsToPixels(50);
 
-			Label message= new Label(composite, SWT.LEFT);
+			Label message= new Label(composite, SWT.LEFT + SWT.WRAP);
 			message.setLayoutData(gd);
-			message.setText(NewWizardMessages.getFormattedString("SourceAttachmentBlock.message", fEntry.getPath().lastSegment())); //$NON-NLS-1$
+			message.setText(Messages.format(NewWizardMessages.SourceAttachmentBlock_message, fEntry.getPath().lastSegment())); 
 			
 			DialogField.createEmptySpace(composite, 1);
 			gd= new GridData(GridData.HORIZONTAL_ALIGN_FILL);
 			gd.widthHint= widthHint;
 			gd.horizontalSpan= 2;
 			Label desc= new Label(composite, SWT.LEFT + SWT.WRAP);
-			desc.setText(NewWizardMessages.getString("SourceAttachmentBlock.filename.description")); //$NON-NLS-1$
+			desc.setText(NewWizardMessages.SourceAttachmentBlock_filename_description); 
 			desc.setLayoutData(gd);
 			DialogField.createEmptySpace(composite, 1);
 			
@@ -268,7 +234,7 @@ public class SourceAttachmentBlock {
 			// label that shows the resolved path for variable jars
 			DialogField.createEmptySpace(composite, 1);	
 			fFullPathResolvedLabel= new CLabel(composite, SWT.LEFT);
-			fFullPathResolvedLabel.setText(getResolvedLabelString(fFileNameField.getText(), true));
+			fFullPathResolvedLabel.setText(getResolvedLabelString());
 			fFullPathResolvedLabel.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
 			DialogField.createEmptySpace(composite, 2);
 			
@@ -278,12 +244,15 @@ public class SourceAttachmentBlock {
 			
 			GridData gd= new GridData(GridData.HORIZONTAL_ALIGN_FILL);
 			gd.horizontalSpan= 3;
+			gd.widthHint= converter.convertWidthInCharsToPixels(50);
 
-			Label message= new Label(composite, SWT.LEFT);
+			Label message= new Label(composite, SWT.LEFT + SWT.WRAP);
 			message.setLayoutData(gd);
-			message.setText(NewWizardMessages.getFormattedString("SourceAttachmentBlock.message", fEntry.getPath().lastSegment())); //$NON-NLS-1$
+			message.setText(Messages.format(NewWizardMessages.SourceAttachmentBlock_message, fEntry.getPath().lastSegment())); 
 			
 			fWorkspaceButton.doFillIntoGrid(composite, 1);
+			((GridData) fWorkspaceButton.getSelectionButton(null).getLayoutData()).verticalAlignment= SWT.END;
+			
 			
 			// archive name field
 			fFileNameField.doFillIntoGrid(composite, 4);
@@ -297,8 +266,10 @@ public class SourceAttachmentBlock {
 		}
 				
 		fFileNameField.postSetFocusOnDialogField(parent.getDisplay());
+		
+		Dialog.applyDialogFont(composite);
 				
-		WorkbenchHelp.setHelp(composite, IJavaHelpContextIds.SOURCE_ATTACHMENT_BLOCK);
+		PlatformUI.getWorkbench().getHelpSystem().setHelp(composite, IJavaHelpContextIds.SOURCE_ATTACHMENT_BLOCK);
 		return composite;
 	}
 	
@@ -331,7 +302,7 @@ public class SourceAttachmentBlock {
 		if (field == fFileNameField) {
 			fNameStatus= updateFileNameStatus();
 		} else if (field == fWorkspaceButton) {
-			IPath jarFilePath= chooseInternalJarFile();
+			IPath jarFilePath= chooseInternal();
 			if (jarFilePath != null) {
 				fFileNameField.setText(jarFilePath.toString());
 			}
@@ -351,7 +322,7 @@ public class SourceAttachmentBlock {
 		
 		// set the resolved path for variable jars
 		if (fFullPathResolvedLabel != null) {
-			fFullPathResolvedLabel.setText(getResolvedLabelString(fFileNameField.getText(), true));
+			fFullPathResolvedLabel.setText(getResolvedLabelString());
 		}
 		
 		IStatus status= StatusUtil.getMostSevere(new IStatus[] { fNameStatus });
@@ -369,14 +340,10 @@ public class SourceAttachmentBlock {
 		return false;
 	}
 	
-	private String getResolvedLabelString(String path, boolean osPath) {
-		IPath resolvedPath= getResolvedPath(new Path(path));
+	private String getResolvedLabelString() {
+		IPath resolvedPath= getResolvedPath(getFilePath());
 		if (resolvedPath != null) {
-			if (osPath) {
-				return resolvedPath.toOSString();
-			} else {
-				return resolvedPath.toString();
-			}
+			return resolvedPath.toOSString();
 		}
 		return ""; //$NON-NLS-1$
 	}	
@@ -396,7 +363,6 @@ public class SourceAttachmentBlock {
 	
 	private IStatus updateFileNameStatus() {
 		StatusInfo status= new StatusInfo();
-		fResolvedFile= null;
 		fFileVariablePath= null;
 		
 		String fileName= fFileNameField.getText();
@@ -405,39 +371,43 @@ public class SourceAttachmentBlock {
 			return status;
 		} else {
 			if (!Path.EMPTY.isValidPath(fileName)) {
-				status.setError(NewWizardMessages.getString("SourceAttachmentBlock.filename.error.notvalid")); //$NON-NLS-1$
+				status.setError(NewWizardMessages.SourceAttachmentBlock_filename_error_notvalid); 
 				return status;
 			}
-			IPath filePath= new Path(fileName);
+			IPath filePath= Path.fromOSString(fileName);
 			IPath resolvedPath;
 			if (isVariableEntry()) {
 				if (filePath.getDevice() != null) {
-					status.setError(NewWizardMessages.getString("SourceAttachmentBlock.filename.error.deviceinpath")); //$NON-NLS-1$
+					status.setError(NewWizardMessages.SourceAttachmentBlock_filename_error_deviceinpath); 
 					return status;
 				}
 				String varName= filePath.segment(0);
 				if (varName == null) {
-					status.setError(NewWizardMessages.getString("SourceAttachmentBlock.filename.error.notvalid")); //$NON-NLS-1$
+					status.setError(NewWizardMessages.SourceAttachmentBlock_filename_error_notvalid); 
 					return status;
 				}
 				fFileVariablePath= JavaCore.getClasspathVariable(varName);
 				if (fFileVariablePath == null) {
-					status.setError(NewWizardMessages.getString("SourceAttachmentBlock.filename.error.varnotexists")); //$NON-NLS-1$
+					status.setError(NewWizardMessages.SourceAttachmentBlock_filename_error_varnotexists); 
 					return status;
 				}
 				resolvedPath= fFileVariablePath.append(filePath.removeFirstSegments(1));
 				
 				if (resolvedPath.isEmpty()) {
-					status.setWarning(NewWizardMessages.getString("SourceAttachmentBlock.filename.warning.varempty")); //$NON-NLS-1$
+					status.setWarning(NewWizardMessages.SourceAttachmentBlock_filename_warning_varempty); 
 					return status;
 				}
 				File file= resolvedPath.toFile();
 				if (!file.exists()) {				
-					String message= NewWizardMessages.getFormattedString("SourceAttachmentBlock.filename.error.filenotexists", resolvedPath.toOSString()); //$NON-NLS-1$
+					String message= Messages.format(NewWizardMessages.SourceAttachmentBlock_filename_error_filenotexists, resolvedPath.toOSString()); 
 					status.setWarning(message);
 					return status;
-				}						
-				fResolvedFile= file;
+				}
+				if (!resolvedPath.isAbsolute()) {
+					String message=  Messages.format(NewWizardMessages.SourceAttachmentBlock_filename_error_notabsolute, filePath.toString()); 
+					status.setError(message);
+					return status;
+				}
 				
 			} else {
 				File file= filePath.toFile();
@@ -446,19 +416,29 @@ public class SourceAttachmentBlock {
 					file= res.getLocation().toFile();
 				}
 				if (!file.exists()) {
-					String message=  NewWizardMessages.getFormattedString("SourceAttachmentBlock.filename.error.filenotexists", filePath.toString()); //$NON-NLS-1$
+					String message=  Messages.format(NewWizardMessages.SourceAttachmentBlock_filename_error_filenotexists, filePath.toString()); 
 					status.setError(message);
 					return status;
 				}
-				fResolvedFile= file;
+				if (res == null) {
+					if (!filePath.isAbsolute()) {
+						String message=  Messages.format(NewWizardMessages.SourceAttachmentBlock_filename_error_notabsolute, filePath.toString()); 
+						status.setError(message);
+						return status;
+					}
+				}
 			}
 			
 		}
 		return status;
 	}
 	
+	private IPath getFilePath() {
+		return Path.fromOSString(fFileNameField.getText()).makeAbsolute();
+	}
+	
 	private IPath chooseExtension() {
-		IPath currPath= new Path(fFileNameField.getText());
+		IPath currPath= getFilePath();
 		if (currPath.isEmpty()) {
 			currPath= fEntry.getPath();
 		}		
@@ -468,13 +448,13 @@ public class SourceAttachmentBlock {
 			
 		String currVariable= currPath.segment(0);
 		JARFileSelectionDialog dialog= new JARFileSelectionDialog(getShell(), false, true);
-		dialog.setTitle(NewWizardMessages.getString("SourceAttachmentBlock.extvardialog.title")); //$NON-NLS-1$
-		dialog.setMessage(NewWizardMessages.getString("SourceAttachmentBlock.extvardialog.description")); //$NON-NLS-1$
+		dialog.setTitle(NewWizardMessages.SourceAttachmentBlock_extvardialog_title); 
+		dialog.setMessage(NewWizardMessages.SourceAttachmentBlock_extvardialog_description); 
 		dialog.setInput(fFileVariablePath.toFile());
 		dialog.setInitialSelection(initialSelection);
-		if (dialog.open() == JARFileSelectionDialog.OK) {
+		if (dialog.open() == Window.OK) {
 			File result= (File) dialog.getResult()[0];
-			IPath returnPath= new Path(result.getPath()).makeAbsolute();
+			IPath returnPath= Path.fromOSString(result.getPath()).makeAbsolute();
 			return modifyPath(returnPath, currVariable);
 		}
 		return null;
@@ -484,7 +464,7 @@ public class SourceAttachmentBlock {
 	 * Opens a dialog to choose a jar from the file system.
 	 */
 	private IPath chooseExtJarFile() {
-		IPath currPath= new Path(fFileNameField.getText());
+		IPath currPath= getFilePath();
 		if (currPath.isEmpty()) {
 			currPath= fEntry.getPath();
 		}		
@@ -494,18 +474,18 @@ public class SourceAttachmentBlock {
 		}
 	
 		FileDialog dialog= new FileDialog(getShell());
-		dialog.setText(NewWizardMessages.getString("SourceAttachmentBlock.extjardialog.text")); //$NON-NLS-1$
+		dialog.setText(NewWizardMessages.SourceAttachmentBlock_extjardialog_text); 
 		dialog.setFilterExtensions(new String[] {"*.jar;*.zip"}); //$NON-NLS-1$
 		dialog.setFilterPath(currPath.toOSString());
 		String res= dialog.open();
 		if (res != null) {
-			return new Path(res).makeAbsolute();
+			return Path.fromOSString(res).makeAbsolute();
 		}
 		return null;
 	}
 	
 	private IPath chooseExtFolder() {
-		IPath currPath= new Path(fFileNameField.getText());
+		IPath currPath= getFilePath();
 		if (currPath.isEmpty()) {
 			currPath= fEntry.getPath();
 		}
@@ -514,27 +494,23 @@ public class SourceAttachmentBlock {
 		}
 	
 		DirectoryDialog dialog= new DirectoryDialog(getShell());
-		dialog.setText(NewWizardMessages.getString("SourceAttachmentBlock.extfolderdialog.text")); //$NON-NLS-1$
+		dialog.setMessage(NewWizardMessages.SourceAttachmentBlock_extfolderdialog_message);
+		dialog.setText(NewWizardMessages.SourceAttachmentBlock_extfolderdialog_text); 
 		dialog.setFilterPath(currPath.toOSString());
 		String res= dialog.open();
 		if (res != null) {
-			return new Path(res).makeAbsolute();
+			return Path.fromOSString(res).makeAbsolute();
 		}
 		return null;
 	}	
 	
-	
-
 	/*
 	 * Opens a dialog to choose an internal jar.
 	 */	
-	private IPath chooseInternalJarFile() {
+	private IPath chooseInternal() {
 		String initSelection= fFileNameField.getText();
 		
-		Class[] acceptedClasses= new Class[] { IFolder.class, IFile.class };
-		TypedElementSelectionValidator validator= new TypedElementSelectionValidator(acceptedClasses, false);
-	
-		ViewerFilter filter= new ArchiveFileFilter(null, false);
+		ViewerFilter filter= new ArchiveFileFilter((List) null, false);
 
 		ILabelProvider lp= new WorkbenchLabelProvider();
 		ITreeContentProvider cp= new WorkbenchContentProvider();
@@ -549,13 +525,12 @@ public class SourceAttachmentBlock {
 
 		FolderSelectionDialog dialog= new FolderSelectionDialog(getShell(), lp, cp);
 		dialog.setAllowMultiple(false);
-		dialog.setValidator(validator);
 		dialog.addFilter(filter);
-		dialog.setTitle(NewWizardMessages.getString("SourceAttachmentBlock.intjardialog.title")); //$NON-NLS-1$
-		dialog.setMessage(NewWizardMessages.getString("SourceAttachmentBlock.intjardialog.message")); //$NON-NLS-1$
+		dialog.setTitle(NewWizardMessages.SourceAttachmentBlock_intjardialog_title); 
+		dialog.setMessage(NewWizardMessages.SourceAttachmentBlock_intjardialog_message); 
 		dialog.setInput(fWorkspaceRoot);
 		dialog.setInitialSelection(initSel);
-		if (dialog.open() == ElementTreeSelectionDialog.OK) {
+		if (dialog.open() == Window.OK) {
 			IResource res= (IResource) dialog.getFirstResult();
 			return res.getFullPath();
 		}
@@ -597,103 +572,25 @@ public class SourceAttachmentBlock {
 
 	/**
 	 * Creates a runnable that sets the source attachment by modifying the project's classpath.
-	 * @deprecated use getRunnable() instead
 	 */
-	public IRunnableWithProgress getRunnable(final IJavaProject jproject, final Shell shell) {
-		fProject= jproject;
-		return getRunnable(shell);
-	}
-
-	/**
-	 * Creates a runnable that sets the source attachment by modifying the
-	 * project's classpath or updating a container.
-	 */
-	public IRunnableWithProgress getRunnable(final Shell shell) {
+	public static IRunnableWithProgress getRunnable(final Shell shell, final IClasspathEntry newEntry, final IJavaProject jproject, final IPath containerPath) {
 		return new IRunnableWithProgress() {
 			public void run(IProgressMonitor monitor) throws InvocationTargetException {				
 				try {
-					attachSource(shell, monitor);
+					BuildPathSupport.modifyClasspathEntry(shell, newEntry, jproject, containerPath, monitor);
 				} catch (CoreException e) {
 					throw new InvocationTargetException(e);
 				}
 			}
 		};
+	}
+
+	/**
+	 * @deprecated Use {@link #getRunnable(Shell, IClasspathEntry, IJavaProject, IPath)}
+	 */
+	public IRunnableWithProgress getRunnable(final Shell shell) {
+		return getRunnable(shell, getNewEntry(), fProject, fContainerPath);
 	}	
 	
 	
-	protected void attachSource(final Shell shell, IProgressMonitor monitor) throws CoreException {
-		boolean isExported= fEntry.isExported();
-		IClasspathEntry newEntry;
-		if (fEntry.getEntryKind() == IClasspathEntry.CPE_VARIABLE) {
-			newEntry= JavaCore.newVariableEntry(fEntry.getPath(), getSourceAttachmentPath(), getSourceAttachmentRootPath(), isExported);
-		} else {
-			newEntry= JavaCore.newLibraryEntry(fEntry.getPath(), getSourceAttachmentPath(), getSourceAttachmentRootPath(), isExported);
-		}
-		if (fContainerPath != null) {
-			updateContainerClasspath(fProject, fContainerPath, newEntry, monitor);
-		} else {
-			updateProjectClasspath(shell, fProject, newEntry, monitor);
-		}
-	}
-
-	private void updateContainerClasspath(IJavaProject jproject, IPath containerPath, IClasspathEntry newEntry, IProgressMonitor monitor) throws CoreException {
-		IClasspathContainer container= JavaCore.getClasspathContainer(containerPath, jproject);
-		IClasspathEntry[] entries= container.getClasspathEntries();
-		IClasspathEntry[] newEntries= new IClasspathEntry[entries.length];
-		for (int i= 0; i < entries.length; i++) {
-			IClasspathEntry curr= entries[i];
-			if (curr.getEntryKind() == newEntry.getEntryKind() && curr.getPath().equals(newEntry.getPath())) {
-				newEntries[i]= newEntry;
-			} else {
-				newEntries[i]= curr;
-			}
-		}
-		IClasspathContainer updatedContainer= new UpdatedClasspathContainer(container, newEntries);
-			
-		ClasspathContainerInitializer initializer= JavaCore.getClasspathContainerInitializer(containerPath.segment(0));
-		initializer.requestClasspathContainerUpdate(containerPath, jproject, updatedContainer);
-		monitor.worked(1);
-	}
-
-	private void updateProjectClasspath(Shell shell, IJavaProject jproject, IClasspathEntry newEntry, IProgressMonitor monitor) throws JavaModelException {
-		IClasspathEntry[] oldClasspath= jproject.getRawClasspath();
-		int nEntries= oldClasspath.length;
-		ArrayList newEntries= new ArrayList(nEntries + 1);
-		int entryKind= newEntry.getEntryKind();
-		IPath jarPath= newEntry.getPath();
-		boolean found= false;
-		for (int i= 0; i < nEntries; i++) {
-			IClasspathEntry curr= oldClasspath[i];
-			if (curr.getEntryKind() == entryKind && curr.getPath().equals(jarPath)) {
-				// add modified entry
-				newEntries.add(newEntry);
-				found= true;
-			} else {
-				newEntries.add(curr);
-			}
-		}
-		if (!found) {
-			if (newEntry.getSourceAttachmentPath() == null || !putJarOnClasspathDialog(shell)) {
-				return;
-			}
-			// add new
-			newEntries.add(newEntry);			
-		}
-		IClasspathEntry[] newClasspath= (IClasspathEntry[]) newEntries.toArray(new IClasspathEntry[newEntries.size()]);
-		jproject.setRawClasspath(newClasspath, monitor);
-	}
-	
-	private boolean putJarOnClasspathDialog(Shell shell) {
-		final boolean[] result= new boolean[1];
-		shell.getDisplay().syncExec(new Runnable() {
-			public void run() {
-				String title= NewWizardMessages.getString("SourceAttachmentBlock.putoncpdialog.title"); //$NON-NLS-1$
-				String message= NewWizardMessages.getString("SourceAttachmentBlock.putoncpdialog.message"); //$NON-NLS-1$
-				result[0]= MessageDialog.openQuestion(JavaPlugin.getActiveWorkbenchShell(), title, message);
-			}
-		});
-		return result[0];
-	}
-	
-	
 }
