@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2005 IBM Corporation and others.
+ * Copyright (c) 2003, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -129,8 +129,8 @@ public class EclipseStarter {
 	 */
 	public static void main(String[] args) throws Exception {
 		URL url = EclipseStarter.class.getProtectionDomain().getCodeSource().getLocation();
-		System.getProperties().put(PROP_FRAMEWORK, url.toExternalForm());
-		String filePart = url.getFile();
+		System.getProperties().put(PROP_FRAMEWORK, decode(url.toExternalForm()));
+		String filePart = decode(url.getFile());
 		System.getProperties().put(PROP_INSTALL_AREA, filePart.substring(0, filePart.lastIndexOf('/')));
 		System.getProperties().put(PROP_NOSHUTDOWN, "true"); //$NON-NLS-1$
 		run(args, null);
@@ -1263,4 +1263,100 @@ public class EclipseStarter {
 			this.start = start;
 		}
 	}
+
+	private static String decode(String urlString) {
+		//try to use Java 1.4 method if available
+		try {
+			Class clazz = URLDecoder.class;
+			Method method = clazz.getDeclaredMethod("decode", new Class[] {String.class, String.class}); //$NON-NLS-1$
+			//first encode '+' characters, because URLDecoder incorrectly converts 
+			//them to spaces on certain class library implementations.
+			if (urlString.indexOf('+') >= 0) {
+				int len = urlString.length();
+				StringBuffer buf = new StringBuffer(len);
+				for (int i = 0; i < len; i++) {
+					char c = urlString.charAt(i);
+					if (c == '+')
+						buf.append("%2B"); //$NON-NLS-1$
+					else
+						buf.append(c);
+				}
+				urlString = buf.toString();
+			}
+			Object result = method.invoke(null, new Object[] {urlString, "UTF-8"}); //$NON-NLS-1$
+			if (result != null)
+				return (String) result;
+		} catch (Exception e) {
+			//JDK 1.4 method not found -- fall through and decode by hand
+		}
+		//decode URL by hand
+		boolean replaced = false;
+		byte[] encodedBytes = urlString.getBytes();
+		int encodedLength = encodedBytes.length;
+		byte[] decodedBytes = new byte[encodedLength];
+		int decodedLength = 0;
+		for (int i = 0; i < encodedLength; i++) {
+			byte b = encodedBytes[i];
+			if (b == '%') {
+				byte enc1 = encodedBytes[++i];
+				byte enc2 = encodedBytes[++i];
+				b = (byte) ((hexToByte(enc1) << 4) + hexToByte(enc2));
+				replaced = true;
+			}
+			decodedBytes[decodedLength++] = b;
+		}
+		if (!replaced)
+			return urlString;
+		try {
+			return new String(decodedBytes, 0, decodedLength, "UTF-8"); //$NON-NLS-1$
+		} catch (UnsupportedEncodingException e) {
+			//use default encoding
+			return new String(decodedBytes, 0, decodedLength);
+		}
+	}
+
+	private static int hexToByte(byte b) {
+		switch (b) {
+			case '0' :
+				return 0;
+			case '1' :
+				return 1;
+			case '2' :
+				return 2;
+			case '3' :
+				return 3;
+			case '4' :
+				return 4;
+			case '5' :
+				return 5;
+			case '6' :
+				return 6;
+			case '7' :
+				return 7;
+			case '8' :
+				return 8;
+			case '9' :
+				return 9;
+			case 'A' :
+			case 'a' :
+				return 10;
+			case 'B' :
+			case 'b' :
+				return 11;
+			case 'C' :
+			case 'c' :
+				return 12;
+			case 'D' :
+			case 'd' :
+				return 13;
+			case 'E' :
+			case 'e' :
+				return 14;
+			case 'F' :
+			case 'f' :
+				return 15;
+			default :
+				throw new IllegalArgumentException("Switch error decoding URL"); //$NON-NLS-1$
+		}
+	}
 }
