@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2005 IBM Corporation and others.
+ * Copyright (c) 2003, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -33,8 +33,9 @@ class StateReader {
 
 	private boolean lazyLoad = true;
 	private int numBundles;
+	private boolean accessedFlag = false;
 
-	public static final byte STATE_CACHE_VERSION = 21;
+	public static final byte STATE_CACHE_VERSION = 22;
 	public static final byte NULL = 0;
 	public static final byte OBJECT = 1;
 	public static final byte INDEX = 2;
@@ -282,6 +283,18 @@ class StateReader {
 				resolved[i] = readBundleDescription(in);
 			result.setResolvedRequires(resolved);
 		}
+
+		int dynamicPkgCnt = in.readInt();
+		if (dynamicPkgCnt > 0) {
+			HashMap dynamicStamps = new HashMap(dynamicPkgCnt);
+			for (int i = 0; i < dynamicPkgCnt; i++) {
+				String pkg = readString(in, false);
+				Long stamp = new Long(in.readLong());
+				dynamicStamps.put(pkg, stamp);
+			}
+			result.setDynamicStamps(dynamicStamps);
+		}
+
 		result.setFullyLoaded(true); // set fully loaded before setting the dependencies
 		// No need to add bundle dependencies for hosts, imports or requires;
 		// This is done by readBundleDescription
@@ -456,7 +469,16 @@ class StateReader {
 		return lazyLoad;
 	}
 
+	boolean getAccessedFlag() {
+		return accessedFlag;
+	}
+
+	void setAccessedFlag(boolean accessedFlag) {
+		this.accessedFlag = accessedFlag;
+	}
+
 	synchronized void fullyLoad() {
+		setAccessedFlag(true);
 		DataInputStream in = null;
 		try {
 			in = openLazyFile();
@@ -475,6 +497,7 @@ class StateReader {
 	}
 
 	synchronized void fullyLoad(BundleDescriptionImpl target) throws IOException {
+		setAccessedFlag(true);
 		DataInputStream in = null;
 		try {
 			in = openLazyFile();
