@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -19,9 +19,9 @@ package org.eclipse.pde.internal.runtime;
  * Window>Preferences>Java>Code Generation.
  */
 public interface IHelpContextIds {
-	public static final String PREFIX = "org.eclipse.pde.doc.user" + ".";
+	public static final String PREFIX = "org.eclipse.pde.doc.user" + "."; //$NON-NLS-1$ //$NON-NLS-2$
 	
-	public static final String LOG_VIEW = PREFIX + "log_view";
-	public static final String REGISTRY_VIEW = PREFIX + "registry_view";
+	public static final String LOG_VIEW = PREFIX + "log_view"; //$NON-NLS-1$
+	public static final String REGISTRY_VIEW = PREFIX + "registry_view"; //$NON-NLS-1$
 
 }
