@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -36,6 +36,8 @@ import org.eclipse.swt.widgets.*;
  * <p>
  * Note: Only one of the styles SINGLE, and MULTI may be specified.
  * </p>
+ * 
+ * @deprecated As of 3.1 use Tree, TreeItem and TreeColumn
  */
 public class TableTree extends Composite {
 	Table table;
@@ -129,7 +131,7 @@ public TableTree(Composite parent, int style) {
 }
 
 int addItem(TableTreeItem item, int index) {
-	if (index < 0 || index > items.length) throw new SWTError(SWT.ERROR_INVALID_ARGUMENT);
+	if (index < 0 || index > items.length) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	TableTreeItem[] newItems = new TableTreeItem[items.length + 1];
 	System.arraycopy(items, 0, newItems, 0, index);
 	newItems[index] = item;
@@ -172,27 +174,34 @@ int addItem(TableTreeItem item, int index) {
  */
 public void addSelectionListener(SelectionListener listener) {
 	checkWidget();
-	if (listener == null) throw new SWTError (SWT.ERROR_NULL_ARGUMENT);
+	if (listener == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
 	TypedListener typedListener = new TypedListener (listener);
 	addListener (SWT.Selection,typedListener);
 	addListener (SWT.DefaultSelection,typedListener);
 }
 
-/**	 
- * Adds the listener to receive tree events.
- * <p>
+/**
+ * Adds the listener to the collection of listeners who will
+ * be notified when an item in the receiver is expanded or collapsed
+ * by sending it one of the messages defined in the <code>TreeListener</code>
+ * interface.
  *
- * @param listener the tree listener
+ * @param listener the listener which should be notified
  *
- * @exception SWTError <ul>
- *	<li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
- *	<li>ERROR_WIDGET_DISPOSED when the widget has been disposed
- *	<li>ERROR_NULL_ARGUMENT when listener is null
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
  * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see TreeListener
+ * @see #removeTreeListener
  */
 public void addTreeListener(TreeListener listener) {
 	checkWidget();
-	if (listener == null) throw new SWTError (SWT.ERROR_NULL_ARGUMENT);
+	if (listener == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
 	TypedListener typedListener = new TypedListener (listener);
 	addListener (SWT.Expand, typedListener);
 	addListener (SWT.Collapse, typedListener);
@@ -217,7 +226,7 @@ public Rectangle computeTrim (int x, int y, int width, int height) {
  * If an item is selected, it is deselected.
  * If an item is not selected, it remains unselected.
  *
- * @exception SWTError <ul>
+ * @exception SWTException <ul>
  *	<li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
  *	<li>ERROR_WIDGET_DISPOSED when the widget has been disposed
  * </ul>
@@ -275,10 +284,9 @@ public int getItemCount () {
  *
  * @return the height of one item in the widget
  *
- * @exception SWTError <ul>
+ * @exception SWTException <ul>
  *	<li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
  *	<li>ERROR_WIDGET_DISPOSED when the widget has been disposed
- *	<li>ERROR_CANNOT_GET_ITEM_HEIGHT when the operation fails
  * </ul>
  */
 public int getItemHeight () {
@@ -306,10 +314,9 @@ public TableTreeItem [] getItems () {
  *
  * @return the selected items in the widget
  *
- * @exception SWTError <ul>
+ * @exception SWTException <ul>
  *		<li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread</li>
  *		<li>ERROR_WIDGET_DISPOSED when the widget has been disposed</li>
- * 		<li>ERROR_CANNOT_GET_SELECTION when the operation fails</li>
  *	</ul>
  */
 public TableTreeItem [] getSelection () {
@@ -330,10 +337,9 @@ public TableTreeItem [] getSelection () {
  *
  * @return the number of selected items in the widget
  *
- * @exception SWTError <ul>
+ * @exception SWTException <ul>
  *		<li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread</li>
  *		<li>ERROR_WIDGET_DISPOSED when the widget has been disposed</li>
- * 		<li>ERROR_CANNOT_GET_COUNT when the operation fails</li>
  *	</ul>
  */
 public int getSelectionCount () {
@@ -466,6 +472,30 @@ void onSelection(Event e) {
 	}
 	notifyListeners(e.type, event);
 }
+/**
+ * Returns the item at the given, zero-relative index in the
+ * receiver. Throws an exception if the index is out of range.
+ *
+ * @param index the index of the item to return
+ * @return the item at the given index
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the list minus 1 (inclusive)</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.1
+ */
+public TableTreeItem getItem (int index) {
+	checkWidget();
+	int count = items.length;
+	if (!(0 <= index && index < count)) SWT.error (SWT.ERROR_INVALID_RANGE);
+	return items [index];
+}
+
 /**
  * Returns the item at the given point in the receiver
  * or null if no such item exists. The point is in the
@@ -475,7 +505,7 @@ void onSelection(Event e) {
  * @return the item at the given point
  *
  * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+ *    <li>ERROR_NULL_ARGUMENT - if the point is null</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -580,10 +610,9 @@ void onMouseDown(Event event) {
  * This operation will fail when an item
  * could not be removed in the OS.
  *
- * @exception SWTError <ul>
+ * @exception SWTException <ul>
  *	<li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
  * 	<li>ERROR_WIDGET_DISPOSED when the widget has been disposed
- * 	<li>ERROR_ITEM_NOT_REMOVED when the operation fails
  * </ul>
  */
 public void removeAll () {
@@ -606,50 +635,60 @@ void removeItem(TableTreeItem item) {
 	items = newItems;
 }
 
-/**	 
- * Removes the listener.
- * <p>
+/**
+ * Removes the listener from the collection of listeners who will
+ * be notified when the receiver's selection changes.
  *
- * @param listener the listener
+ * @param listener the listener which should no longer be notified
  *
- * @exception SWTError <ul>
- *	<li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
- * 	<li>ERROR_WIDGET_DISPOSED when the widget has been disposed
- *	<li>ERROR_NULL_ARGUMENT when listener is null
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
  * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see SelectionListener
+ * @see #addSelectionListener
  */
 public void removeSelectionListener (SelectionListener listener) {
 	checkWidget();
-	if (listener == null) throw new SWTError (SWT.ERROR_NULL_ARGUMENT);
+	if (listener == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
 	removeListener(SWT.Selection, listener);
 	removeListener(SWT.DefaultSelection, listener);
 }
 
-/**	 
- * Removes the listener.
+/**
+ * Removes the listener from the collection of listeners who will
+ * be notified when items in the receiver are expanded or collapsed..
  *
- * @param listener the listener
+ * @param listener the listener which should no longer be notified
  *
- * @exception SWTError <ul>
- *	<li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
- *	<li>ERROR_WIDGET_DISPOSED when the widget has been disposed
- *	<li>ERROR_NULL_ARGUMENT when listener is null
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
  * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see TreeListener
+ * @see #addTreeListener
  */
 public void removeTreeListener (TreeListener listener) {
 	checkWidget();
-	if (listener == null) throw new SWTError (SWT.ERROR_NULL_ARGUMENT);
+	if (listener == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
 	removeListener(SWT.Expand, listener);
 	removeListener(SWT.Collapse, listener);
 }
 
 /**
- * Selects all items.
+ * Selects all of the items in the receiver.
  * <p>
- * If an item is not selected, it is selected.
- * If an item is selected, it remains selected.
+ * If the receiver is single-select, do nothing.
  *
- * @exception SWTError <ul>
+ * @exception SWTException <ul>
  *	<li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
  *	<li>ERROR_WIDGET_DISPOSED when the widget has been disposed
  * </ul>
@@ -687,21 +726,37 @@ public void setMenu (Menu menu) {
 }
 
 /**
- * Sets the selection.
+ * Sets the receiver's selection to be the given array of items.
+ * The current selection is cleared before the new items are selected.
  * <p>
- * @param items new selection
+ * Items that are not in the receiver are ignored.
+ * If the receiver is single-select and multiple items are specified,
+ * then all items are ignored.
  *
- * @exception SWTError <ul>
- *	<li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
- *	<li>ERROR_WIDGET_DISPOSED when the widget has been disposed
- *	<li>ERROR_NULL_ARGUMENT when items is null
+ * @param items the array of items
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the array of items is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if one of the item has been disposed</li>
  * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see TableTree#deselectAll()
  */
 public void setSelection (TableTreeItem[] items) {
-	checkWidget();
-	TableItem[] tableItems = new TableItem[items.length];
-	for (int i = 0; i < items.length; i++) {
-		if (items[i] == null) throw new SWTError(SWT.ERROR_NULL_ARGUMENT);
+	checkWidget ();
+	if (items == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
+	int length = items.length;
+	if (length == 0 || ((table.getStyle() & SWT.SINGLE) != 0 && length > 1)) {
+		deselectAll();
+		return;
+	}
+	TableItem[] tableItems = new TableItem[length];
+	for (int i = 0; i < length; i++) {
+		if (items[i] == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
 		if (!items[i].getVisible()) expandItem (items[i]);
 		tableItems[i] = items[i].tableItem;
 	}
@@ -713,19 +768,26 @@ public void setToolTipText (String string) {
 }
 
 /**
- * Shows the item.
- * <p>
+ * Shows the item.  If the item is already showing in the receiver,
+ * this method simply returns.  Otherwise, the items are scrolled
+ * and expanded until the item is visible.
+ *
  * @param item the item to be shown
  *
- * @exception SWTError <ul>
- *	<li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
- *	<li>ERROR_WIDGET_DISPOSED when the widget has been disposed
- *	<li>ERROR_NULL_ARGUMENT when item is null
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the item is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if the item has been disposed</li>
  * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see TableTree#showSelection()
  */
 public void showItem (TableTreeItem item) {
 	checkWidget();
-	if (item == null) throw new SWTError (SWT.ERROR_NULL_ARGUMENT);
+	if (item == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
 	if (!item.getVisible()) expandItem (item);
 	TableItem tableItem = item.tableItem;
 	table.showItem(tableItem);
@@ -740,7 +802,7 @@ public void showItem (TableTreeItem item) {
  * the top index of the widget is changed such
  * that selection becomes visible.
  *
- * @exception SWTError <ul>
+ * @exception SWTException <ul>
  *	<li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
  *	<li>ERROR_WIDGET_DISPOSED when the widget has been disposed
  * </ul>
