@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -64,20 +64,21 @@ public class OleControlSite extends OleClientSite
  *
  * @param parent a composite widget; must be an OleFrame
  * @param style the bitwise OR'ing of widget styles
- * @param progID the unique program identifier which has been registered for this ActiveX Control; 
+ * @param progId the unique program identifier which has been registered for this ActiveX Control; 
  *               the value of the ProgID key or the value of the VersionIndependentProgID key specified
  *               in the registry for this Control (for example, the VersionIndependentProgID for 
  *               Internet Explorer is Shell.Explorer)
  *
- * @exception SWTError
- * <ul><li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
- *     <li>ERROR_ERROR_NULL_ARGUMENT when the parent is null
+ *@exception IllegalArgumentException <ul>
+ *     <li>ERROR_NULL_ARGUMENT when the parent is null
+ *</ul>
+ * @exception SWTException <ul>
+ *     <li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
  *     <li>ERROR_INVALID_CLASSID when the progId does not map to a registered CLSID
  *     <li>ERROR_CANNOT_CREATE_OBJECT when failed to create OLE Object
  *     <li>ERROR_CANNOT_ACCESS_CLASSFACTORY when Class Factory could not be found
  *     <li>ERROR_CANNOT_CREATE_LICENSED_OBJECT when failed to create a licensed OLE Object
- *     <li>ERROR_INTERFACES_NOT_INITIALIZED when unable to create callbacks for OLE Interfaces</ul>
- *
+ * </ul>
  */
 public OleControlSite(Composite parent, int style, String progId) {
 	super(parent, style);
@@ -154,8 +155,9 @@ public OleControlSite(Composite parent, int style, String progId) {
  * 
  * @param listener the listener
  *
- * @exception SWTError 
- *	<ul><li>ERROR_NULL_ARGUMENT when listener is null</li></ul>
+ * @exception IllegalArgumentException <ul> 
+ *	    <li>ERROR_NULL_ARGUMENT when listener is null</li>
+ * </ul>
  */
 public void addEventListener(int eventID, OleListener listener) {
 	if (listener == null) OLE.error (SWT.ERROR_NULL_ARGUMENT);
@@ -238,8 +240,9 @@ static GUID getDefaultEventSinkGUID(IUnknown unknown) {
  * 
  * @param listener the listener
  *
- * @exception SWTError 
- *	<ul><li>ERROR_NULL_ARGUMENT when listener is null</li></ul>
+ * @exception IllegalArgumentException <ul> 
+ *	   <li>ERROR_NULL_ARGUMENT when listener is null</li>
+ * </ul>
  */
 public void addEventListener(OleAutomation automation, int eventID, OleListener listener) {
 	if (listener == null || automation == null) OLE.error (SWT.ERROR_NULL_ARGUMENT);
@@ -310,11 +313,12 @@ protected void addObjectReferences() {
  * @param propertyID the identifier of the property
  * @param listener the listener
  *
- * @exception SWTError 
- *	<ul><li>ERROR_NULL_ARGUMENT when listener is null</li></ul>
+ * @exception IllegalArgumentException <ul> 
+ *	    <li>ERROR_NULL_ARGUMENT when listener is null</li>
+ * </ul>
  */
 public void addPropertyListener(int propertyID, OleListener listener) {
-	if (listener == null) throw new SWTError (SWT.ERROR_NULL_ARGUMENT);
+	if (listener == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
 	olePropertyChangeSink.addListener(propertyID, listener);	
 }
 
@@ -456,7 +460,7 @@ protected int getLicenseInfo(GUID clsid) {
 		return 0;
 	}
 	int[] pBstrKey = new int[1];
-	if (licinfo != null && licinfo.fRuntimeKeyAvail != 0) {
+	if (licinfo != null && licinfo.fRuntimeKeyAvail) {
 		if (classFactory.RequestLicKey(0, pBstrKey) == COM.S_OK) {
 			classFactory.Release();
 			return pBstrKey[0];
@@ -616,12 +620,13 @@ protected void releaseObjectInterfaces() {
  * 
  * @param listener the listener
  *
- * @exception SWTError
- *	<ul><li>ERROR_NULL_ARGUMENT when listener is null</li></ul>
+ * @exception IllegalArgumentException <ul>
+ *	    <li>ERROR_NULL_ARGUMENT when listener is null</li>
+ * </ul>
  */
 public void removeEventListener(int eventID, OleListener listener) {
 	checkWidget();
-	if (listener == null) throw new SWTError (SWT.ERROR_NULL_ARGUMENT);
+	if (listener == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
 	
 	GUID riid = getDefaultEventSinkGUID(objIUnknown);
 	if (riid != null) {
@@ -642,12 +647,13 @@ public void removeEventListener(int eventID, OleListener listener) {
  * 
  * @param listener the listener
  *
- * @exception SWTError
- *	<ul><li>ERROR_NULL_ARGUMENT when listener is null</li></ul>
+ * @exception IllegalArgumentException <ul>
+ *	    <li>ERROR_NULL_ARGUMENT when listener is null</li>
+ * </ul>
  */
 public void removeEventListener(OleAutomation automation, GUID guid, int eventID, OleListener listener) {
 	checkWidget();
-	if (automation == null || listener == null || guid == null) throw new SWTError (SWT.ERROR_NULL_ARGUMENT);
+	if (automation == null || listener == null || guid == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
 	removeEventListener(automation.getAddress(), guid, eventID, listener);
 }
 /**	 
@@ -661,12 +667,13 @@ public void removeEventListener(OleAutomation automation, GUID guid, int eventID
  * 
  * @param listener the listener
  *
- * @exception SWTError
- *	<ul><li>ERROR_NULL_ARGUMENT when listener is null</li></ul>
+ * @exception IllegalArgumentException <ul>
+ *	    <li>ERROR_NULL_ARGUMENT when listener is null</li>
+ * </ul>
  */
 public void removeEventListener(OleAutomation automation, int eventID, OleListener listener) {
 	checkWidget();
-	if (automation == null || listener == null) throw new SWTError (SWT.ERROR_NULL_ARGUMENT);
+	if (automation == null || listener == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
 	int address = automation.getAddress();
 	IUnknown unknown = new IUnknown(address);
 	GUID riid = getDefaultEventSinkGUID(unknown);
@@ -675,7 +682,7 @@ public void removeEventListener(OleAutomation automation, int eventID, OleListen
 	}
 }
 void removeEventListener(int iunknown, GUID guid, int eventID, OleListener listener) {
-	if (listener == null || guid == null) throw new SWTError (SWT.ERROR_NULL_ARGUMENT);
+	if (listener == null || guid == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
 	for (int i = 0; i < oleEventSink.length; i++) {
 		if (COM.IsEqualGUID(oleEventSinkGUID[i], guid)) {
 			if (iunknown == oleEventSinkIUnknown[i]) {
@@ -716,11 +723,12 @@ void removeEventListener(int iunknown, GUID guid, int eventID, OleListener liste
  *
  * @param listener the listener
  *
- * @exception SWTError
- *	<ul><li>ERROR_NULL_ARGUMENT when listener is null</li></ul>
+ * @exception IllegalArgumentException <ul>
+ *	    <li>ERROR_NULL_ARGUMENT when listener is null</li>
+ * </ul>
  */
 public void removePropertyListener(int propertyID, OleListener listener) {
-	if (listener == null) throw new SWTError (SWT.ERROR_NULL_ARGUMENT);
+	if (listener == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
 	olePropertyChangeSink.removeListener(propertyID, listener);
 }
 public void setBackground (Color color) {
