@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -40,6 +40,7 @@ public final class Printer extends Device {
 	PrinterData data;
 	int printContext, xScreen, xDrawable, xtContext;
 	Font defaultFont;
+	boolean isGCCreated;
 
 	static String APP_NAME = "SWT_Printer";
 	
@@ -238,11 +239,11 @@ static String[] getXPServerList() {
  * You must dispose the printer when it is no longer required. 
  * </p>
  *
- * @exception IllegalArgumentException <ul>
- *    <li>ERROR_UNSPECIFIED - if there are no valid printers
+ * @exception SWTError <ul>
+ *    <li>ERROR_NO_HANDLES - if there are no valid printers
  * </ul>
  *
- * @see #dispose
+ * @see Device#dispose
  */
 public Printer() {
 	this(null);
@@ -259,10 +260,12 @@ public Printer() {
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_INVALID_ARGUMENT - if the specified printer data does not represent a valid printer
- *    <li>ERROR_UNSPECIFIED - if there are no valid printers
+ * </ul>
+ * @exception SWTError <ul>
+ *    <li>ERROR_NO_HANDLES - if there are no valid printers
  * </ul>
  *
- * @see #dispose
+ * @see Device#dispose
  */
 public Printer(PrinterData data) {
 	super(checkNull(data));
@@ -272,6 +275,7 @@ public Printer(PrinterData data) {
  * Creates the printer handle.
  * This method is called internally by the instance creation
  * mechanism of the <code>Device</code> class.
+ * @param deviceData the device data
  */
 protected void create(DeviceData deviceData) {
 	data = (PrinterData)deviceData;
@@ -325,7 +329,7 @@ protected void init() {
 	OS.XpGetPageDimensions(xDisplay, printContext, width, height, rect);
 	xDrawable = OS.XCreateWindow(xDisplay, OS.XRootWindowOfScreen(xScreen), 
 		0, 0, rect.width, rect.height, 0,
-		OS.CopyFromParent, OS.CopyFromParent, OS.CopyFromParent, 0, 0);
+		OS.CopyFromParent, OS.CopyFromParent, OS.CopyFromParent, 0, null);
 	if (xDrawable == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 
 	/* Initialize the default font */
@@ -362,9 +366,8 @@ protected void destroy() {
  * @return the platform specific GC handle
  */
 public int internal_new_GC(GCData data) {
-	int xGC = OS.XCreateGC(xDisplay, xDrawable, 0, null);
-	if (xGC == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 	if (data != null) {
+		if (isGCCreated) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 		int mask = SWT.LEFT_TO_RIGHT | SWT.RIGHT_TO_LEFT;
 		if ((data.style & mask) == 0) {
 			data.style |= SWT.LEFT_TO_RIGHT;
@@ -372,8 +375,7 @@ public int internal_new_GC(GCData data) {
 		data.device = this;
 		data.display = xDisplay;
 		data.drawable = xDrawable;
-		data.fontList = defaultFont.handle;
-		data.codePage = defaultFont.codePage;
+		data.font = defaultFont;
 		data.colormap = OS.XDefaultColormapOfScreen(xScreen);
 		int defaultGC = OS.XDefaultGCOfScreen(xScreen);
 		if (defaultGC != 0) {
@@ -381,8 +383,11 @@ public int internal_new_GC(GCData data) {
 			OS.XGetGCValues(xDisplay, defaultGC, OS.GCBackground | OS.GCForeground, values);
 			data.foreground = values.foreground;
 			data.background = values.background;
-		}	
+		}
+		isGCCreated = true;
 	}
+	int xGC = OS.XCreateGC(xDisplay, xDrawable, 0, null);
+	if (xGC == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 	return xGC;
 }
 
@@ -396,11 +401,12 @@ public int internal_new_GC(GCData data) {
  * application code.
  * </p>
  *
- * @param handle the platform specific GC handle
+ * @param hDC the platform specific GC handle
  * @param data the platform specific GC data 
  */
 public void internal_dispose_GC(int xGC, GCData data) {
 	OS.XFreeGC(xDisplay, xGC);
+	if (data != null) isGCCreated = false;
 }
 
 /**
@@ -413,6 +419,7 @@ public void internal_dispose_GC(int xGC, GCData data) {
  * will result in undefined behavior.
  * </p>
  * 
+ * @param jobName the name of the print job to start
  * @return true if the job started successfully and false otherwise.
  *
  * @exception SWTException <ul>
@@ -605,6 +612,10 @@ public Rectangle getClientArea() {
  * is usually used by passing in the client area (the 'printable
  * area') of the printer. It can also be useful to pass in 0, 0, 0, 0.
  * 
+ * @param x the desired x coordinate of the client area
+ * @param y the desired y coordinate of the client area
+ * @param width the desired width of the client area
+ * @param height the desired height of the client area
  * @return the required bounds to produce the given client area
  *
  * @exception SWTException <ul>
