@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -39,19 +39,20 @@ import org.eclipse.swt.*;
  * </p>
  */
 
-public final class Cursor {
+public final class Cursor extends Resource {
 	
 	/**
 	 * the handle to the OS cursor resource
 	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
 	 */
 	public int handle;
 	
-	/**
-	 * the device where this cursor was created
-	 */
-	Device device;
-	
 	/**
 	 * data and mask used to create a Resize NS Cursor
 	 */
@@ -148,7 +149,7 @@ public Cursor(Device device, int style) {
 		case SWT.CURSOR_ARROW: 		handle = OS.kThemeArrowCursor; break;
 		case SWT.CURSOR_WAIT: 			handle = OS.kThemeSpinningCursor; break;
 		case SWT.CURSOR_CROSS: 		handle = OS.kThemeCrossCursor; break;
-		case SWT.CURSOR_APPSTARTING: 	handle = OS.kThemeWatchCursor; break;
+		case SWT.CURSOR_APPSTARTING: 	handle = OS.kThemeArrowCursor; break;
 		case SWT.CURSOR_HELP: 			handle = OS.kThemeCrossCursor; break;
 		case SWT.CURSOR_SIZEALL: 		handle = OS.kThemeCrossCursor; break;
 		case SWT.CURSOR_SIZENESW: 		handle = OS.kThemeCrossCursor; break;
@@ -184,7 +185,7 @@ public Cursor(Device device, int style) {
 /**	 
  * Constructs a new cursor given a device, image and mask
  * data describing the desired cursor appearance, and the x
- * and y co-ordinates of the <em>hotspot</em> (that is, the point
+ * and y coordinates of the <em>hotspot</em> (that is, the point
  * within the area covered by the cursor which is considered
  * to be where the on-screen pointer is "pointing").
  * <p>
@@ -206,8 +207,7 @@ public Cursor(Device device, int style) {
  *    <li>ERROR_NULL_ARGUMENT - if the source is null</li>
  *    <li>ERROR_NULL_ARGUMENT - if the mask is null and the source does not have a mask</li>
  *    <li>ERROR_INVALID_ARGUMENT - if the source and the mask are not the same 
- *          size, or either is not of depth one, or if the hotspot is outside 
- *          the bounds of the image</li>
+ *          size, or if the hotspot is outside the bounds of the image</li>
  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES - if a handle could not be obtained for cursor creation</li>
@@ -228,14 +228,14 @@ public Cursor(Device device, ImageData source, ImageData mask, int hotspotX, int
 	if (mask.width != source.width || mask.height != source.height) {
 		SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	}
-	/* Check color depths */
-	if (mask.depth != 1) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
-	if (source.depth != 1) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	/* Check the hotspots */
 	if (hotspotX >= source.width || hotspotX < 0 ||
 		hotspotY >= source.height || hotspotY < 0) {
 		SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	}
+	/* Convert depth to 1 */
+	mask = ImageData.convertMask(mask);
+	source = ImageData.convertMask(source);
 
 	/* Find the first non transparent pixel if cursor bigger than 16x16. */
 	int width = source.width;
@@ -312,117 +312,143 @@ public Cursor(Device device, ImageData source, ImageData mask, int hotspotX, int
 	OS.memcpy(handle, cursor, org.eclipse.swt.internal.carbon.Cursor.sizeof);
 }
 
-//3.0 API
-//public Cursor(Device device, ImageData source, int hotspotX, int hotspotY) {
-//	if (device == null) device = Device.getDevice();
-//	if (device == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
-//	this.device = device;
-//	if (source == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
-//	if (hotspotX >= source.width || hotspotX < 0 ||
-//		hotspotY >= source.height || hotspotY < 0) {
-//		SWT.error(SWT.ERROR_INVALID_ARGUMENT);
-//	}
-//	ImageData mask = source.getTransparencyMask();
-//
-//	/* Ensure depth is equal to 1 */
-//	if (source.depth > 1) {
-//		/* Create a destination image with no data */
-//		ImageData newSource = new ImageData(
-//			source.width, source.height, 1, ImageData.bwPalette(),
-//			1, null, 0, null, null, -1, -1, source.type,
-//			source.x, source.y, source.disposalMethod, source.delayTime);
-//
-//		/* Convert the source to a black and white image of depth 1 */
-//		PaletteData palette = source.palette;
-//		if (palette.isDirect) ImageData.blit(ImageData.BLIT_SRC,
-//			source.data, source.depth, source.bytesPerLine, source.getByteOrder(), 0, 0, source.width, source.height, 0, 0, 0,
-//			ImageData.ALPHA_OPAQUE, null, 0, 0, 0,
-//			newSource.data, newSource.depth, newSource.bytesPerLine, newSource.getByteOrder(), 0, 0, newSource.width, newSource.height, 0, 0, 0,
-//			false, false);
-//		else ImageData.blit(ImageData.BLIT_SRC,
-//			source.data, source.depth, source.bytesPerLine, source.getByteOrder(), 0, 0, source.width, source.height, null, null, null,
-//			ImageData.ALPHA_OPAQUE, null, 0, 0, 0,
-//			newSource.data, newSource.depth, newSource.bytesPerLine, newSource.getByteOrder(), 0, 0, newSource.width, newSource.height, null, null, null,
-//			false, false);
-//		source = newSource;
-//	}
-//
-//	/* Find the first non transparent pixel if cursor bigger than 16x16. */
-//	int width = source.width;
-//	int height = source.height;
-//	int minX = 0, minY = 0;
-//	if (width > 16 || height > 16) {
-//		minX = width;
-//		minY = height;
-//		int maxX = 0, maxY = 0;
-//		for (int y = 0; y < height; y++) {
-//			for (int x = 0; x < width; x++) {
-//				if (!(source.getPixel(x, y) == 1 && mask.getPixel(x, y) == 0)) {
-//					minX = Math.min(minX, x);
-//					minY = Math.min(minY, y);
-//					maxX = Math.max(maxX, x);
-//					maxY = Math.max(maxY, y);
-//				}
-//			}
-//		}
-//		width = maxX - minX + 1;
-//		height = maxY - minY + 1;
-//		
-//		/* Stretch cursor if still bigger than 16x16. */
-//		if (width > 16 || height > 16) {
-//			int newWidth = Math.min(width, 16);
-//			int newHeight = Math.min(height, 16);
-//			ImageData newSource =
-//				new ImageData(newWidth, newHeight, source.depth, source.palette,
-//					1, null, 0, null, null, -1, -1, source.type,
-//					source.x, source.y, source.disposalMethod, source.delayTime);
-//			ImageData newMask = new ImageData(newWidth, newHeight, mask.depth,
-//					mask.palette, 1, null, 0, null, null, -1, -1, mask.type,
-//					mask.x, mask.y, mask.disposalMethod, mask.delayTime);
-//			ImageData.blit(ImageData.BLIT_SRC,
-//				source.data, source.depth, source.bytesPerLine, source.getByteOrder(), minX, minY, width, height, null, null, null,
-//				ImageData.ALPHA_OPAQUE, null, 0, minX, minY,
-//				newSource.data, newSource.depth, newSource.bytesPerLine, newSource.getByteOrder(), 0, 0, newWidth, newHeight, null, null, null,
-//				false, false);
-//			ImageData.blit(ImageData.BLIT_SRC,
-//				mask.data, mask.depth, mask.bytesPerLine, mask.getByteOrder(), minX, minY, width, height, null, null, null,
-//				ImageData.ALPHA_OPAQUE, null, 0, minX, minY,
-//				newMask.data, newMask.depth, newMask.bytesPerLine, newMask.getByteOrder(), 0, 0, newWidth, newHeight, null, null, null,
-//				false, false);
-//			width = newWidth;
-//			height = newHeight;
-//			minX = minY = 0;
-//			source = newSource;
-//			mask = newMask;
-//		}
-//	}
-//
-//	/* Create the cursor */
-//	org.eclipse.swt.internal.carbon.Cursor cursor = new org.eclipse.swt.internal.carbon.Cursor();
-//	short[] srcData = cursor.data;
-//	short[] maskData = cursor.mask;
-//	for (int y= 0; y < height; y++) {
-//		short d = 0, m = 0;
-//		for (int x = 0; x < width; x++) {
-//			int bit = 1 << (width - 1 - x);			
-//			if (source.getPixel(x + minX, y + minY) == 0) {
-//				if (mask.getPixel(x + minX, y + minY) != 0) {
-//					d |= bit;
-//					m |= bit;
-//				}
-//			} else {
-//				if (mask.getPixel(x + minX, y + minY) != 0) m |= bit;
-//			}
-//		}
-//		srcData[y] = d;
-//		maskData[y] = m;
-//	}
-//	cursor.hotSpot_h = (short)Math.max(0, Math.min(15, hotspotY - minX));
-//	cursor.hotSpot_v = (short)Math.max(0, Math.min(15, hotspotY - minY));
-//	handle = OS.NewPtr(org.eclipse.swt.internal.carbon.Cursor.sizeof);
-//	if (handle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-//	OS.memcpy(handle, cursor, org.eclipse.swt.internal.carbon.Cursor.sizeof);
-//}
+/**	 
+ * Constructs a new cursor given a device, image data describing
+ * the desired cursor appearance, and the x and y coordinates of
+ * the <em>hotspot</em> (that is, the point within the area
+ * covered by the cursor which is considered to be where the
+ * on-screen pointer is "pointing").
+ * <p>
+ * You must dispose the cursor when it is no longer required. 
+ * </p>
+ *
+ * @param device the device on which to allocate the cursor
+ * @param source the image data for the cursor
+ * @param hotspotX the x coordinate of the cursor's hotspot
+ * @param hotspotY the y coordinate of the cursor's hotspot
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if device is null and there is no current device</li>
+ *    <li>ERROR_NULL_ARGUMENT - if the image is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if the hotspot is outside the bounds of the
+ * 		 image</li>
+ * </ul>
+ * @exception SWTError <ul>
+ *    <li>ERROR_NO_HANDLES - if a handle could not be obtained for cursor creation</li>
+ * </ul>
+ * 
+ * @since 3.0
+ */
+public Cursor(Device device, ImageData source, int hotspotX, int hotspotY) {
+	if (device == null) device = Device.getDevice();
+	if (device == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
+	this.device = device;
+	if (source == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
+	if (hotspotX >= source.width || hotspotX < 0 ||
+		hotspotY >= source.height || hotspotY < 0) {
+		SWT.error(SWT.ERROR_INVALID_ARGUMENT);
+	}
+	ImageData mask = source.getTransparencyMask();
+
+	/* Ensure depth is equal to 1 */
+	if (source.depth > 1) {
+		/* Create a destination image with no data */
+		ImageData newSource = new ImageData(
+			source.width, source.height, 1, ImageData.bwPalette(),
+			1, null, 0, null, null, -1, -1, source.type,
+			source.x, source.y, source.disposalMethod, source.delayTime);
+
+		/* Convert the source to a black and white image of depth 1 */
+		PaletteData palette = source.palette;
+		if (palette.isDirect) ImageData.blit(ImageData.BLIT_SRC,
+			source.data, source.depth, source.bytesPerLine, source.getByteOrder(), 0, 0, source.width, source.height, 0, 0, 0,
+			ImageData.ALPHA_OPAQUE, null, 0, 0, 0,
+			newSource.data, newSource.depth, newSource.bytesPerLine, newSource.getByteOrder(), 0, 0, newSource.width, newSource.height, 0, 0, 0,
+			false, false);
+		else ImageData.blit(ImageData.BLIT_SRC,
+			source.data, source.depth, source.bytesPerLine, source.getByteOrder(), 0, 0, source.width, source.height, null, null, null,
+			ImageData.ALPHA_OPAQUE, null, 0, 0, 0,
+			newSource.data, newSource.depth, newSource.bytesPerLine, newSource.getByteOrder(), 0, 0, newSource.width, newSource.height, null, null, null,
+			false, false);
+		source = newSource;
+	}
+
+	/* Find the first non transparent pixel if cursor bigger than 16x16. */
+	int width = source.width;
+	int height = source.height;
+	int minX = 0, minY = 0;
+	if (width > 16 || height > 16) {
+		minX = width;
+		minY = height;
+		int maxX = 0, maxY = 0;
+		for (int y = 0; y < height; y++) {
+			for (int x = 0; x < width; x++) {
+				if (!(source.getPixel(x, y) == 1 && mask.getPixel(x, y) == 0)) {
+					minX = Math.min(minX, x);
+					minY = Math.min(minY, y);
+					maxX = Math.max(maxX, x);
+					maxY = Math.max(maxY, y);
+				}
+			}
+		}
+		width = maxX - minX + 1;
+		height = maxY - minY + 1;
+		
+		/* Stretch cursor if still bigger than 16x16. */
+		if (width > 16 || height > 16) {
+			int newWidth = Math.min(width, 16);
+			int newHeight = Math.min(height, 16);
+			ImageData newSource =
+				new ImageData(newWidth, newHeight, source.depth, source.palette,
+					1, null, 0, null, null, -1, -1, source.type,
+					source.x, source.y, source.disposalMethod, source.delayTime);
+			ImageData newMask = new ImageData(newWidth, newHeight, mask.depth,
+					mask.palette, 1, null, 0, null, null, -1, -1, mask.type,
+					mask.x, mask.y, mask.disposalMethod, mask.delayTime);
+			ImageData.blit(ImageData.BLIT_SRC,
+				source.data, source.depth, source.bytesPerLine, source.getByteOrder(), minX, minY, width, height, null, null, null,
+				ImageData.ALPHA_OPAQUE, null, 0, minX, minY,
+				newSource.data, newSource.depth, newSource.bytesPerLine, newSource.getByteOrder(), 0, 0, newWidth, newHeight, null, null, null,
+				false, false);
+			ImageData.blit(ImageData.BLIT_SRC,
+				mask.data, mask.depth, mask.bytesPerLine, mask.getByteOrder(), minX, minY, width, height, null, null, null,
+				ImageData.ALPHA_OPAQUE, null, 0, minX, minY,
+				newMask.data, newMask.depth, newMask.bytesPerLine, newMask.getByteOrder(), 0, 0, newWidth, newHeight, null, null, null,
+				false, false);
+			width = newWidth;
+			height = newHeight;
+			minX = minY = 0;
+			source = newSource;
+			mask = newMask;
+		}
+	}
+
+	/* Create the cursor */
+	org.eclipse.swt.internal.carbon.Cursor cursor = new org.eclipse.swt.internal.carbon.Cursor();
+	short[] srcData = cursor.data;
+	short[] maskData = cursor.mask;
+	for (int y= 0; y < height; y++) {
+		short d = 0, m = 0;
+		for (int x = 0; x < width; x++) {
+			int bit = 1 << (width - 1 - x);			
+			if (source.getPixel(x + minX, y + minY) == 0) {
+				if (mask.getPixel(x + minX, y + minY) != 0) {
+					d |= bit;
+					m |= bit;
+				}
+			} else {
+				if (mask.getPixel(x + minX, y + minY) != 0) m |= bit;
+			}
+		}
+		srcData[y] = d;
+		maskData[y] = m;
+	}
+	cursor.hotSpot_h = (short)Math.max(0, Math.min(15, hotspotY - minX));
+	cursor.hotSpot_v = (short)Math.max(0, Math.min(15, hotspotY - minY));
+	handle = OS.NewPtr(org.eclipse.swt.internal.carbon.Cursor.sizeof);
+	if (handle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+	OS.memcpy(handle, cursor, org.eclipse.swt.internal.carbon.Cursor.sizeof);
+}
 
 /**
  * Disposes of the operating system resources associated with
@@ -470,7 +496,7 @@ public boolean equals (Object object) {
 
 /**
  * Returns an integer hash code for the receiver. Any two 
- * objects which return <code>true</code> when passed to 
+ * objects that return <code>true</code> when passed to 
  * <code>equals</code> must return the same value for this
  * method.
  *
