@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -53,7 +53,7 @@ import java.io.*;
  * loading process should use the support provided in class
  * <code>ImageLoader</code>.
  * </p><p>
- * Application code must explicitely invoke the <code>Image.dispose()</code> 
+ * Application code must explicitly invoke the <code>Image.dispose()</code> 
  * method to release the operating system resources managed by each instance
  * when those instances are no longer required.
  * </p>
@@ -62,7 +62,7 @@ import java.io.*;
  * @see ImageData
  * @see ImageLoader
  */
-public final class Image implements Drawable{
+public final class Image extends Resource implements Drawable {
 
 	/**
 	 * specifies whether the receiver is a bitmap or an icon
@@ -71,49 +71,51 @@ public final class Image implements Drawable{
 	public int type;
 	
 	/**
-	 * The handle to the OS image resource.
+	 * the handle to the OS image resource
 	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
 	 */
 	public int handle;
 
 	/**
 	 * The data to the OS image resource.
-	 * Warning: This field is platform dependent.
+	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
 	 */
 	public int data;
-
-	/**
-	 * The device where this image was created.
-	 */
-	Device device;
 	
 	/**
 	 * specifies the transparent pixel
-	 * (Warning: This field is platform dependent)
 	 */
 	int transparentPixel = -1;
 	
 	/**
 	 * The GC the image is currently selected in.
-	 * Warning: This field is platform dependent.
 	 */
 	GC memGC;
 
 	/**
 	 * The alpha data of the image.
-	 * Warning: This field is platform dependent.
 	 */
 	byte[] alphaData;
 	
 	/**
 	 * The global alpha value to be used for every pixel.
-	 * Warning: This field is platform dependent.
 	 */
 	int alpha = -1;
 	
 	/**
 	 * Specifies the default scanline padding.
-	 * Warning: This field is platform dependent.
 	 */
 	static final int DEFAULT_SCANLINE_PAD = 4;
 
@@ -150,7 +152,7 @@ Image() {
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
  * </ul>
  */
-public Image(Device display, int width, int height) {
+public Image(Device device, int width, int height) {
 	if (device == null) device = Device.getDevice();
 	if (device == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
 	init(device, width, height);
@@ -182,6 +184,7 @@ public Image(Device display, int width, int height) {
  * @exception SWTException <ul>
  *    <li>ERROR_INVALID_IMAGE - if the image is not a bitmap or an icon, or
  *          is otherwise in an invalid state</li>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the depth of the Image is not supported</li>
  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
@@ -230,7 +233,7 @@ public Image(Device device, Image srcImage, int flag) {
 		OS.DisposePtr(data);
 		SWT.error(SWT.ERROR_NO_HANDLES);
 	}
-	handle = OS.CGImageCreate(width, height, bpc, bpp, bpr, colorspace, alphaInfo, provider, null, false, 0);
+	handle = OS.CGImageCreate(width, height, bpc, bpp, bpr, colorspace, alphaInfo, provider, null, true, 0);
 	OS.CGDataProviderRelease(provider);
 	if (handle == 0) {
 		OS.DisposePtr(data);
@@ -327,7 +330,7 @@ public Image(Device device, Image srcImage, int flag) {
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
  * </ul>
  */
-public Image(Device display, Rectangle bounds) {
+public Image(Device device, Rectangle bounds) {
 	if (device == null) device = Device.getDevice();
 	if (device == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
 	if (bounds == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
@@ -345,6 +348,9 @@ public Image(Device display, Rectangle bounds) {
  *    <li>ERROR_NULL_ARGUMENT - if device is null and there is no current device</li>
  *    <li>ERROR_NULL_ARGUMENT - if the image data is null</li>
  * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the depth of the ImageData is not supported</li>
+ * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
  * </ul>
@@ -358,10 +364,8 @@ public Image(Device device, ImageData data) {
 /**
  * Constructs an instance of this class, whose type is 
  * <code>SWT.ICON</code>, from the two given <code>ImageData</code>
- * objects. The two images must be the same size, and the mask image
- * must have a color depth of 1. Pixel transparency in either image
- * will be ignored. If either image is an icon to begin with, an
- * exception is thrown.
+ * objects. The two images must be the same size. Pixel transparency
+ * in either image will be ignored.
  * <p>
  * The mask image should contain white wherever the icon is to be visible,
  * and black wherever the icon is to be transparent. In addition,
@@ -376,22 +380,20 @@ public Image(Device device, ImageData data) {
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if device is null and there is no current device</li>
  *    <li>ERROR_NULL_ARGUMENT - if either the source or mask is null </li>
- *    <li>ERROR_INVALID_ARGUMENT - if source and mask are different sizes or
- *          if the mask is not monochrome, or if either the source or mask
- *          is already an icon</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if source and mask are different sizes</li>
  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
  * </ul>
  */
-public Image(Device display, ImageData source, ImageData mask) {
+public Image(Device device, ImageData source, ImageData mask) {
 	if (device == null) device = Device.getDevice();
 	if (source == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
 	if (mask == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
 	if (source.width != mask.width || source.height != mask.height) {
 		SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	}
-	if (mask.depth != 1) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
+	mask = ImageData.convertMask(mask);
 	ImageData image = new ImageData(source.width, source.height, source.depth, source.palette, source.scanlinePad, source.data);
 	image.maskPad = mask.scanlinePad;
 	image.maskData = mask.data;
@@ -402,7 +404,8 @@ public Image(Device display, ImageData source, ImageData mask) {
  * Constructs an instance of this class by loading its representation
  * from the specified input stream. Throws an error if an error
  * occurs while loading the image, or if the result is an image
- * of an unsupported type.
+ * of an unsupported type.  Application code is still responsible
+ * for closing the input stream.
  * <p>
  * This constructor is provided for convenience when loading a single
  * image only. If the stream contains multiple images, only the first
@@ -412,7 +415,20 @@ public Image(Device display, ImageData source, ImageData mask) {
  * This constructor may be used to load a resource as follows:
  * </p>
  * <pre>
- *     new Image(device, clazz.getResourceAsStream("file.gif"));
+ *     static Image loadImage (Display display, Class clazz, String string) {
+ *          InputStream stream = clazz.getResourceAsStream (string);
+ *          if (stream == null) return null;
+ *          Image image = null;
+ *          try {
+ *               image = new Image (display, stream);
+ *          } catch (SWTException ex) {
+ *          } finally {
+ *               try {
+ *                    stream.close ();
+ *               } catch (IOException ex) {}
+ *          }
+ *          return image;
+ *     }
  * </pre>
  *
  * @param device the device on which to create the image
@@ -425,7 +441,9 @@ public Image(Device display, ImageData source, ImageData mask) {
  * @exception SWTException <ul>
  *    <li>ERROR_INVALID_IMAGE - if the image file contains invalid data </li>
  *    <li>ERROR_IO - if an IO error occurs while reading data</li>
- * </ul>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the InputStream describes an image with an unsupported depth</li>
+ *    <li>ERROR_UNSUPPORTED_FORMAT - if the image file contains an unrecognized format</li>
+ *  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
  * </ul>
@@ -456,27 +474,46 @@ public Image(Device device, InputStream stream) {
  * @exception SWTException <ul>
  *    <li>ERROR_INVALID_IMAGE - if the image file contains invalid data </li>
  *    <li>ERROR_IO - if an IO error occurs while reading data</li>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the image file has an unsupported depth</li>
+ *    <li>ERROR_UNSUPPORTED_FORMAT - if the image file contains an unrecognized format</li>
  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
  * </ul>
  */
-public Image(Device display, String filename) {
+public Image(Device device, String filename) {
 	if (device == null) device = Device.getDevice();
 	if (device == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
 	init(device, new ImageData(filename));
 }
 
-void createMask () {
-	if (transparentPixel == -1) return;
+void createAlpha () {
+	if (transparentPixel == -1 && alpha == -1 && alphaData == null) return;
 	int height = OS.CGImageGetHeight(handle);
 	int bpr = OS.CGImageGetBytesPerRow(handle);
 	int dataSize = height * bpr;
 	byte[] srcData = new byte[dataSize];
 	OS.memcpy(srcData, data, dataSize);
-	for (int i=0; i<dataSize; i+=4) {
-		int pixel = ((srcData[i+1] & 0xFF) << 16) | ((srcData[i+2] & 0xFF) << 8) | (srcData[i+3] & 0xFF);
-		srcData[i] = (byte)(pixel == transparentPixel ? 0 : 0xFF); 
+	if (transparentPixel != -1) {
+		for (int i=0; i<dataSize; i+=4) {
+			int pixel = ((srcData[i+1] & 0xFF) << 16) | ((srcData[i+2] & 0xFF) << 8) | (srcData[i+3] & 0xFF);
+			srcData[i] = (byte)(pixel == transparentPixel ? 0 : 0xFF); 
+		}
+	} else if (alpha != -1) {
+		byte a = (byte)this.alpha;
+		for (int i=0; i<dataSize; i+=4) {
+			srcData[i] = a;				
+		}
+	} else {
+		int width = OS.CGImageGetWidth(handle);
+		int offset = 0, alphaOffset = 0;
+		for (int y = 0; y<height; y++) {
+			for (int x = 0; x<width; x++) {
+				srcData[offset] = alphaData[alphaOffset];
+				offset += 4;
+				alphaOffset += 1;
+			}
+		}
 	}
 	OS.memcpy(data, srcData, dataSize);
 }
@@ -489,6 +526,7 @@ void createMask () {
 public void dispose () {
 	if (handle == 0) return;
 	if (device.isDisposed()) return;
+	if (memGC != null) memGC.dispose();
 	OS.CGImageRelease(handle);
 	OS.DisposePtr(data);
 	device = null;
@@ -589,7 +627,8 @@ public ImageData getImageData() {
 	data.transparentPixel = transparentPixel;
 	if (transparentPixel == -1 && type == SWT.ICON) {
 		/* Get the icon mask data */
-		int maskBpl = (((width + 7) / 8) + 3) / 4 * 4;
+		int maskPad = 2;
+		int maskBpl = (((width + 7) / 8) + (maskPad - 1)) / maskPad * maskPad;
 		byte[] maskData = new byte[height * maskBpl];
 		int offset = 0, maskOffset = 0;
 		for (int y = 0; y<height; y++) {
@@ -604,7 +643,10 @@ public ImageData getImageData() {
 			maskOffset += maskBpl;
 		}
 		data.maskData = maskData;
-		data.maskPad = 4;
+		data.maskPad = maskPad;
+	}
+	for (int i = 0; i < srcData.length; i+= 4) {
+		srcData[i] = 0;
 	}
 	data.alpha = alpha;
 	if (alpha == -1 && alphaData != null) {
@@ -643,7 +685,7 @@ public static Image carbon_new(Device device, int type, int handle, int data) {
 
 /**
  * Returns an integer hash code for the receiver. Any two 
- * objects which return <code>true</code> when passed to 
+ * objects that return <code>true</code> when passed to 
  * <code>equals</code> must return the same value for this
  * method.
  *
@@ -673,7 +715,7 @@ void init(Device device, int width, int height) {
 		SWT.error(SWT.ERROR_NO_HANDLES);
 	}
 	int colorspace = device.colorspace;
-	handle = OS.CGImageCreate(width, height, 8, 32, bpr, colorspace, OS.kCGImageAlphaNoneSkipFirst, provider, null, false, 0);
+	handle = OS.CGImageCreate(width, height, 8, 32, bpr, colorspace, OS.kCGImageAlphaNoneSkipFirst, provider, null, true, 0);
 	OS.CGDataProviderRelease(provider);
 	if (handle == 0) {
 		OS.DisposePtr(data);
@@ -683,6 +725,11 @@ void init(Device device, int width, int height) {
 	/* Fill the image with white */
 	int bpc = OS.CGImageGetBitsPerComponent(handle);
 	int context = OS.CGBitmapContextCreate(this.data, width, height, bpc, bpr, colorspace, OS.kCGImageAlphaNoneSkipFirst);
+	if (context == 0) {
+		OS.CGImageRelease(handle);
+		OS.DisposePtr(data);
+		SWT.error(SWT.ERROR_NO_HANDLES);
+	}
 	CGRect rect = new CGRect();
 	rect.width = width; rect.height = height;
 	OS.CGContextSetRGBFillColor(context, 1, 1, 1, 1);
@@ -695,6 +742,11 @@ void init(Device device, ImageData image) {
 	this.device = device;
 	int width = image.width;
 	int height = image.height;
+	PaletteData palette = image.palette;
+	if (!(((image.depth == 1 || image.depth == 2 || image.depth == 4 || image.depth == 8) && !palette.isDirect) ||
+			((image.depth == 8) || (image.depth == 16 || image.depth == 24 || image.depth == 32) && palette.isDirect)))
+				SWT.error(SWT.ERROR_UNSUPPORTED_DEPTH);
+
 	
 	/* Create the image */
 	int dataSize = width * height * 4;
@@ -707,8 +759,8 @@ void init(Device device, ImageData image) {
 	}
 	int colorspace = device.colorspace;
 	int transparency = image.getTransparencyType(); 
-	int alphaInfo = transparency == SWT.TRANSPARENCY_NONE ? OS.kCGImageAlphaNoneSkipFirst : OS.kCGImageAlphaFirst;
-	handle = OS.CGImageCreate(width, height, 8, 32, width * 4, colorspace, alphaInfo, provider, null, false, 0);
+	int alphaInfo = transparency == SWT.TRANSPARENCY_NONE && image.alpha == -1 ? OS.kCGImageAlphaNoneSkipFirst : OS.kCGImageAlphaFirst;
+	handle = OS.CGImageCreate(width, height, 8, 32, width * 4, colorspace, alphaInfo, provider, null, true, 0);
 	OS.CGDataProviderRelease(provider);
 	if (handle == 0) {
 		OS.DisposePtr(data);
@@ -717,8 +769,6 @@ void init(Device device, ImageData image) {
 	
 	/* Initialize data */
 	int bpr = width * 4;
-	PaletteData palette = image.palette;
-	PaletteData newPalette = new PaletteData(0xFF0000, 0xFF00, 0xFF);
 	byte[] buffer = new byte[dataSize];
 	if (palette.isDirect) {
 		ImageData.blit(ImageData.BLIT_SRC,
@@ -742,7 +792,7 @@ void init(Device device, ImageData image) {
 		ImageData.blit(ImageData.BLIT_SRC,
 			image.data, image.depth, image.bytesPerLine, image.getByteOrder(), 0, 0, width, height, srcReds, srcGreens, srcBlues,
 			ImageData.ALPHA_OPAQUE, null, 0, 0, 0,
-			buffer, 32, bpr, ImageData.MSB_FIRST, 0, 0, width, height, newPalette.redMask, newPalette.greenMask, newPalette.blueMask,
+			buffer, 32, bpr, ImageData.MSB_FIRST, 0, 0, width, height, 0xFF0000, 0xFF00, 0xFF,
 			false, false);
 	}
 	
@@ -830,7 +880,6 @@ public int internal_new_GC (GCData data) {
 	if (context == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 	OS.CGContextScaleCTM(context, 1, -1);
 	OS.CGContextTranslateCTM(context, 0, -height);
-	OS.CGContextSetShouldSmoothFonts(context, false);
 	if (data != null) {
 		int mask = SWT.LEFT_TO_RIGHT | SWT.RIGHT_TO_LEFT;
 		if ((data.style & mask) == 0) {
@@ -855,7 +904,7 @@ public int internal_new_GC (GCData data) {
  * application code.
  * </p>
  *
- * @param handle the platform specific GC handle
+ * @param hDC the platform specific GC handle
  * @param data the platform specific GC data 
  */
 public void internal_dispose_GC (int context, GCData data) {
@@ -887,7 +936,7 @@ public boolean isDisposed() {
  * these cases. For example:
  * <pre>
  *    Button b = new Button();
- *    image.setBackground(b.getBackground());>
+ *    image.setBackground(b.getBackground());
  *    b.setImage(image);
  * </pre>
  * </p><p>
