@@ -1,18 +1,21 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.swt.widgets;
 
- 
+
+import org.eclipse.swt.internal.carbon.FontInfo;
+import org.eclipse.swt.internal.carbon.GDevice;
 import org.eclipse.swt.internal.carbon.OS;
 import org.eclipse.swt.internal.carbon.MenuTrackingData;
+import org.eclipse.swt.internal.carbon.Rect;
  
 import org.eclipse.swt.*;
 import org.eclipse.swt.events.*;
@@ -39,10 +42,17 @@ public class Menu extends Widget {
 	/**
 	 * the handle to the OS resource 
 	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
 	 */
 	int handle;
 	short id;
 	int x, y;
+//	int width, height;
 	boolean hasLocation, modified, closed;
 	MenuItem [] items;
 	MenuItem cascade, defaultItem, lastTarget;
@@ -114,7 +124,7 @@ public Menu (Decorations parent, int style) {
  * for the instance so that the instance will be a drop-down
  * menu on the given parent's parent.
  *
- * @param parent a menu which will be the parent of the new instance (cannot be null)
+ * @param parentMenu a menu which will be the parent of the new instance (cannot be null)
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
@@ -138,7 +148,7 @@ public Menu (Menu parentMenu) {
  * for the instance so that the instance will be a drop-down
  * menu on the given parent's parent menu.
  *
- * @param parent a menu item which will be the parent of the new instance (cannot be null)
+ * @param parentItem a menu item which will be the parent of the new instance (cannot be null)
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
@@ -178,12 +188,27 @@ static int checkStyle (int style) {
 void _setVisible (boolean visible) {
 	if ((style & (SWT.BAR | SWT.DROP_DOWN)) != 0) return;
 	if (!visible) return;
-	int left = x, top = y;
-	if (!hasLocation) {
-		org.eclipse.swt.internal.carbon.Point where = new org.eclipse.swt.internal.carbon.Point ();
-		OS.GetGlobalMouse (where);
-		left = where.h; top = where.v;
+	int left = 0, top = 0;
+	org.eclipse.swt.internal.carbon.Point where = new org.eclipse.swt.internal.carbon.Point ();
+	OS.GetGlobalMouse (where);
+	if (hasLocation) {
+		left = x;
+		top = y;
+	} else {
+		left = where.h;
+		top = where.v;
 	}
+	/*
+	* Feature in the Macintosh.  PopUpMenuSelect() does not position
+	* the menu at the specified coordinates. The x coordinate is honoured,
+	* but the top of the menu is placed a few pixels above the y coordinate.
+	* This means that the first item in the menu will be under the mouse
+	* and will draw selected.  This is against the Macintosh User Guidelines.
+	* The fix is to offset the menu by one pixel in both directions when
+	* it coincides with the mouse location. 
+	*/
+	if (left == where.h) left++;
+	if (top == where.v) top++;
 	OS.PopUpMenuSelect (handle, (short)top, (short)left, (short)-1);
 }
 
@@ -255,7 +280,7 @@ void createItem (MenuItem item, int index) {
 	checkWidget ();
 	int count = OS.CountMenuItems (handle);
 	if (!(0 <= index && index <= count)) error (SWT.ERROR_INVALID_RANGE);
-	int attributes = 0;
+	int attributes = OS.kMenuItemAttrAutoRepeat | OS.kMenuItemAttrCustomDraw;
 	if ((item.style & SWT.SEPARATOR) != 0) attributes = OS.kMenuItemAttrSeparator;
 	int result = OS.InsertMenuItemTextWithCFString (handle, 0, (short) index, attributes, 0);
 	if (result != OS.noErr) {
@@ -269,15 +294,8 @@ void createItem (MenuItem item, int index) {
 	System.arraycopy (items, index, items, index + 1, count - index);
 	items [index] = item;
 	modified = true;
-	if ((style & SWT.BAR) != 0) {
-//		Display display = getDisplay ();
-//		short menuID = display.nextMenuId ();
-//		int outMenuRef [] = new int [1];
-//		if (OS.CreateNewMenu (menuID, 0, outMenuRef) != OS.noErr) {
-//			error (SWT.ERROR_NO_HANDLES);
-//		}
-//		OS.SetMenuItemHierarchicalMenu (handle, (short) (index + 1), outMenuRef [0]);
-	}
+	int emptyMenu = item.createEmptyMenu ();
+	if (emptyMenu != 0) OS.SetMenuItemHierarchicalMenu (handle, (short) (index + 1), emptyMenu);
 }
 
 void createWidget () {
@@ -297,15 +315,8 @@ void destroyItem (MenuItem item) {
 	System.arraycopy (items, index + 1, items, index, --count - index);
 	items [count] = null;
 	if (count == 0) items = new MenuItem [4];
+	if (item.menu == null) item.destroyEmptyMenu (index);
 	modified = true;
-	if ((style & SWT.BAR) != 0) {
-//		int [] outMenuRef = new int [1];
-//		OS.GetMenuItemHierarchicalMenu (handle, outIndex [0], outMenuRef);
-//		if (outMenuRef [0] != 0) {
-//			OS.DeleteMenu (OS.GetMenuID (outMenuRef [0]));
-//			OS.DisposeMenu (outMenuRef [0]);
-//		}
-	}
 	OS.DeleteMenuItem (handle, (short) (index + 1));
 }
 
@@ -318,6 +329,24 @@ void destroyWidget () {
 	}
 }
 
+/*public*/ Rectangle getBounds () {
+	checkWidget ();
+	if ((style & SWT.BAR) != 0) {
+		Menu menu = display.getMenuBar ();
+		if (this != menu) return new Rectangle (0, 0, 0, 0);
+		int height = OS.GetMBarHeight ();
+		int gdevice = OS.GetMainDevice ();
+		int [] ptr = new int [1];
+		OS.memcpy (ptr, gdevice, 4);
+		GDevice device = new GDevice ();
+		OS.memcpy (device, ptr [0], GDevice.sizeof);
+		return new Rectangle (0, 0, device.right - device.left, height);
+	}
+	OS.CalcMenuSize (handle);
+	return new Rectangle (x, y, 0, 0);
+//	return new Rectangle (x, y, width, height);
+}
+
 /**
  * Returns the default menu item or null if none has
  * been previously set.
@@ -337,7 +366,7 @@ public MenuItem getDefaultItem () {
 
 /**
  * Returns <code>true</code> if the receiver is enabled, and
- * <code>false</code> otherwise. A disabled control is typically
+ * <code>false</code> otherwise. A disabled menu is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -393,8 +422,8 @@ public int getItemCount () {
 }
 
 /**
- * Returns an array of <code>MenuItem</code>s which are the items
- * in the receiver. 
+ * Returns a (possibly empty) array of <code>MenuItem</code>s which
+ * are the items in the receiver. 
  * <p>
  * Note: This is not the actual structure used by the receiver
  * to maintain its list of items, so modifying the array will
@@ -536,19 +565,185 @@ void hookEvents () {
 	super.hookEvents ();
 	int menuProc = display.menuProc;
 	int [] mask = new int [] {
+		OS.kEventClassMenu, OS.kEventMenuCalculateSize,
 		OS.kEventClassMenu, OS.kEventMenuClosed,
+		OS.kEventClassMenu, OS.kEventMenuCreateFrameView,
+		OS.kEventClassMenu, OS.kEventMenuDrawItem,
+		OS.kEventClassMenu, OS.kEventMenuDrawItemContent,
+		OS.kEventClassMenu, OS.kEventMenuMeasureItemWidth,
 		OS.kEventClassMenu, OS.kEventMenuOpening,
 		OS.kEventClassMenu, OS.kEventMenuTargetItem,
 	};
 	int menuTarget = OS.GetMenuEventTarget (handle);
-	OS.InstallEventHandler (menuTarget, menuProc, mask.length / 2, mask, 0, null);
+	OS.InstallEventHandler (menuTarget, menuProc, mask.length / 2, mask, handle, null);
+}
+
+int kEventMenuCalculateSize (int nextHandler, int theEvent, int userData) {
+	int result = super.kEventMenuCalculateSize (nextHandler, theEvent, userData);
+	if (result == OS.noErr) return result;
+	int [] theControl = new int [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamControlRef, OS.typeControlRef, null, 4, null, theControl);
+	int menuProc = display.menuProc;
+	int [] mask = new int [] {
+		OS.kEventClassMenu, OS.kEventMenuGetFrameBounds,
+	};
+	int controlTarget = OS.GetControlEventTarget (theControl [0]);
+	//TODO - installed multi-times, does this matter?
+	OS.InstallEventHandler (controlTarget, menuProc, mask.length / 2, mask, handle, null);
+	return result;
 }
 
 int kEventMenuClosed (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventMenuClosed (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
 	closed = true;
+//	width = height = 0;
+//	int count = OS.CountMenuItems (handle);
+//	for (int i=0; i<count; i++) {
+//		MenuItem item = items [i];
+//		item.x = item.y = item.width = item.height = 0;
+//	}
 	sendEvent (SWT.Hide);
+	if (hooks (SWT.Hide)) {
+		getShell().update (true);
+	}
+	return OS.eventNotHandledErr;
+}
+
+int kEventMenuCreateFrameView (int nextHandler, int theEvent, int userData) {
+	int result = super.kEventMenuCreateFrameView (nextHandler, theEvent, userData);
+	if (result == OS.noErr) return result;
+	int [] theControl = new int [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamControlRef, OS.typeControlRef, null, 4, null, theControl);
+	int menuProc = display.menuProc;
+	int [] mask = new int [] {
+		OS.kEventClassMenu, OS.kEventMenuGetFrameBounds,
+	};
+	int controlTarget = OS.GetControlEventTarget (theControl [0]);
+	OS.InstallEventHandler (controlTarget, menuProc, mask.length / 2, mask, handle, null);
+	return OS.eventNotHandledErr;
+}
+
+int kEventMenuDrawItem (int nextHandler, int theEvent, int userData) {
+	int result = super.kEventMenuDrawItem (nextHandler, theEvent, userData);
+	if (result == OS.noErr) return result;
+//	short [] index = new short [1];
+//	OS.GetEventParameter (theEvent, OS.kEventParamMenuItemIndex, OS.typeMenuItemIndex, null, 2, null, index);
+//	MenuItem item = items [index [0] - 1];
+//	Rect rect = new Rect ();
+//	OS.GetEventParameter (theEvent, OS.kEventParamMenuItemBounds, OS.typeQDRectangle, null, Rect.sizeof, null, rect);
+//	item.x = rect.left - x;
+//	item.y = rect.top - y;
+//	item.width = rect.right - rect.left;
+//	item.height = rect.bottom - rect.top;
+	return OS.eventNotHandledErr;
+}
+
+int kEventMenuDrawItemContent (int nextHandler, int theEvent, int userData) {
+	int result = super.kEventMenuDrawItemContent (nextHandler, theEvent, userData);
+	if (result == OS.noErr) return result;
+	short [] index = new short [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamMenuItemIndex, OS.typeMenuItemIndex, null, 2, null, index);
+	MenuItem item = items [index [0] - 1];
+	if (item.accelerator == 0) {
+		int accelIndex = item.text.indexOf ('\t');
+		if (accelIndex != -1) {
+			String accelText = item.text.substring (accelIndex + 1);
+			int length = accelText.length ();
+			if (length != 0) {
+				result = OS.CallNextEventHandler (nextHandler, theEvent);
+				Rect rect = new Rect ();
+				OS.GetEventParameter (theEvent, OS.kEventParamMenuItemBounds, OS.typeQDRectangle, null, Rect.sizeof, null, rect);
+				int [] context = new int [1];
+				OS.GetEventParameter (theEvent, OS.kEventParamCGContextRef, OS.typeCGContextRef, null, 4, null, context);
+
+				/* Draw the key */
+				int modifierIndex = modifierIndex (accelText);
+				char [] buffer = new char [length - modifierIndex - 1];
+				accelText.getChars (modifierIndex + 1, length, buffer, 0);
+				int font = OS.kThemeMenuItemFont;
+				if (buffer.length > 1) font = OS.kThemeMenuItemCmdKeyFont;
+				byte [] family = new byte [256];
+				short [] size = new short [1];
+				byte [] style = new byte [1];
+				OS.GetThemeFont ((short) font, (short) OS.smSystemScript, family, size, style);
+				FontInfo info = new FontInfo ();
+				OS.FetchFontInfo (family[0], size[0], style[0], info);
+				int [] metric = new int [1];
+				OS.GetThemeMetric (OS.kThemeMetricMenuIconTrailingEdgeMargin, metric);
+				int str = OS.CFStringCreateWithCharacters (OS.kCFAllocatorDefault, buffer, buffer.length);
+				org.eclipse.swt.internal.carbon.Point size1 = new org.eclipse.swt.internal.carbon.Point ();
+				OS.GetThemeTextDimensions (str, (short) font, 0, false, size1, null);
+				rect.left = (short) (rect.right - Math.max (info.widMax, size1.h) - metric [0]);
+				OS.DrawThemeTextBox (str, (short) font, OS.kThemeStateActive, false, rect, (short) OS.teFlushLeft, context [0]);
+				OS.CFRelease (str);
+				
+				/* Draw the modifiers */
+				if (modifierIndex != -1) {
+					buffer = new char [modifierIndex + 1];
+					accelText.getChars (0, buffer.length, buffer, 0);
+					str = OS.CFStringCreateWithCharacters (OS.kCFAllocatorDefault, buffer, buffer.length);
+					OS.GetThemeTextDimensions (str, (short) OS.kThemeMenuItemCmdKeyFont, 0, false, size1, null);
+					rect.right = rect.left;
+					rect.left = (short) (rect.right - size1.h);
+					OS.DrawThemeTextBox (str, (short) OS.kThemeMenuItemCmdKeyFont, OS.kThemeStateActive, false, rect, (short) OS.teFlushLeft, context [0]);
+					OS.CFRelease (str);
+				}
+				return result;
+			}
+		}			
+	}
+	return OS.eventNotHandledErr;
+}
+
+int kEventMenuGetFrameBounds (int nextHandler, int theEvent, int userData) {
+	int result = super.kEventMenuGetFrameBounds (nextHandler, theEvent, userData);
+	if (result == OS.noErr) return result;
+	result = OS.CallNextEventHandler (nextHandler, theEvent);
+//	CGRect rect = new CGRect ();
+//	OS.GetEventParameter (theEvent, OS.kEventParamBounds, OS.typeHIRect, null, CGRect.sizeof, null, rect);
+//	x = (int) rect.x;
+//	y = (int) rect.y;
+//	width = (int) rect.width;
+//	height = (int) rect.height;
+//	if (cascade != null) {
+//		OS.GetEventParameter (theEvent, OS.kEventParamMenuItemBounds, OS.typeHIRect, null, CGRect.sizeof, null, rect);
+//		cascade.x = (int) rect.x - x;
+//		cascade.y = (int) rect.y - y;
+//		cascade.width = (int) rect.width;
+//		cascade.height = (int) rect.height;
+//	}
+	return result;
+}
+
+int kEventMenuMeasureItemWidth (int nextHandler, int theEvent, int userData) {
+	int result = super.kEventMenuMeasureItemWidth (nextHandler, theEvent, userData);
+	if (result == OS.noErr) return result;
+	short [] index = new short [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamMenuItemIndex, OS.typeMenuItemIndex, null, 2, null, index);
+	MenuItem item = items [index [0] - 1];
+	if (item.accelerator == 0) {
+		int accelIndex = item.text.indexOf ('\t');
+		if (accelIndex != -1) {
+			String accelText = item.text.substring (accelIndex + 1);
+			if (accelText.length () != 0) {
+				result = OS.CallNextEventHandler (nextHandler, theEvent);
+				char [] buffer = new char [accelText.length ()];
+				accelText.getChars (0, buffer.length, buffer, 0);
+				int str = OS.CFStringCreateWithCharacters (OS.kCFAllocatorDefault, buffer, buffer.length);
+				org.eclipse.swt.internal.carbon.Point size = new org.eclipse.swt.internal.carbon.Point ();
+				OS.GetThemeTextDimensions (str, (short) OS.kThemeMenuItemCmdKeyFont, 0, false, size, null);
+				OS.CFRelease (str);
+				short [] width = new short [1];
+				OS.GetEventParameter (theEvent, OS.kEventParamMenuItemWidth, OS.typeSInt16, null, 2, null, width);
+				int [] metric = new int [1];
+				OS.GetThemeMetric (OS.kThemeMetricMenuTextTrailingEdgeMargin, metric);
+				width [0] += metric [0] + size.h;
+				OS.SetEventParameter (theEvent, OS.kEventParamMenuItemWidth, OS.typeSInt16, 2, width);
+				return result;
+			}
+		}			
+	}
 	return OS.eventNotHandledErr;
 }
 
@@ -558,6 +753,9 @@ int kEventMenuOpening (int nextHandler, int theEvent, int userData) {
 	closed = false;
 	sendEvent (SWT.Show);
 	modified = false;
+	if (hooks (SWT.Show)) {
+		getShell().update (true);
+	}
 	return OS.eventNotHandledErr;
 }
 
@@ -567,8 +765,13 @@ int kEventMenuTargetItem (int nextHandler, int theEvent, int userData) {
 	lastTarget = null;
 	short [] index = new short [1];
 	if (OS.GetEventParameter (theEvent, OS.kEventParamMenuItemIndex, OS.typeMenuItemIndex, null, 2, null, index) == OS.noErr) {
-		if (index [0] != 0) lastTarget = items [index [0] - 1];
-		if (lastTarget != null) lastTarget.sendEvent (SWT.Arm);
+		if (index [0] > 0) lastTarget = items [index [0] - 1];
+		if (lastTarget != null) {
+			lastTarget.sendEvent (SWT.Arm);
+			if (lastTarget.hooks (SWT.Arm)) {
+				getShell().update (true);
+			}
+		}
 	}
 	return OS.eventNotHandledErr;
 }
@@ -603,7 +806,7 @@ public int indexOf (MenuItem item) {
 /**
  * Returns <code>true</code> if the receiver is enabled and all
  * of the receiver's ancestors are enabled, and <code>false</code>
- * otherwise. A disabled control is typically not selectable from the
+ * otherwise. A disabled menu is typically not selectable from the
  * user interface and draws with an inactive or "grayed" look.
  *
  * @return the receiver's enabled state
@@ -641,6 +844,26 @@ public boolean isVisible () {
 	return getVisible ();
 }
 
+int modifierIndex (String accelText) {
+	int start = accelText.length () - 1;
+	int index = start;
+	while (index >= 0) {
+		char c = accelText.charAt (index);
+		switch (c) {
+			case ' ':
+				if (index != start) return index;
+				break;
+			case '\u2303':
+			case '\u2325':
+			case '\u21E7':
+			case '\u2318':
+				return index;
+		}
+		index--;
+	}
+	return -1;
+}
+
 void releaseChild () {
 	super.releaseChild ();
 	if (cascade != null) cascade.setMenu (null);
@@ -671,7 +894,7 @@ void releaseWidget () {
  * Removes the listener from the collection of listeners who will
  * be notified when the help events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -695,7 +918,7 @@ public void removeHelpListener (HelpListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the menu events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -738,7 +961,7 @@ public void setDefaultItem (MenuItem item) {
 
 /**
  * Enables the receiver if the argument is <code>true</code>,
- * and disables it otherwise. A disabled control is typically
+ * and disables it otherwise. A disabled menu is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -761,11 +984,15 @@ public void setEnabled (boolean enabled) {
 }
 
 /**
- * Sets the receiver's location to the point specified by
- * the arguments which are relative to the display.
+ * Sets the location of the receiver, which must be a popup,
+ * to the point specified by the arguments which are relative
+ * to the display.
  * <p>
- * Note:  This is different from most widgets where the
+ * Note that this is different from most widgets where the
  * location of the widget is relative to the parent.
+ * </p><p>
+ * Note that the platform window manager ultimately has control
+ * over the location of popup menus.
  * </p>
  *
  * @param x the new x coordinate for the receiver
@@ -784,11 +1011,15 @@ public void setLocation (int x, int y) {
 }
 
 /**
- * Sets the receiver's location to the point specified by
- * the arguments which are relative to the display.
+ * Sets the location of the receiver, which must be a popup,
+ * to the point specified by the argument which is relative
+ * to the display.
  * <p>
- * Note:  This is different from most widgets where the
+ * Note that this is different from most widgets where the
  * location of the widget is relative to the parent.
+ * </p><p>
+ * Note that the platform window manager ultimately has control
+ * over the location of popup menus.
  * </p>
  *
  * @param location the new location for the receiver
