@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -13,6 +13,8 @@ package org.eclipse.swt.widgets;
 
 import org.eclipse.swt.internal.carbon.OS;
 import org.eclipse.swt.internal.carbon.Rect;
+import org.eclipse.swt.internal.carbon.CGPoint;
+import org.eclipse.swt.internal.carbon.CGRect;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.*;
@@ -68,7 +70,7 @@ import org.eclipse.swt.graphics.*;
  * it would be upgraded to <code>APPLICATION_MODAL</code>.
  * <dl>
  * <dt><b>Styles:</b></dt>
- * <dd>BORDER, CLOSE, MIN, MAX, NO_TRIM, RESIZE, TITLE</dd>
+ * <dd>BORDER, CLOSE, MIN, MAX, NO_TRIM, RESIZE, TITLE, ON_TOP, TOOL</dd>
  * <dd>APPLICATION_MODAL, MODELESS, PRIMARY_MODAL, SYSTEM_MODAL</dd>
  * <dt><b>Events:</b></dt>
  * <dd>Activate, Close, Deactivate, Deiconify, Iconify</dd>
@@ -102,8 +104,14 @@ import org.eclipse.swt.graphics.*;
  */
 public class Shell extends Decorations {
 	int shellHandle, windowGroup;
-	boolean resized;
+	boolean resized, moved, drawing, reshape, update, activate, disposed, opened;
+	int invalRgn;
 	Control lastActive;
+	Region region;
+	Rect rgnRect;
+
+	static int DEFAULT_CLIENT_WIDTH = -1;
+	static int DEFAULT_CLIENT_HEIGHT = -1;
 
 /**
  * Constructs a new instance of this class. This is equivalent
@@ -235,6 +243,9 @@ Shell (Display display, Shell parent, int style, int handle) {
 	if (!display.isValidThread ()) {
 		error (SWT.ERROR_THREAD_INVALID_ACCESS);
 	}
+	if (parent != null && parent.isDisposed ()) {
+		error (SWT.ERROR_INVALID_ARGUMENT);	
+	}
 	this.style = checkStyle (style);
 	this.parent = parent;
 	this.display = display;
@@ -257,7 +268,7 @@ Shell (Display display, Shell parent, int style, int handle) {
  * @param parent a shell which will be the parent of the new instance
  *
  * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if the parent is disposed</li> 
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -291,6 +302,9 @@ public Shell (Shell parent) {
  * @param parent a shell which will be the parent of the new instance
  * @param style the style of control to construct
  *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_INVALID_ARGUMENT - if the parent is disposed</li> 
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
  *    <li>ERROR_INVALID_SUBCLASS - if this class is not an allowed subclass</li>
@@ -305,6 +319,8 @@ public Shell (Shell parent) {
  * @see SWT#NO_TRIM
  * @see SWT#SHELL_TRIM
  * @see SWT#DIALOG_TRIM
+ * @see SWT#ON_TOP
+ * @see SWT#TOOL
  * @see SWT#MODELESS
  * @see SWT#PRIMARY_MODAL
  * @see SWT#APPLICATION_MODAL
@@ -354,8 +370,17 @@ public void addShellListener(ShellListener listener) {
 	addListener(SWT.Deiconify,typedListener);
 }
 
-void bringToTop () {
-	OS.SelectWindow (shellHandle);
+void bringToTop (boolean force) {
+	if (getMinimized ()) return;
+	if (force) {
+		forceActive ();
+	} else {
+		setActive ();
+	}
+}
+
+void checkOpen () {
+	if (!opened) resized = false;
 }
 
 /**
@@ -370,6 +395,7 @@ void bringToTop () {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
+ * @see SWT#Close
  * @see #dispose
  */
 public void close () {
@@ -416,17 +442,20 @@ void createHandle () {
 	}
 	int windowClass = OS.kDocumentWindowClass;
 	if ((style & (SWT.CLOSE | SWT.TITLE)) == 0) windowClass = OS.kSheetWindowClass;
-//	int windowClass = parent == null ? OS.kDocumentWindowClass : OS.kSheetWindowClass;
-//	if ((style & SWT.APPLICATION_MODAL) != 0) windowClass = OS.kMovableModalWindowClass;
-//	if ((style & SWT.SYSTEM_MODAL) != 0) windowClass = OS.kModalWindowClass;
-	Rect rect = new Rect ();
-	OS.GetAvailableWindowPositioningBounds (OS.GetMainDevice (), rect);
-	int width = (rect.right - rect.left) * 5 / 8;
-	int height = (rect.bottom - rect.top) * 5 / 8;
-	OS.SetRect (rect, (short) 0, (short) 0, (short) width, (short) height);
+//	if ((style & (SWT.APPLICATION_MODAL | SWT.PRIMARY_MODAL | SWT.SYSTEM_MODAL)) != 0) {
+//		if ((style & SWT.CLOSE) == 0)  {
+//			windowClass = (style & SWT.TITLE) != 0 ? OS.kMovableModalWindowClass : OS.kModalWindowClass;
+//		}
+//	}
+	Monitor monitor = getMonitor ();
+	Rectangle rect = monitor.getClientArea ();
+	int width = rect.width * 5 / 8;
+	int height = rect.height * 5 / 8;
+	Rect bounds = new Rect ();
+	OS.SetRect (bounds, (short) 0, (short) 0, (short) width, (short) height);
 	int [] outWindow = new int [1];
 	attributes &= OS.GetAvailableWindowAttributes (windowClass);
-	OS.CreateNewWindow (windowClass, attributes, rect, outWindow);
+	OS.CreateNewWindow (windowClass, attributes, bounds, outWindow);
 	if (outWindow [0] == 0) error (SWT.ERROR_NO_HANDLES);
 	shellHandle = outWindow [0];
 	OS.RepositionWindow (shellHandle, 0, OS.kWindowCascadeOnMainScreen);
@@ -455,40 +484,97 @@ void createHandle () {
 		OS.SetWindowGroupParent (windowGroup, parentGroup);
 	}
 	OS.SetWindowGroupOwner (windowGroup, shellHandle);
+	CGPoint inMinLimits = new CGPoint (), inMaxLimits = new CGPoint ();
+	OS.GetWindowResizeLimits (shellHandle, inMinLimits, inMaxLimits);
+	if (DEFAULT_CLIENT_WIDTH == -1) DEFAULT_CLIENT_WIDTH = (int) inMinLimits.x;
+	if (DEFAULT_CLIENT_HEIGHT == -1) DEFAULT_CLIENT_HEIGHT = 0;
+	inMinLimits.y = (int) 0;
+	int trim = SWT.TITLE | SWT.CLOSE | SWT.MIN | SWT.MAX;
+	if ((style & SWT.NO_TRIM) != 0 || (style & trim) == 0) {
+		inMinLimits.x = (int) 0;	
+	}
+	OS.SetWindowResizeLimits (shellHandle, inMinLimits, inMaxLimits);
 }
 
 void createWidget () {
 	super.createWidget ();
-	layoutControl (false);
+	resizeBounds ();
 }
 
 void deregister () {
 	super.deregister ();
 	int [] theRoot = new int [1];
 	OS.GetRootControl (shellHandle, theRoot);
-	WidgetTable.remove (theRoot [0]);
+	display.removeWidget (theRoot [0]);
 }
 
 void destroyWidget () {
-	Display display = this.display;
 	int theWindow = shellHandle;
+	/*
+	* Bug in the Macintosh.  Under certain circumstances, yet to
+	* be determined, calling HideWindow() and then DisposeWindow()
+	* causes a segment fault when an application is exiting.  This
+	* seems to happen to large applications.  The fix is to avoid
+	* calling HideWindow() when a shell is about to be disposed.
+	*/
 //	OS.HideWindow (shellHandle);
 	releaseHandle ();
 	if (theWindow != 0) {
-		display.addDisposeWindow (theWindow);
+		OS.DisposeWindow (theWindow);
 	} 
 }
 
+void drawWidget (int control, int damageRgn, int visibleRgn, int theEvent) {
+	super.drawWidget (control, damageRgn, visibleRgn, theEvent);
+	/*
+	* Bug in the Macintosh. In kEventWindowGetRegion, 
+	* Carbon assumes the origin of the Region is (0, 0)
+	* and ignores the actual origin.  This causes the 
+	* window to be shifted for a non zero origin.  Also,
+	* the size of the window is the size of the region
+	* which may be less then the size specified in
+	* setSize or setBounds.
+	* The fix is to include (0, 0) and the bottom 
+	* right corner of the size in the region and to
+	* make these points transparent.
+	*/
+	if (region == null || region.isDisposed ()) return;
+	boolean origin = region.contains (0, 0);
+	boolean limit = region.contains(rgnRect.right - 1, rgnRect.bottom - 1);
+	if (origin && limit) return;
+	
+	int[] context = new int [1];
+	int port = OS.GetWindowPort (shellHandle);
+	Rect portRect = new Rect ();
+	OS.GetPortBounds (port, portRect);
+	OS.QDBeginCGContext (port, context);
+	OS.CGContextScaleCTM (context [0], 1, -1);
+	OS.CGContextTranslateCTM (context [0], 0, portRect.top - portRect.bottom);
+	CGRect cgRect = new CGRect ();
+	cgRect.width = 1;
+	cgRect.height = 1;
+	if (!origin) {
+		OS.CGContextClearRect (context [0], cgRect);
+	}
+	if (!limit) {
+		cgRect.x = rgnRect.right - 1;
+		cgRect.y = rgnRect.bottom - 1;
+		OS.CGContextClearRect (context [0], cgRect);
+	}
+	OS.CGContextSynchronize (context [0]);
+	OS.QDEndCGContext (port, context);
+}
+
 Cursor findCursor () {
 	return cursor;
 }
 
 /**
- * Moves the receiver to the top of the drawing order for
- * the display on which it was created (so that all other
- * shells on that display, which are not the receiver's
- * children will be drawn behind it) and forces the window
- * manager to make the shell active.
+ * If the receiver is visible, moves it to the top of the 
+ * drawing order for the display on which it was created 
+ * (so that all other shells on that display, which are not 
+ * the receiver's children will be drawn behind it) and forces 
+ * the window manager to make the shell active.
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -506,8 +592,10 @@ Cursor findCursor () {
  */
 public void forceActive () {
 	checkWidget ();
-	OS.SetFrontProcess (new int [] {0, OS.kCurrentProcess});
+	if (activate) return;
+	if(!isVisible()) return;
 	OS.SelectWindow (shellHandle);
+	OS.SetFrontProcessWithOptions (new int [] {0, OS.kCurrentProcess}, OS.kSetFrontProcessFrontWindowOnly);
 }
 
 public Rectangle getClientArea () {
@@ -566,13 +654,59 @@ public boolean getMaximized () {
 
 public boolean getMinimized () {
 	checkWidget();
+	if (!getVisible ()) return super.getMinimized ();
 	return OS.IsWindowCollapsed (shellHandle);
 }
 
+/**
+ * Returns a point describing the minimum receiver's size. The
+ * x coordinate of the result is the minimum width of the receiver.
+ * The y coordinate of the result is the minimum height of the
+ * receiver.
+ *
+ * @return the receiver's size
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.1
+ */
+public Point getMinimumSize () {
+	checkWidget();
+	Rect rect = new Rect ();
+	OS.GetWindowStructureWidths (shellHandle, rect);
+	CGPoint inMinLimits = new CGPoint (), inMaxLimits = new CGPoint ();
+	OS.GetWindowResizeLimits (shellHandle, inMinLimits, inMaxLimits);
+	int width = Math.max (1, (int) inMinLimits.x + (rect.left + rect.right));
+	int height = Math.max (1, (int) inMinLimits.y + (rect.top + rect.bottom));
+	return new Point (width, height);
+}
+
 float [] getParentBackground () {
 	return null;
 }
 
+/** 
+ * Returns the region that defines the shape of the shell,
+ * or null if the shell has the default shape.
+ *
+ * @return the region that defines the shape of the shell (or null)
+ *	
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @since 3.0
+ *
+ */
+public Region getRegion () {
+	checkWidget ();
+	return region;
+}
+
 public Shell getShell () {
 	checkWidget();
 	return this;
@@ -640,9 +774,13 @@ void hookEvents () {
 		OS.kEventClassWindow, OS.kEventWindowClose,
 		OS.kEventClassWindow, OS.kEventWindowCollapsed,
 		OS.kEventClassWindow, OS.kEventWindowDeactivated,
+		OS.kEventClassWindow, OS.kEventWindowDrawContent,
 		OS.kEventClassWindow, OS.kEventWindowExpanded,
+		OS.kEventClassWindow, OS.kEventWindowGetRegion,
 		OS.kEventClassWindow, OS.kEventWindowHidden,
+		OS.kEventClassWindow, OS.kEventWindowHitTest,
 		OS.kEventClassWindow, OS.kEventWindowShown,
+		OS.kEventClassWindow, OS.kEventWindowUpdate,
 	};
 	int windowTarget = OS.GetWindowEventTarget (shellHandle);
 	OS.InstallEventHandler (windowTarget, windowProc, mask1.length / 2, mask1, shellHandle, null);
@@ -663,11 +801,37 @@ void invalidateVisibleRegion (int control) {
 	invalidateChildrenVisibleRegion (control);
 }
 
+void invalWindowRgn (int window, int rgn) {
+	/*
+	* Bug in the Macintosh.  InvalidWindowRgn() will not invalidate
+	* the window when it is called from the default kEventWindowUpdate
+	* handler.  The fix is to detect that case, acumulate the region
+	* to invalidate and call InvalWindowRgn() after the default handler
+	* is done.  
+	*/
+	if (update) {
+		if (invalRgn == 0) invalRgn = OS.NewRgn();
+		OS.UnionRgn (rgn, invalRgn, invalRgn);
+	} else {
+		OS.InvalWindowRgn (window, rgn);
+	}
+	
+}
+
 public boolean isEnabled () {
 	checkWidget();
 	return getEnabled ();
 }
 
+boolean isEnabledCursor () {
+	return true;
+}
+
+public boolean isLayoutDeferred () {
+	checkWidget ();
+	return layoutCount > 0;
+}
+
 public boolean isVisible () {
 	checkWidget();
 	return getVisible ();
@@ -686,9 +850,12 @@ int kEventWindowActivated (int nextHandler, int theEvent, int userData) {
 	OS.GetWindowActivationScope (shellHandle, outScope); 
 	if (outScope [0] == OS.kWindowActivationScopeNone) return result;
 	display.setMenuBar (menuBar);
+	if (menuBar != null) OS.DrawMenuBar ();
+	activate = true;
 	sendEvent (SWT.Activate);
 	if (isDisposed ()) return result;
 	restoreFocus ();
+	activate = false;
 	return result;
 }
 
@@ -698,13 +865,24 @@ int kEventWindowBoundsChanged (int nextHandler, int theEvent, int userData) {
 	int [] attributes = new int [1];
 	OS.GetEventParameter (theEvent, OS.kEventParamAttributes, OS.typeUInt32, null, attributes.length * 4, null, attributes);
 	if ((attributes [0] & OS.kWindowBoundsChangeOriginChanged) != 0) {
+		moved = true;
 		sendEvent (SWT.Move);
+		if (isDisposed ()) return OS.noErr;
 	}
 	if ((attributes [0] & OS.kWindowBoundsChangeSizeChanged) != 0) {
 		resized = true;
-		layoutControl (false);
+		resizeBounds ();
 		sendEvent (SWT.Resize);
-		if (layout != null) layout.layout (this, false);
+		if (isDisposed ()) return OS.noErr;
+		if (layout != null) {
+			markLayout (false, false);
+			updateLayout (false);
+		}
+		if (region != null && !region.isDisposed()) {
+			OS.GetEventParameter (theEvent, OS.kEventParamCurrentBounds, OS.typeQDRectangle, null, Rect.sizeof, null, rgnRect);
+			OS.SetRect (rgnRect, (short) 0, (short) 0, (short) (rgnRect.right - rgnRect.left), (short) (rgnRect.bottom - rgnRect.top));
+			OS.ReshapeCustomWindow (shellHandle);
+		}
 	}
 	return result;
 }
@@ -712,13 +890,14 @@ int kEventWindowBoundsChanged (int nextHandler, int theEvent, int userData) {
 int kEventWindowClose (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventWindowClose (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
-	closeWidget ();
+	if (isEnabled ()) closeWidget ();
 	return OS.noErr;
 }
 
 int kEventWindowCollapsed (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventWindowCollapsed (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
+	minimized = true;
 	sendEvent (SWT.Iconify);
 	return result;
 }
@@ -727,6 +906,7 @@ int kEventWindowDeactivated (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventWindowDeactivated (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
 	//TEMPORARY CODE - should be send, but causes a GP
+	Display display = this.display;
 	postEvent (SWT.Deactivate);
 	if (isDisposed ()) return result;
 	saveFocus ();
@@ -741,19 +921,80 @@ int kEventWindowDeactivated (int nextHandler, int theEvent, int userData) {
 		OS.ClearKeyboardFocus (shellHandle);
 		display.ignoreFocus = false;
 		//TEMPORARY CODE - should be send, but causes a GP
-		if (!savedFocus.isDisposed ()) savedFocus.sendFocusEvent (false, true);
+		if (!savedFocus.isDisposed ()) savedFocus.sendFocusEvent (SWT.FocusOut, true);
 	}
 	display.setMenuBar (null);
 	return result;
 }
 
+int kEventWindowDrawContent (int nextHandler, int theEvent, int userData) {
+	drawing = true;
+	int result = OS.CallNextEventHandler (nextHandler, theEvent);
+	drawing = false;	
+	if (reshape) {
+		reshape = false;
+		OS.ReshapeCustomWindow (shellHandle);
+	}
+	return result;
+}
+
 int kEventWindowExpanded (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventWindowExpanded (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
+	minimized = false;
 	sendEvent (SWT.Deiconify);
 	return result;
 }
 
+int kEventWindowGetRegion (int nextHandler, int theEvent, int userData) {
+	int result = super.kEventWindowGetRegion (nextHandler, theEvent, userData);
+	if (result == OS.noErr) return result;
+	if (region == null || region.isDisposed ()) return OS.eventNotHandledErr;
+	short [] regionCode = new short [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamWindowRegionCode, OS.typeWindowRegionCode , null, 2, null, regionCode);
+	int [] temp = new int [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamRgnHandle, OS.typeQDRgnHandle , null, 4, null, temp);
+	int hRegion = temp [0];
+	switch (regionCode [0]) {
+		case OS.kWindowContentRgn:
+		case OS.kWindowStructureRgn:
+			OS.RectRgn (hRegion, rgnRect);
+			OS.SectRgn (hRegion, region.handle, hRegion);
+			/*
+			* Bug in the Macintosh. In kEventWindowGetRegion, 
+			* Carbon assumes the origin of the Region is (0, 0)
+			* and ignores the actual origin.  This causes the 
+			* window to be shifted for a non zero origin.  Also,
+			* the size of the window is the size of the region
+			* which may be less then the size specified in
+			* setSize or setBounds.
+			* The fix is to include (0, 0) and the bottom 
+			* right corner of the size in the region and to
+			* make these points transparent.
+			*/
+			if (!region.contains (0, 0)) {
+				Rect r = new Rect ();
+				OS.SetRect (r, (short)0, (short)0, (short)1, (short)1);
+				int rectRgn = OS.NewRgn ();
+				OS.RectRgn (rectRgn, r);
+				OS.UnionRgn (rectRgn, hRegion, hRegion);
+				OS.DisposeRgn (rectRgn);
+			}
+			if (!region.contains (rgnRect.right - 1, rgnRect.bottom - 1)) {
+				Rect r = new Rect ();
+				OS.SetRect (r, (short) (rgnRect.right - 1), (short) (rgnRect.bottom - 1), rgnRect.right, rgnRect.bottom);
+				int rectRgn = OS.NewRgn ();
+				OS.RectRgn (rectRgn, r);
+				OS.UnionRgn (rectRgn, hRegion, hRegion);
+				OS.DisposeRgn (rectRgn);
+			}
+			return OS.noErr;
+		default:
+			OS.DiffRgn (hRegion, hRegion, hRegion);
+			return OS.noErr;
+	}
+}
+
 int kEventWindowHidden (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventWindowHidden (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
@@ -765,6 +1006,25 @@ int kEventWindowHidden (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
+int kEventWindowHitTest (int nextHandler, int theEvent, int userData) {
+	int result = super.kEventWindowHitTest (nextHandler, theEvent, userData);
+	if (result == OS.noErr) return result;
+	if (region == null || region.isDisposed ()) return OS.eventNotHandledErr;
+	org.eclipse.swt.internal.carbon.Point pt = new org.eclipse.swt.internal.carbon.Point ();
+	int sizeof = org.eclipse.swt.internal.carbon.Point.sizeof;
+	OS.GetEventParameter (theEvent, OS.kEventParamMouseLocation, OS.typeQDPoint, null, sizeof, null, pt);
+	Rect rect = new Rect ();
+	OS.GetWindowBounds (shellHandle, (short) OS.kWindowContentRgn, rect);
+	OS.SetPt (pt, (short) (pt.h - rect.left), (short) (pt.v - rect.top));
+	int rgn = OS.NewRgn ();
+	OS.RectRgn (rgn, rgnRect);
+	OS.SectRgn (rgn, region.handle, rgn);
+	short inData = OS.PtInRgn (pt, rgn) ? OS.wInContent: OS.wNoHit;
+	OS.DisposeRgn (rgn);
+	OS.SetEventParameter (theEvent, OS.kEventParamWindowDefPart, OS.typeWindowDefPartCode, 2, new short [] {inData});
+	return OS.noErr;
+}
+
 int kEventWindowShown (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventWindowShown (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
@@ -778,12 +1038,24 @@ int kEventWindowShown (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
-void layoutControl (boolean events) {
+int kEventWindowUpdate (int nextHandler, int theEvent, int userData) {
+	update = true;
+	int result = OS.CallNextEventHandler (nextHandler, theEvent);
+	update = false;
+	if (invalRgn != 0) {
+		OS.InvalWindowRgn (shellHandle, invalRgn);
+		OS.DisposeRgn (invalRgn);
+		invalRgn = 0;
+	}
+	return result;
+}
+
+void resizeBounds () {
 	Rect rect = new Rect ();
 	OS.GetWindowBounds (shellHandle, (short)  OS.kWindowContentRgn, rect);
 	int control = scrolledHandle != 0 ? scrolledHandle : handle;
 	setBounds (control, 0, 0, rect.right - rect.left, rect.bottom - rect.top, false, true, false);
-	super.layoutControl (events);
+	resizeClientArea ();
 }
 
 /**
@@ -791,8 +1063,8 @@ void layoutControl (boolean events) {
  * the display on which it was created (so that all other
  * shells on that display, which are not the receiver's
  * children will be drawn behind it), marks it visible,
- * and sets focus to its default button (if it has one)
- * and asks the window manager to make the shell active.
+ * sets the focus and asks the window manager to make the
+ * shell active.
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -809,16 +1081,21 @@ void layoutControl (boolean events) {
  */
 public void open () {
 	checkWidget();
-	OS.SelectWindow (shellHandle);
+	bringToTop (false);
 	setVisible (true);
-	if (!restoreFocus ()) traverseGroup (true);
+	if (isDisposed ()) return;
+	if (!restoreFocus () && !traverseGroup (true)) setFocus ();
 }
 
 void register () {
 	super.register ();
 	int [] theRoot = new int [1];
 	OS.GetRootControl (shellHandle, theRoot);
-	WidgetTable.put (theRoot [0], this);
+	display.addWidget (theRoot [0], this);
+}
+
+void releaseChild () {
+	/* Do nothing */
 }
 
 void releaseHandle () {
@@ -835,11 +1112,15 @@ void releaseShells () {
 }
 
 void releaseWidget () {
+	disposed = true;
 	releaseShells ();
 	super.releaseWidget ();
 	if (windowGroup != 0) OS.ReleaseWindowGroup (windowGroup);
-	windowGroup = 0;
+	display.updateQuitMenu ();
+	if (invalRgn != 0) OS.DisposeRgn (invalRgn);
+	invalRgn = windowGroup = 0;
 	lastActive = null;
+	region = null;
 }
 
 /**
@@ -871,11 +1152,11 @@ public void removeShellListener(ShellListener listener) {
 }
 
 /**
- * Moves the receiver to the top of the drawing order for
- * the display on which it was created (so that all other
- * shells on that display, which are not the receiver's
- * children will be drawn behind it) and asks the window
- * manager to make the shell active.
+ * If the receiver is visible, moves it to the top of the 
+ * drawing order for the display on which it was created 
+ * (so that all other shells on that display, which are not 
+ * the receiver's children will be drawn behind it) and asks 
+ * the window manager to make the shell active 
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -893,6 +1174,8 @@ public void removeShellListener(ShellListener listener) {
  */
 public void setActive () {
 	checkWidget ();
+	if (activate) return;
+	if(!isVisible()) return;
 	OS.SelectWindow (shellHandle);
 }
 
@@ -933,13 +1216,36 @@ void setActiveControl (Control control) {
 	}
 }
 
-public void setBounds (int x, int y, int width, int height) {
-	checkWidget ();
-	width = Math.max (0, width);
-	height = Math.max (0, height);
+int setBounds (int x, int y, int width, int height, boolean move, boolean resize, boolean events) {
 	Rect rect = new Rect ();
+	if (!move) {
+		OS.GetWindowBounds (shellHandle, (short) OS.kWindowStructureRgn, rect);
+		x = rect.left;
+		y = rect.top;
+	}
+	if (!resize) {
+		OS.GetWindowBounds (shellHandle, (short) OS.kWindowStructureRgn, rect);
+		width = rect.right - rect.left;
+		height = rect.bottom - rect.top;
+	} else {
+		OS.GetWindowStructureWidths (shellHandle, rect);
+		CGPoint inMinLimits = new CGPoint (), inMaxLimits = new CGPoint ();
+		OS.GetWindowResizeLimits (shellHandle, inMinLimits, inMaxLimits);
+		width = Math.max (1, Math.max (width, (int) inMinLimits.x + (rect.left + rect.right)));
+		height = Math.max (1, Math.max (height, (int) inMinLimits.y + (rect.top + rect.bottom)));
+	}
 	OS.SetRect (rect, (short) x, (short) y, (short) (x + width), (short) (y + height));
 	OS.SetWindowBounds (shellHandle, (short) OS.kWindowStructureRgn, rect);
+	return 0;
+}
+
+public void setEnabled (boolean enabled) {
+	checkWidget();
+	if (((state & DISABLED) == 0) == enabled) return;
+	super.setEnabled (enabled);
+	if (enabled && OS.IsWindowActive (shellHandle)) {
+		if (!restoreFocus ()) traverseGroup (false);
+	}
 }
 
 public void setMenuBar (Menu menu) {
@@ -970,39 +1276,136 @@ public void setImeInputMode (int mode) {
 	checkWidget();
 }
 
-public void setLocation (int x, int y) {
-	checkWidget();
-	Rect rect = new Rect ();
-	OS.GetWindowBounds (shellHandle, (short) OS.kWindowStructureRgn, rect);
-	int width = rect.right - rect.left;
-	int height = rect.bottom - rect.top;
-	OS.SetRect (rect, (short) x, (short) y, (short) (x + width), (short) (y + height));
-	OS.SetWindowBounds (shellHandle, (short) OS.kWindowStructureRgn, rect);
-}
-
 public void setMaximized (boolean maximized) {
 	checkWidget();
 	super.setMaximized (maximized);
 	org.eclipse.swt.internal.carbon.Point pt = new org.eclipse.swt.internal.carbon.Point ();
+	if (maximized) {
+		Rect rect = new Rect ();
+		int gdevice = OS.GetMainDevice ();
+		OS.GetAvailableWindowPositioningBounds (gdevice, rect);
+		pt.h = (short) (rect.right - rect.left);
+		pt.v = (short) (rect.bottom - rect.top);
+	}
 	short inPartCode = (short) (maximized ? OS.inZoomOut : OS.inZoomIn);
-	//FIXME - returns -50 errParam
 	OS.ZoomWindowIdeal (shellHandle, inPartCode, pt);
 }
 
 public void setMinimized (boolean minimized) {
 	checkWidget();
+	if (this.minimized == minimized) return;
 	super.setMinimized (minimized);
-	OS.CollapseWindow (shellHandle, true);
+	if (!minimized && OS.IsWindowCollapsed (shellHandle)) {
+		if (!activate) OS.SelectWindow (shellHandle);
+	}
+	OS.CollapseWindow (shellHandle, minimized);
 }
 
-public void setSize (int width, int height) {
+/**
+ * Sets the receiver's minimum size to the size specified by the arguments.
+ * If the new minimum size is larger than the current size of the receiver,
+ * the receiver is resized to the new minimum size.
+ *
+ * @param width the new minimum width for the receiver
+ * @param height the new minimum height for the receiver
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.1
+ */
+public void setMinimumSize (int width, int height) {
 	checkWidget();
-	width = Math.max (0, width);
-	height = Math.max (0, height);
+	int clientWidth = 0, clientHeight = 0;
+	int trim = SWT.TITLE | SWT.CLOSE | SWT.MIN | SWT.MAX;
+	if ((style & SWT.NO_TRIM) == 0 && (style & trim) != 0) {
+		clientWidth = DEFAULT_CLIENT_WIDTH;
+		clientHeight = DEFAULT_CLIENT_HEIGHT;
+	}
 	Rect rect = new Rect ();
-	OS.GetWindowBounds (shellHandle, (short) OS.kWindowStructureRgn, rect);
-	OS.SetRect (rect, rect.left, rect.top, (short)(rect.left + width), (short)(rect.top + height));
-	OS.SetWindowBounds (shellHandle, (short) OS.kWindowStructureRgn, rect);
+	OS.GetWindowStructureWidths (shellHandle, rect);
+	CGPoint inMinLimits = new CGPoint (), inMaxLimits = new CGPoint ();
+	OS.GetWindowResizeLimits (shellHandle, inMinLimits, inMaxLimits);
+	width = Math.max (width, clientWidth + rect.left + rect.right);
+	height = Math.max (height, clientHeight + rect.top + rect.bottom);
+	inMinLimits.x = width - (rect.left + rect.right);
+	inMinLimits.y = height - (rect.top + rect.bottom);
+	OS.SetWindowResizeLimits (shellHandle, inMinLimits, inMaxLimits);
+	Point size = getSize ();
+	int newWidth = Math.max (size.x, width), newHeight = Math.max (size.y, height);
+	if (newWidth != size.x || newHeight != size.y) setSize (newWidth, newHeight);
+}
+
+/**
+ * Sets the receiver's minimum size to the size specified by the argument.
+ * If the new minimum size is larger than the current size of the receiver,
+ * the receiver is resized to the new minimum size.
+ *
+ * @param size the new minimum size for the receiver
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the point is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.1
+ */
+public void setMinimumSize (Point size) {
+	checkWidget();
+	if (size == null) error (SWT.ERROR_NULL_ARGUMENT);
+	setMinimumSize (size.x, size.y);
+}
+
+/**
+ * Sets the shape of the shell to the region specified
+ * by the argument.  When the argument is null, the
+ * default shape of the shell is restored.  The shell
+ * must be created with the style SWT.NO_TRIM in order
+ * to specify a region.
+ *
+ * @param region the region that defines the shape of the shell (or null)
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_INVALID_ARGUMENT - if the region has been disposed</li>
+ * </ul>  
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @since 3.0
+ *
+ */
+public void setRegion (Region region) {
+	checkWidget ();
+	if ((style & SWT.NO_TRIM) == 0) return;
+	if (region != null && region.isDisposed()) error (SWT.ERROR_INVALID_ARGUMENT);
+	if (region == null) {
+		rgnRect = null;
+	} else {
+		if (rgnRect == null) {
+			rgnRect = new Rect ();
+			OS.GetWindowBounds (shellHandle, (short) OS.kWindowStructureRgn, rgnRect);
+			OS.SetRect (rgnRect, (short) 0, (short) 0, (short) (rgnRect.right - rgnRect.left), (short) (rgnRect.bottom - rgnRect.top));	
+		}
+	}
+	this.region = region;
+	/*
+	* Bug in the Macintosh.  Calling ReshapeCustomWindow() from a
+	* kEventWindowDrawContent handler originating from ShowWindow()
+	* will deadlock.  The fix is to detected this case and only call
+	* ReshapeCustomWindow() after the default handler is done.
+	*/
+	if (drawing) {
+		reshape = true;
+	} else {
+		OS.ReshapeCustomWindow (shellHandle);
+	}
 }
 
 public void setText (String string) {
@@ -1032,12 +1435,23 @@ public void setVisible (boolean visible) {
 void setWindowVisible (boolean visible) {
 	if (OS.IsWindowVisible (shellHandle) == visible) return;
 	if (visible) {
-		if (!resized) {
-			sendEvent (SWT.Resize);
-			if (layout != null) layout.layout (this, false);
-		}
 		sendEvent (SWT.Show);
 		if (isDisposed ()) return;
+		opened = true;
+		if (!moved) {
+			moved = true;
+			sendEvent (SWT.Move);
+			if (isDisposed ()) return;
+		}
+		if (!resized) {
+			resized = true;
+			sendEvent (SWT.Resize);
+			if (isDisposed ()) return;
+			if (layout != null) {
+				markLayout (false, false);
+				updateLayout (false);
+			}
+		}
 		int inModalKind = OS.kWindowModalityNone;
 		if ((style & SWT.PRIMARY_MODAL) != 0) inModalKind = OS.kWindowModalityWindowModal;
 		if ((style & SWT.APPLICATION_MODAL) != 0) inModalKind = OS.kWindowModalityAppModal;
@@ -1053,19 +1467,36 @@ void setWindowVisible (boolean visible) {
 		int [] scope = new int [1];
 		if ((style & SWT.ON_TOP) != 0) {
 			OS.GetWindowActivationScope (shellHandle, scope);
-	    	OS.SetWindowActivationScope (shellHandle, OS.kWindowActivationScopeNone);
+			OS.SetWindowActivationScope (shellHandle, OS.kWindowActivationScopeNone);
 		}
+		int shellHandle = this.shellHandle;
+		OS.RetainWindow (shellHandle);
 		OS.ShowWindow (shellHandle);
-		if ((style & SWT.ON_TOP) != 0) {
-			OS.SetWindowActivationScope (shellHandle, scope [0]);
+		OS.ReleaseWindow (shellHandle);
+		if (!isDisposed()) {
+			if (minimized != OS.IsWindowCollapsed (shellHandle)) {
+				OS.CollapseWindow (shellHandle, minimized);
+			}
+			if ((style & SWT.ON_TOP) != 0) {
+				OS.SetWindowActivationScope (shellHandle, scope [0]);
+			}
 		}
 	} else {
-    	OS.HideWindow (shellHandle);
+		/*
+		* Bug in the Macintosh.  Under certain circumstances, yet to
+		* be determined, calling HideWindow() and then DisposeWindow()
+		* causes a segment fault when an application is exiting.  This
+		* seems to happen to large applications.  The fix is to avoid
+		* calling HideWindow() when a shell is about to be disposed.
+		*/
+		if (!disposed) OS.HideWindow (shellHandle);
+		if (isDisposed ()) return;
 		int topHandle = topHandle ();
 		OS.SetControlVisibility (topHandle, false, false);
 		invalidateVisibleRegion (topHandle);
 		sendEvent (SWT.Hide);
 	}
+	display.updateQuitMenu ();
 }
 
 void setZOrder () {
