@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -28,7 +28,7 @@ import org.eclipse.swt.graphics.*;
  * <p>
  * <dl>
  * <dt><b>Styles:</b></dt>
- * <dd>CENTER, LEFT, MULTI, SINGLE, RIGHT, READ_ONLY, WRAP</dd>
+ * <dd>CENTER, LEFT, MULTI, PASSWORD, SINGLE, RIGHT, READ_ONLY, WRAP</dd>
  * <dt><b>Events:</b></dt>
  * <dd>DefaultSelection, Modify, Verify</dd>
  * </dl>
@@ -42,8 +42,22 @@ public class Text extends Scrollable {
 	int txnObject, txnFrameID;
 	int textLimit = LIMIT;
 	char echoCharacter;
+	/**
+	* The maximum number of characters that can be entered
+	* into a text widget.
+	* <p>
+	* Note that this value is platform dependent, based upon
+	* the native widget implementation.
+	* </p>
+	*/
 	public static final int LIMIT;
+	/**
+	* The delimiter used by multi-line text widgets.  When text
+	* is queried and from the widget, it will be delimited using
+	* this delimiter.
+	*/
 	public static final String DELIMITER;
+	static final char PASSWORD = '\245';
 
 	/*
 	* These values can be different on different platforms.
@@ -52,7 +66,7 @@ public class Text extends Scrollable {
 	*/
 	static {
 		LIMIT = 0x7FFFFFFF;
-		DELIMITER = "\n";
+		DELIMITER = "\r";
 	}
 
 /**
@@ -196,7 +210,7 @@ public void append (String string) {
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (hooks (SWT.Verify) || filters (SWT.Verify)) {
 		int charCount = getCharCount ();
-		string = verifyText (string, charCount, charCount);
+		string = verifyText (string, charCount, charCount, null);
 		if (string == null) return;
 	}
 	setTXNText (OS.kTXNEndOffset, OS.kTXNEndOffset, string);
@@ -206,14 +220,25 @@ public void append (String string) {
 }
 
 static int checkStyle (int style) {
+	if ((style & SWT.SINGLE) != 0 && (style & SWT.MULTI) != 0) {
+		style &= ~SWT.MULTI;
+	}
 	style = checkBits (style, SWT.LEFT, SWT.CENTER, SWT.RIGHT, 0, 0, 0);
 	if ((style & SWT.SINGLE) != 0) style &= ~(SWT.H_SCROLL | SWT.V_SCROLL | SWT.WRAP);
-	if ((style & SWT.WRAP) != 0) style |= SWT.MULTI;
+	if ((style & SWT.WRAP) != 0) {
+		style |= SWT.MULTI;
+		style &= ~SWT.H_SCROLL;
+	}
+	if ((style & SWT.MULTI) != 0) style &= ~SWT.PASSWORD;
 	if ((style & (SWT.SINGLE | SWT.MULTI)) != 0) return style;
 	if ((style & (SWT.H_SCROLL | SWT.V_SCROLL)) != 0) return style | SWT.MULTI;
 	return style | SWT.SINGLE;
 }
 
+int callFocusEventHandler (int nextHandler, int theEvent) {
+	return OS.noErr;
+}
+
 /**
  * Clears the selection.
  *
@@ -322,20 +347,27 @@ void createHandle () {
 		OS.HIViewAddSubview (handle, scrollBar [0]);
 	}
 	
-	/* Configure the TXNOBject */
+	/* Configure the TXNObject */
 	int ptr = OS.NewPtr (Rect.sizeof);
 	Rect rect = new Rect ();
-	OS.SetRect (rect, (short) 1, (short) 1, (short) 1, (short) 1);
+	if (hasBorder ()) {
+		OS.SetRect (rect, (short) 1, (short) 1, (short) 1, (short) 1);
+	}
 	OS.memcpy (ptr, rect, Rect.sizeof);
 	int [] tags = new int [] {
 		OS.kTXNDisableDragAndDropTag,
 		OS.kTXNIOPrivilegesTag,
 		OS.kTXNMarginsTag,
+		OS.kTXNJustificationTag,
 	};
+	int just = OS.kTXNFlushLeft;
+	if ((style & SWT.CENTER) != 0) just = OS.kTXNCenter;
+	if ((style & SWT.RIGHT) != 0) just = OS.kTXNFlushRight;
 	int [] datas = new int [] {
 		1,
 		(style & SWT.READ_ONLY) != 0 ? 1 : 0,
 		ptr,
+		just,
 	};
 	OS.TXNSetTXNObjectControls (txnObject, false, tags.length, tags, datas);
 	OS.TXNSetFrameBounds (txnObject, 0, 0, 0, 0, txnFrameID);
@@ -350,10 +382,15 @@ void createHandle () {
 	OS.TXNSetData (txnObject, OS.kTXNUnicodeTextData, buffer, 0, OS.kTXNStartOffset, OS.kTXNEndOffset);
 }
 
-ScrollBar createScrollBar (int type) {
+ScrollBar createScrollBar (int style) {
 	return createStandardBar (style);
 }
 
+void createWidget () {
+	super.createWidget ();
+	if ((style & SWT.PASSWORD) != 0) setEchoChar (PASSWORD);
+}
+
 /**
  * Cuts the selected text.
  * <p>
@@ -368,23 +405,43 @@ ScrollBar createScrollBar (int type) {
  */
 public void cut () {
 	checkWidget();
+	if ((style & SWT.READ_ONLY) != 0) return;
+	boolean cut = true;
 	Point oldSelection = getSelection ();
 	if (hooks (SWT.Verify) || filters (SWT.Verify)) {
 		if (oldSelection.x != oldSelection.y) {
-			String newText = verifyText ("", oldSelection.x, oldSelection.y);
+			String newText = verifyText ("", oldSelection.x, oldSelection.y, null);
 			if (newText == null) return;
 			if (newText.length () != 0) {
 				setTXNText (OS.kTXNUseCurrentSelection, OS.kTXNUseCurrentSelection, newText);
+				OS.TXNShowSelection (txnObject, false);
+				cut = false;
 			}
 		}
 	}
-	OS.TXNCut (txnObject);
+	if (cut) {
+		OS.TXNCut (txnObject);
+
+		/*
+		* Feature in the Macintosh.  When an empty string is set in the TXNObject,
+		* the font attributes are cleared.  The fix is to reset them.
+		*/
+		if (OS.TXNDataSize (txnObject) / 2 == 0) setFontStyle (font);
+	}
 	Point newSelection = getSelection ();
-	if (!oldSelection.equals (newSelection)) sendEvent (SWT.Modify);
+	if (!cut || !oldSelection.equals (newSelection)) sendEvent (SWT.Modify);
+}
+
+Color defaultBackground () {
+	return display.getSystemColor (SWT.COLOR_LIST_BACKGROUND);
+}
+
+Color defaultForeground () {
+	return display.getSystemColor (SWT.COLOR_LIST_FOREGROUND);
 }
 
 void drawBackground (int control) {
-	drawFocus (control, hasFocus (), hasBorder (), getParentBackground (), inset ());
+	drawFocus (control, hasFocus () && drawFocusRing (), hasBorder (), getParentBackground (), inset ());
 }
 
 void drawWidget (int control, int damageRgn, int visibleRgn, int theEvent) {
@@ -393,7 +450,7 @@ void drawWidget (int control, int damageRgn, int visibleRgn, int theEvent) {
 }
 
 /**
- * Gets the line number of the caret.
+ * Returns the line number of the caret.
  * <p>
  * The line number of the caret is returned.
  * </p>
@@ -412,7 +469,8 @@ public int getCaretLineNumber () {
 }
 
 /**
- * Gets the location the caret.
+ * Returns a point describing the receiver's location relative
+ * to its parent (or its display if its parent is null).
  * <p>
  * The location of the caret is returned.
  * </p>
@@ -436,9 +494,9 @@ public Point getCaretLocation () {
 }
 
 /**
- * Gets the position of the caret.
+ * Returns the character position of the caret.
  * <p>
- * The character position of the caret is returned.
+ * Indexing is zero based.
  * </p>
  *
  * @return the position of the caret
@@ -456,7 +514,7 @@ public int getCaretPosition () {
 }
 
 /**
- * Gets the number of characters.
+ * Returns the number of characters.
  *
  * @return number of characters in the widget
  *
@@ -466,17 +524,29 @@ public int getCaretPosition () {
  * </ul>
  */
 public int getCharCount () {
-	checkWidget();
+	checkWidget ();
 	return OS.TXNDataSize (txnObject) / 2;
 }
 
+String getClipboardText () {
+	int [] scrap = new int [1];
+	OS.GetCurrentScrap (scrap);
+	int [] size = new int [1];
+	if (OS.GetScrapFlavorSize (scrap [0], OS.kScrapFlavorTypeUnicode, size) != OS.noErr || size [0] == 0) return "";
+	char [] buffer = new char [size [0] / 2];
+	if (OS.GetScrapFlavorData (scrap [0], OS.kScrapFlavorTypeUnicode, size, buffer) != OS.noErr) return "";
+	return new String (buffer);
+}
+
 /**
- * Gets the double click enabled flag.
+ * Returns the double click enabled flag.
  * <p>
  * The double click flag enables or disables the
  * default action of the text widget when the user
  * double clicks.
  * </p>
+ * 
+ * @return whether or not double click is enabled
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -490,17 +560,21 @@ public boolean getDoubleClickEnabled () {
 }
 
 /**
- * Gets the echo character.
+ * Returns the echo character.
  * <p>
  * The echo character is the character that is
  * displayed when the user enters text or the
  * text is changed by the programmer.
  * </p>
+ * 
+ * @return the echo character
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see #setEchoChar
  */
 public char getEchoChar () {
 	checkWidget();
@@ -508,8 +582,10 @@ public char getEchoChar () {
 }
 
 /**
- * Gets the editable state.
+ * Returns the editable state.
  *
+ * @return whether or not the reciever is editable
+ * 
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -521,7 +597,7 @@ public boolean getEditable () {
 }
 
 /**
- * Gets the number of lines.
+ * Returns the number of lines.
  *
  * @return the number of lines in the widget
  *
@@ -538,7 +614,7 @@ public int getLineCount () {
 }
 
 /**
- * Gets the line delimiter.
+ * Returns the line delimiter.
  *
  * @return a string that is the line delimiter
  *
@@ -546,6 +622,8 @@ public int getLineCount () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see #DELIMITER
  */
 public String getLineDelimiter () {
 	checkWidget();
@@ -553,7 +631,7 @@ public String getLineDelimiter () {
 }
 
 /**
- * Gets the height of a line.
+ * Returns the height of a line.
  *
  * @return the height of a row of text
  *
@@ -570,9 +648,10 @@ public int getLineHeight () {
 }
 
 /**
- * Returns the orientation of the receiver.
+ * Returns the orientation of the receiver, which will be one of the
+ * constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.RIGHT_TO_LEFT</code>.
  *
- * @return the orientation bit.
+ * @return the orientation style
  * 
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -587,14 +666,17 @@ public int getOrientation () {
 }
 
 /**
- * Gets the position of the selected text.
+ * Returns a <code>Point</code> whose x coordinate is the
+ * character position representing the start of the selected
+ * text, and whose y coordinate is the character position
+ * representing the end of the selection. An "empty" selection
+ * is indicated by the x and y coordinates having the same value.
  * <p>
- * Indexing is zero based.  The range of
- * a selection is from 0..N where N is
- * the number of characters in the widget.
+ * Indexing is zero based.  The range of a selection is from
+ * 0..N where N is the number of characters in the widget.
  * </p>
- * 
- * @return the start and end of the selection
+ *
+ * @return a point representing the selection start and end
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -609,7 +691,7 @@ public Point getSelection () {
 }
 
 /**
- * Gets the number of selected characters.
+ * Returns the number of selected characters.
  *
  * @return the number of selected characters.
  *
@@ -626,7 +708,7 @@ public int getSelectionCount () {
 }
 
 /**
- * Gets the selected text.
+ * Gets the selected text, or an empty string if there is no current selection.
  *
  * @return the selected text
  * 
@@ -641,7 +723,7 @@ public String getSelectionText () {
 }
 
 /**
- * Gets the number of tabs.
+ * Returns the number of tabs.
  * <p>
  * Tab stop spacing is specified in terms of the
  * space (' ') character.  The width of a single
@@ -662,9 +744,10 @@ public int getTabs () {
 }
 
 /**
- * Gets the widget text.
+ * Returns the widget text.
  * <p>
- * The text for a text widget is the characters in the widget.
+ * The text for a text widget is the characters in the widget, or
+ * an empty string if this has never been set.
  * </p>
  *
  * @return the widget text
@@ -680,7 +763,8 @@ public String getText () {
 }
 
 /**
- * Gets a range of text.
+ * Returns a range of text.  Returns an empty string if the
+ * start of the range is greater than the end.
  * <p>
  * Indexing is zero based.  The range of
  * a selection is from 0..N-1 where N is
@@ -698,8 +782,11 @@ public String getText () {
  */
 public String getText (int start, int end) {
 	checkWidget ();
-	if (start < 0 || start > end) return "";
-	return getTXNText (start, end);
+	if (!(start <= end && 0 <= end)) return "";
+	int length = OS.TXNDataSize (txnObject) / 2;
+	start = Math.max (0, start);
+	end = Math.min (end, length - 1);
+	return getTXNText (start, end + 1);
 }
 
 /**
@@ -715,6 +802,8 @@ public String getText (int start, int end) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see #LIMIT
  */
 public int getTextLimit () {
 	checkWidget();
@@ -742,7 +831,7 @@ public int getTopIndex () {
 }
 
 /**
- * Gets the top pixel.
+ * Returns the top pixel.
  * <p>
  * The top pixel is the pixel position of the line
  * that is currently at the top of the widget.  On
@@ -790,11 +879,13 @@ String getTXNText (int iStartOffset, int iEndOffset) {
 Rect inset () {
 	Rect rect = new Rect ();
 	int [] outMetric = new int [1];
-	OS.GetThemeMetric (OS.kThemeMetricFocusRectOutset, outMetric);
-	rect.left += outMetric [0];
-	rect.top += outMetric [0];
-	rect.right += outMetric [0];
-	rect.bottom += outMetric [0];
+	if (drawFocusRing ()) {
+		OS.GetThemeMetric (OS.kThemeMetricFocusRectOutset, outMetric);
+		rect.left += outMetric [0];
+		rect.top += outMetric [0];
+		rect.right += outMetric [0];
+		rect.bottom += outMetric [0];
+	}
 	if (hasBorder ()) {
 		OS.GetThemeMetric (OS.kThemeMetricEditTextFrameOutset, outMetric);
 		rect.left += outMetric [0];
@@ -813,6 +904,9 @@ Rect inset () {
  *
  * @param string the string
  *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the string is <code>null</code></li>
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -823,7 +917,7 @@ public void insert (String string) {
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (hooks (SWT.Verify) || filters (SWT.Verify)) {
 		Point selection = getSelection ();
-		string = verifyText (string, selection.x, selection.y);
+		string = verifyText (string, selection.x, selection.y, null);
 		if (string == null) return;
 	}
 	setTXNText (OS.kTXNUseCurrentSelection, OS.kTXNUseCurrentSelection, string);
@@ -868,6 +962,10 @@ int kEventControlDeactivate (int nextHandler, int theEvent, int userData) {
 	return result;
 }
 
+int kEventControlGetFocusPart (int nextHandler, int theEvent, int userData) {
+	return OS.noErr;
+}
+
 int kEventControlSetCursor (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventControlSetCursor (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
@@ -877,23 +975,26 @@ int kEventControlSetCursor (int nextHandler, int theEvent, int userData) {
 
 int kEventControlSetFocusPart (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventControlSetFocusPart (nextHandler, theEvent, userData);
-	if (result == OS.noErr) return result;
-	short [] part = new short [1];
-	OS.GetEventParameter (theEvent, OS.kEventParamControlPart, OS.typeControlPartCode, null, 2, null, part);
-	drawFocusClipped (handle, part [0] != 0, hasBorder (), getParentBackground (), inset ());
-	OS.TXNDraw (txnObject, 0);
-	OS.TXNFocus (txnObject, part [0] != 0);
-	return OS.noErr;
+	if (result == OS.noErr) {
+		short [] part = new short [1];
+		OS.GetEventParameter (theEvent, OS.kEventParamControlPart, OS.typeControlPartCode, null, 2, null, part);
+		drawFocusClipped (handle, part [0] != OS.kControlFocusNoPart && drawFocusRing (), hasBorder (), getParentBackground (), inset ());
+		OS.TXNDraw (txnObject, 0);
+		OS.TXNFocus (txnObject, part [0] != OS.kControlFocusNoPart);
+	}
+	return result;
 }
 
-int kEventRawKey (int nextHandler, int theEvent, int userData) {
-	int result = super.kEventRawKey (nextHandler, theEvent, userData);
+int kEventTextInputUnicodeForKeyEvent (int nextHandler, int theEvent, int userData) {
+	int result = super.kEventTextInputUnicodeForKeyEvent (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
+	int [] keyboardEvent = new int [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamTextInputSendKeyboardEvent, OS.typeEventRef, null, keyboardEvent.length * 4, null, keyboardEvent);
 	int [] modifiers = new int [1];
-	OS.GetEventParameter (theEvent, OS.kEventParamKeyModifiers, OS.typeUInt32, null, 4, null, modifiers);
+	OS.GetEventParameter (keyboardEvent [0], OS.kEventParamKeyModifiers, OS.typeUInt32, null, 4, null, modifiers);
 	if (modifiers [0] == OS.cmdKey) {
 		int [] keyCode = new int [1];
-		OS.GetEventParameter (theEvent, OS.kEventParamKeyCode, OS.typeUInt32, null, keyCode.length * 4, null, keyCode);
+		OS.GetEventParameter (keyboardEvent [0], OS.kEventParamKeyCode, OS.typeUInt32, null, keyCode.length * 4, null, keyCode);
 		switch (keyCode [0]) {
 			case 7: /* X */
 				cut ();
@@ -908,7 +1009,7 @@ int kEventRawKey (int nextHandler, int theEvent, int userData) {
 	}
 	if ((style & SWT.SINGLE) != 0) {
 		int [] keyCode = new int [1];
-		OS.GetEventParameter (theEvent, OS.kEventParamKeyCode, OS.typeUInt32, null, keyCode.length * 4, null, keyCode);
+		OS.GetEventParameter (keyboardEvent [0], OS.kEventParamKeyCode, OS.typeUInt32, null, keyCode.length * 4, null, keyCode);
 		switch (keyCode [0]) {
 			/*
 			* Feature in the Macintosh.  Tab and Return characters are inserted into a
@@ -941,8 +1042,22 @@ int kEventRawKey (int nextHandler, int theEvent, int userData) {
  */
 public void paste () {
 	checkWidget();
-	//NOT DONE - get clipboard text and verify or use undo?
-	OS.TXNPaste (txnObject);
+	if ((style & SWT.READ_ONLY) != 0) return;
+	boolean paste = true;
+	if (hooks (SWT.Verify) || filters (SWT.Verify)) {
+		String oldText = getClipboardText ();
+		if (oldText != null) {
+			Point selection = getSelection ();
+			String newText = verifyText (oldText, selection.x, selection.y, null);
+			if (newText == null) return;
+			if (!newText.equals (oldText)) {
+				setTXNText (OS.kTXNUseCurrentSelection, OS.kTXNUseCurrentSelection, newText);
+				OS.TXNShowSelection (txnObject, false);
+				paste = false;
+			}
+		}
+	}
+	if (paste) OS.TXNPaste (txnObject);
 	sendEvent (SWT.Modify);
 }
 
@@ -980,7 +1095,7 @@ public void removeModifyListener (ModifyListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1005,7 +1120,7 @@ public void removeSelectionListener(SelectionListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is verified.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1092,7 +1207,7 @@ boolean sendKeyEvent (int type, Event event) {
 			if (event.character != '\t' && event.character < 0x20) return true;
 			oldText = new String (new char [] {event.character});
 	}
-	String newText = verifyText (oldText, start, end);
+	String newText = verifyText (oldText, start, end, event);
 	if (newText == null) return false;
 	if (charCount - (end - start) + newText.length () > textLimit) {
 		return false;
@@ -1120,8 +1235,8 @@ void setBackground (float [] color) {
 	OS.TXNSetBackground (txnObject, txnColor);
 }
 
-int setBounds (int control, int x, int y, int width, int height, boolean move, boolean resize, boolean events) {
-	int result = super.setBounds(control, x, y, width, height, move, resize, events);
+int setBounds (int x, int y, int width, int height, boolean move, boolean resize, boolean events) {
+	int result = super.setBounds(x, y, width, height, move, resize, events);
 	if ((result & (RESIZED | MOVED)) != 0) setTXNBounds ();
 	return result;
 }
@@ -1132,6 +1247,9 @@ int setBounds (int control, int x, int y, int width, int height, boolean move, b
  * The double click flag enables or disables the
  * default action of the text widget when the user
  * double clicks.
+ * </p><p>
+ * Note: This operation is a hint and is not supported on
+ * platforms that do not have this concept.
  * </p>
  * 
  * @param doubleClick the new double click flag
@@ -1155,8 +1273,9 @@ public void setDoubleClickEnabled (boolean doubleClick) {
  * the echo character to '\0' clears the echo
  * character and redraws the original text.
  * If for any reason the echo character is invalid,
- * the default echo character for the platform
- * is used.
+ * or if the platform does not allow modification
+ * of the echo character, the default echo character
+ * for the platform is used.
  * </p>
  *
  * @param echo the new echo character
@@ -1168,6 +1287,7 @@ public void setDoubleClickEnabled (boolean doubleClick) {
  */
 public void setEchoChar (char echo) {
 	checkWidget();
+	if ((style & SWT.MULTI) != 0) return;
 	echoCharacter = echo;
 	OS.TXNEchoMode (txnObject, echo, 0, echo != '\0');
 }
@@ -1198,7 +1318,7 @@ void setForeground (float [] color) {
 	if (color == null) {	
 		rgb = new RGBColor ();
 	} else {
-		rgb = toRGBColor (foreground);
+		rgb = toRGBColor (color);
 	}
 	OS.memcpy (ptr2, rgb, RGBColor.sizeof);
 	int [] attribs = new int [] {
@@ -1237,10 +1357,13 @@ void setFontStyle (Font font) {
 
 /**
  * Sets the orientation of the receiver, which must be one
- * of the constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.LEFT_TO_RIGHT</code>.
+ * of the constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.RIGHT_TO_LEFT</code>.
  * <p>
+ * Note: This operation is a hint and is not supported on
+ * platforms that do not have this concept.
+ * </p>
  *
- * @param orientation new orientation bit
+ * @param orientation new orientation style
  * 
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -1282,7 +1405,8 @@ public void setSelection (int start) {
 }
 
 /**
- * Sets the selection.
+ * Sets the selection to the range specified
+ * by the given start and end indices.
  * <p>
  * Indexing is zero based.  The range of
  * a selection is from 0..N where N is
@@ -1307,13 +1431,18 @@ public void setSelection (int start) {
  */
 public void setSelection (int start, int end) {
 	checkWidget();
-	if (start < 0 || start > end) return;
-	OS.TXNSetSelection (txnObject, start, end);
+	int length = OS.TXNDataSize (txnObject) / 2;
+	int nStart = Math.min (Math.max (Math.min (start, end), 0), length);
+	int nEnd = Math.min (Math.max (Math.max (start, end), 0), length);
+	OS.TXNSetSelection (txnObject, nStart, nEnd);
 	OS.TXNShowSelection (txnObject, false);
 }
 
 /**
- * Sets the selection.
+ * Sets the selection to the range specified
+ * by the given point, where the x coordinate
+ * represents the start index and the y coordinate
+ * represents the end index.
  * <p>
  * Indexing is zero based.  The range of
  * a selection is from 0..N where N is
@@ -1370,7 +1499,7 @@ public void setTabs (int tabs) {
  * SINGLE and the argument contains multiple lines of text, the result of this
  * operation is undefined and may vary from platform to platform.
  *
- * @param text the new text
+ * @param string the new text
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the string is null</li>
@@ -1384,7 +1513,7 @@ public void setText (String string) {
 	checkWidget();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (hooks (SWT.Verify) || filters (SWT.Verify)) {
-		string = verifyText (string, 0, getCharCount ());
+		string = verifyText (string, 0, getCharCount (), null);
 		if (string == null) return;
 	}
 	setTXNText (OS.kTXNStartOffset, OS.kTXNEndOffset, string);
@@ -1394,6 +1523,9 @@ public void setText (String string) {
 }
 
 void setTXNBounds () {
+	Rect viewRect = new Rect ();
+	OS.TXNGetViewRect (txnObject, viewRect);
+
 	Rect rect = new Rect ();
 	OS.GetControlBounds (handle, rect);
 	Rect inset = inset ();
@@ -1415,21 +1547,19 @@ void setTXNBounds () {
 	* scroll the widget back.  The fix is to save the 
 	* current selection, set the selection to the start
 	* of the text and then restore the selection.  This
-	* will cause the widget text widget to recompute the
-	* left scroll position.
-	* 
-	* NOTE: Currently, this work around is only applied
-	* to text widgets that are not visible.  If the widget
-	* is resized when it is visible, this is fine because
-	* the user has already seen that the text is scrolled.
+	* will cause the widget to recompute the left scroll
+	* position.
 	*/
-	if (OS.IsControlVisible (handle)) return;
-	int [] oStartOffset = new int [1], oEndOffset = new int [1];
-	OS.TXNGetSelection (txnObject, oStartOffset, oEndOffset);
-	OS.TXNSetSelection (txnObject, OS.kTXNStartOffset, OS.kTXNStartOffset);
-	OS.TXNShowSelection (txnObject, false);
-	OS.TXNSetSelection (txnObject, oStartOffset [0], oEndOffset [0]);
-	OS.TXNShowSelection (txnObject, false);
+	int width = viewRect.left - viewRect.right;
+	int height = viewRect.bottom - viewRect.top;
+	if (width <= (inset.left + inset.right) && height <= (inset.top + inset.bottom)) {
+		int [] oStartOffset = new int [1], oEndOffset = new int [1];
+		OS.TXNGetSelection (txnObject, oStartOffset, oEndOffset);
+		OS.TXNSetSelection (txnObject, OS.kTXNStartOffset, OS.kTXNStartOffset);
+		OS.TXNShowSelection (txnObject, false);
+		OS.TXNSetSelection (txnObject, oStartOffset [0], oEndOffset [0]);
+		OS.TXNShowSelection (txnObject, false);
+	}
 }
 
 void setTXNText (int iStartOffset, int iEndOffset, String string) {
@@ -1440,6 +1570,12 @@ void setTXNText (int iStartOffset, int iEndOffset, String string) {
 	if (readOnly) OS.TXNSetTXNObjectControls (txnObject, false, 1, tag, new int [] {0});
 	OS.TXNSetData (txnObject, OS.kTXNUnicodeTextData, buffer, buffer.length * 2, iStartOffset, iEndOffset);
 	if (readOnly) OS.TXNSetTXNObjectControls (txnObject, false, 1, tag, new int [] {1});
+
+	/*
+	* Feature in the Macintosh.  When an empty string is set in the TXNObject,
+	* the font attributes are cleared.  The fix is to reset them.
+	*/
+	if (OS.TXNDataSize (txnObject) / 2 == 0) setFontStyle (font);
 }
 
 /**
@@ -1450,6 +1586,8 @@ void setTXNText (int iStartOffset, int iEndOffset, String string) {
  * creating a read-only text widget.
  * </p><p>
  * To reset this value to the default, use <code>setTextLimit(Text.LIMIT)</code>.
+ * Specifying a limit value larger than <code>Text.LIMIT</code> sets the
+ * receiver's limit to <code>Text.LIMIT</code>.
  * </p>
  *
  * @param limit new text limit
@@ -1461,6 +1599,8 @@ void setTXNText (int iStartOffset, int iEndOffset, String string) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see #LIMIT
  */
 public void setTextLimit (int limit) {
 	checkWidget();
@@ -1536,11 +1676,16 @@ int traversalCode (int key, int theEvent) {
 	return bits;
 }
 
-String verifyText (String string, int start, int end) {
+String verifyText (String string, int start, int end, Event keyEvent) {
 	Event event = new Event ();
 	event.text = string;
 	event.start = start;
 	event.end = end;
+	if (keyEvent != null) {
+		event.character = keyEvent.character;
+		event.keyCode = keyEvent.keyCode;
+		event.stateMask = keyEvent.stateMask;
+	}
 	/*
 	 * It is possible (but unlikely), that application
 	 * code could have disposed the widget in the verify
