@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -12,6 +12,7 @@ package org.eclipse.swt.widgets;
 
 
 import org.eclipse.swt.internal.*;
+import org.eclipse.swt.internal.carbon.CGRect;
 import org.eclipse.swt.internal.carbon.OS;
 import org.eclipse.swt.internal.carbon.RGBColor;
 import org.eclipse.swt.internal.carbon.Rect;
@@ -55,26 +56,21 @@ public abstract class Widget {
 	Object data;
 
 	/* Global state flags */
-//	static final int AUTOMATIC		= 1 << 0;
-//	static final int ACTIVE			= 1 << 1;
-	static final int GRAB			= 1 << 2;
-//	static final int MULTIEXPOSE	= 1 << 3;
-//	static final int RESIZEREDRAW	= 1 << 4;
-//	static final int WRAP			= 1 << 5;
-	static final int DISABLED		= 1 << 6;
-	static final int HIDDEN			= 1 << 7;
-//	static final int FOREGROUND		= 1 << 8;
-//	static final int BACKGROUND		= 1 << 9;
-	static final int DISPOSED		= 1 << 10;
-//	static final int HANDLE			= 1 << 11;
-	static final int CANVAS			= 1 << 12;
-	static final int MOVED			= 1 << 13;
-	static final int RESIZED		= 1 << 14;
-	static final int KEYED_DATA		= 1 << 15;
+	static final int DISPOSED         = 1 << 0;
+	static final int CANVAS           = 1 << 1;
+	static final int KEYED_DATA       = 1 << 2;
+	static final int DISABLED         = 1 << 3;
+	static final int HIDDEN           = 1 << 4;
+	static final int GRAB	             = 1 << 5;
+	static final int MOVED            = 1 << 6;
+	static final int RESIZED          = 1 << 7;
+	static final int EXPANDING        = 1 << 8;
+	static final int LAYOUT_NEEDED    = 1 << 9;
+	static final int LAYOUT_CHANGED   = 1 << 10;
+	static final int IGNORE_WHEEL     = 1 << 11;
 
 	static final int DEFAULT_WIDTH	= 64;
 	static final int DEFAULT_HEIGHT	= 64;
-	static final char Mnemonic = '&';
 	
 	static final Rect EMPTY_RECT = new Rect ();
 
@@ -100,6 +96,7 @@ Widget () {
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if the parent is disposed</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -148,6 +145,10 @@ public void addListener (int eventType, Listener handler) {
 	eventTable.hook (eventType, handler);
 }
 
+int callPaintEventHandler (int control, int damageRgn, int visibleRgn, int theEvent, int nextHandler) {
+	return OS.CallNextEventHandler (nextHandler, theEvent);
+}
+
 /**
  * Adds the listener to the collection of listeners who will
  * be notifed when the widget is disposed. When the widget is
@@ -224,6 +225,10 @@ void calculateVisibleRegion (int control, int visibleRgn, boolean clipChildren)
 	OS.DisposeRgn (tempRgn);
 }
 
+void checkOpen () {
+	/* Do nothing */
+}
+
 void checkOrientation (Widget parent) {
 	style &= ~SWT.MIRRORED;
 	if ((style & (SWT.LEFT_TO_RIGHT | SWT.RIGHT_TO_LEFT)) == 0) {
@@ -237,7 +242,9 @@ void checkOrientation (Widget parent) {
 
 void checkParent (Widget parent) {
 	if (parent == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (parent.isDisposed ()) error (SWT.ERROR_INVALID_ARGUMENT);
 	parent.checkWidget ();
+	parent.checkOpen ();
 }
 
 /**
@@ -320,6 +327,7 @@ int controlProc (int nextHandler, int theEvent, int userData) {
 		case OS.kEventControlSetCursor:				return kEventControlSetCursor (nextHandler, theEvent, userData);
 		case OS.kEventControlSetFocusPart:			return kEventControlSetFocusPart (nextHandler, theEvent, userData);
 		case OS.kEventControlTrack:					return kEventControlTrack (nextHandler, theEvent, userData);
+		case OS.kEventControlGetFocusPart:			return kEventControlGetFocusPart (nextHandler, theEvent, userData); 
 	}
 	return OS.eventNotHandledErr;
 }
@@ -405,6 +413,117 @@ int createCIcon (Image image) {
 void createHandle () {
 }
 
+int createIconRef (Image image) {
+	int imageHandle = image.handle;
+	int imageData = image.data;
+	int width = OS.CGImageGetWidth(imageHandle);
+	int height = OS.CGImageGetHeight(imageHandle);
+	int bpr = OS.CGImageGetBytesPerRow(imageHandle);
+	int alphaInfo = OS.CGImageGetAlphaInfo(imageHandle);
+	
+	int type = 0, maskType = 0;
+	if (width == 16 && height == 16) {
+		type = OS.kSmall32BitData;
+		maskType = OS.kSmall8BitMask;
+	} else if (width == 32 && height == 32) {
+		type = OS.kLarge32BitData;
+		maskType = OS.kLarge8BitMask;
+	} else if (width == 48 && height == 48) {
+		type = OS.kHuge32BitData;
+		maskType = OS.kHuge8BitMask;
+	} else if (width == 128 && height == 128) {
+		type = OS.kThumbnail32BitData;
+		maskType = OS.kThumbnail8BitMask;
+	} else {
+		type = OS.kSmall32BitData;
+		maskType = OS.kSmall8BitMask;
+		int size = 16;
+		if (width > 16 || height > 16) {
+			type = OS.kHuge32BitData;
+			maskType = OS.kHuge8BitMask;
+			size = 32;
+		}
+		if (width > 32 || height > 32) {
+			type = OS.kHuge32BitData;
+			maskType = OS.kHuge8BitMask;
+			size = 48;			
+		}
+		if (width > 48 || height > 48) {
+			type = OS.kThumbnail32BitData;
+			maskType = OS.kThumbnail8BitMask;
+			size = 128;
+		}
+		width = height = size;
+		bpr = width * 4;
+		int dataSize = height * bpr;
+		imageData = OS.NewPtr(dataSize);
+		if (imageData == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+		int colorspace = OS.CGColorSpaceCreateDeviceRGB();
+		if (colorspace == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+		int context = OS.CGBitmapContextCreate(imageData, width, height, 8, bpr, colorspace, OS.kCGImageAlphaNoneSkipFirst);
+		OS.CGColorSpaceRelease(colorspace);
+		if (context == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+		CGRect rect = new CGRect();
+		rect.width = width;
+		rect.height = height;
+		OS.CGContextDrawImage(context, rect, imageHandle);
+		OS.CGContextRelease(context);		
+	}
+	if (type == 0) return 0;
+	
+	int iconFamily = OS.NewHandle(0);
+	if (iconFamily == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+	
+	int dataSize = height * bpr;
+	int dataHandle = OS.NewHandle(dataSize);
+	if (dataHandle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+	int[] dataPtr = new int[1];
+	OS.HLock(dataHandle);
+	OS.memcpy(dataPtr, dataHandle, 4);
+	OS.memcpy(dataPtr[0], imageData, dataSize);
+	OS.HUnlock(dataHandle);
+	OS.SetIconFamilyData(iconFamily, type, dataHandle);
+	OS.DisposeHandle(dataHandle);
+
+	/* Initialize the mask data */
+	int maskSize = width * height;
+	int maskHandle = OS.NewHandle (maskSize);	
+	if (maskHandle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+	OS.HLock(maskHandle);
+	int[] maskPtr = new int[1];
+	OS.memcpy(maskPtr, maskHandle, 4);
+	if (alphaInfo != OS.kCGImageAlphaFirst) {
+		OS.memset(maskPtr[0], 0xFF, maskSize);
+	} else {
+		byte[] srcData = new byte[dataSize];
+		OS.memcpy(srcData, imageData, dataSize);
+		byte[] maskData = new byte[maskSize];
+		int offset = 0, maskOffset = 0;
+		for (int y = 0; y<height; y++) {
+			for (int x = 0; x<width; x++) {
+				maskData[maskOffset++] = srcData[offset];
+				offset += 4;
+			}
+		}
+		OS.memcpy(maskPtr[0], maskData, maskData.length);
+	}
+	OS.HUnlock(maskHandle);
+	OS.SetIconFamilyData(iconFamily, maskType, maskHandle);
+	OS.DisposeHandle(maskHandle);
+	
+	if (imageData != image.data) OS.DisposePtr(imageData);
+
+	/* Create the icon ref */
+	int[] iconRef = new int[1];
+	OS.HLock(iconFamily);
+	int[] iconPtr = new int[1];
+	OS.memcpy(iconPtr, iconFamily, 4);
+	OS.GetIconRefFromIconFamilyPtr(iconPtr[0], OS.GetHandleSize(iconFamily), iconRef);
+	OS.HUnlock(iconFamily);	
+	OS.DisposeHandle(iconFamily);
+	return iconRef[0];
+}
+
 void createWidget () {
 	createHandle ();
 	register ();
@@ -510,12 +629,22 @@ void drawFocus (int control, boolean hasFocus, boolean hasBorder, float[] backgr
 	rect.top += inset.top;
 	rect.right -= inset.right;
 	rect.bottom -= inset.bottom;
-	int state = OS.IsControlActive (control) ? OS.kThemeStateActive : OS.kThemeStateInactive;
+	int state;
+	if (OS.IsControlEnabled (control)) {
+		state = OS.IsControlActive (control) ? OS.kThemeStateActive : OS.kThemeStateInactive;
+	} else {
+		state = OS.IsControlActive (control) ? OS.kThemeStateUnavailable : OS.kThemeStateUnavailableInactive;
+	}
 	if (hasFocus) {
 		if (hasBorder) OS.DrawThemeEditTextFrame (rect, state);
 		OS.DrawThemeFocusRect (rect, true);
 	} else {
-		OS.DrawThemeFocusRect (rect, false);
+		/*
+		* This code is intentionaly commented.
+		*  
+		* NOTE: the focus ring is erased by drawBackground() above. 
+		*/
+//		OS.DrawThemeFocusRect (rect, false);
 		if (hasBorder) OS.DrawThemeEditTextFrame (rect, state);
 	}
 }
@@ -549,6 +678,18 @@ boolean filters (int eventType) {
 	return display.filters (eventType);
 }
 
+int fixMnemonic (char [] buffer) {
+	int i=0, j=0;
+	while (i < buffer.length) {
+		if ((buffer [j++] = buffer [i++]) == '&') {
+			if (i == buffer.length) {continue;}
+			if (buffer [i] == '&') {i++; continue;}
+			j--;
+		}
+	}
+	return j;
+}
+
 Rect getControlBounds (int control) {
 	Rect rect = new Rect();
 	OS.GetControlBounds (control, rect);
@@ -601,7 +742,7 @@ Rect getControlSize (int control) {
  *    <li>ERROR_THREAD_INVALID_ACCESS - when called from the wrong thread</li>
  * </ul>
  *
- * @see #setData
+ * @see #setData(Object)
  */
 public Object getData () {
 	checkWidget();
@@ -630,7 +771,7 @@ public Object getData () {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
- * @see #setData
+ * @see #setData(String, Object)
  */
 public Object getData (String key) {
 	checkWidget();
@@ -657,7 +798,6 @@ public Object getData (String key) {
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
- *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  */
 public Display getDisplay () {
@@ -676,9 +816,9 @@ Rect getInset () {
 
 String getName () {
 	String string = getClass ().getName ();
-	int index = string.lastIndexOf(".");
+	int index = string.lastIndexOf ('.');
 	if (index == -1) return string;
-	return string.substring(index + 1, string.length ());
+	return string.substring (index + 1, string.length ());
 }
 
 String getNameText () {
@@ -736,6 +876,10 @@ boolean hooks (int eventType) {
 void invalidateVisibleRegion (int control) {
 }
 
+void invalWindowRgn (int window, int rgn) {
+	OS.InvalWindowRgn (window, rgn);
+}
+
 /**
  * Returns <code>true</code> if the widget has been disposed,
  * and <code>false</code> otherwise.
@@ -772,7 +916,7 @@ boolean isEnabled () {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  */
-protected boolean isListening (int eventType) {
+public boolean isListening (int eventType) {
 	checkWidget();
 	return hooks (eventType);
 }
@@ -846,7 +990,7 @@ int kEventControlDraw (int nextHandler, int theEvent, int userData) {
 		OS.GetClip (oldClip);
 		OS.SetClip (visibleRgn);
 		drawBackground (theControl [0]);
-		OS.CallNextEventHandler (nextHandler, theEvent);
+		callPaintEventHandler (theControl [0], region [0], visibleRgn, theEvent, nextHandler);
 		drawWidget (theControl [0], region [0], visibleRgn, theEvent);
 		OS.SetClip (oldClip);
 		OS.DisposeRgn (oldClip);
@@ -856,6 +1000,10 @@ int kEventControlDraw (int nextHandler, int theEvent, int userData) {
 	return OS.noErr;
 }
 
+int kEventControlGetFocusPart (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
 int kEventControlHit (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
@@ -872,10 +1020,34 @@ int kEventControlTrack (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
+int kEventMenuCalculateSize (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
 int kEventMenuClosed (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
+int kEventMenuCreateFrameView (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
+int kEventMenuDrawItem (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
+int kEventMenuDrawItemContent (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
+int kEventMenuGetFrameBounds (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
+int kEventMenuMeasureItemWidth (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
 int kEventMenuOpening (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
@@ -904,19 +1076,56 @@ int kEventMouseWheelMoved (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
-int kEventRawKeyUp (int nextHandler, int theEvent, int userData) {
+int kEventRawKeyDown (int nextHandler, int theEvent, int userData) {
+	return kEventRawKeyPressed (nextHandler, theEvent, userData);
+}
+
+int kEventRawKeyModifiersChanged (int nextHandler, int theEvent, int userData) {
+	Display display = this.display;
+	int [] modifiers = new int [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamKeyModifiers, OS.typeUInt32, null, modifiers.length * 4, null, modifiers);
+	int lastModifiers = display.lastModifiers;
+	int chord = OS.GetCurrentEventButtonState ();
+	int type = SWT.KeyUp;
+	if ((modifiers [0] & OS.alphaLock) != 0 && (lastModifiers & OS.alphaLock) == 0) type = SWT.KeyDown;
+	if ((modifiers [0] & OS.shiftKey) != 0 && (lastModifiers & OS.shiftKey) == 0) type = SWT.KeyDown;
+	if ((modifiers [0] & OS.controlKey) != 0 && (lastModifiers & OS.controlKey) == 0) type = SWT.KeyDown;
+	if ((modifiers [0] & OS.cmdKey) != 0 && (lastModifiers & OS.cmdKey) == 0) type = SWT.KeyDown;
+	if ((modifiers [0] & OS.optionKey) != 0 && (lastModifiers & OS.optionKey) == 0) type = SWT.KeyDown;
+	if (type == SWT.KeyUp && (modifiers [0] & OS.alphaLock) == 0 && (lastModifiers & OS.alphaLock) != 0) {
+		Event event = new Event ();
+		event.keyCode = SWT.CAPS_LOCK;
+		setInputState (event, SWT.KeyDown, chord, modifiers [0]);
+		sendKeyEvent (SWT.KeyDown, event);
+	}
+	Event event = new Event ();
+	setInputState (event, type, chord, modifiers [0]);
+	if (event.keyCode == 0 && event.character == 0) return OS.eventNotHandledErr;
+	boolean result = sendKeyEvent (type, event);
+	if (type == SWT.KeyDown && (modifiers [0] & OS.alphaLock) != 0 && (lastModifiers & OS.alphaLock) == 0) {
+		event = new Event ();
+		event.keyCode = SWT.CAPS_LOCK;
+		setInputState (event, SWT.KeyUp, chord, modifiers [0]);
+		sendKeyEvent (SWT.KeyUp, event);
+	}
+	display.lastModifiers = modifiers [0];
+	return result ? OS.eventNotHandledErr : OS.noErr;
+}
+
+int kEventRawKeyPressed (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
 int kEventRawKeyRepeat (int nextHandler, int theEvent, int userData) {
+	return kEventRawKeyPressed (nextHandler, theEvent, userData);
+}
+
+int kEventRawKeyUp (int nextHandler, int theEvent, int userData) {
+	if (!sendKeyEvent (SWT.KeyUp, theEvent)) return OS.noErr;
 	return OS.eventNotHandledErr;
 }
 
-int kEventRawKeyModifiersChanged (int nextHandler, int theEvent, int userData) {
-	return OS.eventNotHandledErr;
-}
-
-int kEventRawKeyDown (int nextHandler, int theEvent, int userData) {
+int kEventTextInputUnicodeForKeyEvent (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
@@ -940,18 +1149,34 @@ int kEventWindowDeactivated (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
+int kEventWindowDrawContent (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
 int kEventWindowExpanded (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
+int kEventWindowGetRegion (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
 int kEventWindowHidden (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
+int kEventWindowHitTest (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
 int kEventWindowShown (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
+int kEventWindowUpdate (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
 int keyboardProc (int nextHandler, int theEvent, int userData) {
 	int eventKind = OS.GetEventKind (theEvent);
 	switch (eventKind) {
@@ -966,9 +1191,15 @@ int keyboardProc (int nextHandler, int theEvent, int userData) {
 int menuProc (int nextHandler, int theEvent, int userData) {	
 	int eventKind = OS.GetEventKind (theEvent);
 	switch (eventKind) {
-		case OS.kEventMenuClosed:		return kEventMenuClosed (nextHandler, theEvent, userData);
-		case OS.kEventMenuOpening:		return kEventMenuOpening (nextHandler, theEvent, userData);
-		case OS.kEventMenuTargetItem:	return kEventMenuTargetItem (nextHandler, theEvent, userData);
+		case OS.kEventMenuCalculateSize:		return kEventMenuCalculateSize (nextHandler, theEvent, userData);
+		case OS.kEventMenuClosed:				return kEventMenuClosed (nextHandler, theEvent, userData);
+		case OS.kEventMenuCreateFrameView:	return kEventMenuCreateFrameView (nextHandler, theEvent, userData);
+		case OS.kEventMenuDrawItem: 			return kEventMenuDrawItem (nextHandler, theEvent, userData);
+		case OS.kEventMenuDrawItemContent: 	return kEventMenuDrawItemContent (nextHandler, theEvent, userData);
+		case OS.kEventMenuGetFrameBounds: 	return kEventMenuGetFrameBounds (nextHandler, theEvent, userData);
+		case OS.kEventMenuMeasureItemWidth: 	return kEventMenuMeasureItemWidth (nextHandler, theEvent, userData);
+		case OS.kEventMenuOpening:			return kEventMenuOpening (nextHandler, theEvent, userData);
+		case OS.kEventMenuTargetItem:			return kEventMenuTargetItem (nextHandler, theEvent, userData);
 	}
 	return OS.eventNotHandledErr;
 }
@@ -995,9 +1226,6 @@ int mouseProc (int nextHandler, int theEvent, int userData) {
  * @param eventType the type of event which has occurred
  * @param event the event data
  *
- * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the event is null</li>
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -1005,11 +1233,8 @@ int mouseProc (int nextHandler, int theEvent, int userData) {
  */
 public void notifyListeners (int eventType, Event event) {
 	checkWidget();
-	if (event == null) error (SWT.ERROR_NULL_ARGUMENT);
-	if (eventTable == null) return;
-	event.type = eventType;
-	event.widget = this;
-	eventTable.sendEvent (event);
+	if (event == null) event = new Event ();
+	sendEvent (eventType, event);
 }
 
 void postEvent (int eventType) {
@@ -1024,7 +1249,7 @@ void redrawWidget (int control, boolean children) {
 	if (!isDrawing (control)) return;
 	int window = OS.GetControlOwner (control);
 	int visibleRgn = getVisibleRegion (control, !children);
-	OS.InvalWindowRgn (window, visibleRgn);
+	invalWindowRgn (window, visibleRgn);
 	OS.DisposeRgn (visibleRgn);
 }
 
@@ -1040,7 +1265,7 @@ void redrawWidget (int control, int x, int y, int width, int height, boolean chi
 	int visibleRgn = getVisibleRegion (control, !children);
 	OS.SectRgn (rectRgn, visibleRgn, visibleRgn);
 	int window = OS.GetControlOwner (control);
-	OS.InvalWindowRgn (window, visibleRgn);
+	invalWindowRgn (window, visibleRgn);
 	OS.DisposeRgn (rectRgn);
 	OS.DisposeRgn (visibleRgn);
 }
@@ -1182,60 +1407,119 @@ void sendEvent (int eventType, Event event, boolean send) {
 	}
 }
 
+
+boolean sendKeyEvent (int type, int theEvent) {
+	int [] length = new int [1];
+	int status = OS.GetEventParameter (theEvent, OS.kEventParamKeyUnicodes, OS.typeUnicodeText, null, 4, length, (char[])null);
+	if (status == OS.noErr && length [0] > 2) {
+		int count = 0;
+		int [] chord = new int [1];
+		OS.GetEventParameter (theEvent, OS.kEventParamMouseChord, OS.typeUInt32, null, 4, null, chord);
+		int [] modifiers = new int [1];
+		OS.GetEventParameter (theEvent, OS.kEventParamKeyModifiers, OS.typeUInt32, null, 4, null, modifiers);
+		char [] chars = new char [length [0] / 2];
+		OS.GetEventParameter (theEvent, OS.kEventParamKeyUnicodes, OS.typeUnicodeText, null, chars.length * 2, null, chars);
+		for (int i=0; i<chars.length; i++) {
+			Event event = new Event ();
+			event.character = chars [i];
+			setInputState (event, type, chord [0], modifiers [0]);
+			if (sendKeyEvent (type, event)) chars [count++] = chars [i];
+		}
+		if (count == 0) return false;
+		if (count != chars.length - 1) {
+			OS.SetEventParameter (theEvent, OS.kEventParamKeyUnicodes, OS.typeUnicodeText, count * 2, chars);
+		}
+		return true;
+	} else {
+		Event event = new Event ();
+		if (!setKeyState (event, type, theEvent)) return true;
+		return sendKeyEvent (type, event);
+	}
+}
+
+boolean sendKeyEvent (int type, Event event) {
+	sendEvent (type, event);
+	// widget could be disposed at this point
+	
+	/*
+	* It is possible (but unlikely), that application
+	* code could have disposed the widget in the key
+	* events.  If this happens, end the processing of
+	* the key by returning false.
+	*/
+	if (isDisposed ()) return false;
+	return event.doit;
+}
+
 int setBounds (int control, int x, int y, int width, int height, boolean move, boolean resize, boolean events) {
-	Rect inset = getInset ();
+	/* Compute the old bounds */
 	Rect oldBounds = new Rect ();
 	OS.GetControlBounds (control, oldBounds);
+	int [] theRoot = new int [1];
+	int window = OS.GetControlOwner (control);
+	OS.GetRootControl (window, theRoot);
+	int [] parentHandle = new int [1];
+	OS.GetSuperControl (control, parentHandle);
+	Rect parentRect = new Rect ();
+	if (parentHandle [0] != theRoot [0]) {
+		OS.GetControlBounds (parentHandle [0], parentRect);
+		OS.OffsetRect (oldBounds, (short) -parentRect.left, (short) -parentRect.top);
+	}
+	Rect inset = getInset ();
 	oldBounds.left -= inset.left;
 	oldBounds.top -= inset.top;
 	oldBounds.right += inset.right;
 	oldBounds.bottom += inset.bottom;
-	x += inset.left;
-	y += inset.top;
-	width -= (inset.left + inset.right);
-	height -= (inset.top + inset.bottom);
-	int window = OS.GetControlOwner (control);
-	if (move) {
-		int [] theRoot = new int [1];
-		OS.GetRootControl (window, theRoot);
-		int [] parentHandle = new int [1];
-		OS.GetSuperControl (control, parentHandle);
-		if (parentHandle [0] != theRoot [0]) {
-			Rect rect = new Rect ();
-			OS.GetControlBounds (parentHandle [0], rect);
-			x += rect.left;
-			y += rect.top;
-		}
-	} else {
+	
+	/* Compute the new bounds */
+	if (!move) {
 		x = oldBounds.left;
 		y = oldBounds.top;
 	}
 	if (!resize) {
 		width = oldBounds.right - oldBounds.left;
 		height = oldBounds.bottom - oldBounds.top;
-	}
-	width = Math.max (0, width);
-	height = Math.max (0, height);
-	boolean sameOrigin = x == oldBounds.left && y == oldBounds.top;
-	boolean sameExtent = width == (oldBounds.right - oldBounds.left) && height == (oldBounds.bottom - oldBounds.top);
-	if (sameOrigin && sameExtent) return 0;
+	}	
 	Rect newBounds = new Rect ();
-	newBounds.left = (short) x;
-	newBounds.top = (short) y;
-	newBounds.right = (short) (x + width);
-	newBounds.bottom = (short) (y + height);
+	newBounds.left = (short) (parentRect.left + x + inset.left);
+	newBounds.top = (short) (parentRect.top + y + inset.top);
+	newBounds.right = (short) (newBounds.left + width - inset.right - inset.left);
+	newBounds.bottom = (short) (newBounds.top + height - inset.bottom - inset.top);	
+	if (newBounds.bottom < newBounds.top) newBounds.bottom = newBounds.top;
+	if (newBounds.right < newBounds.left) newBounds.right = newBounds.left;
+
+	/* Get bounds again, since the one above is in SWT coordinates */
+	OS.GetControlBounds (control, oldBounds);
+	
+	/* Check if anything changed */
+	boolean sameOrigin = newBounds.left == oldBounds.left && newBounds.top == oldBounds.top;
+	boolean sameExtent = (newBounds.right - newBounds.left) == (oldBounds.right - oldBounds.left) && (newBounds.bottom - newBounds.top) == (oldBounds.bottom - oldBounds.top);
+	if (sameOrigin && sameExtent) return 0;
+
+	/* Apply changes and invalidate appropriate rectangles */
+	int tempRgn = 0;
 	boolean visible = OS.IsControlVisible (control);
-	if (visible) OS.InvalWindowRect (window, oldBounds);
+	if (visible) {
+		tempRgn = OS.NewRgn ();
+		OS.GetControlRegion (control, (short) OS.kControlStructureMetaPart, tempRgn);
+		invalWindowRgn (window, tempRgn);
+	}
 	OS.SetControlBounds (control, newBounds);
 	invalidateVisibleRegion (control);
-	if (visible) OS.InvalWindowRect (window, newBounds);
+	if (visible) {
+		OS.GetControlRegion (control, (short) OS.kControlStructureMetaPart, tempRgn);
+		invalWindowRgn (window, tempRgn);
+		OS.DisposeRgn(tempRgn);
+	}
+	
+	/* Send events */
 	int result = 0;
 	if (move && !sameOrigin) {
 		if (events) sendEvent (SWT.Move);
 		result |= MOVED;
 	}
 	if (resize && !sameExtent) {
-		if (events) sendEvent (SWT.Move);
+		if (events) sendEvent (SWT.Resize);
 		result |= RESIZED;
 	}
 	return result;
@@ -1260,6 +1544,8 @@ int setBounds (int control, int x, int y, int width, int height, boolean move, b
  *    <li>ERROR_WIDGET_DISPOSED - when the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - when called from the wrong thread</li>
  * </ul>
+ * 
+ * @see #getData()
  */
 public void setData (Object data) {
 	checkWidget();
@@ -1292,7 +1578,7 @@ public void setData (Object data) {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
- * @see #getData
+ * @see #getData(String)
  */
 public void setData (String key, Object value) {
 	checkWidget();
@@ -1339,116 +1625,182 @@ public void setData (String key, Object value) {
 	}
 }
 
-void setInputState (Event event, int theEvent) {
-	short [] button = new short [1];
-	OS.GetEventParameter (theEvent, OS.kEventParamMouseButton, OS.typeMouseButton, null, 2, null, button);
-	int [] chord = new int [1];
-	OS.GetEventParameter (theEvent, OS.kEventParamMouseChord, OS.typeUInt32, null, 4, null, chord);
-	int [] modifiers = new int [1];
-	OS.GetEventParameter (theEvent, OS.kEventParamKeyModifiers, OS.typeUInt32, null, 4, null, modifiers);
-	setInputState (event, button [0], chord [0], modifiers [0]);
-}
-
-void setInputState (Event event, short button, int chord, int modifiers) {
-	switch (button) {
-		case 1: event.button = 1; break;
-		case 2: event.button = 3; break;
-		case 3: event.button = 2; break;
-	}
+boolean setInputState (Event event, int type, int chord, int modifiers) {
 	if ((chord & 0x01) != 0) event.stateMask |= SWT.BUTTON1;
 	if ((chord & 0x02) != 0) event.stateMask |= SWT.BUTTON3;
 	if ((chord & 0x04) != 0) event.stateMask |= SWT.BUTTON2;
+	if ((chord & 0x08) != 0) event.stateMask |= SWT.BUTTON4;
+	if ((chord & 0x10) != 0) event.stateMask |= SWT.BUTTON5;
+	
 	if ((modifiers & OS.optionKey) != 0) event.stateMask |= SWT.ALT;
 	if ((modifiers & OS.shiftKey) != 0) event.stateMask |= SWT.SHIFT;
 	if ((modifiers & OS.controlKey) != 0) event.stateMask |= SWT.CONTROL;
 	if ((modifiers & OS.cmdKey) != 0) event.stateMask |= SWT.COMMAND;
-	switch (event.type) {
+	switch (type) {
 		case SWT.MouseDown:
 		case SWT.MouseDoubleClick:
 			if (event.button == 1) event.stateMask &= ~SWT.BUTTON1;
 			if (event.button == 2) event.stateMask &= ~SWT.BUTTON2;
 			if (event.button == 3)  event.stateMask &= ~SWT.BUTTON3;
+			if (event.button == 4)  event.stateMask &= ~SWT.BUTTON4;
+			if (event.button == 5)  event.stateMask &= ~SWT.BUTTON5;
 			break;
 		case SWT.MouseUp:
 			if (event.button == 1) event.stateMask |= SWT.BUTTON1;
 			if (event.button == 2) event.stateMask |= SWT.BUTTON2;
 			if (event.button == 3) event.stateMask |= SWT.BUTTON3;
+			if (event.button == 4) event.stateMask |= SWT.BUTTON4;
+			if (event.button == 5) event.stateMask |= SWT.BUTTON5;
 			break;
 		case SWT.KeyDown:
 		case SWT.Traverse: {
-			if (event.keyCode != 0 || event.character != 0) return;
+			if (event.keyCode != 0 || event.character != 0) return true;
 			int lastModifiers = display.lastModifiers;
+			if ((modifiers & OS.alphaLock) != 0 && (lastModifiers & OS.alphaLock) == 0) {
+				event.keyCode = SWT.CAPS_LOCK;
+				return true;
+			}
 			if ((modifiers & OS.shiftKey) != 0 && (lastModifiers & OS.shiftKey) == 0) {
 				event.stateMask &= ~SWT.SHIFT;
 				event.keyCode = SWT.SHIFT;
-				return;
+				return true;
 			}
 			if ((modifiers & OS.controlKey) != 0 && (lastModifiers & OS.controlKey) == 0) {
 				event.stateMask &= ~SWT.CONTROL;
 				event.keyCode = SWT.CONTROL;
-				return;
+				return true;
 			}
 			if ((modifiers & OS.cmdKey) != 0 && (lastModifiers & OS.cmdKey) == 0) {
 				event.stateMask &= ~SWT.COMMAND;
 				event.keyCode = SWT.COMMAND;
-				return;
+				return true;
 			}	
 			if ((modifiers & OS.optionKey) != 0 && (lastModifiers & OS.optionKey) == 0) {
 				event.stateMask &= ~SWT.ALT;
 				event.keyCode = SWT.ALT;
-				return;
+				return true;
 			}
 			break;
 		}
 		case SWT.KeyUp: {
-			if (event.keyCode != 0 || event.character != 0) return;
+			if (event.keyCode != 0 || event.character != 0) return true;
 			int lastModifiers = display.lastModifiers;
+			if ((modifiers & OS.alphaLock) == 0 && (lastModifiers & OS.alphaLock) != 0) {
+				event.keyCode = SWT.CAPS_LOCK;
+				return true;
+			}
 			if ((modifiers & OS.shiftKey) == 0 && (lastModifiers & OS.shiftKey) != 0) {
 				event.stateMask |= SWT.SHIFT;
 				event.keyCode = SWT.SHIFT;
-				return;
+				return true;
 			}
 			if ((modifiers & OS.controlKey) == 0 && (lastModifiers & OS.controlKey) != 0) {
 				event.stateMask |= SWT.CONTROL;
 				event.keyCode = SWT.CONTROL;
-				return;
+				return true;
 			}
 			if ((modifiers & OS.cmdKey) == 0 && (lastModifiers & OS.cmdKey) != 0) {
 				event.stateMask |= SWT.COMMAND;
 				event.keyCode = SWT.COMMAND;
-				return;
-			}	
-			if ((modifiers & OS.optionKey) != 0 && (lastModifiers & OS.optionKey) == 0) {
+				return true;
+			}
+			if ((modifiers & OS.optionKey) == 0 && (lastModifiers & OS.optionKey) != 0) {
 				event.stateMask |= SWT.ALT;
 				event.keyCode = SWT.ALT;
-				return;
+				return true;
 			}
 			break;
 		}
 	}
+	return true; 
 }
 
-void setKeyState (Event event, int theEvent) {
+boolean setKeyState (Event event, int type, int theEvent) {
+	boolean isNull = false;
 	int [] keyCode = new int [1];
 	OS.GetEventParameter (theEvent, OS.kEventParamKeyCode, OS.typeUInt32, null, keyCode.length * 4, null, keyCode);
 	event.keyCode = Display.translateKey (keyCode [0]);
 	switch (event.keyCode) {
-		case 0:
-		case SWT.BS:
-		case SWT.CR:
-		case SWT.DEL:
-		case SWT.ESC:
-		case SWT.TAB: {
-			byte [] charCode = new byte [1];
-			OS.GetEventParameter (theEvent, OS.kEventParamKeyMacCharCodes, OS.typeChar, null, charCode.length, null, charCode);
-			event.character = (char) charCode [0];
+		case SWT.LF: {
+			/*
+			* Feature in the Macintosh.  When the numeric key pad
+			* Enter key is pressed, it generates '\n'.  This is the
+			* correct platform behavior but is not portable.  The
+			* fix is to convert the '\n' into '\r'.
+			*/
+			event.keyCode = SWT.KEYPAD_CR;
+			event.character = '\r';
 			break;
 		}
-		case SWT.LF:
-			event.character = '\n';
+		case SWT.BS: event.character = '\b'; break;
+		case SWT.CR: event.character = '\r'; break;
+		case SWT.DEL: event.character = 0x7F; break;
+		case SWT.ESC: event.character = 0x1B; break;
+		case SWT.TAB: event.character = '\t'; break;
+		default: {
+			if (event.keyCode == 0 || (SWT.KEYPAD_MULTIPLY <= event.keyCode && event.keyCode <= SWT.KEYPAD_CR)) {
+				int [] length = new int [1];
+				int status = OS.GetEventParameter (theEvent, OS.kEventParamKeyUnicodes, OS.typeUnicodeText, null, 4, length, (char[])null);
+				if (status == OS.noErr && length [0] != 0) {
+					char [] chars = new char [1];
+					OS.GetEventParameter (theEvent, OS.kEventParamKeyUnicodes, OS.typeUnicodeText, null, 2, null, chars);
+					event.character = chars [0];
+				}
+				/*
+				* Bug in the Mactonish.  For some reason, Ctrl+Shift+'2' and Ctrl+Shift+'6'
+				* fail to give 0x0 (^@ or ASCII NUL) and 0x1e (^^).  Other control character
+				* key sequences such as ^A or even Ctrl+Shift+'-' (^_ or 0x1f) are correctly
+				* translated to control characters.  Since it is not possible to know which
+				* key combination gives '@' on an international keyboard, there is no way to
+				* test for either character and convert it to a control character (Shift+'2'
+				* gives '@' only on an English keyboard) to work around the problem.
+				*
+				* There is no fix at this time.
+				*/
+			}
+			if (event.keyCode == 0) {
+				int kchrPtr = OS.GetScriptManagerVariable ((short) OS.smKCHRCache);
+				if (display.kchrPtr != kchrPtr) {
+					display.kchrPtr = kchrPtr;
+					display.kchrState [0] = 0;
+				}
+				int result = OS.KeyTranslate (display.kchrPtr, (short)keyCode [0], display.kchrState);
+				if (result <= 0x7f) {
+					event.keyCode = result & 0x7f;
+				} else {
+					int [] encoding = new int [1];
+					short keyScript = (short) OS.GetScriptManagerVariable ((short) OS.smKeyScript);
+					short regionCode = (short) OS.GetScriptManagerVariable ((short) OS.smRegionCode);
+					if (OS.UpgradeScriptInfoToTextEncoding (keyScript, (short) OS.kTextLanguageDontCare, regionCode, null, encoding) == OS.paramErr) {
+						if (OS.UpgradeScriptInfoToTextEncoding (keyScript, (short) OS.kTextLanguageDontCare, (short) OS.kTextRegionDontCare, null, encoding) == OS.paramErr) {
+							encoding [0] = OS.kTextEncodingMacRoman;
+						}
+					}
+					int [] encodingInfo = new int [1];
+					OS.CreateTextToUnicodeInfoByEncoding (encoding [0], encodingInfo);
+					if (encodingInfo [0] != 0) {
+						char [] chars = new char [1];
+						int [] nchars = new int [1];
+						byte [] buffer = new byte [2];
+						buffer [0] = 1;
+						buffer [1] = (byte) (result & 0xFF);
+						OS.ConvertFromPStringToUnicode (encodingInfo [0], buffer, chars.length * 2, nchars, chars);
+						OS.DisposeTextToUnicodeInfo (encodingInfo);
+						event.keyCode = chars [0];
+					}
+				}
+			}
 			break;
+		}
 	}
-	setInputState (event, theEvent);
+	if (event.keyCode == 0 && event.character == 0) {
+		if (!isNull) return false;
+	}
+	int [] chord = new int [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamMouseChord, OS.typeUInt32, null, 4, null, chord);
+	int [] modifiers = new int [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamKeyModifiers, OS.typeUInt32, null, 4, null, modifiers);
+	return setInputState (event, type, chord [0], modifiers [0]);
 }
 
 void setVisible (int control, boolean visible) {
@@ -1460,7 +1812,7 @@ void setVisible (int control, boolean visible) {
 	if (drawing && visible) visibleRgn = getVisibleRegion (control, false);
 	if (drawing) {
 		int window = OS.GetControlOwner (control);
-		OS.InvalWindowRgn (window, visibleRgn);
+		invalWindowRgn (window, visibleRgn);
 		OS.DisposeRgn (visibleRgn);
 	}
 }
@@ -1480,12 +1832,20 @@ void setZOrder (int control, int otheControl, boolean above) {
 			OS.DiffRgn (oldRgn, newRgn, newRgn);
 		}
 		int window = OS.GetControlOwner (control);
-		OS.InvalWindowRgn (window, newRgn);
+		invalWindowRgn (window, newRgn);
 		OS.DisposeRgn (oldRgn);
 		OS.DisposeRgn (newRgn);
 	}
 }
 
+int textInputProc (int nextHandler, int theEvent, int userData) {
+	int eventKind = OS.GetEventKind (theEvent);
+	switch (eventKind) {
+		case OS.kEventTextInputUnicodeForKeyEvent: return kEventTextInputUnicodeForKeyEvent (nextHandler, theEvent, userData);
+	}
+	return OS.eventNotHandledErr;
+}
+
 RGBColor toRGBColor (float [] color) {
 	RGBColor rgb = new RGBColor ();
 	rgb.red = (short) (color [0] * 0xffff);
@@ -1522,9 +1882,13 @@ int windowProc (int nextHandler, int theEvent, int userData) {
 		case OS.kEventWindowClose:				return kEventWindowClose (nextHandler, theEvent, userData);
 		case OS.kEventWindowCollapsed:			return kEventWindowCollapsed (nextHandler, theEvent, userData);
 		case OS.kEventWindowDeactivated:		return kEventWindowDeactivated (nextHandler, theEvent, userData);
+		case OS.kEventWindowDrawContent:		return kEventWindowDrawContent (nextHandler, theEvent, userData);
 		case OS.kEventWindowExpanded:			return kEventWindowExpanded (nextHandler, theEvent, userData);
-		case OS.kEventWindowHidden:			return kEventWindowHidden (nextHandler, theEvent, userData);
+		case OS.kEventWindowGetRegion:			return kEventWindowGetRegion (nextHandler, theEvent, userData);
+		case OS.kEventWindowHidden:				return kEventWindowHidden (nextHandler, theEvent, userData);
+		case OS.kEventWindowHitTest:			return kEventWindowHitTest (nextHandler, theEvent, userData);
 		case OS.kEventWindowShown:				return kEventWindowShown (nextHandler, theEvent, userData);
+		case OS.kEventWindowUpdate:				return kEventWindowUpdate (nextHandler, theEvent, userData);
 	}
 	return OS.eventNotHandledErr;
 }
