@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -46,29 +46,36 @@ public abstract class Widget {
 	/**
 	 * the handle to the OS resource 
 	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
 	 */
-	public int handle;
-	
+	public int /*long*/ handle;
 	int style, state;
+	Display display;
 	EventTable eventTable;
 	Object data;
-	String [] keys;
-	Object [] values;
 	
 	/* Global state flags */
-//	static final int AUTOMATIC		= 1<<0;
-//	static final int ACTIVE			= 1<<1;
-//	static final int AUTOGRAB		= 1<<2;
-//	static final int MULTIEXPOSE	= 1<<3;
-//	static final int RESIZEREDRAW	= 1<<4;
-//	static final int WRAP			= 1<<5;
-	static final int DISABLED		= 1<<6;
-//	static final int HIDDEN			= 1<<7;
-//	static final int FOREGROUND		= 1<<8;
-//	static final int BACKGROUND		= 1<<9;
-	static final int DISPOSED		= 1<<10;
-	static final int HANDLE			= 1<<11;
-	static final int CANVAS			= 1<<12;
+	static final int DISPOSED                = 1<<0;
+	static final int CANVAS                  = 1<<1;
+	static final int KEYED_DATA           = 1<<2;
+	static final int HANDLE                   = 1<<3;
+	static final int DISABLED                = 1<<4;
+	static final int MENU                      = 1<<5;
+	static final int OBSCURED              = 1<<6;
+	static final int MOVED                   = 1<<7;
+	static final int RESIZED                  = 1<<8;
+	static final int LAYOUT_NEEDED    = 1<<9;
+	static final int LAYOUT_CHANGED = 1<<10;
+	static final int ZERO_SIZED = 1<<11;
+	static final int HIDDEN = 1<<12;
+	static final int FOREGROUND = 1<<13;
+	static final int BACKGROUND = 1<<14;
+	static final int FONT = 1<<15;
 	
 	/* Default widths for widgets */
 	static final int DEFAULT_WIDTH	= 64;
@@ -88,33 +95,44 @@ public abstract class Widget {
 	static final int ENTER_NOTIFY_EVENT = 11;
 	static final int EVENT = 12;
 	static final int EVENT_AFTER = 13;
-	static final int EXPOSE_EVENT = 14;
-	static final int FOCUS_IN_EVENT = 15;
-	static final int FOCUS_OUT_EVENT = 16;
-	static final int HIDE = 17;
-	static final int INSERT_TEXT = 18;
-	static final int KEY_PRESS_EVENT = 19;
-	static final int KEY_RELEASE_EVENT = 20;
-	static final int LEAVE_NOTIFY_EVENT = 21;
-	static final int MAP_EVENT = 22;
-	static final int MNEMONIC_ACTIVATE = 23;
-	static final int MOTION_NOTIFY_EVENT = 24;
-	static final int POPUP_MENU = 25;
-	static final int PREEDIT_CHANGED = 26;
-	static final int REALIZE = 40;
-	static final int ROW_ACTIVATED = 27;
-	static final int ROW_COLLAPSED = 28;
-	static final int ROW_EXPANDED = 29;
-	static final int SELECT = 30;
-	static final int SELECT_CHILD = 31;
-	static final int SHOW = 32;
-	static final int SHOW_HELP = 33;
-	static final int SIZE_ALLOCATE = 34;
-	static final int SWITCH_PAGE = 35;
-	static final int TOGGLED = 36;
-	static final int UNMAP_EVENT = 37;
-	static final int UNREALIZE = 38;
-	static final int VALUE_CHANGED = 39;
+	static final int EXPAND_COLLAPSE_CURSOR_ROW = 14;
+	static final int EXPOSE_EVENT = 15;
+	static final int FOCUS = 16;
+	static final int FOCUS_IN_EVENT = 17;
+	static final int FOCUS_OUT_EVENT = 18;
+	static final int HIDE = 19;
+	static final int INPUT = 20;
+	static final int INSERT_TEXT = 21;
+	static final int KEY_PRESS_EVENT = 22;
+	static final int KEY_RELEASE_EVENT = 23;
+	static final int LEAVE_NOTIFY_EVENT = 24;
+	static final int MAP = 25;
+	static final int MAP_EVENT = 26;
+	static final int MNEMONIC_ACTIVATE = 27;
+	static final int MOTION_NOTIFY_EVENT = 28;
+	static final int MOVE_FOCUS = 29;
+	static final int OUTPUT = 30;
+	static final int POPUP_MENU = 31;
+	static final int PREEDIT_CHANGED = 32;
+	static final int REALIZE = 33;
+	static final int ROW_ACTIVATED = 34;
+	static final int SCROLL_CHILD = 35;
+	static final int SCROLL_EVENT = 36;
+	static final int SELECT = 37;
+	static final int SHOW = 38;
+	static final int SHOW_HELP = 39;
+	static final int SIZE_ALLOCATE = 40;
+	static final int STYLE_SET = 41;
+	static final int SWITCH_PAGE = 42;
+	static final int TEST_COLLAPSE_ROW = 43;
+	static final int TEST_EXPAND_ROW = 44;
+	static final int TOGGLED = 45;
+	static final int UNMAP = 46;
+	static final int UNMAP_EVENT = 47;
+	static final int UNREALIZE = 48;
+	static final int VALUE_CHANGED = 49;
+	static final int VISIBILITY_NOTIFY_EVENT = 50;
+	static final int WINDOW_STATE_EVENT = 51;
 
 /**
  * Prevents uninitialized instances from being created outside the package.
@@ -139,6 +157,7 @@ Widget () {}
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if the parent is disposed</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -153,6 +172,7 @@ public Widget (Widget parent, int style) {
 	checkSubclass ();
 	checkParent (parent);
 	this.style = style;
+	display = parent.display;
 }
 
 /**
@@ -208,6 +228,16 @@ public void addDisposeListener (DisposeListener listener) {
 	addListener (SWT.Dispose, typedListener);
 }
 
+int /*long*/ textCellDataProc (int /*long*/ tree_column, int /*long*/ cell, int /*long*/ tree_model, int /*long*/ iter, int /*long*/ data) {
+	return 0;
+}
+int /*long*/ paintWindow () {
+	return 0;
+}
+int /*long*/ pixbufCellDataProc (int /*long*/ tree_column, int /*long*/ cell, int /*long*/ tree_model, int /*long*/ iter, int /*long*/ data) {
+	return 0;
+}
+
 static int checkBits (int style, int int0, int int1, int int2, int int3, int int4, int int5) {
 	int mask = int0 | int1 | int2 | int3 | int4 | int5;
 	if ((style & mask) == 0) style |= int0;
@@ -220,6 +250,10 @@ static int checkBits (int style, int int0, int int1, int int2, int int3, int int
 	return style;
 }
 
+void checkOpen () {
+	/* Do nothing */
+}
+
 void checkOrientation (Widget parent) {
 	style &= ~SWT.MIRRORED;
 	if ((style & (SWT.LEFT_TO_RIGHT | SWT.RIGHT_TO_LEFT)) == 0) {
@@ -245,8 +279,9 @@ void checkOrientation (Widget parent) {
  */
 void checkParent (Widget parent) {
 	if (parent == null) error (SWT.ERROR_NULL_ARGUMENT);
-	if (!parent.isValidThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
-	if (parent.isDisposed()) error (SWT.ERROR_INVALID_ARGUMENT);
+	if (parent.isDisposed ()) error (SWT.ERROR_INVALID_ARGUMENT);
+	parent.checkWidget ();
+	parent.checkOpen ();
 }
 
 /**
@@ -305,8 +340,10 @@ protected void checkSubclass () {
  * </ul>
  */
 protected void checkWidget () {
-	if (!isValidThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
-	if (!isValidWidget ()) error (SWT.ERROR_WIDGET_DISPOSED);
+	Display display = this.display;
+	if (display == null) error (SWT.ERROR_WIDGET_DISPOSED);
+	if (display.thread != Thread.currentThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
+	if ((state & DISPOSED) != 0) error (SWT.ERROR_WIDGET_DISPOSED);
 }
 
 void createHandle (int index) {
@@ -314,17 +351,18 @@ void createHandle (int index) {
 
 void createWidget (int index) {
 	createHandle (index);
+	setOrientation ();
 	hookEvents ();
 	register ();
 }
 
 void deregister () {
 	if (handle == 0) return;
-	if ((state & HANDLE) != 0) WidgetTable.remove (handle);
+	if ((state & HANDLE) != 0) display.removeWidget (handle);
 }
 
 void destroyWidget () {
-	int topHandle = topHandle ();
+	int /*long*/ topHandle = topHandle ();
 	releaseHandle ();
 	if (topHandle != 0 && (state & HANDLE) != 0) {
 		OS.gtk_widget_destroy (topHandle);
@@ -390,11 +428,11 @@ void error (int code) {
  *    <li>ERROR_THREAD_INVALID_ACCESS - when called from the wrong thread</li>
  * </ul>
  *
- * @see #setData
+ * @see #setData(Object)
  */
 public Object getData () {
-	checkWidget ();
-	return data;
+	checkWidget();
+	return (state & KEYED_DATA) != 0 ? ((Object []) data) [0] : data;
 }
 /**
  * Returns the application defined property of the receiver
@@ -418,14 +456,16 @@ public Object getData () {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
- * @see #setData
+ * @see #setData(String, Object)
  */
 public Object getData (String key) {
-	checkWidget ();
+	checkWidget();
 	if (key == null) error (SWT.ERROR_NULL_ARGUMENT);
-	if (keys == null) return null;
-	for (int i=0; i<keys.length; i++) {
-		if (keys [i].equals (key)) return values [i];
+	if ((state & KEYED_DATA) != 0) {
+		Object [] table = (Object []) data;
+		for (int i=1; i<table.length; i+=2) {
+			if (key.equals (table [i])) return table [i+1];
+		}
 	}
 	return null;
 }
@@ -443,10 +483,13 @@ public Object getData (String key) {
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
- *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  */
-public abstract Display getDisplay ();
+public Display getDisplay () {
+	Display display = this.display;
+	if (display == null) error (SWT.ERROR_WIDGET_DISPOSED);
+	return display;
+}
 
 String getName () {
 //	String string = getClass ().getName ();
@@ -454,7 +497,7 @@ String getName () {
 //	if (index == -1) return string;	
 	String string = getClass ().getName ();
 	int index = string.length ();
-	while ((--index > 0) && (string.charAt (index) != '.'));
+	while ((--index > 0) && (string.charAt (index) != '.')) {}
 	return string.substring (index + 1, string.length ());
 }
 
@@ -488,174 +531,222 @@ public int getStyle () {
 }
 
 
-int gtk_activate (int widget) {
+int /*long*/ gtk_activate (int /*long*/ widget) {
 	return 0;
 }
 
-int gtk_button_press_event (int widget, int event) {
+int /*long*/ gtk_button_press_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_button_release_event (int widget, int event) {
+int /*long*/ gtk_button_release_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_changed (int widget) {
+int /*long*/ gtk_changed (int /*long*/ widget) {
 	return 0;
 }
 
-int gtk_clicked (int widget) {
+int /*long*/ gtk_clicked (int /*long*/ widget) {
 	return 0;
 }
 
-int gtk_commit (int imcontext, int text) {
+int /*long*/ gtk_commit (int /*long*/ imcontext, int /*long*/ text) {
 	return 0;
 }
 
-int gtk_configure_event (int widget, int event) {
+int /*long*/ gtk_configure_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_delete_event (int widget, int event) {
+int /*long*/ gtk_delete_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_delete_range (int widget, int iter1, int iter2) {
+int /*long*/ gtk_delete_range (int /*long*/ widget, int /*long*/ iter1, int /*long*/ iter2) {
 	return 0;
 }
 
-int gtk_delete_text (int widget, int start_pos, int end_pos) {
+int /*long*/ gtk_delete_text (int /*long*/ widget, int /*long*/ start_pos, int /*long*/ end_pos) {
 	return 0;
 }
 
-int gtk_enter_notify_event (int widget, int event) {
+int /*long*/ gtk_enter_notify_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_event (int widget, int event) {
+int /*long*/ gtk_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_event_after (int widget, int event) {
+int /*long*/ gtk_event_after (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_expose_event (int widget, int event) {
+int /*long*/ gtk_expand_collapse_cursor_row (int /*long*/ widget, int /*long*/ logical, int /*long*/ expand, int /*long*/ open_all) {
 	return 0;
 }
 
-int gtk_focus_in_event (int widget, int event) {
+int /*long*/ gtk_expose_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_focus_out_event (int widget, int event) {
+int /*long*/ gtk_focus (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_hide (int widget) {
+int /*long*/ gtk_focus_in_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_insert_text (int widget, int new_text, int new_text_length, int position) {
+int /*long*/ gtk_focus_out_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_key_press_event (int widget, int event) {
+int /*long*/ gtk_hide (int /*long*/ widget) {
 	return 0;
 }
 
-int gtk_key_release_event (int widget, int event) {
+int /*long*/ gtk_input (int /*long*/ widget, int /*long*/ arg1) {
 	return 0;
 }
 
-int gtk_leave_notify_event (int widget, int event) {
+int /*long*/ gtk_insert_text (int /*long*/ widget, int /*long*/ new_text, int /*long*/ new_text_length, int /*long*/ position) {
 	return 0;
 }
 
-int gtk_map_event (int widget, int event) {
+int /*long*/ gtk_key_press_event (int /*long*/ widget, int /*long*/ event) {
+	GdkEventKey gdkEvent = new GdkEventKey ();
+	OS.memmove (gdkEvent, event, GdkEventKey.sizeof);
+	return sendKeyEvent (SWT.KeyDown, gdkEvent) ? 0 : 1;
+}
+
+int /*long*/ gtk_key_release_event (int /*long*/ widget, int /*long*/ event) {
+	GdkEventKey gdkEvent = new GdkEventKey ();
+	OS.memmove (gdkEvent, event, GdkEventKey.sizeof);
+	return sendKeyEvent (SWT.KeyUp, gdkEvent) ? 0 : 1;
+}
+
+int /*long*/ gtk_leave_notify_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_mnemonic_activate (int widget, int arg1) {
+int /*long*/ gtk_map (int /*long*/ widget) {
 	return 0;
 }
 
-int gtk_motion_notify_event (int widget, int event) {
+int /*long*/ gtk_map_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_popup_menu (int widget) {
+int /*long*/ gtk_mnemonic_activate (int /*long*/ widget, int /*long*/ arg1) {
 	return 0;
 }
 
-int gtk_preedit_changed (int imcontext) {
+int /*long*/ gtk_motion_notify_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_realize (int widget) {
+int /*long*/ gtk_move_focus (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_row_activated (int tree, int path, int column) {
+int /*long*/ gtk_output (int /*long*/ widget) {
 	return 0;
 }
 
-int gtk_row_collapsed (int tree, int iter, int path) {
+int /*long*/ gtk_popup_menu (int /*long*/ widget) {
 	return 0;
 }
 
-int gtk_row_expanded (int tree, int iter, int path) {
+int /*long*/ gtk_preedit_changed (int /*long*/ imcontext) {
 	return 0;
 }
 
-int gtk_select (int item) {
+int /*long*/ gtk_realize (int /*long*/ widget) {
 	return 0;
 }
 
-int gtk_select_child (int list, int widget) {
+int /*long*/ gtk_row_activated (int /*long*/ tree, int /*long*/ path, int /*long*/ column) {
 	return 0;
 }
 
-int gtk_show (int widget) {
+int /*long*/ gtk_scroll_child (int /*long*/ widget, int /*long*/ scrollType, int /*long*/ horizontal) {
 	return 0;
 }
 
-int gtk_show_help (int widget, int helpType) {
+int /*long*/ gtk_scroll_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
-int gtk_size_allocate (int widget, int allocation) {
+int /*long*/ gtk_select (int /*long*/ item) {
 	return 0;
 }
 
-int gtk_switch_page (int widget, int page, int page_num) {
+int /*long*/ gtk_show (int /*long*/ widget) {
 	return 0;
 }
 
-int gtk_timer () {
+int /*long*/ gtk_show_help (int /*long*/ widget, int /*long*/ helpType) {
 	return 0;
 }
 
-int gtk_toggled (int renderer, int pathStr) {
+int /*long*/ gtk_size_allocate (int /*long*/ widget, int /*long*/ allocation) {
 	return 0;
 }
 
-int gtk_unmap_event (int widget, int event) {
+int /*long*/ gtk_style_set (int /*long*/ widget, int /*long*/ previousStyle) {
 	return 0;
 }
 
-int gtk_unrealize (int widget) {
+int /*long*/ gtk_switch_page (int /*long*/ widget, int /*long*/ page, int /*long*/ page_num) {
 	return 0;
 }
 
-int gtk_value_changed (int adjustment) {
+int /*long*/ gtk_test_collapse_row (int /*long*/ tree, int /*long*/ iter, int /*long*/ path) {
 	return 0;
 }
 
-int fontHeight (int font, int widgetHandle) {
-	int context = OS.gtk_widget_get_pango_context (widgetHandle);
-	int lang = OS.pango_context_get_language (context);
-	int metrics = OS.pango_context_get_metrics (context, font, lang);
+int /*long*/ gtk_test_expand_row (int /*long*/ tree, int /*long*/ iter, int /*long*/ path) {
+	return 0;
+}
+
+int /*long*/ gtk_timer () {
+	return 0;
+}
+
+int /*long*/ gtk_toggled (int /*long*/ renderer, int /*long*/ pathStr) {
+	return 0;
+}
+
+int /*long*/ gtk_unmap (int /*long*/ widget) {
+	return 0;
+}
+
+int /*long*/ gtk_unmap_event (int /*long*/ widget, int /*long*/ event) {
+	return 0;
+}
+
+int /*long*/ gtk_unrealize (int /*long*/ widget) {
+	return 0;
+}
+
+int /*long*/ gtk_value_changed (int /*long*/ adjustment) {
+	return 0;
+}
+
+int /*long*/ gtk_visibility_notify_event (int /*long*/ widget, int /*long*/ event) {
+	return 0;
+}
+
+int /*long*/ gtk_window_state_event (int /*long*/ widget, int /*long*/ event) {
+	return 0;
+}
+
+int fontHeight (int /*long*/ font, int /*long*/ widgetHandle) {
+	int /*long*/ context = OS.gtk_widget_get_pango_context (widgetHandle);
+	int /*long*/ lang = OS.pango_context_get_language (context);
+	int /*long*/ metrics = OS.pango_context_get_metrics (context, font, lang);
 	int ascent = OS.pango_font_metrics_get_ascent (metrics);
 	int descent = OS.pango_font_metrics_get_descent (metrics);
 	OS.pango_font_metrics_unref (metrics);
@@ -663,27 +754,35 @@ int fontHeight (int font, int widgetHandle) {
 }
 
 boolean filters (int eventType) {
-	Display display = getDisplay ();
 	return display.filters (eventType);
 }
 
+int /*long*/ filterProc (int /*long*/ xEvent, int /*long*/ gdkEvent, int /*long*/ data) {
+	return 0;
+}
+
+int /*long*/ fixedMapProc (int /*long*/ widget) {
+	return 0;
+}
+
 char [] fixMnemonic (String string) {
 	int length = string.length ();
 	char [] text = new char [length];
 	string.getChars (0, length, text, 0);
 	int i = 0, j = 0;
-	char [] result = new char [length * 2 + 1];
+	char [] result = new char [length * 2];
 	while (i < length) {
 		switch (text [i]) {
-			case '&': {
-				if (i + 1 < length && text[i + 1] == '&') {
+			case '&':
+				if (i + 1 < length && text [i + 1] == '&') {
 					i++; 
-					break;
+				} else {
+					text [i] = '_';
 				}
-				text [i] = '_';
 				break;
-			}
-			case '_': result [j++] = '_'; break;
+			case '_':
+				result [j++] = '_';
+				break;
 		}
 		result [j++] = text [i++];
 	}
@@ -720,7 +819,7 @@ public boolean isDisposed () {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  */
-protected boolean isListening (int eventType) {
+public boolean isListening (int eventType) {
 	checkWidget ();
 	return hooks (eventType);
 }
@@ -760,25 +859,26 @@ boolean hooks (int eventType) {
 	return eventTable.hooks (eventType);
 }
 
-int hoverProc (int widget) {
+int /*long*/ hoverProc (int /*long*/ widget) {
 	return 0;
 }
 
-char mbcsToWcs (char ch) {
-	int key = ch & 0xFFFF;
-	if (key <= 0x7F) return ch;
-	byte [] buffer;
-	if (key <= 0xFF) {
-		buffer = new byte [1];
-		buffer [0] = (byte) key;
-	} else {
-		buffer = new byte [2];
-		buffer [0] = (byte) ((key >> 8) & 0xFF);
-		buffer [1] = (byte) (key & 0xFF);
-	}
-	char [] result = Converter.mbcsToWcs (null, buffer);
-	if (result.length == 0) return 0;
-	return result [0];
+int /*long*/ menuPositionProc (int /*long*/ menu, int /*long*/ x, int /*long*/ y, int /*long*/ push_in, int /*long*/ user_data) {
+	return 0;
+}
+
+boolean mnemonicHit (int /*long*/ mnemonicHandle, char key) {
+	if (!mnemonicMatch (mnemonicHandle, key)) return false;
+	OS.g_signal_handlers_block_matched (mnemonicHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, MNEMONIC_ACTIVATE);
+	boolean result = OS.gtk_widget_mnemonic_activate (mnemonicHandle, false);
+	OS.g_signal_handlers_unblock_matched (mnemonicHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, MNEMONIC_ACTIVATE);
+	return result;
+}
+
+boolean mnemonicMatch (int /*long*/ mnemonicHandle, char key) {
+	int keyval1 = OS.gdk_keyval_to_lower (OS.gdk_unicode_to_keyval (key));
+	int keyval2 = OS.gdk_keyval_to_lower (OS.gtk_label_get_mnemonic_keyval (mnemonicHandle)); 
+	return keyval1 == keyval2;
 }
 
 /**
@@ -789,9 +889,6 @@ char mbcsToWcs (char ch) {
  * @param eventType the type of event which has occurred
  * @param event the event data
  *
- * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the event is null</li>
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -799,7 +896,7 @@ char mbcsToWcs (char ch) {
  */
 public void notifyListeners (int eventType, Event event) {
 	checkWidget();
-	if (event == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (event == null) event = new Event ();
 	sendEvent (eventType, event);
 }
 
@@ -813,7 +910,7 @@ void postEvent (int eventType, Event event) {
 
 void register () {
 	if (handle == 0) return;
-	if ((state & HANDLE) != 0) WidgetTable.put (handle, this);
+	if ((state & HANDLE) != 0) display.addWidget (handle, this);
 }
 
 void releaseChild () {
@@ -823,6 +920,7 @@ void releaseChild () {
 void releaseHandle () {
 	handle = 0;
 	state |= DISPOSED;
+	display = null;
 }
 
 void releaseResources () {
@@ -933,7 +1031,6 @@ void sendEvent (int eventType, Event event) {
 }
 
 void sendEvent (int eventType, Event event, boolean send) {
-	Display display = getDisplay ();
 	if (eventTable == null && !display.filters (eventType)) {
 		return;
 	}
@@ -951,6 +1048,87 @@ void sendEvent (int eventType, Event event, boolean send) {
 	}
 }
 
+boolean sendKeyEvent (int type, GdkEventKey keyEvent) {
+	int length = keyEvent.length;
+	if (keyEvent.string == 0 || OS.g_utf8_strlen (keyEvent.string, length) <= 1) {
+		Event event = new Event ();
+		event.time = keyEvent.time;
+		if (!setKeyState (event, keyEvent)) return true;
+		sendEvent (type, event);
+		// widget could be disposed at this point
+	
+		/*
+		* It is possible (but unlikely), that application
+		* code could have disposed the widget in the key
+		* events.  If this happens, end the processing of
+		* the key by returning false.
+		*/
+		if (isDisposed ()) return false;
+		return event.doit;
+	}
+	byte [] buffer = new byte [length];
+	OS.memmove (buffer, keyEvent.string, length);
+	char [] chars = Converter.mbcsToWcs (null, buffer);
+	return sendIMKeyEvent (type, keyEvent, chars) != null;
+}
+
+char [] sendIMKeyEvent (int type, GdkEventKey keyEvent, char  [] chars) {
+	int index = 0, count = 0, state = 0;
+	int /*long*/ ptr = 0;
+	if (keyEvent == null) {
+		ptr = OS.gtk_get_current_event ();
+		if (ptr != 0) {
+			keyEvent = new GdkEventKey ();
+			OS.memmove (keyEvent, ptr, GdkEventKey.sizeof);
+			switch (keyEvent.type) {
+				case OS.GDK_KEY_PRESS:
+				case OS.GDK_KEY_RELEASE:
+					state = keyEvent.state;
+					break;
+				default:
+					keyEvent = null;
+					break;
+			}
+		}
+	}
+	if (keyEvent == null) {
+		int [] buffer = new int [1];
+		OS.gtk_get_current_event_state (buffer);
+		state = buffer [0];
+	}
+	while (index < chars.length) {
+		Event event = new Event ();
+		if (keyEvent != null && chars.length <= 1) {
+			setKeyState (event, keyEvent);
+		} else {
+			setInputState (event, state);
+		}
+		event.character = chars [index];
+		sendEvent (type, event);
+	
+		/*
+		* It is possible (but unlikely), that application
+		* code could have disposed the widget in the key
+		* events.  If this happens, end the processing of
+		* the key by returning null.
+		*/
+		if (isDisposed ()) {
+			if (ptr != 0) OS.gdk_event_free (ptr);
+			return null;
+		}
+		if (event.doit) chars [count++] = chars [index];
+		index++;
+	}
+	if (ptr != 0) OS.gdk_event_free (ptr);
+	if (count == 0) return null;
+	if (index != count) {
+		char [] result = new char [count];
+		System.arraycopy (chars, 0, result, 0, count);
+		return result;
+	}
+	return chars;
+}
+
 /**
  * Sets the application defined widget data associated
  * with the receiver to be the argument. The <em>widget
@@ -970,10 +1148,16 @@ void sendEvent (int eventType, Event event, boolean send) {
  *    <li>ERROR_WIDGET_DISPOSED - when the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - when called from the wrong thread</li>
  * </ul>
+ * 
+ * @see #getData()
  */
 public void setData (Object data) {
 	checkWidget();
-	this.data = data;
+	if ((state & KEYED_DATA) != 0) {
+		((Object []) this.data) [0] = data;
+	} else {
+		this.data = data;
+	}
 }
 
 /**
@@ -998,95 +1182,106 @@ public void setData (Object data) {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
- * @see #getData
+ * @see #getData(String)
  */
 public void setData (String key, Object value) {
-	checkWidget ();
+	checkWidget();
 	if (key == null) error (SWT.ERROR_NULL_ARGUMENT);
-	
-	/* Remove the key/value pair */
-	if (value == null) {
-		if (keys == null) return;
-		int index = 0;
-		while (index < keys.length && !keys [index].equals (key)) index++;
-		if (index == keys.length) return;
-		if (keys.length == 1) {
-			keys = null;
-			values = null;
+	int index = 1;
+	Object [] table = null;
+	if ((state & KEYED_DATA) != 0) {
+		table = (Object []) data;
+		while (index < table.length) {
+			if (key.equals (table [index])) break;
+			index += 2;
+		}
+	}
+	if (value != null) {
+		if ((state & KEYED_DATA) != 0) {
+			if (index == table.length) {
+				Object [] newTable = new Object [table.length + 2];
+				System.arraycopy (table, 0, newTable, 0, table.length);
+				data = table = newTable;
+			}
 		} else {
-			String [] newKeys = new String [keys.length - 1];
-			Object [] newValues = new Object [values.length - 1];
-			System.arraycopy (keys, 0, newKeys, 0, index);
-			System.arraycopy (keys, index + 1, newKeys, index, newKeys.length - index);
-			System.arraycopy (values, 0, newValues, 0, index);
-			System.arraycopy (values, index + 1, newValues, index, newValues.length - index);
-			keys = newKeys;
-			values = newValues;
+			table = new Object [3];
+			table [0] = data;
+			data = table;
+			state |= KEYED_DATA;
 		}
-		return;
-	}
-	
-	/* Add the key/value pair */
-	if (keys == null) {
-		keys = new String [] {key};
-		values = new Object [] {value};
-		return;
-	}
-	for (int i=0; i<keys.length; i++) {
-		if (keys [i].equals (key)) {
-			values [i] = value;
-			return;
+		table [index] = key;
+		table [index + 1] = value;
+	} else {
+		if ((state & KEYED_DATA) != 0) {
+			if (index != table.length) {
+				int length = table.length - 2;
+				if (length == 1) {
+					data = table [0];
+					state &= ~KEYED_DATA;
+				} else {
+					Object [] newTable = new Object [length];
+					System.arraycopy (table, 0, newTable, 0, index);
+					System.arraycopy (table, index + 2, newTable, index, length - index);
+					data = newTable;
+				}
+			}
 		}
 	}
-	String [] newKeys = new String [keys.length + 1];
-	Object [] newValues = new Object [values.length + 1];
-	System.arraycopy (keys, 0, newKeys, 0, keys.length);
-	System.arraycopy (values, 0, newValues, 0, values.length);
-	newKeys [keys.length] = key;
-	newValues [values.length] = value;
-	keys = newKeys;
-	values = newValues;
 }
 
-void setInputState (Event event, int state) {
+boolean setInputState (Event event, int state) {
 	if ((state & OS.GDK_MOD1_MASK) != 0) event.stateMask |= SWT.ALT;
 	if ((state & OS.GDK_SHIFT_MASK) != 0) event.stateMask |= SWT.SHIFT;
 	if ((state & OS.GDK_CONTROL_MASK) != 0) event.stateMask |= SWT.CONTROL;
 	if ((state & OS.GDK_BUTTON1_MASK) != 0) event.stateMask |= SWT.BUTTON1;
 	if ((state & OS.GDK_BUTTON2_MASK) != 0) event.stateMask |= SWT.BUTTON2;
 	if ((state & OS.GDK_BUTTON3_MASK) != 0) event.stateMask |= SWT.BUTTON3;
+	return true;
 }
 
-void setKeyState (Event event, GdkEventKey keyEvent) {
-	if (keyEvent.length <= 1) {
-		event.keyCode = Display.translateKey (keyEvent.keyval);
-		switch (keyEvent.keyval) {
-			case OS.GDK_BackSpace:		event.character = '\b'; break;
-			case OS.GDK_Linefeed:		event.character = '\n'; break;
-			case OS.GDK_Return: 		event.character = '\r'; break;
-			case OS.GDK_Delete:		event.character = 0x7F; break;
-			case OS.GDK_Cancel:
-			case OS.GDK_Escape:		event.character = 0x1B; break;
-			case OS.GDK_Tab:
-			case OS.GDK_ISO_Left_Tab: 	event.character = '\t'; break;
-//			case OS.GDK_Clear:			event.character = 0xB; break;
-//			case OS.GDK_Pause:			event.character = 0x13; break;
-//			case OS.GDK_Scroll_Lock:	event.character = 0x14; break;
-			default: {
-				if (event.keyCode == 0) {
-					int key = keyEvent.keyval;
-					if ((keyEvent.state & OS.GDK_CONTROL_MASK) != 0 && (0 <= key && key <= 0x7F)) {
-						if ('a'  <= key && key <= 'z') key -= 'a' - 'A';
-						if (64 <= key && key <= 95) key -= 64;
-						event.character = (char) key;
-					} else {
-						event.character = (char) OS.gdk_keyval_to_unicode (key);
-					}
+boolean setKeyState (Event event, GdkEventKey keyEvent) {
+	if (keyEvent.string != 0 && OS.g_utf8_strlen (keyEvent.string, keyEvent.length) > 1) return false;
+	boolean isNull = false;
+	event.keyCode = Display.translateKey (keyEvent.keyval);
+	switch (keyEvent.keyval) {
+		case OS.GDK_BackSpace:		event.character = SWT.BS; break;
+		case OS.GDK_Linefeed:		event.character = SWT.LF; break;
+		case OS.GDK_KP_Enter:
+		case OS.GDK_Return: 		event.character = SWT.CR; break;
+		case OS.GDK_KP_Delete:
+		case OS.GDK_Delete:			event.character = SWT.DEL; break;
+		case OS.GDK_Escape:			event.character = SWT.ESC; break;
+		case OS.GDK_Tab:
+		case OS.GDK_ISO_Left_Tab: 	event.character = SWT.TAB; break;
+		default: {
+			if (event.keyCode == 0) {
+				int [] keyval = new int [1], effective_group= new int [1], level = new int [1], consumed_modifiers = new int [1];
+				if (OS.gdk_keymap_translate_keyboard_state(OS.gdk_keymap_get_default (), keyEvent.hardware_keycode, 0, keyEvent.group, keyval, effective_group, level, consumed_modifiers)) {
+					event.keyCode = OS.gdk_keyval_to_unicode (keyval [0]);
 				}
 			}
+			int key = keyEvent.keyval;
+			if ((keyEvent.state & OS.GDK_CONTROL_MASK) != 0 && (0 <= key && key <= 0x7F)) {
+				if ('a'  <= key && key <= 'z') key -= 'a' - 'A';
+				if (64 <= key && key <= 95) key -= 64;
+				event.character = (char) key;
+				isNull = keyEvent.keyval == '@' && key == 0;
+			} else {
+				event.character = (char) OS.gdk_keyval_to_unicode (key);
+			}
 		}
 	}
-	setInputState (event, keyEvent.state);
+	if (event.keyCode == 0 && event.character == 0) {
+		if (!isNull) return false;
+	}
+	return setInputState (event, keyEvent.state);
+}
+
+void setOrientation () {
+}
+
+int /*long*/ shellMapProc (int /*long*/ handle, int /*long*/ arg0, int /*long*/ user_data) {
+	return 0;
 }
 
 /**
@@ -1104,15 +1299,15 @@ public String toString () {
 	return getName () + " {" + string + "}";
 }
 
-int topHandle () {
+int /*long*/ topHandle () {
 	return handle;
 }
 
-int timerProc (int widget) {
+int /*long*/ timerProc (int /*long*/ widget) {
 	return 0;
 }
 
-int treeSelectionProc (int model, int path, int iter, int[] selection, int length) {
+int /*long*/ treeSelectionProc (int /*long*/ model, int /*long*/ path, int /*long*/ iter, int [] selection, int length) {
 	return 0;
 }
 
@@ -1120,36 +1315,36 @@ boolean translateTraversal (int event) {
 	return false;
 }
 
-char wcsToMbcs (char ch) {
-	int key = ch & 0xFFFF;
-	if (key <= 0x7F) return ch;
-	byte [] buffer = Converter.wcsToMbcs (null, new char [] {ch}, false);
-	if (buffer.length == 1) return (char) buffer [0];
-	if (buffer.length == 2) {
-		return (char) (((buffer [0] & 0xFF) << 8) | (buffer [1] & 0xFF));
-	}
-	return 0;
-}
-
-int windowProc (int handle, int user_data) {
-	switch (user_data) {
+int /*long*/ windowProc (int /*long*/ handle, int /*long*/ user_data) {
+	switch ((int)/*64*/user_data) {
 		case ACTIVATE: return gtk_activate (handle);
 		case CHANGED: return gtk_changed (handle);
 		case CLICKED: return gtk_clicked (handle);
 		case HIDE: return gtk_hide (handle);
+		case MAP: return gtk_map (handle);
+		case OUTPUT: return gtk_output (handle);
 		case POPUP_MENU: return gtk_popup_menu (handle);
 		case PREEDIT_CHANGED: return gtk_preedit_changed (handle);
 		case REALIZE: return gtk_realize (handle);
 		case SELECT: return gtk_select (handle);
 		case SHOW: return gtk_show (handle);
 		case VALUE_CHANGED: return gtk_value_changed (handle);
+		case UNMAP: return gtk_unmap (handle);
 		case UNREALIZE: return gtk_unrealize (handle);
 		default: return 0;
 	}
 }
 
-int windowProc (int handle, int arg0, int user_data) {
-	switch (user_data) {
+int /*long*/ windowProc (int /*long*/ handle, int /*long*/ arg0, int /*long*/ user_data) {
+	switch ((int)/*64*/user_data) {
+		case -EXPOSE_EVENT: {
+			GdkEventExpose gdkEvent = new GdkEventExpose ();
+			OS.memmove (gdkEvent, arg0, GdkEventExpose.sizeof);
+			int /*long*/ paintWindow = paintWindow();
+			int /*long*/ window = gdkEvent.window;
+			if (window != paintWindow) return 0;
+			return (state & OBSCURED) != 0 ? 1 : 0;
+		}
 		case -BUTTON_PRESS_EVENT:
 		case -BUTTON_RELEASE_EVENT:
 		case -MOTION_NOTIFY_EVENT: {
@@ -1164,37 +1359,45 @@ int windowProc (int handle, int arg0, int user_data) {
 		case EVENT: return gtk_event (handle, arg0);
 		case EVENT_AFTER: return gtk_event_after (handle, arg0);
 		case EXPOSE_EVENT: return gtk_expose_event (handle, arg0);
+		case FOCUS: return gtk_focus (handle, arg0);
 		case FOCUS_IN_EVENT: return gtk_focus_in_event (handle, arg0);
 		case FOCUS_OUT_EVENT: return gtk_focus_out_event (handle, arg0);
 		case KEY_PRESS_EVENT: return gtk_key_press_event (handle, arg0);
 		case KEY_RELEASE_EVENT: return gtk_key_release_event (handle, arg0);
+		case INPUT: return gtk_input (handle, arg0);
 		case LEAVE_NOTIFY_EVENT: return gtk_leave_notify_event (handle, arg0);
 		case MAP_EVENT: return gtk_map_event (handle, arg0);
 		case MNEMONIC_ACTIVATE: return gtk_mnemonic_activate (handle, arg0);
 		case MOTION_NOTIFY_EVENT: return gtk_motion_notify_event (handle, arg0);
-		case SELECT_CHILD: return gtk_select_child (handle, arg0);
+		case MOVE_FOCUS: return gtk_move_focus (handle, arg0);
+		case SCROLL_EVENT:	return gtk_scroll_event (handle, arg0);
 		case SHOW_HELP: return gtk_show_help (handle, arg0);
 		case SIZE_ALLOCATE: return gtk_size_allocate (handle, arg0);
+		case STYLE_SET: return gtk_style_set (handle, arg0);
 		case TOGGLED: return gtk_toggled (handle, arg0);
 		case UNMAP_EVENT: return gtk_unmap_event (handle, arg0);
+		case VISIBILITY_NOTIFY_EVENT: return gtk_visibility_notify_event (handle, arg0);
+		case WINDOW_STATE_EVENT: return gtk_window_state_event (handle, arg0);
 		default: return 0;
 	}
 }
 
-int windowProc (int handle, int arg0, int arg1, int user_data) {
-	switch (user_data) {
+int /*long*/ windowProc (int /*long*/ handle, int /*long*/ arg0, int /*long*/ arg1, int /*long*/ user_data) {
+	switch ((int)/*64*/user_data) {
 		case DELETE_RANGE: return gtk_delete_range (handle, arg0, arg1);
 		case DELETE_TEXT: return gtk_delete_text (handle, arg0, arg1);
 		case ROW_ACTIVATED: return gtk_row_activated (handle, arg0, arg1);
-		case ROW_COLLAPSED: return gtk_row_collapsed (handle, arg0, arg1);
-		case ROW_EXPANDED: return gtk_row_expanded(handle, arg0, arg1);
+		case SCROLL_CHILD: return gtk_scroll_child (handle, arg0, arg1);
 		case SWITCH_PAGE: return gtk_switch_page (handle, arg0, arg1);
+		case TEST_COLLAPSE_ROW: return gtk_test_collapse_row (handle, arg0, arg1);
+		case TEST_EXPAND_ROW: return gtk_test_expand_row(handle, arg0, arg1);
 		default: return 0;
 	}
 }
 
-int windowProc (int handle, int arg0, int arg1, int arg2, int user_data) {
-	switch (user_data) {
+int /*long*/ windowProc (int /*long*/ handle, int /*long*/ arg0, int /*long*/ arg1, int /*long*/ arg2, int /*long*/ user_data) {
+	switch ((int)/*64*/user_data) {
+		case EXPAND_COLLAPSE_CURSOR_ROW: return gtk_expand_collapse_cursor_row (handle, arg0, arg1, arg2);
 		case INSERT_TEXT: return gtk_insert_text (handle, arg0, arg1, arg2);
 		default: return 0;
 	}
