@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -23,16 +23,28 @@ import org.eclipse.swt.internal.motif.*;
  */
 public abstract class Device implements Drawable {
 	/**
-	* the handle to the X Display
-	* (Warning: This field is platform dependent)
-	*/
+	 * the handle to the X Display
+	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
+	 */
 	public int xDisplay;
 	
 	/**
-	* whether the XLFD resolution should match the
-	* resolution of the device when fonts are created
-	* (Warning: This field is platform dependent)
-	*/
+	 * whether the XLFD resolution should match the
+	 * resolution of the device when fonts are created
+	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
+	 */
 	// TEMPORARY CODE
 	public boolean setDPI;
 	
@@ -59,15 +71,45 @@ public abstract class Device implements Drawable {
 	
 	/* System Font */
 	Font systemFont;
-
-	/* Warning and Error Handlers */
-	boolean warnings = true;
-	Callback xErrorCallback, xtWarningCallback, xIOErrorCallback, xtErrorCallback;
-	int xErrorProc, xtWarningProc, xIOErrorProc, xtErrorProc;
-	int xNullErrorProc, xtNullWarningProc, xNullIOErrorProc, xtNullErrorProc;
 	
+	int shellHandle;
+
 	/* Parsing Tables */
-	int tabMapping, crMapping, tabPointer, crPointer;
+	int tabPointer, crPointer;
+	/**
+	 * parse table mappings for tab and cr
+	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
+	 */
+	// TEMPORARY CODE
+	public int tabMapping, crMapping;
+
+	/* Xt Warning and Error Handlers */
+	boolean warnings = true;
+	Callback xtWarningCallback, xtErrorCallback;
+	int xtWarningProc, xtErrorProc, xtNullWarningProc, xtNullErrorProc;	
+	
+	/* X Warning and Error Handlers */
+	static Callback XErrorCallback, XIOErrorCallback;
+	static int XErrorProc, XIOErrorProc, XNullErrorProc, XNullIOErrorProc;
+	static Device[] Devices = new Device[4];
+	
+	static final Object CREATE_LOCK = new Object();
+
+	/* Initialize X and Xt */
+	static {
+		/*
+		* This code is intentionally commented.
+		*/
+//		OS.XInitThreads ();
+//		OS.XtToolkitThreadInitialize ();
+		OS.XtToolkitInitialize ();
+	}
 	
 	/*
 	* TEMPORARY CODE. When a graphics object is
@@ -92,13 +134,28 @@ public abstract class Device implements Drawable {
 /* 
 * TEMPORARY CODE 
 */
-static Device getDevice () {
+static synchronized Device getDevice () {
 	if (DeviceFinder != null) DeviceFinder.run();
 	Device device = CurrentDevice;
 	CurrentDevice = null;
 	return device;
 }
 
+/**
+ * Constructs a new instance of this class.
+ * <p>
+ * You must dispose the device when it is no longer required. 
+ * </p>
+ *
+ * @see #create
+ * @see #init
+ * 
+ * @since 3.1
+ */
+public Device() {
+	this(null);
+}
+
 /**
  * Constructs a new instance of this class.
  * <p>
@@ -112,22 +169,33 @@ static Device getDevice () {
  * @see DeviceData
  */
 public Device(DeviceData data) {
-	if (data != null) {
-		display_name = data.display_name;
-		application_name = data.application_name;
-		application_class = data.application_class;
-		tracking = data.tracking;
-		debug = data.debug;
+	synchronized (CREATE_LOCK) {
+		if (data != null) {
+			display_name = data.display_name;
+			application_name = data.application_name;
+			application_class = data.application_class;
+			tracking = data.tracking;
+			debug = data.debug;
+		}
+		if (tracking) {
+			errors = new Error [128];
+			objects = new Object [128];
+		}
+		create (data);
+		init ();
+		register (this);
+		
+		/* Initialize the system font slot */
+		systemFont = getSystemFont ();
 	}
-	if (tracking) {
-		errors = new Error [128];
-		objects = new Object [128];
+}
+
+void checkCairo() {
+	try {
+		Class.forName("org.eclipse.swt.internal.cairo.Cairo");
+	} catch (Throwable t) {
+		SWT.error(SWT.ERROR_NO_GRAPHICS_LIBRARY, t, " [Cairo required]");
 	}
-	create (data);
-	init ();
-	
-	/* Initialize the system font slot */
-	systemFont = getSystemFont ();
 }
 
 /**
@@ -171,6 +239,12 @@ protected void checkDevice () {
 protected void create (DeviceData data) {
 }
 
+synchronized static void deregister (Device device) {
+	for (int i=0; i<Devices.length; i++) {
+		if (device == Devices [i]) Devices [i] = null;
+	}
+}
+
 /**
  * Destroys the device in the operating system and releases
  * the device's handle.  If the device does not have a handle,
@@ -203,6 +277,7 @@ public void dispose () {
 	checkDevice ();
 	release ();
 	destroy ();
+	deregister (this);
 	xDisplay = 0;
 	if (tracking) {
 		objects = null;
@@ -220,6 +295,16 @@ void dispose_Object (Object object) {
 	}
 }
 
+static synchronized Device findDevice (int xDisplay) {
+	for (int i=0; i<Devices.length; i++) {
+		Device device = Devices [i];
+		if (device != null && device.xDisplay == xDisplay) {
+			return device;
+		}
+	}
+	return null;
+}
+
 /**
  * Returns a rectangle describing the receiver's size and location.
  *
@@ -342,7 +427,7 @@ public Point getDPI () {
  * <code>faceName</code> is null, all fonts will be returned.
  *
  * @param faceName the name of the font to look for, or null
- * @param scalable true if scalable fonts should be returned.
+ * @param scalable if true only scalable fonts are returned, otherwise only non-scalable fonts are returned.
  * @return the matching font data
  *
  * @exception SWTException <ul>
@@ -378,10 +463,14 @@ public FontData [] getFontList (String faceName, boolean scalable) {
 		OS.memmove (buffer2, charPtr, length);
 		/* Use the character encoding for the default locale */
 		char [] chars = Converter.mbcsToWcs (null, buffer2);
-		FontData data = FontData.motif_new (new String (chars));
-		boolean isScalable = data.averageWidth == 0 && data.pixels == 0 && data.points == 0;
-		if (isScalable == scalable) {
-			fd [fdIndex++] = data;
+		try {
+			FontData data = FontData.motif_new (new String (chars));
+			boolean isScalable = data.averageWidth == 0 && data.pixels == 0 && data.points == 0;
+			if (isScalable == scalable) {
+				fd [fdIndex++] = data;
+			}
+		} catch (Exception e) {
+			/* do not add the font to the list */
 		}
 		ptr += 4;
 	}
@@ -398,7 +487,7 @@ public FontData [] getFontList (String faceName, boolean scalable) {
  * specified in class <code>SWT</code>. Any value other
  * than one of the SWT color constants which is passed
  * in will result in the color black. This color should
- * not be free'd because it was allocated by the system,
+ * not be freed because it was allocated by the system,
  * not the application.
  *
  * @param id the color constant
@@ -412,7 +501,6 @@ public FontData [] getFontList (String faceName, boolean scalable) {
  */
 public Color getSystemColor (int id) {
 	checkDevice ();
-	XColor xColor = null;
 	switch (id) {
 		case SWT.COLOR_BLACK: 				return COLOR_BLACK;
 		case SWT.COLOR_DARK_RED: 			return COLOR_DARK_RED;
@@ -431,15 +519,14 @@ public Color getSystemColor (int id) {
 		case SWT.COLOR_CYAN: 				return COLOR_CYAN;
 		case SWT.COLOR_WHITE: 				return COLOR_WHITE;
 	}
-	if (xColor == null) return COLOR_BLACK;
-	return Color.motif_new (this, xColor);
+	return COLOR_BLACK;
 }
 
 /**
  * Returns a reasonable font for applications to use.
  * On some platforms, this will match the "default font"
  * or "system font" if such can be found.  This font
- * should not be free'd because it was allocated by the
+ * should not be freed because it was allocated by the
  * system, not the application.
  * <p>
  * Typically, applications which want the default look
@@ -473,6 +560,10 @@ public Font getSystemFont () {
  */
 public boolean getWarnings () {
 	checkDevice ();
+	return _getWarnings();
+}
+
+boolean _getWarnings () {
 	return warnings;
 }
 
@@ -489,32 +580,33 @@ public boolean getWarnings () {
  * @see #create
  */
 protected void init () {
+	if (debug) OS.XSynchronize (xDisplay, true);
 		
 	/* Create the warning and error callbacks */
-	xErrorCallback = new Callback (this, "xErrorProc", 2);
-	xNullErrorProc = xErrorCallback.getAddress ();
-	if (xNullErrorProc == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
+	Class clazz = getClass ();
+	synchronized (clazz) {
+		if (XErrorCallback == null) {
+			XErrorCallback = new Callback (clazz, "XErrorProc", 2);
+			XNullErrorProc = XErrorCallback.getAddress ();
+			if (XNullErrorProc == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
+			XErrorProc = OS.XSetErrorHandler (XNullErrorProc);
+		}
+		if (XIOErrorCallback == null) {
+			XIOErrorCallback = new Callback (clazz, "XIOErrorProc", 1);
+			XNullIOErrorProc = XIOErrorCallback.getAddress ();
+			if (XNullIOErrorProc == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
+			XIOErrorProc = OS.XSetIOErrorHandler (XNullIOErrorProc);
+		}
+	}
 	xtWarningCallback = new Callback (this, "xtWarningProc", 1);
 	xtNullWarningProc = xtWarningCallback.getAddress ();
 	if (xtNullWarningProc == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
-	xIOErrorCallback = new Callback (this, "xIOErrorProc", 1);
-	xNullIOErrorProc = xIOErrorCallback.getAddress ();
-	if (xNullIOErrorProc == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
 	xtErrorCallback = new Callback (this, "xtErrorProc", 1);
 	xtNullErrorProc = xtErrorCallback.getAddress ();
 	if (xtNullErrorProc == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
-	
-	/* Set the warning and error handlers */
-	if (debug) OS.XSynchronize (xDisplay, true);
 	int xtContext = OS.XtDisplayToApplicationContext (xDisplay);
-	xErrorProc = OS.XSetErrorHandler (xNullErrorProc);
-	if (!debug) OS.XSetErrorHandler (xErrorProc);
 	xtWarningProc = OS.XtAppSetWarningHandler (xtContext, xtNullWarningProc);
-	if (!debug) OS.XtAppSetWarningHandler (xtContext, xtWarningProc);
-	xIOErrorProc = OS.XSetIOErrorHandler (xNullIOErrorProc);
-	if (!debug) OS.XSetIOErrorHandler (xIOErrorProc);
 	xtErrorProc = OS.XtAppSetErrorHandler (xtContext, xtNullErrorProc);
-	if (!debug) OS.XtAppSetErrorHandler (xtContext, xtErrorProc);
 	
 	/* Only use palettes for <= 8 bpp default visual */
 	int xScreenPtr = OS.XDefaultScreenOfDisplay (xDisplay);
@@ -547,6 +639,10 @@ protected void init () {
 	COLOR_CYAN = new Color (this, 0,0xFF,0xFF);
 	COLOR_WHITE = new Color (this, 0xFF,0xFF,0xFF);
 
+	int widgetClass = OS.topLevelShellWidgetClass ();
+	shellHandle = OS.XtAppCreateShell (null, null, widgetClass, xDisplay, null, 0);
+	if (shellHandle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+
 	/* Create the parsing tables */
 	byte[] tabBuffer = {(byte) '\t', 0};
 	tabPointer = OS.XtMalloc (tabBuffer.length);
@@ -595,7 +691,7 @@ public abstract int internal_new_GC (GCData data);
  * application code.
  * </p>
  *
- * @param handle the platform specific GC handle
+ * @param hDC the platform specific GC handle
  * @param data the platform specific GC data 
  */
 public abstract void internal_dispose_GC (int handle, GCData data);
@@ -632,6 +728,19 @@ void new_Object (Object object) {
 	errors = newErrors;
 }
 
+static synchronized void register (Device device) {
+	for (int i=0; i<Devices.length; i++) {
+		if (Devices [i] == null) {
+			Devices [i] = device;
+			return;
+		}
+	}
+	Device [] newDevices = new Device [Devices.length + 4];
+	System.arraycopy (Devices, 0, newDevices, 0, Devices.length);
+	newDevices [Devices.length] = device;
+	Devices = newDevices;
+}
+
 /**
  * Releases any internal resources back to the operating
  * system and clears all fields except the device handle.
@@ -663,6 +772,9 @@ protected void release () {
 	OS.XmParseMappingFree(crMapping);
 	tabPointer = crPointer = tabMapping = crMapping = 0;
 
+	if (shellHandle != 0) OS.XtDestroyWidget (shellHandle);
+	shellHandle = 0;
+
 	/*
 	* Free the palette.  Note that this disposes all colors on
 	* the display that were allocated using the Color constructor.
@@ -700,36 +812,42 @@ protected void release () {
 	xtWarningCallback.dispose (); xtWarningCallback = null;
 	xtNullWarningProc = xtWarningProc = 0;
 	
-	/* Free the X IO error handler */
-	OS.XSetIOErrorHandler (xIOErrorProc);
-	xIOErrorCallback.dispose (); xIOErrorCallback = null;
-	xNullIOErrorProc = xIOErrorProc = 0;
-	
-	/* Free the X error handler */
-	/*
-	* Bug in Motif.  For some reason, when a pixmap is
-	* set into a button or label, despite the fact that
-	* the pixmap is cleared from the widget before it
-	* is disposed, Motif still references the pixmap
-	* and attempts to dispose it in XtDestroyApplicationContext().
-	* The fix is to install the null error handler to avoid the
-	* warning.
-	*
-	* NOTE: The warning callback is leaked.
-	*/
-	OS.XSetErrorHandler (xNullErrorProc);
-//	OS.XSetErrorHandler (xErrorProc);
-//	xErrorCallback.dispose (); xErrorCallback = null;
-//	xNullErrorProc = xErrorProc = 0;
+	int count = 0;
+	for (int i = 0; i < Devices.length; i++){
+		if (Devices [i] != null) count++;
+	}
+	if (count == 1) {
+		/* Free the X IO error handler */
+		OS.XSetIOErrorHandler (XIOErrorProc);
+		XIOErrorCallback.dispose (); XIOErrorCallback = null;
+		XNullIOErrorProc = XIOErrorProc = 0;
+		
+		/* Free the X error handler */
+		/*
+		* Bug in Motif.  For some reason, when a pixmap is
+		* set into a button or label, despite the fact that
+		* the pixmap is cleared from the widget before it
+		* is disposed, Motif still references the pixmap
+		* and attempts to dispose it in XtDestroyApplicationContext().
+		* The fix is to avoid warnings by leaving our handler
+		* and settings warnings to false.
+		*
+		* NOTE: The warning callback is leaked.
+		*/
+		warnings = false;
+//		OS.XSetErrorHandler (XErrorProc);
+//		XErrorCallback.dispose (); XErrorCallback = null;
+//		XNullErrorProc = XErrorProc = 0;
+	}
 }
 
 /**
  * If the underlying window system supports printing warning messages
- * to the console, setting warnings to <code>true</code> prevents these
- * messages from being printed. If the argument is <code>false</code>
+ * to the console, setting warnings to <code>false</code> prevents these
+ * messages from being printed. If the argument is <code>true</code> then
  * message printing is not blocked.
  *
- * @param warnings <code>true</code>if warnings should be handled, and <code>false</code> otherwise
+ * @param warnings <code>true</code>if warnings should be printed, and <code>false</code> otherwise
  *
  * @exception SWTException <ul>
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
@@ -737,47 +855,57 @@ protected void release () {
  */
 public void setWarnings (boolean warnings) {
 	checkDevice ();
-	this.warnings = warnings;
-	if (debug) return;
-	int xtContext = OS.XtDisplayToApplicationContext (xDisplay);
-	if (warnings) {
-		OS.XSetErrorHandler (xErrorProc);
-		OS.XtAppSetWarningHandler (xtContext, xtWarningProc);
-	} else {
-		OS.XSetErrorHandler (xNullErrorProc);
-		OS.XtAppSetWarningHandler (xtContext, xtNullWarningProc);
-	}
+	_setWarnings(warnings);
 }
 
-int xErrorProc (int xDisplay, int xErrorEvent) {
-	if (DEBUG || (debug && warnings)) {
-		new SWTError ().printStackTrace ();
-		OS.Call (xErrorProc, xDisplay, xErrorEvent);
+void _setWarnings (boolean warnings) {
+	this.warnings = warnings;
+}
+
+static int XErrorProc (int xDisplay, int xErrorEvent) {
+	Device device = findDevice (xDisplay);
+	if (device != null) {
+		if (device.warnings) {
+			if (DEBUG || device.debug) {
+				new SWTError ().printStackTrace ();
+			}
+			OS.Call (XErrorProc, xDisplay, xErrorEvent);
+		}
+	} else {
+		if (DEBUG) new SWTError ().printStackTrace ();
+		OS.Call (XErrorProc, xDisplay, xErrorEvent);
 	}
 	return 0;
 }
 
-int xIOErrorProc (int xDisplay) {
-	if (DEBUG || debug) {
-		new SWTError ().printStackTrace ();
-		OS.Call (xIOErrorProc, xDisplay, 0);
+static int XIOErrorProc (int xDisplay) {
+	Device device = findDevice (xDisplay);
+	if (device != null) {
+		if (DEBUG || device.debug) {
+			new SWTError ().printStackTrace ();
+		}
+	} else {
+		if (DEBUG) new SWTError ().printStackTrace ();
 	}
+	OS.Call (XIOErrorProc, xDisplay, 0);
 	return 0;
 }
 
 int xtErrorProc (int message) {
 	if (DEBUG || debug) {
 		new SWTError ().printStackTrace ();
-		OS.Call (xtErrorProc, message, 0);
 	}
+	OS.Call (xtErrorProc, message, 0);
 	return 0;
 }
 
 int xtWarningProc (int message) {
-	if (DEBUG || (debug && warnings)) {
-		new SWTError ().printStackTrace ();
+	if (warnings) {
+		if (DEBUG || debug) {
+			new SWTError ().printStackTrace ();
+		}
 		OS.Call (xtWarningProc, message, 0);
-	}
+	}	
 	return 0;
 }
 
