@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -43,34 +43,28 @@ import org.eclipse.swt.events.*;
  * @see #checkSubclass
  */
 public abstract class Widget {
+	/**
+	 * the handle to the OS resource
+	 * (WARNING: This field is platform dependend)
+	 */
 	public int handle;
 	int style, state;
+	Display display;
 	EventTable eventTable;
 	Object data;
-	String [] keys;
-	Object [] values;
 	
 	/* Global state flags */
-//	static final int AUTOMATIC		= 0x00000001;
-//	static final int ACTIVE			= 0x00000002;
-//	static final int AUTOGRAB		= 0x00000004;
-//	static final int MULTIEXPOSE	= 0x00000008;
-//	static final int RESIZEREDRAW	= 0x00000010;
-//	static final int WRAP			= 0x00000020;
-//	static final int DISABLED		= 0x00000040;
-//	static final int HIDDEN			= 0x00000080;
-//	static final int FOREGROUND		= 0x00000100;
-//	static final int BACKGROUND		= 0x00000200;
-	static final int DISPOSED		= 0x00000400;
-	static final int HANDLE			= 0x00000800;
-	static final int CANVAS			= 0x00001000;
+	static final int DISPOSED		= 1<<0;
+	static final int CANVAS			= 1<<1;
+	static final int KEYED_DATA		= 1<<2;
+	static final int HANDLE			= 1<<3;
+	static final int FOCUS_FORCED	= 1<<4;
+	static final int LAYOUT_NEEDED	= 1<<5;
+	static final int LAYOUT_CHANGED	= 1<<6;
 	
 	static final int DEFAULT_WIDTH	= 64;
 	static final int DEFAULT_HEIGHT	= 64;
 	
-	/* Global widget variables */
-	static final char Mnemonic = '&';
-	
 	/* Events and Callback constants */		
 	static final int BUTTON_PRESS = 1;
 	static final int BUTTON_RELEASE = 2;
@@ -93,17 +87,18 @@ public abstract class Widget {
 	static final int MODIFY_VERIFY_CALLBACK = 19;
 	static final int PAGE_DECREMENT_CALLBACK = 20;
 	static final int PAGE_INCREMENT_CALLBACK = 21;
-	static final int SELECTION_CALLBACK = 22;
-	static final int TO_BOTTOM_CALLBACK = 23;
-	static final int TO_TOP_CALLBACK = 24;
-	static final int VALUE_CHANGED_CALLBACK = 25;
-	static final int NON_MASKABLE  = 26;
-	static final int POINTER_MOTION  = 27;
-	static final int STRUCTURE_NOTIFY  = 28;
-	static final int MAP_CALLBACK = 29;
-	static final int UNMAP_CALLBACK  = 30;
-	static final int DELETE_WINDOW = 31;
-	static final int EXPOSURE_CALLBACK  = 32;	
+	static final int TO_BOTTOM_CALLBACK = 22;
+	static final int TO_TOP_CALLBACK = 23;
+	static final int VALUE_CHANGED_CALLBACK = 24;
+	static final int NON_MASKABLE  = 25;
+	static final int POINTER_MOTION  = 26;
+	static final int STRUCTURE_NOTIFY  = 27;
+	static final int MAP_CALLBACK = 28;
+	static final int UNMAP_CALLBACK  = 29;
+	static final int DELETE_WINDOW = 30;
+	static final int EXPOSURE_CALLBACK  = 31;
+	static final int MULTIPLE_SELECTION_CALLBACK  = 32;
+	static final int PROPERTY_CHANGE = 33;
 
 Widget () {
 	/* Do nothing */
@@ -126,6 +121,7 @@ Widget () {
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if the parent is disposed</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -140,6 +136,7 @@ public Widget (Widget parent, int style) {
 	checkSubclass ();
 	checkParent (parent);
 	this.style = style;
+	display = parent.display;
 }
 /**
  * Adds the listener to the collection of listeners who will
@@ -203,6 +200,9 @@ static int checkBits (int style, int int0, int int1, int int2, int int3, int int
 	if ((style & int5) != 0) style = (style & ~mask) | int5;
 	return style;
 }
+void checkOpen () {
+	/* Do nothing */
+}
 void checkOrientation (Widget parent) {
 	style &= ~SWT.MIRRORED;
 	if ((style & (SWT.LEFT_TO_RIGHT | SWT.RIGHT_TO_LEFT)) == 0) {
@@ -215,8 +215,9 @@ void checkOrientation (Widget parent) {
 }
 void checkParent (Widget parent) {
 	if (parent == null) error (SWT.ERROR_NULL_ARGUMENT);
-	if (!parent.isValidThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
-	if (parent.isDisposed()) error (SWT.ERROR_INVALID_ARGUMENT);
+	if (parent.isDisposed ()) error (SWT.ERROR_INVALID_ARGUMENT);
+	parent.checkWidget ();
+	parent.checkOpen ();
 }
 /**
  * Checks that this class can be subclassed.
@@ -273,8 +274,10 @@ protected void checkSubclass () {
  * </ul>
  */
 protected void checkWidget () {
-	if (!isValidThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
-	if (isDisposed ()) error (SWT.ERROR_WIDGET_DISPOSED);
+	Display display = this.display;
+	if (display == null) error (SWT.ERROR_WIDGET_DISPOSED);
+	if (display.thread != Thread.currentThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
+	if ((state & DISPOSED) != 0) error (SWT.ERROR_WIDGET_DISPOSED);
 }
 void createHandle (int index) {
 	/* Do nothing */
@@ -287,7 +290,7 @@ void createWidget (int index) {
 }
 void deregister () {
 	if (handle == 0) return;
-	WidgetTable.remove (handle);
+	display.removeWidget (handle);
 }
 void destroyWidget () {
 	int topHandle = topHandle ();
@@ -338,9 +341,28 @@ void error (int code) {
 	SWT.error(code);
 }
 boolean filters (int eventType) {
-	Display display = getDisplay ();
 	return display.filters (eventType);
 }
+char fixMnemonic (char [] buffer) {
+	int i=0, j=0;
+	char mnemonic=0;
+	while (i < buffer.length) {
+		if ((buffer [j++] = buffer [i++]) == '&') {
+			if (i == buffer.length) {continue;}
+			if (buffer [i] == '&') {i++; continue;}
+			if (mnemonic == 0) mnemonic = buffer [i];
+			j--;
+		}
+	}
+	while (j < buffer.length) buffer [j++] = 0;
+	return mnemonic;
+}
+int focusProc (int w, int client_data, int call_data, int continue_to_dispatch) {
+	return 0;
+}
+String getCodePage () {
+	return null;
+}
 /**
  * Returns the application defined widget data associated
  * with the receiver, or null if it has not been set. The
@@ -361,11 +383,11 @@ boolean filters (int eventType) {
  *    <li>ERROR_THREAD_INVALID_ACCESS - when called from the wrong thread</li>
  * </ul>
  *
- * @see #setData
+ * @see #setData(Object)
  */
 public Object getData () {
 	checkWidget();
-	return data;
+	return (state & KEYED_DATA) != 0 ? ((Object []) data) [0] : data;
 }
 
 /**
@@ -390,14 +412,16 @@ public Object getData () {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
- * @see #setData
+ * @see #setData(String, Object)
  */
 public Object getData (String key) {
 	checkWidget();
 	if (key == null) error (SWT.ERROR_NULL_ARGUMENT);
-	if (keys == null) return null;
-	for (int i=0; i<keys.length; i++) {
-		if (keys [i].equals (key)) return values [i];
+	if ((state & KEYED_DATA) != 0) {
+		Object [] table = (Object []) data;
+		for (int i=1; i<table.length; i+=2) {
+			if (key.equals (table [i])) return table [i+1];
+		}
 	}
 	return null;
 }
@@ -415,10 +439,13 @@ public Object getData (String key) {
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
- *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  */
-public abstract Display getDisplay ();
+public Display getDisplay () {
+	Display display = this.display;
+	if (display == null) error (SWT.ERROR_WIDGET_DISPOSED);
+	return display;
+}
 String getName () {
 	String string = getClass ().getName ();
 	int index = string.lastIndexOf ('.');
@@ -488,7 +515,7 @@ public boolean isDisposed () {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  */
-protected boolean isListening (int eventType) {
+public boolean isListening (int eventType) {
 	checkWidget();
 	return hooks (eventType);
 }
@@ -501,25 +528,6 @@ boolean isValidThread () {
 void manageChildren () {
 	/* Do nothing */
 }
-char mbcsToWcs (int ch) {
-	return mbcsToWcs (ch, null);
-}
-char mbcsToWcs (int ch, String codePage) {
-	int key = ch & 0xFFFF;
-	if (key <= 0x7F) return (char) ch;
-	byte [] buffer;
-	if (key <= 0xFF) {
-		buffer = new byte [1];
-		buffer [0] = (byte) key;
-	} else {
-		buffer = new byte [2];
-		buffer [0] = (byte) ((key >> 8) & 0xFF);
-		buffer [1] = (byte) (key & 0xFF);
-	}
-	char [] result = Converter.mbcsToWcs (codePage, buffer);
-	if (result.length == 0) return 0;
-	return result [0];
-}
 /**
  * Notifies all of the receiver's listeners for events
  * of the given type that one such event has occurred by
@@ -528,9 +536,6 @@ char mbcsToWcs (int ch, String codePage) {
  * @param eventType the type of event which has occurred
  * @param event the event data
  *
- * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the event is null</li>
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -538,7 +543,7 @@ char mbcsToWcs (int ch, String codePage) {
  */
 public void notifyListeners (int eventType, Event event) {
 	checkWidget();
-	if (event == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (event == null) event = new Event ();
 	sendEvent (eventType, event);
 }
 void postEvent (int eventType) {
@@ -547,7 +552,7 @@ void postEvent (int eventType) {
 void postEvent (int eventType, Event event) {
 	sendEvent (eventType, event, false);
 }
-void propagateHandle (boolean enabled, int widgetHandle) {
+void propagateHandle (boolean enabled, int widgetHandle, int cursor) {
 	int xDisplay = OS.XtDisplay (widgetHandle);
 	if (xDisplay == 0) return;
 	int xWindow = OS.XtWindow (widgetHandle);
@@ -571,27 +576,36 @@ void propagateHandle (boolean enabled, int widgetHandle) {
 		event_mask &= ~(do_not_propagate_mask | OS.EnterWindowMask | OS.LeaveWindowMask);
 		do_not_propagate_mask = 0;
 	}
+	int mask = OS.CWDontPropagate | OS.CWEventMask | OS.CWCursor;
 	XSetWindowAttributes attributes = new XSetWindowAttributes ();
 	attributes.event_mask = event_mask;
 	attributes.do_not_propagate_mask = do_not_propagate_mask;
-	OS.XChangeWindowAttributes (xDisplay, xWindow, OS.CWDontPropagate | OS.CWEventMask, attributes);
+	attributes.cursor = cursor;
+	OS.XChangeWindowAttributes (xDisplay, xWindow, mask, attributes);
 }
-void redrawHandle (int x, int y, int width, int height, int widgetHandle) {
+void redrawHandle (int x, int y, int width, int height, boolean redrawAll, int widgetHandle) {
 	int display = OS.XtDisplay (widgetHandle);
 	if (display == 0) return;
 	int window = OS.XtWindow (widgetHandle);
 	if (window == 0) return;
-	int [] argList = {OS.XmNborderWidth, 0, OS.XmNborderColor, 0};
-	OS.XtGetValues (widgetHandle, argList, argList.length / 2);
-	if (argList [1] != 0) {
-		/* Force the border to repaint by setting the color */
-		OS.XtSetValues (widgetHandle, argList, argList.length / 2);
+	if (redrawAll) {
+		OS.XClearArea (display, window, 0, 0, 0, 0, true);
+	} else {
+		if (width > 0 && height > 0) {
+			int [] argList = {
+				OS.XmNwidth, 0, 		/* 1 */
+				OS.XmNheight, 0, 		/* 3 */
+			};
+			OS.XtGetValues (widgetHandle, argList, argList.length / 2);
+			if ((x < argList [1]) && (y < argList [3]) && (x + width > 0) && (y + height > 0)) {
+				OS.XClearArea (display, window, x, y, width, height, true);
+			}
+		}
 	}
-	OS.XClearArea (display, window, x, y, width, height, true);
 }
 void register () {
 	if (handle == 0) return;
-	WidgetTable.put (handle, this);
+	display.addWidget (handle, this);
 }
 void releaseChild () {
 	/* Do nothing */
@@ -599,6 +613,7 @@ void releaseChild () {
 void releaseHandle () {
 	handle = 0;
 	state |= DISPOSED;
+	display = null;
 }
 void releaseResources () {
 	releaseWidget ();
@@ -609,8 +624,6 @@ void releaseWidget () {
 	deregister ();
 	eventTable = null;
 	data = null;
-	keys = null;
-	values = null;
 }
 /**
  * Removes the listener from the collection of listeners who will
@@ -689,120 +702,44 @@ public void removeDisposeListener (DisposeListener listener) {
 	if (eventTable == null) return;
 	eventTable.unhook (SWT.Dispose, listener);
 }
-void setInputState (Event event, int state) {
+boolean setInputState (Event event, int state) {
 	if ((state & OS.Mod1Mask) != 0) event.stateMask |= SWT.ALT;
 	if ((state & OS.ShiftMask) != 0) event.stateMask |= SWT.SHIFT;
 	if ((state & OS.ControlMask) != 0) event.stateMask |= SWT.CONTROL;
 	if ((state & OS.Button1Mask) != 0) event.stateMask |= SWT.BUTTON1;
 	if ((state & OS.Button2Mask) != 0) event.stateMask |= SWT.BUTTON2;
-	if ((state & OS.Button3Mask) != 0) event.stateMask |= SWT.BUTTON3;	
+	if ((state & OS.Button3Mask) != 0) event.stateMask |= SWT.BUTTON3;
+	return true;
 }
-void setInputState (Event event, XInputEvent xEvent) {
-	setInputState (event, xEvent.state);	
-}
-void setKeyState (Event event, XKeyEvent xEvent) {
-	if (xEvent.keycode != 0) {
-		byte [] buffer = new byte [5];
-		int [] keysym = new int [1];
-		OS.XLookupString (xEvent, buffer, buffer.length, keysym, null);
-		
-		/*
-		* Bug in MOTIF.  On Solaris only, XK_F11 and XK_F12 are not
-		* translated correctly by XLookupString().  They are mapped
-		* to 0x1005FF10 and 0x1005FF11 respectively.  The fix is to
-		* look for these values explicitly and correct them.
-		*/
-		if (OS.IsSunOS && keysym [0] != 0) {
-			switch (keysym [0]) {
-				case 0x1005FF10: 
-					keysym [0] = OS.XK_F11;
-					buffer [0] = 0;
-					break;
-				case 0x1005FF11:
-					keysym [0] = OS.XK_F12;
-					buffer [0] = 0;
-					break;
-			}
-			/*
-			* Bug in MOTIF.  On Solaris only, there is garbage in the
-			* high 16-bits for Keysyms such as XK_Down.  Since Keysyms
-			* must be 16-bits to fit into a Character, mask away the
-			* high 16-bits on all platforms.
-			*/
-			keysym [0] &= 0xFFFF;
-		}
-		
-		/*
-		* Feature in MOTIF. For some reason, XLookupString() fails 
-		* to translate both the keysym and the character when the
-		* control key is down.  For example, Ctrl+2 has the correct
-		* keysym value (50) but no character value, while Ctrl+/ has
-		* the keysym value (2F) but an invalid character value
-		* (1F).  It seems that Motif is applying the algorithm to
-		* convert a character to a control character for characters
-		* that are not valid control characters.  The fix is to test
-		* for 7-bit ASCII keysym values that fall outside of the
-		* the valid control character range and use the keysym value
-		* as the character, not the incorrect value that XLookupString()
-		* returns.  Even though lower case values are not strictly
-		* valid control characters, they are included in the range.
-		* 
-		* Some other cases include Ctrl+3..Ctr+8, Ctrl+[.
-		*/
-		if ((xEvent.state & OS.ControlMask) != 0) {
-			int key = keysym [0];
-			if (0 <= key && key <= 0x7F) {
-				if ('a' <= key && key <= 'z') {
-					key -= 'a' - 'A';
-				}
-				if (!(64 <= key && key <= 95)) {
-					buffer [0] = (byte) key;
-				}
-			}
-		}
-		
-		/*
-		* Bug in Motif.  On HP-UX only, Shift+F9, Shift+F10, Shift+F11
-		* and Shift+F12 are not translated correctly by XLookupString.
-		* The fix is to look for these values explicitly and correct them.
-		*/
-		if (OS.IsHPUX && keysym [0] != 0) {
-			switch (keysym [0]) {
-				case 0xFF91:
-					keysym [0] = OS.XK_F9;
-					break;
-				case 0xFF92:
-					keysym [0] = OS.XK_F10;
-					break;
-				case 0xFF93:
-					keysym [0] = OS.XK_F11;
-					break;
-				case 0xFF94:
-					keysym [0] = OS.XK_F12;
-					break;
-			}
-		}
-		
-		/*
-		* Bug in Motif.  There are some keycodes for which 
-		* XLookupString() does not translate the character.
-		* Some of examples are Shift+Tab and Ctrl+Space.
-		*/
-		switch (keysym [0]) {
-			case OS.XK_ISO_Left_Tab: buffer [0] = '\t'; break;
-			case OS.XK_space: buffer [0] = ' '; break;
-		}
-			
-		/* Fill in the event keyCode or character */
-		if (keysym [0] != 0) {
-			event.keyCode = Display.translateKey (keysym [0]);
-		}
-		if (buffer [0] != 0) {
-			char [] result = Converter.mbcsToWcs (null, buffer);
-			if (result.length != 0) event.character = result [0];
+boolean setKeyState (Event event, XKeyEvent xEvent) {
+	if (xEvent.keycode == 0) return false;
+	byte [] buffer = new byte [5];
+	int [] keysym = new int [1];
+	OS.XLookupString (xEvent, buffer, buffer.length, keysym, null);
+	boolean isNull = display.fixKey (keysym, buffer, xEvent.state);
+	if (keysym [0] != 0) {
+		event.keyCode = Display.translateKey (keysym [0]);
+	}
+	if (event.keyCode == 0) {
+		byte [] buffer1 = new byte [5];
+		int [] keysym1 = new int [1];
+		int oldState = xEvent.state;
+		xEvent.state = 0;
+		OS.XLookupString (xEvent, buffer1, buffer1.length, keysym1, null);
+		xEvent.state = oldState;
+		if (buffer1 [0] != 0) {
+			char [] result = Converter.mbcsToWcs (null, buffer1);
+			if (result.length != 0) event.keyCode = result [0];
 		}
 	}
-	setInputState (event, xEvent);
+	if (buffer [0] != 0) {
+		char [] result = Converter.mbcsToWcs (null, buffer);
+		if (result.length != 0) event.character = result [0];
+	}
+	if (event.keyCode == 0 && event.character == 0) {
+		if (!isNull) return false;
+	}
+	return setInputState (event, xEvent.state);
 }
 void sendEvent (Event event) {
 	Display display = event.display;
@@ -817,7 +754,6 @@ void sendEvent (int eventType, Event event) {
 	sendEvent (eventType, event, true);
 }
 void sendEvent (int eventType, Event event, boolean send) {
-	Display display = getDisplay ();
 	if (eventTable == null && !display.filters (eventType)) {
 		return;
 	}
@@ -834,6 +770,106 @@ void sendEvent (int eventType, Event event, boolean send) {
 		display.postEvent (event);
 	}
 }
+boolean sendIMKeyEvent (int type, XKeyEvent xEvent) {
+	return sendIMKeyEvent (type, xEvent, 0);
+}
+boolean sendIMKeyEvent (int type, XKeyEvent xEvent, int textHandle) {
+	/*
+	* Bug in Motif. On Linux only, XmImMbLookupString () does not return 
+	* XBufferOverflow as the status if the buffer is too small. The fix
+	* is to pass a large buffer.
+	*/
+	byte [] buffer = new byte [512];
+	int [] status = new int [1], unused = new int [1];
+	int focusHandle = OS.XtWindowToWidget (xEvent.display, xEvent.window);
+	int length = OS.XmImMbLookupString (focusHandle, xEvent, buffer, buffer.length, unused, status);
+	if (status [0] == OS.XBufferOverflow) {
+		buffer = new byte [length];
+		length = OS.XmImMbLookupString (focusHandle, xEvent, buffer, length, unused, status);
+	}
+	if (length == 0) return true;
+	
+	/* Convert from MBCS to UNICODE and send the event */
+	/* Use the character encoding for the default locale */
+	char [] chars = Converter.mbcsToWcs (null, buffer);
+	int index = 0, count = 0;
+	while (index < chars.length) {
+		if (chars [index] == 0) {
+			chars [count] = 0;
+			break;
+		}
+		Event event = new Event ();
+		event.time = xEvent.time;
+		event.character = chars [index];
+		setInputState (event, xEvent.state);
+		sendEvent (type, event);
+		// widget could be disposed at this point
+	
+		/*
+		* It is possible (but unlikely), that application
+		* code could have disposed the widget in the key
+		* events.  If this happens, end the processing of
+		* the key by returning false.
+		*/
+		if (isDisposed ()) return false;
+		if (event.doit) chars [count++] = chars [index];
+		index++;
+	}
+	if (count == 0) return false;
+	if (textHandle != 0) {
+		/*
+		* Bug in Motif. On Solaris and Linux, XmImMbLookupString() clears
+		* the characters from the IME. This causes the characters to be
+		* stolen from the text widget. The fix is to detect that the IME
+		* has been cleared and use XmTextInsert() to insert the stolen
+		* characters. This problem does not happen on AIX.
+		*/
+		byte [] testBuffer = new byte [5];
+		int testLength = OS.XmImMbLookupString (textHandle, xEvent, testBuffer, testBuffer.length, unused, unused);
+		if (testLength == 0 || index != count) {
+			int [] start = new int [1], end = new int [1];
+			OS.XmTextGetSelectionPosition (textHandle, start, end);
+			if (start [0] == end [0]) {
+				start [0] = end [0] = OS.XmTextGetInsertionPosition (textHandle);
+			}
+			boolean warnings = display.getWarnings ();
+			display.setWarnings (false);
+			if (index != count) {
+				buffer = Converter.wcsToMbcs (getCodePage (), chars, true);
+			}
+			OS.XmTextReplace (textHandle, start [0], end [0], buffer);
+			int position = start [0] + count;
+			OS.XmTextSetInsertionPosition (textHandle, position);
+			display.setWarnings (warnings);
+			return false;
+		}
+	}
+	return true;
+}
+boolean sendKeyEvent (int type, XKeyEvent xEvent) {
+	Event event = new Event ();
+	event.time = xEvent.time;
+	if (!setKeyState (event, xEvent)) return true;
+	Widget control = this;
+	if ((state & CANVAS) != 0) {
+		if ((style & SWT.NO_FOCUS) != 0) {
+			control = display.getFocusControl ();
+		}
+	}
+	if (control != null) {
+		control.sendEvent (type, event);
+		// widget could be disposed at this point
+	
+		/*
+		* It is possible (but unlikely), that application
+		* code could have disposed the widget in the key
+		* events.  If this happens, end the processing of
+		* the key by returning false.
+		*/
+		if (isDisposed ()) return false;
+	}
+	return event.doit;
+}
 /**
  * Sets the application defined widget data associated
  * with the receiver to be the argument. The <em>widget
@@ -853,10 +889,16 @@ void sendEvent (int eventType, Event event, boolean send) {
  *    <li>ERROR_WIDGET_DISPOSED - when the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - when called from the wrong thread</li>
  * </ul>
+ * 
+ * @see #getData()
  */
 public void setData (Object data) {
 	checkWidget();
-	this.data = data;
+	if ((state & KEYED_DATA) != 0) {
+		((Object []) this.data) [0] = data;
+	} else {
+		this.data = data;
+	}
 }
 
 /**
@@ -881,54 +923,51 @@ public void setData (Object data) {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
- * @see #getData
+ * @see #getData(String)
  */
 public void setData (String key, Object value) {
 	checkWidget();
 	if (key == null) error (SWT.ERROR_NULL_ARGUMENT);
-	
-	/* Remove the key/value pair */
-	if (value == null) {
-		if (keys == null) return;
-		int index = 0;
-		while (index < keys.length && !keys [index].equals (key)) index++;
-		if (index == keys.length) return;
-		if (keys.length == 1) {
-			keys = null;
-			values = null;
+	int index = 1;
+	Object [] table = null;
+	if ((state & KEYED_DATA) != 0) {
+		table = (Object []) data;
+		while (index < table.length) {
+			if (key.equals (table [index])) break;
+			index += 2;
+		}
+	}
+	if (value != null) {
+		if ((state & KEYED_DATA) != 0) {
+			if (index == table.length) {
+				Object [] newTable = new Object [table.length + 2];
+				System.arraycopy (table, 0, newTable, 0, table.length);
+				data = table = newTable;
+			}
 		} else {
-			String [] newKeys = new String [keys.length - 1];
-			Object [] newValues = new Object [values.length - 1];
-			System.arraycopy (keys, 0, newKeys, 0, index);
-			System.arraycopy (keys, index + 1, newKeys, index, newKeys.length - index);
-			System.arraycopy (values, 0, newValues, 0, index);
-			System.arraycopy (values, index + 1, newValues, index, newValues.length - index);
-			keys = newKeys;
-			values = newValues;
+			table = new Object [3];
+			table [0] = data;
+			data = table;
+			state |= KEYED_DATA;
 		}
-		return;
-	}
-	
-	/* Add the key/value pair */
-	if (keys == null) {
-		keys = new String [] {key};
-		values = new Object [] {value};
-		return;
-	}
-	for (int i=0; i<keys.length; i++) {
-		if (keys [i].equals (key)) {
-			values [i] = value;
-			return;
+		table [index] = key;
+		table [index + 1] = value;
+	} else {
+		if ((state & KEYED_DATA) != 0) {
+			if (index != table.length) {
+				int length = table.length - 2;
+				if (length == 1) {
+					data = table [0];
+					state &= ~KEYED_DATA;
+				} else {
+					Object [] newTable = new Object [length];
+					System.arraycopy (table, 0, newTable, 0, index);
+					System.arraycopy (table, index + 2, newTable, index, length - index);
+					data = newTable;
+				}
+			}
 		}
 	}
-	String [] newKeys = new String [keys.length + 1];
-	Object [] newValues = new Object [values.length + 1];
-	System.arraycopy (keys, 0, newKeys, 0, keys.length);
-	System.arraycopy (values, 0, newValues, 0, values.length);
-	newKeys [keys.length] = key;
-	newValues [values.length] = value;
-	keys = newKeys;
-	values = newValues;
 }
 
 /**
@@ -948,10 +987,10 @@ public String toString () {
 int topHandle () {
 	return handle;
 }
-boolean translateAccelerator (int key, int keysym, XKeyEvent xEvent) {
+boolean translateAccelerator (char key, int keysym, XKeyEvent xEvent, boolean doit) {
 	return false;
 }
-boolean translateMnemonic (int key, XKeyEvent xEvent) {
+boolean translateMnemonic (char key, int keysym, XKeyEvent xEvent) {
 	return false;
 }
 boolean translateTraversal (int key, XKeyEvent xEvent) {
@@ -964,26 +1003,17 @@ boolean XmProcessTraversal (int widget, int direction) {
 	* is to post focus events and run them when the handler
 	* has returned.
 	*/
-	Display display = getDisplay ();
+	Display display = this.display;
 	boolean oldFocusOut = display.postFocusOut;
 	display.postFocusOut = true;
 	boolean result = OS.XmProcessTraversal (widget, direction);
 	display.postFocusOut = oldFocusOut;
-	if (!display.postFocusOut) display.runFocusOutEvents ();
-	return result;
-}
-int wcsToMbcs (char ch) {
-	return wcsToMbcs (ch, null);
-}
-int wcsToMbcs (char ch, String codePage) {
-	int key = ch & 0xFFFF;
-	if (key <= 0x7F) return ch;
-	byte [] buffer = Converter.wcsToMbcs (codePage, new char [] {ch}, false);
-	if (buffer.length == 1) return (char) buffer [0];
-	if (buffer.length == 2) {
-		return (char) (((buffer [0] & 0xFF) << 8) | (buffer [1] & 0xFF));
+	if (!display.postFocusOut) {
+		display.focusEvent = SWT.FocusOut;
+		display.runFocusOutEvents ();
+		display.focusEvent = SWT.None;
 	}
-	return 0;
+	return result;
 }
 int hoverProc (int widget) {
 	return 0;
@@ -1001,6 +1031,7 @@ int windowProc (int w, int client_data, int call_data, int continue_to_dispatch)
 		case KEY_PRESS:					return XKeyPress (w, client_data, call_data, continue_to_dispatch);
 		case KEY_RELEASE:					return XKeyRelease (w, client_data, call_data, continue_to_dispatch);
 		case LEAVE_WINDOW:					return XLeaveWindow (w, client_data, call_data, continue_to_dispatch);
+		case PROPERTY_CHANGE:				return XPropertyChange (w, client_data, call_data, continue_to_dispatch);
 		case ACTIVATE_CALLBACK:			return XmNactivateCallback (w, client_data, call_data);
 		case ARM_CALLBACK:					return XmNarmCallback (w, client_data, call_data);
 		case BROWSE_SELECTION_CALLBACK:	return XmNbrowseSelectionCallback (w, client_data, call_data);
@@ -1012,9 +1043,9 @@ int windowProc (int w, int client_data, int call_data, int continue_to_dispatch)
 		case HELP_CALLBACK:				return XmNhelpCallback (w, client_data, call_data);
 		case INCREMENT_CALLBACK:			return XmNincrementCallback (w, client_data, call_data);
 		case MODIFY_VERIFY_CALLBACK:		return XmNmodifyVerifyCallback (w, client_data, call_data);
+		case MULTIPLE_SELECTION_CALLBACK:		return XmNmultipleSelectionCallback (w, client_data, call_data);
 		case PAGE_DECREMENT_CALLBACK:		return XmNpageDecrementCallback (w, client_data, call_data);
 		case PAGE_INCREMENT_CALLBACK:		return XmNpageIncrementCallback (w, client_data, call_data);
-		case SELECTION_CALLBACK:			return XmNselectionCallback (w, client_data, call_data);
 		case TO_BOTTOM_CALLBACK:			return XmNtoBottomCallback (w, client_data, call_data);
 		case TO_TOP_CALLBACK:				return XmNtoTopCallback (w, client_data, call_data);
 		case VALUE_CHANGED_CALLBACK:		return XmNvalueChangedCallback (w, client_data, call_data);
@@ -1047,9 +1078,27 @@ int XFocusChange (int w, int client_data, int call_data, int continue_to_dispatc
 	return 0;
 }
 int XKeyPress (int w, int client_data, int call_data, int continue_to_dispatch) {
+	XKeyEvent xEvent = new XKeyEvent ();
+	OS.memmove (xEvent, call_data, XKeyEvent.sizeof);
+	boolean doit = true;
+	if (xEvent.keycode != 0) {
+		doit = sendKeyEvent (SWT.KeyDown, xEvent);
+	} else {
+		doit = sendIMKeyEvent (SWT.KeyDown, xEvent);
+	}
+	if (!doit) {
+		OS.memmove (continue_to_dispatch, new int [1], 4);
+		return 1;
+	}
 	return 0;
 }
 int XKeyRelease (int w, int client_data, int call_data, int continue_to_dispatch) {
+	XKeyEvent xEvent = new XKeyEvent ();
+	OS.memmove (xEvent, call_data, XKeyEvent.sizeof);
+	if (!sendKeyEvent (SWT.KeyUp, xEvent)) {
+		OS.memmove (continue_to_dispatch, new int [1], 4);
+		return 1;
+	}
 	return 0;
 }
 int XLeaveWindow (int w, int client_data, int call_data, int continue_to_dispatch) {
@@ -1058,6 +1107,9 @@ int XLeaveWindow (int w, int client_data, int call_data, int continue_to_dispatc
 int XPointerMotion (int w, int client_data, int call_data, int continue_to_dispatch) {
 	return 0;
 }
+int XPropertyChange (int w, int client_data, int call_data, int continue_to_dispatch) {
+	return 0;
+}
 int XmNactivateCallback (int w, int client_data, int call_data) {
 	return 0;
 }
@@ -1097,15 +1149,15 @@ int XmNmapCallback (int w, int client_data, int call_data) {
 int XmNmodifyVerifyCallback (int w, int client_data, int call_data) {
 	return 0;
 }
+int XmNmultipleSelectionCallback (int w, int client_data, int call_data) {
+	return 0;
+}
 int XmNpageDecrementCallback (int w, int client_data, int call_data) {
 	return 0;
 }
 int XmNpageIncrementCallback (int w, int client_data, int call_data) {
 	return 0;
 }
-int XmNselectionCallback (int w, int client_data, int call_data) {
-	return 0;
-}
 int XmNtoBottomCallback (int w, int client_data, int call_data) {
 	return 0;
 }
