@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -26,7 +26,8 @@ import org.eclipse.swt.accessibility.*;
  * <dd>LEFT_TO_RIGHT, RIGHT_TO_LEFT</dd>
  * <dt><b>Events:</b>
  * <dd>FocusIn, FocusOut, Help, KeyDown, KeyUp, MouseDoubleClick, MouseDown, MouseEnter,
- *     MouseExit, MouseHover, MouseUp, MouseMove, Move, Paint, Resize</dd>
+ *     MouseExit, MouseHover, MouseUp, MouseMove, Move, Paint, Resize, Traverse,
+ *     DragDetect, MenuDetect</dd>
  * </dl>
  * <p>
  * Only one of LEFT_TO_RIGHT or RIGHT_TO_LEFT may be specified.
@@ -34,8 +35,6 @@ import org.eclipse.swt.accessibility.*;
  * IMPORTANT: This class is intended to be subclassed <em>only</em>
  * within the SWT implementation.
  * </p>
- * 
- * Note: Only one of LEFT_TO_RIGHT and RIGHT_TO_LEFT may be specified.
  */
 public abstract class Control extends Widget implements Drawable {
 	Composite parent;
@@ -348,12 +347,15 @@ public boolean forceFocus () {
 	*/
 	Shell shell = getShell ();
 	if (shell.activeMenu != null) return false;
-	int shellHandle = shell.shellHandle;
-	OS.PtWindowToFront (shellHandle);
+	shell.bringToTop (false);
 	OS.PtContainerGiveFocus (handle, null);
 	return hasFocus ();
 }
 
+void checkBuffered () {
+	style &= ~SWT.DOUBLE_BUFFERED;
+}
+
 /**
  * Returns the preferred size of the receiver.
  * <p>
@@ -378,7 +380,7 @@ public boolean forceFocus () {
  * @see #getBorderWidth
  * @see #getBounds
  * @see #getSize
- * @see #pack
+ * @see #pack(boolean)
  * @see "computeTrim, getClientArea for controls that implement them"
  */
 public Point computeSize (int wHint, int hHint) {
@@ -416,7 +418,7 @@ public Point computeSize (int wHint, int hHint) {
  * @see #getBorderWidth
  * @see #getBounds
  * @see #getSize
- * @see #pack
+ * @see #pack(boolean)
  * @see "computeTrim, getClientArea for controls that implement them"
  */
 public Point computeSize (int wHint, int hHint, boolean changed) {
@@ -463,26 +465,28 @@ Control [] computeTabList () {
 void createWidget (int index) {
 	checkOrientation (parent);
 	super.createWidget (index);
+	checkBuffered ();
 	setZOrder ();
 	realizeWidget ();
 	setDefaultFont ();
 }
 
 int defaultBackground () {
-	Display display = getDisplay ();
 	return display.WIDGET_BACKGROUND;
 }
 
 byte [] defaultFont () {
-	Display display = getDisplay ();
 	return display.TEXT_FONT;
 }
 
 int defaultForeground () {
-	Display display = getDisplay ();
 	return display.WIDGET_FOREGROUND;
 }
 
+boolean drawGripper (int x, int y, int width, int height) {
+	return false;
+}
+
 int drawProc (int widget, int damage) {
 	drawWidget (widget, damage);
 	if (!hooks(SWT.Paint) && !filters (SWT.Paint)) return OS.Pt_CONTINUE;
@@ -533,7 +537,7 @@ int drawProc (int widget, int damage) {
 			event.y = tile.rect_ul_y;
 			event.width = tile.rect_lr_x - tile.rect_ul_x + 1;
 			event.height = tile.rect_lr_y - tile.rect_ul_y + 1;
-			Region region = Region.photon_new (tile.next);
+			Region region = Region.photon_new (display, tile.next);
 			GCData data = new GCData();
 			if (OS.QNX_MAJOR > 6 || (OS.QNX_MAJOR == 6 && (OS.QNX_MINOR > 2 || (OS.QNX_MINOR == 2 && OS.QNX_MICRO >= 1)))) {
 				data.paint = true;
@@ -599,7 +603,7 @@ public Color getBackground () {
 	checkWidget();
 	int [] args = {OS.Pt_ARG_FILL_COLOR, 0, 0};
 	OS.PtGetResources (handle, args.length / 3, args);
-	return Color.photon_new (getDisplay (), args [1]);
+	return Color.photon_new (display, args [1]);
 }
 
 /**
@@ -633,7 +637,7 @@ public Font getFont () {
 		font = new byte [length + 1];
 		OS.memmove (font, ptr, length);
 	}
-	return Font.photon_new (getDisplay (), font);
+	return Font.photon_new (display, font);
 }
 
 /**
@@ -650,7 +654,7 @@ public Color getForeground () {
 	checkWidget();
 	int [] args = {OS.Pt_ARG_COLOR, 0, 0};
 	OS.PtGetResources (handle, args.length / 3, args);
-	return Color.photon_new (getDisplay (), args [1]);
+	return Color.photon_new (display, args [1]);
 }
 
 /**
@@ -684,7 +688,9 @@ public int getBorderWidth () {
 
 /**
  * Returns a rectangle describing the receiver's size and location
- * relative to its parent (or its display if its parent is null).
+ * relative to its parent (or its display if its parent is null),
+ * unless the receiver is a shell. In this case, the location is
+ * relative to the display.
  *
  * @return the receiver's bounding rectangle
  *
@@ -701,22 +707,6 @@ public Rectangle getBounds () {
 	return new Rectangle (area.pos_x, area.pos_y, area.size_w, area.size_h);
 }
 
-/**
- * Returns the display that the receiver was created on.
- *
- * @return the receiver's display
- *
- * @exception SWTException <ul>
- *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
- *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
- * </ul>
- */
-public Display getDisplay () {
-	Composite parent = this.parent;
-	if (parent == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return parent.getDisplay ();
-}
-
 /**
  * Returns <code>true</code> if the receiver is enabled, and
  * <code>false</code> otherwise. A disabled control is typically
@@ -754,7 +744,9 @@ public Object getLayoutData () {
 
 /**
  * Returns a point describing the receiver's location relative
- * to its parent (or its display if its parent is null).
+ * to its parent (or its display if its parent is null), unless
+ * the receiver is a shell. In this case, the point is 
+ * relative to the display. 
  *
  * @return the receiver's location
  *
@@ -791,6 +783,45 @@ public Menu getMenu () {
 	return menu;
 }
 
+/**
+ * Returns the receiver's monitor.
+ * 
+ * @return the receiver's monitor
+ * 
+ * @since 3.0
+ */
+public Monitor getMonitor () {
+	checkWidget();
+	Monitor [] monitors = display.getMonitors ();
+	if (monitors.length == 1) return monitors [0];
+	int index = -1, value = -1;
+	Rectangle bounds = getBounds ();
+	if (this != getShell ()) {
+		bounds = display.map (this.parent, null, bounds);
+	}
+	for (int i=0; i<monitors.length; i++) {
+		Rectangle rect = bounds.intersection (monitors [i].getBounds ());
+		int area = rect.width * rect.height;
+		if (area > 0 && area > value) {
+			index = i;
+			value = area;
+		}
+	}
+	if (index >= 0) return monitors [index];
+	int centerX = bounds.x + bounds.width / 2, centerY = bounds.y + bounds.height / 2;
+	for (int i=0; i<monitors.length; i++) {
+		Rectangle rect = monitors [i].getBounds ();
+		int x = centerX < rect.x ? rect.x - centerX : centerX > rect.x + rect.width ? centerX - rect.x - rect.width : 0;
+		int y = centerY < rect.y ? rect.y - centerY : centerY > rect.y + rect.height ? centerY - rect.y - rect.height : 0;
+		int distance = x * x + y * y;
+		if (index == -1 || distance < value) {
+			index = i;
+			value = distance;
+		} 
+	}
+	return monitors [index];
+}
+
 /**
  * Returns the receiver's parent, which must be a <code>Composite</code>
  * or null when the receiver is a shell that was created with null or
@@ -910,7 +941,7 @@ boolean hasFocus () {
 }
 
 void hookEvents () {
-	int windowProc = getDisplay ().windowProc;
+	int windowProc = display.windowProc;
 	int focusHandle = focusHandle ();
 	OS.PtAddFilterCallback (handle, OS.Ph_EV_KEY, windowProc, OS.Ph_EV_KEY);
 	OS.PtAddEventHandler (handle, OS.Ph_EV_BUT_PRESS, windowProc, OS.Ph_EV_BUT_PRESS);
@@ -953,7 +984,7 @@ public int internal_new_GC (GCData data) {
 	}
 	int [] args = {OS.Pt_ARG_COLOR, 0, 0, OS.Pt_ARG_FILL_COLOR, 0, 0};
 	OS.PtGetResources (handle, args.length / 3, args);
-	data.device = getDisplay ();
+	data.device = display;
 	data.widget = handle;
 	data.topWidget = topHandle ();
 	data.foreground = args [1];
@@ -972,7 +1003,7 @@ public int internal_new_GC (GCData data) {
  * application code.
  * </p>
  *
- * @param handle the platform specific GC handle
+ * @param hDC the platform specific GC handle
  * @param data the platform specific GC data 
  */
 public void internal_dispose_GC (int phGC, GCData data) {
@@ -982,9 +1013,10 @@ public void internal_dispose_GC (int phGC, GCData data) {
 
 /**
  * Returns <code>true</code> if the receiver is enabled and all
- * of the receiver's ancestors are enabled, and <code>false</code>
- * otherwise. A disabled control is typically not selectable from the
- * user interface and draws with an inactive or "grayed" look.
+ * ancestors up to and including the receiver's nearest ancestor
+ * shell are enabled.  Otherwise, <code>false</code> is returned.
+ * A disabled control is typically not selectable from the user
+ * interface and draws with an inactive or "grayed" look.
  *
  * @return the receiver's enabled state
  *
@@ -1000,8 +1032,7 @@ public boolean isEnabled () {
 	return getEnabled () && parent.isEnabled ();
 }
 
-public boolean isFocusAncestor () {
-	Display display = getDisplay ();
+boolean isFocusAncestor () {
 	Control control = display.getFocusControl ();
 	while (control != null && control != this) {
 		control = control.parent;
@@ -1092,8 +1123,8 @@ boolean isTabItem () {
 
 /**
  * Returns <code>true</code> if the receiver is visible and all
- * of the receiver's ancestors are visible and <code>false</code>
- * otherwise.
+ * ancestors up to and including the receiver's nearest ancestor
+ * shell are visible. Otherwise, <code>false</code> is returned.
  *
  * @return the receiver's visibility state
  *
@@ -1109,6 +1140,10 @@ public boolean isVisible () {
 	return OS.PtWidgetIsRealized (handle);
 }
 
+void markLayout (boolean changed, boolean all) {
+	/* Do nothing */
+}
+
 Decorations menuShell () {
 	return parent.menuShell ();
 }
@@ -1120,7 +1155,7 @@ Decorations menuShell () {
  * the top of the drawing order will not be covered by other
  * controls even if they occupy intersecting areas.
  *
- * @param the sibling control (or null)
+ * @param control the sibling control (or null)
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_INVALID_ARGUMENT - if the control has been disposed</li> 
@@ -1129,6 +1164,9 @@ Decorations menuShell () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see Control#moveBelow
+ * @see Composite#getChildren
  */
 public void moveAbove (Control control) {
 	checkWidget();
@@ -1151,7 +1189,7 @@ public void moveAbove (Control control) {
  * the bottom of the drawing order will be covered by all other
  * controls which occupy intersecting areas.
  *
- * @param the sibling control (or null)
+ * @param control the sibling control (or null)
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_INVALID_ARGUMENT - if the control has been disposed</li> 
@@ -1160,6 +1198,9 @@ public void moveAbove (Control control) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see Control#moveAbove
+ * @see Composite#getChildren
  */
 public void moveBelow (Control control) {
 	checkWidget();
@@ -1185,7 +1226,7 @@ public void moveBelow (Control control) {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
- * @see #computeSize
+ * @see #computeSize(int, int, boolean)
  */
 public void pack () {
 	checkWidget();
@@ -1204,12 +1245,14 @@ public void pack () {
  * manager caches can be retained. 
  * </p>
  *
+ * @param changed whether or not the receiver's contents have changed
+ * 
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
- * @see #computeSize
+ * @see #computeSize(int, int, boolean)
  */
 public void pack (boolean changed) {
 	checkWidget();
@@ -1227,23 +1270,32 @@ int Ph_EV_BOUNDARY (int widget, int info) {
 	if (data == 0) return OS.Pt_END;
 	PhPointerEvent_t pe = new PhPointerEvent_t ();
 	OS.memmove (pe, data, PhPointerEvent_t.sizeof);
-	Event event = new Event ();
-	event.time = ev.timestamp;
-	setMouseState (event, pe, ev);
 	switch ((int) ev.subtype) {
 		case OS.Ph_EV_PTR_ENTER:
-		case OS.Ph_EV_PTR_ENTER_FROM_CHILD:
+		case OS.Ph_EV_PTR_ENTER_FROM_CHILD:{
+			Event event = new Event ();
+			event.time = ev.timestamp;
+			setMouseState (event, SWT.MouseEnter, pe, ev);
 			sendEvent (SWT.MouseEnter, event);
 			break;
+		}
 		case OS.Ph_EV_PTR_LEAVE:
-		case OS.Ph_EV_PTR_LEAVE_TO_CHILD:
+		case OS.Ph_EV_PTR_LEAVE_TO_CHILD: {
+			Event event = new Event ();
+			event.time = ev.timestamp;
+			setMouseState (event, SWT.MouseExit, pe, ev);
 			sendEvent (SWT.MouseExit, event);
 			break;
-		case OS.Ph_EV_PTR_STEADY:
+		}
+		case OS.Ph_EV_PTR_STEADY: {
+			Event event = new Event ();
+			event.time = ev.timestamp;
+			setMouseState (event, SWT.MouseHover, pe, ev);
 			postEvent (SWT.MouseHover, event);
 			destroyToolTip (toolTipHandle);
 			toolTipHandle = createToolTip (toolTipText, handle, getFont ().handle);
 			break;
+		}
 		case OS.Ph_EV_PTR_UNSTEADY:
 			destroyToolTip (toolTipHandle);
 			toolTipHandle = 0;
@@ -1270,24 +1322,29 @@ int Ph_EV_BUT_PRESS (int widget, int info) {
 	OS.memmove (pe, data, PhPointerEvent_t.sizeof);
 	Event event = new Event ();
 	event.time = ev.timestamp;
-	setMouseState (event, pe, ev);
+	setMouseState (event, SWT.MouseDown, pe, ev);
 	postEvent (SWT.MouseDown, event);
 	if (pe.click_count == 2) {
 		Event clickEvent = new Event ();
-		clickEvent.time = event.time;
-		clickEvent.x = event.x;
-		clickEvent.y = event.y;
-		clickEvent.button = event.button;
-		clickEvent.stateMask = event.stateMask;
+		setMouseState (clickEvent, SWT.MouseDoubleClick, pe, ev);
 		postEvent (SWT.MouseDoubleClick, clickEvent);
 	}
 	if (event.button == 3) {
-		if (menu != null && !menu.isDisposed ()) {
-			Display display = getDisplay ();
-			display.runDeferredEvents ();
-			menu.setVisible (true);
+		Event menuEvent = new Event ();
+		menuEvent.x = pe.pos_x;
+		menuEvent.y = pe.pos_y;
+		sendEvent (SWT.MenuDetect, menuEvent);
+		if (menuEvent.doit) {
+			if (menu != null && !menu.isDisposed ()) {
+				if (menuEvent.x != event.x || menuEvent.y != event.y) {
+					menu.setLocation (menuEvent.x, menuEvent.y);
+				}
+				menu.setVisible (true);
+			}
 		}
 	}
+	display.dragStartX = pe.pos_x + ev.translation_x;
+	display.dragStartY = pe.pos_y + ev.translation_y;
 	/*
 	* It is possible that the shell may be
 	* disposed at this point.  If this happens
@@ -1322,7 +1379,7 @@ int Ph_EV_BUT_RELEASE (int widget, int info) {
 	OS.memmove (pe, data, PhPointerEvent_t.sizeof);
 	Event event = new Event ();
 	event.time = ev.timestamp;
-	setMouseState (event, pe, ev);
+	setMouseState (event, SWT.MouseUp, pe, ev);
 	postEvent (SWT.MouseUp, event);
 	return OS.Pt_CONTINUE;
 }
@@ -1346,7 +1403,7 @@ int Ph_EV_DRAG (int widget, int info) {
 	OS.memmove (pe, data, PhPointerEvent_t.sizeof);
 	Event event = new Event ();
 	event.time = ev.timestamp;
-	setMouseState (event, pe, ev);
+	setMouseState (event, SWT.MouseMove, pe, ev);
 	postEvent (SWT.MouseMove, event);
 	return OS.Pt_CONTINUE;
 }
@@ -1438,15 +1495,14 @@ int Ph_EV_KEY (int widget, int info) {
 					return OS.Pt_PROCESS;
 				}
 				// widget could be disposed at this point
-				if (isDisposed ()) return OS.Pt_PROCESS;
+				if (isDisposed ()) return OS.Pt_CONSUME;
 			}
 		}
 	}
 
 	Event event = new Event ();
 	event.time = ev.timestamp;
-	setKeyState (event, ke);
-	Display display = getDisplay ();
+	if (!setKeyState (event, type, ke)) return OS.Pt_PROCESS;
 	if (type == SWT.KeyDown) {
 		display.lastKey = event.keyCode;
 		display.lastAscii = event.character;
@@ -1454,8 +1510,10 @@ int Ph_EV_KEY (int widget, int info) {
 		if (event.keyCode == 0) event.keyCode = display.lastKey;
 		if (event.character == 0) event.character = (char) display.lastAscii;
 	}
-	postEvent (type, event);
-	return OS.Pt_PROCESS;
+	sendEvent (type, event);
+	// widget could be disposed at this point
+	if (isDisposed ()) return OS.Pt_CONSUME;
+	return event.doit ? OS.Pt_PROCESS :OS.Pt_CONSUME; 
 }
 
 int Ph_EV_PTR_MOTION (int widget, int info) {
@@ -1479,7 +1537,7 @@ int Ph_EV_PTR_MOTION (int widget, int info) {
 	OS.memmove (pe, data, PhPointerEvent_t.sizeof);
 	Event event = new Event ();
 	event.time = ev.timestamp;
-	setMouseState (event, pe, ev);
+	setMouseState (event, SWT.MouseMove, pe, ev);
 	postEvent (SWT.MouseMove, event);
 	return OS.Pt_CONTINUE;
 }
@@ -1518,7 +1576,7 @@ int Pt_CB_LOST_FOCUS (int widget, int info) {
 	* events.
 	*/
 	if (!shell.isDisposed ()) {
-		Display display = shell.getDisplay ();
+		Display display = shell.display;
 		Control control = display.getFocusControl ();
 		if (control == null || shell != control.getShell () ) {
 			shell.setActiveControl (null);
@@ -1539,6 +1597,10 @@ void realizeWidget() {
 	}
 }
 
+void releaseChild () {
+	parent.removeControl (this);
+}
+
 void releaseWidget () {
 	super.releaseWidget ();
 	if (toolTipHandle != 0) destroyToolTip (toolTipHandle);
@@ -1554,7 +1616,8 @@ void releaseWidget () {
 /**
  * Causes the entire bounds of the receiver to be marked
  * as needing to be redrawn. The next time a paint request
- * is processed, the control will be completely painted.
+ * is processed, the control will be completely painted,
+ * including the background.
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -1562,6 +1625,11 @@ void releaseWidget () {
  * </ul>
  *
  * @see #update
+ * @see PaintListener
+ * @see SWT#Paint
+ * @see SWT#NO_BACKGROUND
+ * @see SWT#NO_REDRAW_RESIZE
+ * @see SWT#NO_MERGE_PAINTS
  */
 public void redraw () {
 	checkWidget();
@@ -1572,11 +1640,12 @@ public void redraw () {
  * Causes the rectangular area of the receiver specified by
  * the arguments to be marked as needing to be redrawn. 
  * The next time a paint request is processed, that area of
- * the receiver will be painted. If the <code>all</code> flag
- * is <code>true</code>, any children of the receiver which
- * intersect with the specified area will also paint their
- * intersecting areas. If the <code>all</code> flag is 
- * <code>false</code>, the children will not be painted.
+ * the receiver will be painted, including the background.
+ * If the <code>all</code> flag is <code>true</code>, any
+ * children of the receiver which intersect with the specified
+ * area will also paint their intersecting areas. If the
+ * <code>all</code> flag is <code>false</code>, the children
+ * will not be painted.
  *
  * @param x the x coordinate of the area to draw
  * @param y the y coordinate of the area to draw
@@ -1590,6 +1659,11 @@ public void redraw () {
  * </ul>
  *
  * @see #update
+ * @see PaintListener
+ * @see SWT#Paint
+ * @see SWT#NO_BACKGROUND
+ * @see SWT#NO_REDRAW_RESIZE
+ * @see SWT#NO_MERGE_PAINTS
  */
 public void redraw (int x, int y, int width, int height, boolean allChildren) {
 	checkWidget ();
@@ -1606,7 +1680,7 @@ public void redraw (int x, int y, int width, int height, boolean allChildren) {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is moved or resized.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1631,7 +1705,7 @@ public void removeControlListener (ControlListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the control gains or loses focus.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1656,7 +1730,7 @@ public void removeFocusListener(FocusListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the help events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1680,7 +1754,7 @@ public void removeHelpListener (HelpListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when keys are pressed and released on the system keyboard.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1705,7 +1779,7 @@ public void removeKeyListener(KeyListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the mouse passes or hovers over controls.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1731,7 +1805,7 @@ public void removeMouseTrackListener(MouseTrackListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when mouse buttons are pressed and released.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1757,7 +1831,7 @@ public void removeMouseListener (MouseListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the mouse moves.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1781,7 +1855,7 @@ public void removeMouseMoveListener(MouseMoveListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the receiver needs to be painted.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1805,7 +1879,7 @@ public void removePaintListener(PaintListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when traversal events occur.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1881,7 +1955,9 @@ int setBounds (int x, int y, int width, int height, boolean move, boolean resize
  * Sets the receiver's size and location to the rectangular
  * area specified by the arguments. The <code>x</code> and 
  * <code>y</code> arguments are relative to the receiver's
- * parent (or its display if its parent is null).
+ * parent (or its display if its parent is null), unless 
+ * the receiver is a shell. In this case, the <code>x</code>
+ * and <code>y</code> arguments are relative to the display.
  * <p>
  * Note: Attempting to set the width or height of the
  * receiver to a negative number will cause that
@@ -2015,7 +2091,8 @@ public void setEnabled (boolean enabled) {
 
 /**
  * Causes the receiver to have the <em>keyboard focus</em>, 
- * such that all keyboard events will be delivered to it.
+ * such that all keyboard events will be delivered to it.  Focus
+ * reassignment will respect applicable platform constraints.
  *
  * @return <code>true</code> if the control got focus, and <code>false</code> if it was unable to.
  *
@@ -2028,6 +2105,7 @@ public void setEnabled (boolean enabled) {
  */
 public boolean setFocus () {
 	checkWidget();
+	if ((style & SWT.NO_FOCUS) != 0) return false;
 	return forceFocus ();
 }
 
@@ -2063,7 +2141,6 @@ void setBackgroundPixel (int pixel) {
 }
 
 void setDefaultFont () {
-	Display display = getDisplay ();
 	if (display.defaultFont != null) setFont (defaultFont ());
 }
 
@@ -2160,7 +2237,9 @@ public void setLayoutData (Object layoutData) {
 /**
  * Sets the receiver's location to the point specified by
  * the arguments which are relative to the receiver's
- * parent (or its display if its parent is null).
+ * parent (or its display if its parent is null), unless 
+ * the receiver is a shell. In this case, the point is 
+ * relative to the display. 
  *
  * @param x the new x coordinate for the receiver
  * @param y the new y coordinate for the receiver
@@ -2177,8 +2256,10 @@ public void setLocation (int x, int y) {
 
 /**
  * Sets the receiver's location to the point specified by
- * the argument which is relative to the receiver's
- * parent (or its display if its parent is null).
+ * the arguments which are relative to the receiver's
+ * parent (or its display if its parent is null), unless 
+ * the receiver is a shell. In this case, the point is 
+ * relative to the display. 
  *
  * @param location the new location for the receiver
  *
@@ -2233,11 +2314,12 @@ public void setMenu (Menu menu) {
  * @return <code>true</code> if the parent is changed and <code>false</code> otherwise.
  *
  * @exception IllegalArgumentException <ul>
- *    <li>ERROR_INVALID_ARGUMENT - if the argument has been disposed</li> 
+ *    <li>ERROR_INVALID_ARGUMENT - if the argument has been disposed</li>
+ *    <li>ERROR_NULL_ARGUMENT - if the parent is <code>null</code></li> 
  * </ul>
- * @exception SWTError <ul>
- *		<li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread</li>
- *		<li>ERROR_WIDGET_DISPOSED when the widget has been disposed</li>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  *	</ul>
  */
 public boolean setParent (Composite parent) {
@@ -2255,7 +2337,6 @@ public boolean setParent (Composite parent) {
  * </p>
  *
  * @param size the new size for the receiver
- * @param height the new height for the receiver
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the point is null</li>
@@ -2277,7 +2358,7 @@ boolean setTabGroupFocus () {
 
 boolean setTabItemFocus () {
 	if (!isShowing ()) return false;
-	return setFocus ();
+	return forceFocus ();
 }
 
 boolean setRadioSelection (boolean value) {
@@ -2290,7 +2371,8 @@ boolean setRadioSelection (boolean value) {
  * can occur in the receiver until the flag is set to true.
  * Graphics operations that occurred while the flag was
  * <code>false</code> are lost. When the flag is set to <code>true</code>,
- * the entire widget is marked as needing to be redrawn.
+ * the entire widget is marked as needing to be redrawn.  Nested calls
+ * to this method are stacked.
  * <p>
  * Note: This operation is a hint and may not be supported on some
  * platforms or for some widgets.
@@ -2303,7 +2385,7 @@ boolean setRadioSelection (boolean value) {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  * 
- * @see #redraw
+ * @see #redraw(int, int, int, int, boolean)
  * @see #update
  */
 public void setRedraw (boolean redraw) {
@@ -2354,15 +2436,23 @@ public void setSize (int width, int height) {
 public void setVisible (boolean visible) {
 	checkWidget ();
 	int topHandle = topHandle ();
+	int oldFlags = OS.PtWidgetFlags (topHandle);
 	int flags = visible ? 0 : OS.Pt_DELAY_REALIZE;
 	OS.PtSetResource (topHandle, OS.Pt_ARG_FLAGS, flags, OS.Pt_DELAY_REALIZE);
-	if (parent != null && !OS.PtWidgetIsRealized (parent.handle)) return;
-	if (visible == OS.PtWidgetIsRealized (topHandle)) return;
+	if ((oldFlags & OS.Pt_DELAY_REALIZE) == flags) return;
 	if (visible) {
 		sendEvent (SWT.Show);
+		if (isDisposed ()) return;
 		OS.PtRealizeWidget (topHandle);
 	} else {
 		OS.PtUnrealizeWidget (topHandle);
+		/*
+		 * It is possible (but unlikely), that application
+		 * code could have disposed the widget in the FocusOut
+		 * event that is triggered by PtUnrealizeWidget if the widget
+		 * being hidden has focus.  If this happens, just return.
+		 */
+		if (isDisposed ()) return;
 		sendEvent(SWT.Hide);
 	}
 }
@@ -2535,9 +2625,9 @@ boolean translateTraversal (int key_sym, PhKeyEvent_t phEvent) {
 			return false;
 	}
 	Event event = new Event ();
-	event.doit = (code & detail) != 0 || (phEvent.key_mods & OS.Pk_KM_Ctrl) != 0;
+	event.doit = (code & detail) != 0;
 	event.detail = detail;
-	setKeyState (event, phEvent);
+	if (!setKeyState (event, SWT.Traverse, phEvent)) return false;
 	Shell shell = getShell ();
 	Control control = this;
 	do {
@@ -2567,7 +2657,7 @@ boolean traverse (Event event) {
 	* event processing.
 	*/	
 	sendEvent (SWT.Traverse, event);
-	if (isDisposed ()) return false;
+	if (isDisposed ()) return true;
 	if (!event.doit) return false;
 	switch (event.detail) {
 		case SWT.TRAVERSE_NONE:				return true;
@@ -2601,7 +2691,6 @@ boolean traverse (Event event) {
  */
 public boolean traverse (int traversal) {
 	checkWidget ();
-	if (!isFocusControl () && !setFocus ()) return false;
 	Event event = new Event ();
 	event.doit = true;
 	event.detail = traversal;
@@ -2633,7 +2722,7 @@ boolean traverseGroup (boolean next) {
 	while ((index = ((index + offset + length) % length)) != start) {
 		Control control = list [index];
 		if (!control.isDisposed () && control.setTabGroupFocus ()) {
-			if (!isDisposed () && !isFocusControl ()) return true;
+			return true;
 		}
 	}
 	if (group.isDisposed ()) return false;
@@ -2654,6 +2743,7 @@ boolean traverseItem (boolean next) {
 	* or out events.  Ensure that a disposed widget is
 	* not accessed.
 	*/
+	if (index == length) return false;
 	int start = index, offset = (next) ? 1 : -1;
 	while ((index = (index + offset + length) % length) != start) {
 		Control child = children [index];
@@ -2686,13 +2776,19 @@ boolean traverseReturn () {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
- * @see #redraw
+ * @see #redraw(int, int, int, int, boolean)
+ * @see PaintListener
+ * @see SWT#Paint
  */
 public void update () {
 	checkWidget();
 	OS.PtFlush ();
 }
 
+void updateLayout (boolean all) {
+	/* Do nothing */
+}
+
 int widgetClass () {
 	return 0;
 }
