@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -89,14 +89,14 @@ import org.eclipse.swt.graphics.*;
  * @see #wake
  * @see #readAndDispatch
  * @see #sleep
- * @see #dispose
+ * @see Device#dispose
  */
 public class Display extends Device {
 
 	/* TEMPORARY CODE FOR EMULATED TABLE */
 	int textHighlightThickness = 0;
 	
-	/* TEMPORARY HACK FOR PHOTON */
+	/* TEMPORARY CODE FOR EMBEDDED */
 	public boolean embedded;
 	
 	/* Photon Only Public Fields */
@@ -120,6 +120,12 @@ public class Display extends Device {
 	/* Display Shutdown */
 	Runnable [] disposeList;
 	
+	/* System Tray */
+	Tray tray;
+	
+	/* Drag origin */
+	int dragStartX, dragStartY;
+
 	/* Timers */
 	int [] timerIds;
 	Runnable [] timerList;
@@ -139,6 +145,8 @@ public class Display extends Device {
 		{OS.Pk_Shift_R,   SWT.SHIFT},
 		{OS.Pk_Control_L, SWT.CONTROL},
 		{OS.Pk_Control_R, SWT.CONTROL},
+//		{OS.Pk_????, SWT.COMMAND},
+//		{OS.Pk_????, SWT.COMMAND},
 			
 //		{OS.VK_LBUTTON, SWT.BUTTON1},
 //		{OS.VK_MBUTTON, SWT.BUTTON3},
@@ -160,7 +168,6 @@ public class Display extends Device {
 		{OS.Pk_Return,		SWT.CR},
 		{OS.Pk_Delete,		SWT.DEL},
 		{OS.Pk_Escape,		SWT.ESC},
-		{OS.Pk_Cancel,		SWT.ESC},
 		{OS.Pk_Linefeed,	SWT.LF},
 		{OS.Pk_Tab,			SWT.TAB},
 		{OS.Pk_KP_Tab,		SWT.TAB},
@@ -178,27 +185,38 @@ public class Display extends Device {
 		{OS.Pk_F10,	SWT.F10},
 		{OS.Pk_F11,	SWT.F11},
 		{OS.Pk_F12,	SWT.F12},
-		
-		
+		{OS.Pk_F13,	SWT.F13},
+		{OS.Pk_F14,	SWT.F14},
+		{OS.Pk_F15,	SWT.F15},
+
 		/* Numeric Keypad Keys */
-		/*
-		{OS.XK_KP_Add,		SWT.KP_PLUS},
-		{OS.XK_KP_Subtract,	SWT.KP_MINUS},
-		{OS.XK_KP_Multiply,	SWT.KP_TIMES},
-		{OS.XK_KP_Divide,	SWT.KP_DIVIDE},
-		{OS.XK_KP_Decimal,	SWT.KP_PERIOD},
-		{OS.XK_KP_Enter,	SWT.KP_ENTER},
-		{OS.XK_KP_0,		SWT.KP_0},
-		{OS.XK_KP_1,		SWT.KP_1},
-		{OS.XK_KP_2,		SWT.KP_2},
-		{OS.XK_KP_3,		SWT.KP_3},
-		{OS.XK_KP_4,		SWT.KP_4},
-		{OS.XK_KP_5,		SWT.KP_5},
-		{OS.XK_KP_6,		SWT.KP_6},
-		{OS.XK_KP_7,		SWT.KP_7},
-		{OS.XK_KP_8,		SWT.KP_8},
-		{OS.XK_KP_9,		SWT.KP_9},
-		*/
+		{OS.Pk_KP_Multiply,	SWT.KEYPAD_MULTIPLY},
+		{OS.Pk_KP_Add,		SWT.KEYPAD_ADD},
+		{OS.Pk_KP_Enter,	SWT.KEYPAD_CR},
+		{OS.Pk_KP_Subtract,	SWT.KEYPAD_SUBTRACT},
+		{OS.Pk_KP_Decimal,	SWT.KEYPAD_DECIMAL},
+		{OS.Pk_KP_Divide,	SWT.KEYPAD_DIVIDE},
+		{OS.Pk_KP_0,		SWT.KEYPAD_0},
+		{OS.Pk_KP_1,		SWT.KEYPAD_1},
+		{OS.Pk_KP_2,		SWT.KEYPAD_2},
+		{OS.Pk_KP_3,		SWT.KEYPAD_3},
+		{OS.Pk_KP_4,		SWT.KEYPAD_4},
+		{OS.Pk_KP_5,		SWT.KEYPAD_5},
+		{OS.Pk_KP_6,		SWT.KEYPAD_6},
+		{OS.Pk_KP_7,		SWT.KEYPAD_7},
+		{OS.Pk_KP_8,		SWT.KEYPAD_8},
+		{OS.Pk_KP_9,		SWT.KEYPAD_9},
+		{OS.Pk_KP_Equal,	SWT.KEYPAD_EQUAL},
+
+		/* Other keys */
+		{OS.Pk_Caps_Lock,	SWT.CAPS_LOCK},
+		{OS.Pk_Num_Lock,	SWT.NUM_LOCK},
+		{OS.Pk_Scroll_Lock,	SWT.SCROLL_LOCK},
+		{OS.Pk_Pause,		SWT.PAUSE},
+		{OS.Pk_Break,		SWT.BREAK},
+		{OS.Pk_Print,		SWT.PRINT_SCREEN},
+		{OS.Pk_Help,		SWT.HELP},
+
 	};
 	
 	/* Multiple Displays. */
@@ -210,21 +228,18 @@ public class Display extends Device {
 	int PtButton;
 	int PtList;
 	int PtLabel;
-	int PtWindow;
 	int PtToggleButton;
 	int PtComboBox;
 	int PtText;
 	int PtMultiText;
 	int PtScrollbar;
-	int PtScrollContainer;
-	int PtScrollArea;
 	int PtContainer;
 	int PtProgress;
 	int PtPanelGroup;
-	int PtPane;
 	int PtSlider;
 	int PtSeparator;
 	int PtToolbar;
+	int PtNumericInteger;
 				
 	/* Colors */
 	int WIDGET_DARK_SHADOW, WIDGET_NORMAL_SHADOW, WIDGET_LIGHT_SHADOW;
@@ -235,6 +250,9 @@ public class Display extends Device {
 	/* Fonts */
 	byte [] defaultFont;
 	byte [] TEXT_FONT, LIST_FONT, TITLE_FONT, GAUGE_FONT, GROUP_FONT;
+
+	/* System Cursors */
+	Cursor [] cursors = new Cursor [SWT.CURSOR_HAND + 1];
 	
 	/* Images */
 	int nullImage;
@@ -295,7 +313,7 @@ static void setDevice (Device device) {
  * </p>
  *
  * @exception SWTException <ul>
- *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if called from a thread that already created an existing display</li>
  *    <li>ERROR_INVALID_SUBCLASS - if this class is not an allowed subclass</li>
  * </ul>
  *
@@ -308,6 +326,11 @@ public Display () {
 	this (null);
 }
 
+/**
+ * Constructs a new instance of this class using the parameter.
+ * 
+ * @param data the device data
+ */
 public Display (DeviceData data) {
 	super (data);
 }
@@ -315,9 +338,19 @@ public Display (DeviceData data) {
 /**
  * Adds the listener to the collection of listeners who will
  * be notifed when an event of the given type occurs anywhere
- * in SWT. When the event does occur, the listener is notified
- * by sending it the <code>handleEvent()</code> message.
- *
+ * in a widget. When the event does occur, the listener is
+ * notified by sending it the <code>handleEvent()</code> message.
+ * <p>
+ * Setting the type of an event to <code>SWT.None</code> from
+ * within the <code>handleEvent()</code> method can be used to
+ * change the event type and stop subsequent Java listeners
+ * from running. Because event filters run before other listeners,
+ * event filters can both block other listeners and set arbitrary
+ * fields within an event. For this reason, event filters are both
+ * powerful and dangerous. They should generally be avoided for
+ * performance, debugging and code maintenance reasons.
+ * </p>
+ * 
  * @param eventType the type of event to listen for
  * @param listener the listener which should be notified when the event occurs
  *
@@ -326,15 +359,16 @@ public Display (DeviceData data) {
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
  *
  * @see Listener
  * @see #removeFilter
  * @see #removeListener
  * 
- * @since 2.1 
+ * @since 3.0 
  */
-void addFilter (int eventType, Listener listener) {
+public void addFilter (int eventType, Listener listener) {
 	checkDevice ();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (filterTable == null) filterTable = new EventTable ();
@@ -375,9 +409,16 @@ public void addListener (int eventType, Listener listener) {
  * be invoked by the user-interface thread at the next 
  * reasonable opportunity. The caller of this method continues 
  * to run in parallel, and is not notified when the
- * runnable has completed.
+ * runnable has completed.  Specifying <code>null</code> as the
+ * runnable simply wakes the user-interface thread when run.
+ * <p>
+ * Note that at the time the runnable is invoked, widgets 
+ * that have the receiver as their display may have been
+ * disposed. Therefore, it is necessary to check for this
+ * case inside the runnable before accessing the widget.
+ * </p>
  *
- * @param runnable code to run on the user-interface thread.
+ * @param runnable code to run on the user-interface thread or <code>null</code>
  *
  * @exception SWTException <ul>
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
@@ -405,14 +446,16 @@ public void beep () {
 }
 
 protected void checkDevice () {
-	if (!isValidThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
+	if (thread == null) error (SWT.ERROR_WIDGET_DISPOSED);
+	if (thread != Thread.currentThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
 	if (isDisposed ()) error (SWT.ERROR_DEVICE_DISPOSED);
 }
 
-static synchronized void checkDisplay (Thread thread) {
+static synchronized void checkDisplay (Thread thread, boolean multiple) {
 	for (int i=0; i<Displays.length; i++) {
-		if (Displays [i] != null && Displays [i].thread == thread) {
-			SWT.error (SWT.ERROR_THREAD_INVALID_ACCESS);
+		if (Displays [i] != null) {
+			if (!multiple) SWT.error (SWT.ERROR_NOT_IMPLEMENTED, null, " [multiple displays]");
+			if (Displays [i].thread == thread) SWT.error (SWT.ERROR_THREAD_INVALID_ACCESS);
 		}
 	}
 }
@@ -442,7 +485,7 @@ protected void checkSubclass () {
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
  *
- * @see #dispose
+ * @see Device#dispose
  * 
  * @since 2.0
  */
@@ -493,7 +536,7 @@ String convertToLf (String text) {
  */
 protected void create (DeviceData data) {
 	checkSubclass ();
-	checkDisplay (thread = Thread.currentThread ());
+	checkDisplay (thread = Thread.currentThread (), false);
 	createDisplay (data);
 	register ();
 	if (Default == null) Default = this;
@@ -518,7 +561,7 @@ synchronized void deregister () {
  * <p>
  * This method is called after <code>release</code>.
  * </p>
- * @see #dispose
+ * @see Device#dispose
  * @see #release
  */
 protected void destroy () {
@@ -534,7 +577,8 @@ void destroyDisplay () {
 /**
  * Causes the <code>run()</code> method of the runnable to
  * be invoked by the user-interface thread just before the
- * receiver is disposed.
+ * receiver is disposed.  Specifying a <code>null</code> runnable
+ * is ignored.
  *
  * @param runnable code to run at dispose time.
  * 
@@ -580,7 +624,9 @@ void error (int code) {
 /**
  * Returns the display which the given thread is the
  * user-interface thread for, or null if the given thread
- * is not a user-interface thread for any display.
+ * is not a user-interface thread for any display.  Specifying
+ * <code>null</code> as the thread will return <code>null</code>
+ * for the display. 
  *
  * @param thread the user-interface thread
  * @return the display for the given thread
@@ -610,6 +656,10 @@ boolean filters (int eventType) {
  * the instance of the <code>Widget</code> subclass which
  * represents it in the currently running application, if
  * such exists, or null if no matching widget can be found.
+ * <p>
+ * <b>IMPORTANT:</b> This method should not be called from
+ * application code. The arguments are platform-specific.
+ * </p>
  *
  * @param handle the handle for the widget
  * @return the SWT widget that the handle represents
@@ -624,6 +674,32 @@ public Widget findWidget (int handle) {
 	return WidgetTable.get (handle);
 }
 
+/**
+ * Given the operating system handle for a widget,
+ * and widget-specific id, returns the instance of
+ * the <code>Widget</code> subclass which represents
+ * the handle/id pair in the currently running application,
+ * if such exists, or null if no matching widget can be found.
+ * <p>
+ * <b>IMPORTANT:</b> This method should not be called from
+ * application code. The arguments are platform-specific.
+ * </p>
+ *
+ * @param handle the handle for the widget
+ * @param id the id for the subwidget (usually an item)
+ * @return the SWT widget that the handle/id pair represents
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
+ * </ul>
+ * 
+ * @since 3.1
+ */
+public Widget findWidget (int handle, int id) {
+	return null;
+}
+
 /**
  * Returns the currently active <code>Shell</code>, or null
  * if no shell belonging to the currently running application
@@ -696,6 +772,23 @@ public Control getCursorControl () {
 	return null;
 }
 
+/**
+ * Returns an array containing the recommended cursor sizes.
+ *
+ * @return the array of cursor sizes
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
+ * </ul>
+ * 
+ * @since 3.0
+ */
+public Point [] getCursorSizes () {
+	checkDevice ();
+	return new Point [] {new Point (16, 16), new Point (32, 32)};
+}
+
 /**
  * Returns the location of the on-screen pointer relative
  * to the top left corner of the screen.
@@ -746,9 +839,10 @@ public static synchronized Display getDefault () {
 /**
  * On platforms which support it, sets the application name
  * to be the argument. On Motif, for example, this can be used
- * to set the name used for resource lookup.
+ * to set the name used for resource lookup.  Specifying
+ * <code>null</code> for the name clears it.
  *
- * @param name the new app name
+ * @param name the new app name or <code>null</code>
  */
 public static void setAppName (String name) {
 	/* Do nothing */
@@ -829,9 +923,30 @@ int getLastEventTime () {
 }
 
 /**
- * Returns the maximum allowed depth of icons on this display.
- * On some platforms, this may be different than the actual
- * depth of the display.
+ * Returns true when the high contrast mode is enabled.
+ * Otherwise, false is returned.
+ * <p>
+ * Note: This operation is a hint and is not supported on
+ * platforms that do not have this concept.
+ * </p>
+ *
+ * @return the high contrast mode
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
+ * </ul>
+ * 
+ * @since 3.0
+ */
+public boolean getHighContrast () {
+	checkDevice ();
+	return false;
+}
+
+/**
+ * Returns the maximum allowed depth of icons on this display, in bits per pixel.
+ * On some platforms, this may be different than the actual depth of the display.
  *
  * @return the maximum icon depth
  *
@@ -839,14 +954,95 @@ int getLastEventTime () {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
+ * 
+ * @see Device#getDepth
  */
 public int getIconDepth () {
 	return getDepth ();
 }
 
 /**
- * Returns an array containing all shells which have not been
- * disposed and have the receiver as their display.
+ * Returns an array containing the recommended icon sizes.
+ *
+ * @return the array of icon sizes
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
+ * </ul>
+ * 
+ * @see Decorations#setImages(Image[])
+ * 
+ * @since 3.0
+ */
+public Point [] getIconSizes () {
+	checkDevice ();
+	return new Point [] {new Point (15, 15), new Point (43, 43)};	
+}
+
+int getMessageCount () {
+	return synchronizer.getMessageCount ();
+}
+
+/**
+ * Returns an array of monitors attached to the device.
+ * 
+ * @return the array of monitors
+ * 
+ * @since 3.0
+ */
+public Monitor [] getMonitors () {
+	checkDevice ();	
+	int cnt = OS.PhQueryRids (0, 0, 0, OS.Ph_GRAFX_REGION, 0, 0, null, null, 0);
+	int [] rids = new int [cnt];
+	cnt = OS.PhQueryRids (0, 0, 0, OS.Ph_GRAFX_REGION, 0, 0, null, rids, rids.length);
+	PhRect_t rect = new PhRect_t ();
+	Monitor [] monitors = new Monitor [cnt];
+	for (int i = 0; i < cnt; i++) {
+		Monitor monitor = new Monitor ();
+		monitor.handle = rids [i];
+		OS.PhWindowQueryVisible (OS.Ph_QUERY_CONSOLE, rids [i], OS.PhInputGroup (0), rect);
+		monitor.x = rect.ul_x;
+		monitor.y = rect.ul_y;
+		monitor.width = rect.lr_x - rect.ul_x + 1;
+		monitor.height = rect.lr_y - rect.ul_y + 1;
+		OS.PhWindowQueryVisible (OS.Ph_QUERY_WORKSPACE, rids [i], OS.PhInputGroup (0), rect);
+		monitor.clientX = rect.ul_x;
+		monitor.clientY = rect.ul_y;
+		monitor.clientWidth = rect.lr_x - rect.ul_x + 1;
+		monitor.clientHeight = rect.lr_y - rect.ul_y + 1;
+		monitors [i] = monitor;
+	}
+	return monitors;
+}
+
+/**
+ * Returns the primary monitor for that device.
+ * 
+ * @return the primary monitor
+ * 
+ * @since 3.0
+ */
+public Monitor getPrimaryMonitor () {
+	checkDevice ();
+	/*
+	* Note.  Photon does not define a primary monitor.
+	* The workaround is to arbitrarily return the first
+	* monitor whose coordinates are (0, 0), or the first
+	* monitor returned by getMonitors().
+	*/
+	Monitor [] monitors = getMonitors ();
+	if (monitors.length == 1) return monitors [0];
+	for (int i = 0; i < monitors.length; i++) {
+		Monitor monitor = monitors [i];
+		if (monitor.x == 0 && monitor.y == 0) return monitor;
+	}
+	return monitors [0];
+}
+
+/**
+ * Returns a (possibly empty) array containing all shells which have
+ * not been disposed and have the receiver as their display.
  *
  * @return the receiver's shells
  *
@@ -867,7 +1063,7 @@ public Shell [] getShells () {
 	Shell [] shells = WidgetTable.shells ();
 	for (int i=0; i<shells.length; i++) {
 		Shell shell = shells [i];
-		if (!shell.isDisposed () && this == shell.getDisplay ()) {
+		if (!shell.isDisposed () && this == shell.display) {
 			count++;
 		}
 	}
@@ -876,7 +1072,7 @@ public Shell [] getShells () {
 	Shell [] result = new Shell [count];
 	for (int i=0; i<shells.length; i++) {
 		Shell shell = shells [i];
-		if (!shell.isDisposed () && this == shell.getDisplay ()) {
+		if (!shell.isDisposed () && this == shell.display) {
 			result [index++] = shell;
 		}
 	}
@@ -931,6 +1127,57 @@ public Color getSystemColor (int id) {
 	return Color.photon_new (this, color);
 }
 
+/**
+ * Returns the matching standard platform cursor for the given
+ * constant, which should be one of the cursor constants
+ * specified in class <code>SWT</code>. This cursor should
+ * not be free'd because it was allocated by the system,
+ * not the application.  A value of <code>null</code> will
+ * be returned if the supplied constant is not an swt cursor
+ * constant. 
+ *
+ * @param id the swt cursor constant
+ * @return the corresponding cursor or <code>null</code>
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
+ * </ul>
+ *
+ * @see SWT#CURSOR_ARROW
+ * @see SWT#CURSOR_WAIT
+ * @see SWT#CURSOR_CROSS
+ * @see SWT#CURSOR_APPSTARTING
+ * @see SWT#CURSOR_HELP
+ * @see SWT#CURSOR_SIZEALL
+ * @see SWT#CURSOR_SIZENESW
+ * @see SWT#CURSOR_SIZENS
+ * @see SWT#CURSOR_SIZENWSE
+ * @see SWT#CURSOR_SIZEWE
+ * @see SWT#CURSOR_SIZEN
+ * @see SWT#CURSOR_SIZES
+ * @see SWT#CURSOR_SIZEE
+ * @see SWT#CURSOR_SIZEW
+ * @see SWT#CURSOR_SIZENE
+ * @see SWT#CURSOR_SIZESE
+ * @see SWT#CURSOR_SIZESW
+ * @see SWT#CURSOR_SIZENW
+ * @see SWT#CURSOR_UPARROW
+ * @see SWT#CURSOR_IBEAM
+ * @see SWT#CURSOR_NO
+ * @see SWT#CURSOR_HAND
+ * 
+ * @since 3.0
+ */
+public Cursor getSystemCursor (int id) {
+	checkDevice ();
+	if (!(0 <= id && id < cursors.length)) return null;
+	if (cursors [id] == null) {
+		cursors [id] = new Cursor (this, id);
+	}
+	return cursors [id];
+}
+
 /**
  * Returns a reasonable font for applications to use.
  * On some platforms, this will match the "default font"
@@ -958,6 +1205,54 @@ public Font getSystemFont () {
 	return Font.photon_new (this, font);
 }
 
+/**
+ * Returns the matching standard platform image for the given
+ * constant, which should be one of the icon constants
+ * specified in class <code>SWT</code>. This image should
+ * not be free'd because it was allocated by the system,
+ * not the application.  A value of <code>null</code> will
+ * be returned either if the supplied constant is not an
+ * swt icon constant or if the platform does not define an
+ * image that corresponds to the constant. 
+ *
+ * @param id the swt icon constant
+ * @return the corresponding image or <code>null</code>
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
+ * </ul>
+ *
+ * @see SWT#ICON_ERROR
+ * @see SWT#ICON_INFORMATION
+ * @see SWT#ICON_QUESTION
+ * @see SWT#ICON_WARNING
+ * @see SWT#ICON_WORKING
+ * 
+ * @since 3.0
+ */
+public Image getSystemImage (int id) {
+	checkDevice ();
+	return null;
+}
+
+/**
+ * Returns the single instance of the system tray or null
+ * when there is no system tray available for the platform.
+ *
+ * @return the system tray or <code>null</code>
+ * 
+ * @exception SWTException <ul>
+ *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
+ * </ul>
+ *
+ * @since 3.0
+ */
+public Tray getSystemTray () {
+	checkDevice ();
+	return null;
+}
+
 /**
  * Returns the thread that has invoked <code>syncExec</code>
  * or null if no such runnable is currently being invoked by
@@ -1101,27 +1396,29 @@ void initializeWidgetClasses () {
 		OS.PtCreateWidgetClass (OS.PtSlider (), 0, args.length / 3, args), 0, 0,
 		OS.PtCreateWidgetClass (OS.PtSeparator (), 0, args.length / 3, args), 0, 0,
 		OS.PtCreateWidgetClass (OS.PtToolbar (), 0, args.length / 3, args), 0, 0,
+		OS.PtCreateWidgetClass (OS.PtNumericInteger (), 0, args.length / 3, args), 0, 0,
 	};
 	ClassesPtr = OS.malloc (buffer.length * 4);
 	OS.memmove (ClassesPtr, buffer, buffer.length * 4);
 	PtButton = ClassesPtr;
 	PtList = ClassesPtr + 12;
 	PtLabel = ClassesPtr + 24;
-	PtWindow = ClassesPtr + 36;
+	//PtWindow = ClassesPtr + 36;
 	PtToggleButton = ClassesPtr + 48;
 	PtComboBox = ClassesPtr + 60;
 	PtText = ClassesPtr + 72;
 	PtMultiText = ClassesPtr + 84;	
 	PtScrollbar = ClassesPtr + 96;
-	PtScrollContainer = ClassesPtr + 108;
-	PtScrollArea = ClassesPtr + 120;
+	//PtScrollContainer = ClassesPtr + 108;
+	//PtScrollArea = ClassesPtr + 120;
 	PtContainer = ClassesPtr + 132;
 	PtProgress = ClassesPtr + 144;
 	PtPanelGroup = ClassesPtr + 156;
-	PtPane = ClassesPtr + 168;
+	//PtPane = ClassesPtr + 168;
 	PtSlider = ClassesPtr + 180;
 	PtSeparator = ClassesPtr + 192;
 	PtToolbar = ClassesPtr + 204;
+	PtNumericInteger = ClassesPtr + 216;
 }
 
 void initializeWidgetColors () {
@@ -1238,7 +1535,7 @@ void initializeImages () {
 int inputProc (int data, int rcvid, int message, int size) {
 	if (embedded) {
 		runDeferredEvents ();
-		runAsyncMessages ();
+		if (runAsyncMessages (false)) wakeThread ();
 	}
 	return OS.Pt_CONTINUE;
 }
@@ -1256,12 +1553,12 @@ int inputProc (int data, int rcvid, int message, int size) {
  * @param data the platform specific GC data 
  * @return the platform specific GC handle
  * 
- * @exception SWTError <ul>
- *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
+ * @exception SWTError <ul>
+ *    <li>ERROR_NO_HANDLES if a handle could not be obtained for gc creation</li>
+ * </ul>
  */
 public int internal_new_GC (GCData data) {
 	if (isDisposed()) SWT.error(SWT.ERROR_DEVICE_DISPOSED);
@@ -1287,7 +1584,7 @@ public int internal_new_GC (GCData data) {
  * application code.
  * </p>
  *
- * @param handle the platform specific GC handle
+ * @param hDC the platform specific GC handle
  * @param data the platform specific GC data 
  */
 public void internal_dispose_GC (int phGC, GCData data) {
@@ -1305,7 +1602,7 @@ boolean isValidThread () {
  * Applications may have associated arbitrary objects with the
  * receiver in this fashion. If the objects stored in the
  * properties need to be notified when the display is disposed
- * of, it is the application's responsibility provide a
+ * of, it is the application's responsibility to provide a
  * <code>disposeExec()</code> handler which does so.
  * </p>
  *
@@ -1320,8 +1617,8 @@ boolean isValidThread () {
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
  *
- * @see #setData
- * @see #disposeExec
+ * @see #setData(String, Object)
+ * @see #disposeExec(Runnable)
  */
 public Object getData (String key) {
 	checkDevice ();
@@ -1342,7 +1639,7 @@ public Object getData (String key) {
  * Applications may put arbitrary objects in this field. If
  * the object stored in the display specific data needs to
  * be notified when the display is disposed of, it is the
- * application's responsibility provide a
+ * application's responsibility to provide a
  * <code>disposeExec()</code> handler which does so.
  * </p>
  *
@@ -1353,8 +1650,8 @@ public Object getData (String key) {
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
  *
- * @see #setData
- * @see #disposeExec
+ * @see #setData(Object)
+ * @see #disposeExec(Runnable)
  */
 public Object getData () {
 	checkDevice ();
@@ -1391,10 +1688,12 @@ static boolean isValidClass (Class clazz) {
  * @param point to be mapped 
  * @return point with mapped coordinates 
  * 
- * @exception SWTException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the rectangle is null</li> 
- *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the point is null</li>
  *    <li>ERROR_INVALID_ARGUMENT - if the Control from or the Control to have been disposed</li> 
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
  * 
@@ -1428,13 +1727,15 @@ public Point map (Control from, Control to, Point point) {
  * 
  * @param from the source <code>Control</code> or <code>null</code>
  * @param to the destination <code>Control</code> or <code>null</code>
- * @param int x coordinates to be mapped
- * @param int y coordinates to be mapped
+ * @param x coordinates to be mapped
+ * @param y coordinates to be mapped
  * @return point with mapped coordinates
  * 
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_INVALID_ARGUMENT - if the Control from or the Control to have been disposed</li> 
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
- *    <li>ERROR_INVALID_ARGUMENT - if the Control from or the Control to have been disposed</li> 
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
  * 
@@ -1485,10 +1786,12 @@ public Point map (Control from, Control to, int x, int y) {
  * @param rectangle to be mapped
  * @return rectangle with mapped coordinates
  * 
- * @exception SWTException <ul>
+ * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the rectangle is null</li>
- *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  *    <li>ERROR_INVALID_ARGUMENT - if the Control from or the Control to have been disposed</li> 
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
  * 
@@ -1522,15 +1825,17 @@ public Rectangle map (Control from, Control to, Rectangle rectangle) {
  * 
  * @param from the source <code>Control</code> or <code>null</code>
  * @param to the destination <code>Control</code> or <code>null</code>
- * @param int x coordinates to be mapped
- * @param int y coordinates to be mapped
- * @param int width coordinates to be mapped
- * @param int heigth coordinates to be mapped
+ * @param x coordinates to be mapped
+ * @param y coordinates to be mapped
+ * @param width coordinates to be mapped
+ * @param height coordinates to be mapped
  * @return rectangle with mapped coordinates
  * 
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_INVALID_ARGUMENT - if the Control from or the Control to have been disposed</li> 
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
- *    <li>ERROR_INVALID_ARGUMENT - if the Control from or the Control to have been disposed</li> 
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
  * 
@@ -1555,7 +1860,66 @@ public Rectangle map (Control from, Control to, int x, int y, int width, int hei
 	}
 	return rect;
 }
-	
+
+/**
+ * Generate a low level system event.
+ * 
+ * <code>post</code> is used to generate low level keyboard
+ * and mouse events. The intent is to enable automated UI
+ * testing by simulating the input from the user.  Most
+ * SWT applications should never need to call this method.
+ * <p>
+ * Note that this operation can fail when the operating system
+ * fails to generate the event for any reason.  For example,
+ * this can happen when there is no such key or mouse button
+ * or when the system event queue is full.
+ * </p>
+ * <p>
+ * <b>Event Types:</b>
+ * <p>KeyDown, KeyUp
+ * <p>The following fields in the <code>Event</code> apply:
+ * <ul>
+ * <li>(in) type KeyDown or KeyUp</li>
+ * <p> Either one of:
+ * <li>(in) character a character that corresponds to a keyboard key</li>
+ * <li>(in) keyCode the key code of the key that was typed,
+ *          as defined by the key code constants in class <code>SWT</code></li>
+ * </ul>
+ * <p>MouseDown, MouseUp</p>
+ * <p>The following fields in the <code>Event</code> apply:
+ * <ul>
+ * <li>(in) type MouseDown or MouseUp
+ * <li>(in) button the button that is pressed or released
+ * </ul>
+ * <p>MouseMove</p>
+ * <p>The following fields in the <code>Event</code> apply:
+ * <ul>
+ * <li>(in) type MouseMove
+ * <li>(in) x the x coordinate to move the mouse pointer to in screen coordinates
+ * <li>(in) y the y coordinate to move the mouse pointer to in screen coordinates
+ * </ul>
+ * </dl>
+ * 
+ * @param event the event to be generated
+ * 
+ * @return true if the event was generated or false otherwise
+ * 
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the event is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
+ * </ul>
+ *
+ * @since 3.0
+ * 
+ */
+public boolean post (Event event) {
+	if (isDisposed ()) error (SWT.ERROR_DEVICE_DISPOSED);
+	if (event == null) error (SWT.ERROR_NULL_ARGUMENT);
+	return false;
+}
+
 void postEvent (Event event) {
 	/*
 	* Place the event at the end of the event queue.
@@ -1576,7 +1940,7 @@ void postEvent (Event event) {
 		eventQueue = newQueue;
 	}
 	eventQueue [index] = event;
-	if (embedded) wake ();
+	if (embedded) wakeThread ();
 }
 
 /**
@@ -1597,6 +1961,7 @@ void postEvent (Event event) {
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_FAILED_EXEC - if an exception occurred while running an inter-thread message</li>
  * </ul>
  *
  * @see #sleep
@@ -1612,7 +1977,7 @@ public boolean readAndDispatch () {
 	OS.PtAppRemoveWorkProc (app_context, id);
 	boolean result = true;
 	if (idle) {
-		result = runAsyncMessages ();
+		result = runAsyncMessages (false);
 	} else {
 		runDeferredEvents ();
 	}
@@ -1655,7 +2020,7 @@ synchronized void register () {
  * </p>
  * This method is called before <code>destroy</code>.
  * 
- * @see #dispose
+ * @see Device#dispose
  * @see #destroy
  */
 protected void release () {
@@ -1664,10 +2029,12 @@ protected void release () {
 	for (int i=0; i<shells.length; i++) {
 		Shell shell = shells [i];
 		if (!shell.isDisposed ()) {
-			if (this == shell.getDisplay ()) shell.dispose ();
+			if (this == shell.display) shell.dispose ();
 		}
 	}
-	while (readAndDispatch ()) {};
+	if (tray != null) tray.dispose ();
+	tray = null;
+	while (readAndDispatch ()) {}
 	if (disposeList != null) {
 		for (int i=0; i<disposeList.length; i++) {
 			if (disposeList [i] != null) disposeList [i].run ();
@@ -1720,13 +2087,19 @@ void releaseDisplay () {
 	if (nullImage != 0) {
 		PhImage_t phImage = new PhImage_t();
 		OS.memmove(phImage, nullImage, PhImage_t.sizeof);
-		phImage.flags = OS.Ph_RELEASE_IMAGE_ALL;
+		phImage.flags = (byte)OS.Ph_RELEASE_IMAGE_ALL;
 		OS.memmove(nullImage, phImage, PhImage_t.sizeof);
 		OS.PhReleaseImage(nullImage);
 		OS.free(nullImage);
 		nullImage = 0;
 	}
-		
+
+	/* Release the System Cursors */
+	for (int i = 0; i < cursors.length; i++) {
+		if (cursors [i] != null) cursors [i].dispose ();
+	}
+	cursors = null;
+
 	/* Release references */
 	thread = null;
 	data = null;
@@ -1736,7 +2109,8 @@ void releaseDisplay () {
 
 /**
  * Removes the listener from the collection of listeners who will
- * be notifed when an event of the given type occurs anywhere in SWT.
+ * be notifed when an event of the given type occurs anywhere in
+ * a widget.
  *
  * @param eventType the type of event to listen for
  * @param listener the listener which should no longer be notified when the event occurs
@@ -1752,9 +2126,9 @@ void releaseDisplay () {
  * @see #addFilter
  * @see #addListener
  * 
- * @since 2.1 
+ * @since 3.0
  */
-void removeFilter (int eventType, Listener listener) {
+public void removeFilter (int eventType, Listener listener) {
 	checkDevice ();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (filterTable == null) return;
@@ -1789,8 +2163,8 @@ public void removeListener (int eventType, Listener listener) {
 	eventTable.unhook (eventType, listener);
 }
 
-boolean runAsyncMessages () {
-	return synchronizer.runAsyncMessages ();
+boolean runAsyncMessages (boolean all) {
+	return synchronizer.runAsyncMessages (all);
 }
 
 boolean runDeferredEvents () {
@@ -1905,8 +2279,8 @@ public void setCursorLocation (Point point) {
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
  *
- * @see #setData
- * @see #disposeExec
+ * @see #getData(String)
+ * @see #disposeExec(Runnable)
  */
 public void setData (String key, Object value) {
 	checkDevice ();
@@ -1976,8 +2350,8 @@ public void setData (String key, Object value) {
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
  *
- * @see #getData
- * @see #disposeExec
+ * @see #getData()
+ * @see #disposeExec(Runnable)
  */
 public void setData (Object data) {
 	checkDevice ();
@@ -1996,13 +2370,14 @@ public void setData (Object data) {
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_FAILED_EXEC - if an exception occurred while running an inter-thread message</li>
  * </ul>
  */
 public void setSynchronizer (Synchronizer synchronizer) {
 	checkDevice ();
 	if (synchronizer == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (this.synchronizer != null) {
-		this.synchronizer.runAsyncMessages();
+		this.synchronizer.runAsyncMessages(true);
 	}
 	this.synchronizer = synchronizer;
 }
@@ -2023,6 +2398,7 @@ public void setSynchronizer (Synchronizer synchronizer) {
  */
 public boolean sleep () {
 	checkDevice ();
+	if (getMessageCount () != 0) return true;
 	OS.PtFlush ();
 	OS.PtHold ();
 	OS.PtAppProcessEvent (app_context);
@@ -2035,9 +2411,16 @@ public boolean sleep () {
  * Causes the <code>run()</code> method of the runnable to
  * be invoked by the user-interface thread at the next 
  * reasonable opportunity. The thread which calls this method
- * is suspended until the runnable completes.
- *
- * @param runnable code to run on the user-interface thread.
+ * is suspended until the runnable completes.  Specifying <code>null</code>
+ * as the runnable simply wakes the user-interface thread.
+ * <p>
+ * Note that at the time the runnable is invoked, widgets 
+ * that have the receiver as their display may have been
+ * disposed. Therefore, it is necessary to check for this
+ * case inside the runnable before accessing the widget.
+ * </p>
+ * 
+ * @param runnable code to run on the user-interface thread or <code>null</code>
  *
  * @exception SWTException <ul>
  *    <li>ERROR_FAILED_EXEC - if an exception occured when executing the runnable</li>
@@ -2065,6 +2448,12 @@ int textWidth (String string, byte[] font) {
  * be invoked by the user-interface thread after the specified
  * number of milliseconds have elapsed. If milliseconds is less
  * than zero, the runnable is not executed.
+ * <p>
+ * Note that at the time the runnable is invoked, widgets 
+ * that have the receiver as their display may have been
+ * disposed. Therefore, it is necessary to check for this
+ * case inside the runnable before accessing the widget.
+ * </p>
  *
  * @param milliseconds the delay before running the runnable
  * @param runnable code to run on the user-interface thread
@@ -2156,21 +2545,21 @@ static int untranslateKey (int key) {
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
  * </ul>
  * 
- * @see Control#update
+ * @see Control#update()
  */
 public void update() {
 	checkDevice ();
 	Shell[] shells = WidgetTable.shells ();
 	for (int i=0; i<shells.length; i++) {
 		Shell shell = shells [i];
-		if (!shell.isDisposed () && this == shell.getDisplay ()) {
+		if (!shell.isDisposed () && this == shell.display) {
 			shell.update ();
 		}
 	}
 }
 
 /**
- * If the receiver's user-interface thread was <code>sleep</code>'ing, 
+ * If the receiver's user-interface thread was <code>sleep</code>ing, 
  * causes it to be awakened and start running again. Note that this
  * method may be called from any thread.
  * 
@@ -2182,7 +2571,11 @@ public void update() {
  */
 public void wake () {
 	if (isDisposed ()) error (SWT.ERROR_DEVICE_DISPOSED);
-	if (!embedded && thread == Thread.currentThread ()) return;
+	if (thread == Thread.currentThread ()) return;
+	wakeThread ();
+}
+
+void wakeThread () {
 //	int flags = OS.PtEnter (0);	
 	OS.PtAppPulseTrigger (app_context, pulse);
 //	if (flags >= 0) OS.PtLeave (flags);
