@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -31,11 +31,13 @@ import org.eclipse.swt.events.*;
  * </p><p>
  * <dl>
  * <dt><b>Styles:</b></dt>
- * <dd>(none)</dd>
+ * <dd>TOP, BOTTOM</dd>
  * <dt><b>Events:</b></dt>
  * <dd>Selection</dd>
  * </dl>
  * <p>
+ * Note: Only one of the styles TOP and BOTTOM may be specified.
+ * </p><p>
  * IMPORTANT: This class is <em>not</em> intended to be subclassed.
  * </p>
  */
@@ -77,6 +79,7 @@ public TabFolder (Composite parent, int style) {
 }
 
 static int checkStyle (int style) {
+	style = checkBits (style, SWT.TOP, SWT.BOTTOM, 0, 0, 0, 0);
 	/*
 	* Even though it is legal to create this widget
 	* with scroll bars, they serve no useful purpose
@@ -131,7 +134,9 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 	int width = dim.w, height = dim.h;
 	Point size;
 	if (layout != null) {
+		changed |= (state & LAYOUT_CHANGED) != 0;
 		size = layout.computeSize (this, wHint, hHint, changed);
+		state &= ~LAYOUT_CHANGED;
 	} else {
 		size = minimumSize (wHint, hHint, changed);
 	}
@@ -160,7 +165,12 @@ public Rectangle computeTrim (int x, int y, int width, int height) {
 	};
 	OS.PtGetResources(handle, args.length / 3, args);
 	int trimX = x - args [10];
-	int trimY = y - (dim.h - args [1]);
+	int trimY;
+	if ((style & SWT.BOTTOM) != 0) {
+		trimY = y - args [4];
+	} else {
+		trimY = y - (dim.h - args [1]);
+	}
 	int trimWidth = width + args [7] + args [10];
 	int trimHeight = height + dim.h;
 	return new Rectangle (trimX, trimY, trimWidth, trimHeight);
@@ -168,7 +178,6 @@ public Rectangle computeTrim (int x, int y, int width, int height) {
 
 void createHandle (int index) {
 	state |= HANDLE;
-	Display display = getDisplay ();
 	int parentHandle = parent.parentingHandle ();
 	int [] args = {
 		OS.Pt_ARG_RESIZE_FLAGS, 0, OS.Pt_RESIZE_XY_BITS,
@@ -178,6 +187,7 @@ void createHandle (int index) {
 	int clazz = display.PtPanelGroup;
 	args = new int []{
 		OS.Pt_ARG_RESIZE_FLAGS, 0, OS.Pt_RESIZE_XY_BITS,
+		OS.Pt_ARG_PG_FLAGS,  (style & SWT.BOTTOM) != 0 ? OS.Pt_PG_SELECTOR_ON_BOTTOM : 0, OS.Pt_PG_SELECTOR_ON_BOTTOM,  
 	};
 	handle = OS.PtCreateWidget (clazz, parentingHandle, args.length / 3, args);
 	if (handle == 0) error (SWT.ERROR_NO_HANDLES);
@@ -207,6 +217,7 @@ void createItem (TabItem item, int index) {
 	for (int i=0; i<=count; i++) {
 		if (i == index) {
 			str [0] = OS.malloc (1);
+			OS.memset(str [0], 0, 1);
 		} else {
 			OS.memmove (address, oldPtr + (j++ * 4), 4);
 			str [0] = OS.strdup (address [0]);
@@ -388,7 +399,7 @@ public int getSelectionIndex () {
 
 void hookEvents () {
 	super.hookEvents ();
-	int windowProc = getDisplay ().windowProc;
+	int windowProc = display.windowProc;
 	OS.PtAddCallback (handle, OS.Pt_CB_PG_PANEL_SWITCHING, windowProc, OS.Pt_CB_PG_PANEL_SWITCHING);
 	OS.PtAddCallback (handle, OS.Pt_CB_REALIZED, windowProc, OS.Pt_CB_REALIZED);
 }
@@ -506,10 +517,19 @@ void releaseWidget () {
 		TabItem item = items [i];
 		if (!item.isDisposed ()) item.releaseResources ();
 	}
+	itemCount = 0;
 	items = null;
 	super.releaseWidget ();
 }
 
+void removeControl (Control control) {
+	super.removeControl (control);
+	for (int i=0; i<itemCount; i++) {
+		TabItem item = items [i];
+		if (item.control == control) item.setControl (null);
+	}
+}
+
 /**
  * Removes the listener from the collection of listeners who will
  * be notified when the receiver's selection changes.
@@ -556,7 +576,7 @@ int setBounds (int x, int y, int width, int height, boolean move, boolean resize
 /**
  * Selects the item at the given zero-relative index in the receiver. 
  * If the item at the index was already selected, it remains selected.
- * The current selected is first cleared, then the new items are
+ * The current selection is first cleared, then the new items are
  * selected. Indices that are out of range are ignored.
  *
  * @param index the index of the item to select
@@ -568,6 +588,7 @@ int setBounds (int x, int y, int width, int height, boolean move, boolean resize
  */
 public void setSelection (int index) {
 	checkWidget();
+	if (!(0 <= index && index < itemCount)) return;
 	setSelection (index, false);
 }
 
@@ -617,6 +638,9 @@ void setSelection (int index, boolean notify) {
  *
  * @param items the array of items
  *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the items array is null</li>
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -626,12 +650,12 @@ public void setSelection (TabItem [] items) {
 	checkWidget();
 	if (items == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (items.length == 0) {
-		setSelection (-1);
-		return;
-	}
-	for (int i=items.length-1; i>=0; --i) {
-		int index = indexOf (items [i]);
-		if (index != -1) setSelection (index);
+		setSelection (-1, false);
+	} else {
+		for (int i=items.length-1; i>=0; --i) {
+			int index = indexOf (items [i]);
+			if (index != -1) setSelection (index, false);
+		}
 	}
 }
 
