@@ -1,10 +1,10 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -76,7 +76,9 @@ public class ToolItem extends Item {
  * @see Widget#getStyle
  */
 public ToolItem (ToolBar parent, int style) {
-	this(parent, style, parent.getItemCount ());
+	super (parent, checkStyle (style));
+	this.parent = parent;
+	parent.createItem (this, parent.getItemCount ());
 }
 
 /**
@@ -161,6 +163,10 @@ protected void checkSubclass () {
 	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
 }
 
+void click () {
+	click (handle);
+}
+
 int createArrowImage () {
 	short width = 5;
 	short height = 4;
@@ -289,7 +295,16 @@ public Rectangle getBounds () {
 	int topHandle = topHandle ();
 	PhArea_t area = new PhArea_t ();
 	OS.PtWidgetArea (topHandle, area);
-	return new Rectangle (area.pos_x, area.pos_y, area.size_w, area.size_h);
+	int x = area.pos_x, y = area.pos_y;
+	int width = area.size_w, height = area.size_h;
+	/* Check if the item is scrolled */
+	int child = OS.PtWidgetChildBack (parent.handle);
+	if (child != 0) {
+		OS.PtWidgetArea (child, area);
+		x += area.pos_x;
+		y += area.pos_y;
+	}
+	return new Rectangle (x, y, width, height);
 }
 
 /**
@@ -327,12 +342,6 @@ public Image getDisabledImage () {
 	return disabledImage;
 }
 
-public Display getDisplay () {
-	ToolBar parent = this.parent;
-	if (parent == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return parent.getDisplay ();
-}
-
 /**
  * Returns <code>true</code> if the receiver is enabled, and
  * <code>false</code> otherwise. A disabled control is typically
@@ -451,7 +460,7 @@ boolean hasFocus () {
 void hookEvents () {
 	super.hookEvents ();
 	if ((style & SWT.SEPARATOR) != 0) return;
-	int windowProc = getDisplay ().windowProc;
+	int windowProc = display.windowProc;
 	OS.PtAddEventHandler (handle, OS.Ph_EV_BOUNDARY, windowProc, OS.Ph_EV_BOUNDARY);	
 	OS.PtAddCallback (button, OS.Pt_CB_ACTIVATE, windowProc, OS.Pt_CB_ACTIVATE);
 	if ((style & SWT.DROP_DOWN) != 0) {
@@ -460,6 +469,11 @@ void hookEvents () {
 	OS.PtAddCallback (handle, OS.Pt_CB_LOST_FOCUS, windowProc,  OS.Pt_CB_LOST_FOCUS);
 }
 
+int hotkeyProc (int widget, int data, int info) {
+	if (setFocus ()) click ();
+	return OS.Pt_CONTINUE;
+}
+
 /**
  * Returns <code>true</code> if the receiver is enabled and all
  * of the receiver's ancestors are enabled, and <code>false</code>
@@ -549,6 +563,8 @@ void releaseHandle () {
 }
 
 void releaseWidget () {
+	// reparent the control back to the toolbar
+	if (control != null) setControl (null);
 	super.releaseWidget ();
 	if (toolTipHandle != 0) destroyToolTip (toolTipHandle);
 	if (parent.lastFocus == this) parent.lastFocus = null;
@@ -564,7 +580,7 @@ void releaseWidget () {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -638,7 +654,6 @@ public void setControl (Control control) {
 }
 
 void setDefaultFont () {
-	Display display = getDisplay ();
 	if (display.defaultFont != null) setFont (parent.defaultFont ());
 }
 
@@ -862,17 +877,29 @@ public void setText (String string) {
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if ((style & SWT.SEPARATOR) != 0) return;
 	super.setText (string);
-	byte [] buffer = Converter.wcsToMbcs (null, string, true);
-	int ptr = OS.malloc (buffer.length);
-	OS.memmove (ptr, buffer, buffer.length);
+	char [] text = new char [string.length ()];
+	string.getChars (0, text.length, text, 0);
+	char mnemonic = fixMnemonic (text);
+	byte [] buffer = Converter.wcsToMbcs (null, text, true);
+	int ptr1 = OS.malloc (buffer.length);
+	OS.memmove (ptr1, buffer, buffer.length);
+	int ptr2 = 0;
+	if (mnemonic != 0) {
+		byte [] buffer2 = Converter.wcsToMbcs (null, new char []{mnemonic}, true);
+		ptr2 = OS.malloc (buffer2.length);
+		OS.memmove (ptr2, buffer2, buffer2.length);
+	}
+	replaceMnemonic (mnemonic, true, true);
 	int type = OS.Pt_Z_STRING;
 	if (image != null) type = OS.Pt_TEXT_IMAGE;
 	int [] args = {
-		OS.Pt_ARG_TEXT_STRING, ptr, 0,
+		OS.Pt_ARG_TEXT_STRING, ptr1, 0,
 		OS.Pt_ARG_LABEL_TYPE, type, 0,
+		OS.Pt_ARG_ACCEL_KEY, ptr2, 0,
 	};
 	OS.PtSetResources (button, args.length / 3, args);
-	if (ptr != 0) OS.free (ptr);
+	OS.free (ptr1);
+	OS.free (ptr2);
 	
 	/*
 	* Bug on Photon.  When a the text/image is set on a
@@ -904,7 +931,7 @@ public void setToolTipText (String string) {
 }
 
 /**
- * Sets the width of the receiver.
+ * Sets the width of the receiver, for <code>SEPARATOR</code> ToolItems.
  *
  * @param width the new width
  *
