@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -43,35 +43,37 @@ import org.eclipse.swt.events.*;
  * @see #checkSubclass
  */
 public abstract class Widget {
+	/**
+	 * the handle to the resource 
+	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
+	 */
 	public int handle;
 	int style, state;
+	Display display;
 	EventTable eventTable;
 	Object data;
-	String [] keys;
-	Object [] values;
 	
 	/* Global state flags */
-//	static final int AUTOMATIC		= 1 << 0;
-//	static final int ACTIVE			= 1 << 1;
-	static final int GRAB		= 1 << 2;
-//	static final int MULTIEXPOSE	= 1 << 3;
-//	static final int RESIZEREDRAW	= 1 << 4;
-//	static final int WRAP			= 1 << 5;
-	static final int DISABLED	= 1 << 6;
-//	static final int HIDDEN		= 1 << 7;
-//	static final int FOREGROUND		= 1 << 8;
-//	static final int BACKGROUND		= 1 << 9;
-	static final int DISPOSED	= 1 << 10;
-	static final int HANDLE		= 1 << 11;
-	static final int CANVAS		= 1 << 12;
-	static final int MOVED		= 1 << 13;
-	static final int RESIZED	= 1 << 14;
+	static final int DISPOSED		= 1 << 0;
+	static final int CANVAS			= 1 << 1;
+	static final int KEYED_DATA		= 1 << 2;
+	static final int HANDLE			= 1 << 3;
+	static final int DISABLED		= 1 << 4;
+	static final int MOVED			= 1 << 5;
+	static final int RESIZED		= 1 << 6;
+	static final int GRAB			= 1 << 7;
+	static final int LAYOUT_CHANGED	= 1 << 8;
+	static final int LAYOUT_NEEDED  = 1 << 9;
 	
 	static final int DEFAULT_WIDTH	= 64;
 	static final int DEFAULT_HEIGHT = 64;
 
-	static final char Mnemonic = '&';
-
 Widget () {
 	/* Do nothing */
 }
@@ -94,6 +96,7 @@ Widget () {
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if the parent is disposed</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -108,6 +111,7 @@ public Widget (Widget parent, int style) {
 	checkSubclass ();
 	checkParent (parent);
 	this.style = style;
+	display = parent.display;
 }
 
 static int checkBits (int style, int int0, int int1, int int2, int int3, int int4, int int5) {
@@ -122,25 +126,6 @@ static int checkBits (int style, int int0, int int1, int int2, int int3, int int
 	return style;
 }
 
-/**
- * Returns the provided string without mnemonic indicators.
- * 
- * @param string the string to demangle
- */
-static String stripMnemonics (String string) {
-	char [] text = new char [string.length ()];
-	string.getChars (0, text.length, text, 0);
-	int j = 0;
-	for (int i = 0; i < text.length;) {
-		if ((text[j++] = text[i++]) == Mnemonic) {
-			if (i != text.length) {
-				if (text[i] == Mnemonic) i++; else j--;
-			}
-		}
-	}
-	return new String(text, 0, j);
-}
-
 void checkOrientation (Widget parent) {
 	style &= ~SWT.MIRRORED;
 	if ((style & (SWT.LEFT_TO_RIGHT | SWT.RIGHT_TO_LEFT)) == 0) {
@@ -154,8 +139,8 @@ void checkOrientation (Widget parent) {
 
 void checkParent (Widget parent) {
 	if (parent == null) error (SWT.ERROR_NULL_ARGUMENT);
-	if (!parent.isValidThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
-	if (parent.isDisposed()) error (SWT.ERROR_INVALID_ARGUMENT);
+	if (parent.isDisposed ()) error (SWT.ERROR_INVALID_ARGUMENT);
+	parent.checkWidget ();
 }
 
 /**
@@ -214,8 +199,10 @@ protected void checkSubclass () {
  * </ul>
  */
 protected void checkWidget () {
-	if (!isValidThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
-	if (isDisposed ()) error (SWT.ERROR_WIDGET_DISPOSED);
+	Display display = this.display;
+	if (display == null) error (SWT.ERROR_WIDGET_DISPOSED);
+	if (display.thread != Thread.currentThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
+	if ((state & DISPOSED) != 0) error (SWT.ERROR_WIDGET_DISPOSED);
 }
 
 int copyPhImage(int image) {
@@ -307,6 +294,37 @@ public void addDisposeListener (DisposeListener listener) {
 	addListener (SWT.Dispose, typedListener);
 }
 
+void click (int widget) {
+	int rid = OS.PtWidgetRid (widget);
+	if (rid == 0) return;
+	PhEvent_t event = new PhEvent_t ();
+	event.emitter_rid = rid;
+	event.emitter_handle = widget;
+	event.collector_rid = rid;
+	event.collector_handle = widget;
+	event.flags = (short) OS.Ph_EVENT_DIRECT;
+	event.processing_flags = (short) OS.Ph_FAKE_EVENT;
+	event.type = OS.Ph_EV_BUT_PRESS;
+	event.num_rects = 1;
+	PhPointerEvent_t pe = new PhPointerEvent_t ();
+	pe.click_count = 1;
+	pe.buttons = (short) OS.Ph_BUTTON_SELECT;
+	PhRect_t rect = new PhRect_t ();
+	int ptr = OS.malloc (PhEvent_t.sizeof + PhPointerEvent_t.sizeof + PhRect_t.sizeof);
+	OS.memmove (ptr, event, PhEvent_t.sizeof);
+	OS.memmove (ptr + PhEvent_t.sizeof,  rect, PhRect_t.sizeof);
+	OS.memmove (ptr + PhEvent_t.sizeof + PhRect_t.sizeof, pe, PhPointerEvent_t.sizeof);
+	OS.PtSendEventToWidget (widget, ptr);	
+	OS.PtFlush ();
+	event.type = OS.Ph_EV_BUT_RELEASE;
+	event.subtype = (short) OS.Ph_EV_RELEASE_REAL;
+	OS.memmove (ptr, event, PhEvent_t.sizeof);
+	OS.memmove (ptr + PhEvent_t.sizeof,  rect, PhRect_t.sizeof);
+	OS.memmove (ptr + PhEvent_t.sizeof + PhRect_t.sizeof, pe, PhPointerEvent_t.sizeof);
+	OS.PtSendEventToWidget (widget, ptr);	
+	OS.free (ptr);
+}
+
 void createHandle (int index) {
 	/* Do nothing */
 }
@@ -318,7 +336,6 @@ int createToolTip (String string, int handle, byte [] font) {
 
 	int shellHandle = OS.PtFindDisjoint (handle);
 	byte [] buffer = Converter.wcsToMbcs (null, string, true);
-	Display display = getDisplay ();
 	int fill = display.INFO_BACKGROUND;
 	int text_color = display.INFO_FOREGROUND;
 	int toolTipHandle = OS.PtInflateBalloon (shellHandle, handle, OS.Pt_BALLOON_RIGHT, buffer, font, fill, text_color);
@@ -419,10 +436,24 @@ static void error (int code) {
 }
 
 boolean filters (int eventType) {
-	Display display = getDisplay ();
 	return display.filters (eventType);
 }
 
+char fixMnemonic (char [] buffer) {
+	int i=0, j=0;
+	char mnemonic=0;
+	while (i < buffer.length) {
+		if ((buffer [j++] = buffer [i++]) == '&') {
+			if (i == buffer.length) {continue;}
+			if (buffer [i] == '&') {i++; continue;}
+			if (mnemonic == 0) mnemonic = buffer [i];
+			j--;
+		}
+	}
+	while (j < buffer.length) buffer [j++] = 0;
+	return mnemonic;
+}
+
 /**
  * Returns the application defined widget data associated
  * with the receiver, or null if it has not been set. The
@@ -443,11 +474,11 @@ boolean filters (int eventType) {
  *    <li>ERROR_THREAD_INVALID_ACCESS - when called from the wrong thread</li>
  * </ul>
  *
- * @see #setData
+ * @see #setData(Object)
  */
 public Object getData () {
 	checkWidget();
-	return data;
+	return (state & KEYED_DATA) != 0 ? ((Object []) data) [0] : data;
 }
 
 /**
@@ -472,14 +503,16 @@ public Object getData () {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
- * @see #setData
+ * @see #setData(String, Object)
  */
 public Object getData (String key) {
 	checkWidget();
 	if (key == null) error (SWT.ERROR_NULL_ARGUMENT);
-	if (keys == null) return null;
-	for (int i=0; i<keys.length; i++) {
-		if (keys [i].equals (key)) return values [i];
+	if ((state & KEYED_DATA) != 0) {
+		Object [] table = (Object []) data;
+		for (int i=1; i<table.length; i+=2) {
+			if (key.equals (table [i])) return table [i+1];
+		}
 	}
 	return null;
 }
@@ -497,10 +530,13 @@ public Object getData (String key) {
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
- *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  */
-public abstract Display getDisplay ();
+public Display getDisplay () {
+	Display display = this.display;
+	if (display == null) error (SWT.ERROR_WIDGET_DISPOSED);
+	return display;
+}
 
 String getName () {
 	String string = getClass ().getName ();
@@ -585,7 +621,7 @@ boolean isValidSubclass () {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  */
-protected boolean isListening (int eventType) {
+public boolean isListening (int eventType) {
 	checkWidget();
 	return hooks (eventType);
 }
@@ -602,9 +638,6 @@ boolean isValidThread () {
  * @param eventType the type of event which has occurred
  * @param event the event data
  *
- * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the event is null</li>
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -612,7 +645,7 @@ boolean isValidThread () {
  */
 public void notifyListeners (int eventType, Event event) {
 	checkWidget();
-	if (event == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (event == null) event = new Event ();
 	sendEvent (eventType, event);
 }
 
@@ -668,6 +701,10 @@ int Pt_CB_MODIFY_VERIFY (int widget, int info) {
 	return OS.Pt_CONTINUE;
 }
 
+int Pt_CB_NUMERIC_CHANGED (int widget, int info) {
+	return OS.Pt_CONTINUE;
+}
+
 int Pt_CB_OUTBOUND (int widget, int info) {
 	return OS.Pt_CONTINUE;
 }
@@ -724,6 +761,7 @@ void register () {
 void releaseHandle () {
 	handle = 0;
 	state |= DISPOSED;
+	display = null;
 }
 
 void releaseResources () {
@@ -736,8 +774,6 @@ void releaseWidget () {
 	deregister ();
 	eventTable = null;
 	data = null;
-	keys = null;
-	values = null;
 }
 
 /**
@@ -821,7 +857,6 @@ public void removeDisposeListener (DisposeListener listener) {
 }
 
 void replaceMnemonic (int mnemonic, boolean normal, boolean alt) {
-	Display display = getDisplay ();
 	int [] args = {OS.Pt_ARG_ACCEL_KEY, 0, 0};
 	OS.PtGetResources (handle, args.length / 3, args);
 	if (args [1] != 0) {
@@ -834,7 +869,7 @@ void replaceMnemonic (int mnemonic, boolean normal, boolean alt) {
 				char key = Character.toLowerCase (accelText [0]);
 				if (normal) {
 					//TEMPORARY CODE
-//					OS.PtRemoveHotkeyHandler (handle, key, 0, (short)0, SWT.Activate, display.windowProc);
+//					OS.PtRemoveHotkeyHandler (handle, key, 0, (short)0, handle, display.hotkeyProc);
 				}
 				if (alt) {
 					OS.PtRemoveHotkeyHandler (handle, key, OS.Pk_KM_Alt, (short)0, handle, display.hotkeyProc);
@@ -869,7 +904,6 @@ void sendEvent (int eventType, Event event) {
 }
 
 void sendEvent (int eventType, Event event, boolean send) {
-	Display display = getDisplay ();
 	if (eventTable == null && !display.filters (eventType)) {
 		return;
 	}
@@ -906,10 +940,16 @@ void sendEvent (int eventType, Event event, boolean send) {
  *    <li>ERROR_WIDGET_DISPOSED - when the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - when called from the wrong thread</li>
  * </ul>
+ * 
+ * @see #getData()
  */
 public void setData (Object data) {
 	checkWidget();
-	this.data = data;
+	if ((state & KEYED_DATA) != 0) {
+		((Object []) this.data) [0] = data;
+	} else {
+		this.data = data;
+	}
 }
 
 /**
@@ -934,108 +974,72 @@ public void setData (Object data) {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
- * @see #getData
+ * @see #getData(String)
  */
 public void setData (String key, Object value) {
 	checkWidget();
 	if (key == null) error (SWT.ERROR_NULL_ARGUMENT);
-	
-	/* Remove the key/value pair */
-	if (value == null) {
-		if (keys == null) return;
-		int index = 0;
-		while (index < keys.length && !keys [index].equals (key)) index++;
-		if (index == keys.length) return;
-		if (keys.length == 1) {
-			keys = null;
-			values = null;
+	int index = 1;
+	Object [] table = null;
+	if ((state & KEYED_DATA) != 0) {
+		table = (Object []) data;
+		while (index < table.length) {
+			if (key.equals (table [index])) break;
+			index += 2;
+		}
+	}
+	if (value != null) {
+		if ((state & KEYED_DATA) != 0) {
+			if (index == table.length) {
+				Object [] newTable = new Object [table.length + 2];
+				System.arraycopy (table, 0, newTable, 0, table.length);
+				data = table = newTable;
+			}
 		} else {
-			String [] newKeys = new String [keys.length - 1];
-			Object [] newValues = new Object [values.length - 1];
-			System.arraycopy (keys, 0, newKeys, 0, index);
-			System.arraycopy (keys, index + 1, newKeys, index, newKeys.length - index);
-			System.arraycopy (values, 0, newValues, 0, index);
-			System.arraycopy (values, index + 1, newValues, index, newValues.length - index);
-			keys = newKeys;
-			values = newValues;
+			table = new Object [3];
+			table [0] = data;
+			data = table;
+			state |= KEYED_DATA;
 		}
-		return;
-	}
-	
-	/* Add the key/value pair */
-	if (keys == null) {
-		keys = new String [] {key};
-		values = new Object [] {value};
-		return;
-	}
-	for (int i=0; i<keys.length; i++) {
-		if (keys [i].equals (key)) {
-			values [i] = value;
-			return;
+		table [index] = key;
+		table [index + 1] = value;
+	} else {
+		if ((state & KEYED_DATA) != 0) {
+			if (index != table.length) {
+				int length = table.length - 2;
+				if (length == 1) {
+					data = table [0];
+					state &= ~KEYED_DATA;
+				} else {
+					Object [] newTable = new Object [length];
+					System.arraycopy (table, 0, newTable, 0, index);
+					System.arraycopy (table, index + 2, newTable, index, length - index);
+					data = newTable;
+				}
+			}
 		}
 	}
-	String [] newKeys = new String [keys.length + 1];
-	Object [] newValues = new Object [values.length + 1];
-	System.arraycopy (keys, 0, newKeys, 0, keys.length);
-	System.arraycopy (values, 0, newValues, 0, values.length);
-	newKeys [keys.length] = key;
-	newValues [values.length] = value;
-	keys = newKeys;
-	values = newValues;
 }
 
-void setInputState(Event event, int key_mods, int button_state) {
+boolean setInputState (Event event, int type, int key_mods, int button_state) {
 	if ((key_mods & OS.Pk_KM_Alt) != 0) event.stateMask |= SWT.ALT;
 	if ((key_mods & OS.Pk_KM_Shift) != 0) event.stateMask |= SWT.SHIFT;
 	if ((key_mods & OS.Pk_KM_Ctrl) != 0) event.stateMask |= SWT.CONTROL;
 	if ((button_state & OS.Ph_BUTTON_SELECT) != 0) event.stateMask |= SWT.BUTTON1;
 	if ((button_state & OS.Ph_BUTTON_ADJUST) != 0) event.stateMask |= SWT.BUTTON2;
 	if ((button_state & OS.Ph_BUTTON_MENU) != 0) event.stateMask |= SWT.BUTTON3;
-}
-
-void setKeyState(Event event, PhKeyEvent_t ke) {
-	/*
-	* Fetuare in Photon.  The key_sym value is not valid when Ctrl
-	* or Alt is pressed. The fix is to detect this case and try to
-	* use the key_cap value.
-	*/
-	int key = 0;
-	if ((ke.key_flags & OS.Pk_KF_Sym_Valid) != 0) {
-		 key = ke.key_sym;
-	} else if ((ke.key_flags & OS.Pk_KF_Cap_Valid) != 0) {
-		if ((ke.key_flags & (OS.Pk_KF_Key_Down | OS.Pk_KF_Key_Repeat)) != 0) {
-			key = ke.key_cap;
-			if ((ke.key_mods & OS.Pk_KM_Ctrl) != 0) {
-				if ('a'  <= key && key <= 'z') key -= 'a' - 'A';
-				if (64 <= key && key <= 95) key -= 64;
-			}
-		}
-	}
-	event.keyCode = Display.translateKey (key);
-	switch (key) {
-		case OS.Pk_BackSpace:	event.character = '\b'; break;
-		case OS.Pk_Linefeed:	event.character = '\n'; break;
-		case OS.Pk_Return: 	event.character = '\r'; break;
-		case OS.Pk_Delete:		event.character = 0x7F; break;
-		case OS.Pk_Cancel:
-		case OS.Pk_Escape:		event.character = 0x1B; break;
-		case OS.Pk_KP_Tab:
-		case OS.Pk_Tab: 		event.character = '\t'; break;
-		/* These keys have no mapping in SWT yet */
-		case OS.Pk_Clear:
-		case OS.Pk_Pause:
-		case OS.Pk_Print:
-		case OS.Pk_Menu:
-		case OS.Pk_Hyper_L:
-		case OS.Pk_Hyper_R:
-		case OS.Pk_Caps_Lock:
-		case OS.Pk_Num_Lock:
-		case OS.Pk_Scroll_Lock:	break;
-		default:
-			if (event.keyCode == 0) event.character = (char) key;
-	}
-	setInputState(event, ke.key_mods, ke.button_state);
-	switch (event.type) {
+	switch (type) {
+		case SWT.MouseDown:
+		case SWT.MouseDoubleClick:
+			if (event.button == 1) event.stateMask &= ~SWT.BUTTON1;
+			if (event.button == 2) event.stateMask &= ~SWT.BUTTON2;
+			if (event.button == 3) event.stateMask &= ~SWT.BUTTON3;
+			break;
+		case SWT.MouseUp:
+			if (event.button == 1) event.stateMask |= SWT.BUTTON1;
+			if (event.button == 2) event.stateMask |= SWT.BUTTON2;
+			if (event.button == 3) event.stateMask |= SWT.BUTTON3;
+			break;
 		case SWT.KeyDown:
 		case SWT.Traverse:
 			if (event.keyCode == SWT.ALT) event.stateMask &= ~SWT.ALT;
@@ -1048,9 +1052,89 @@ void setKeyState(Event event, PhKeyEvent_t ke) {
 			if (event.keyCode == SWT.CONTROL) event.stateMask |= SWT.CONTROL;
 			break;
 	}
+	return true;
 }
 
-void setMouseState(Event event, PhPointerEvent_t pe, PhEvent_t ev) {
+boolean setKeyState (Event event, int type, PhKeyEvent_t ke) {
+	int key = 0;
+	boolean isNull = false;
+	if ((ke.key_flags & OS.Pk_KF_Cap_Valid) != 0) {
+		key = ke.key_cap;
+		if ((ke.key_mods & OS.Pk_KM_Num_Lock) == 0) {
+			switch (key) {
+				case OS.Pk_KP_0: key = OS.Pk_Insert; break;
+				case OS.Pk_KP_1: key = OS.Pk_End; break;
+				case OS.Pk_KP_2: key = OS.Pk_Down; break;
+				case OS.Pk_KP_3: key = OS.Pk_Pg_Down; break;
+				case OS.Pk_KP_4: key = OS.Pk_Left; break;
+				case OS.Pk_KP_5: break;
+				case OS.Pk_KP_6: key = OS.Pk_Right; break;
+				case OS.Pk_KP_7: key = OS.Pk_Home; break;
+				case OS.Pk_KP_8: key = OS.Pk_Up; break;
+				case OS.Pk_KP_9: key = OS.Pk_Pg_Up; break;
+				case OS.Pk_KP_Decimal: key = OS.Pk_Delete; break;
+			}
+			
+		}
+		event.keyCode = Display.translateKey (key);
+	}
+	switch (key) {
+		case OS.Pk_BackSpace:		event.character = '\b'; break;
+		case OS.Pk_Linefeed:		event.character = '\n'; break;
+		case OS.Pk_KP_Enter:
+		case OS.Pk_Return: 		event.character = '\r'; break;
+		case OS.Pk_Delete:		event.character = 0x7F; break;
+		case OS.Pk_Escape:		event.character = 0x1B; break;
+		case OS.Pk_KP_Tab:
+		case OS.Pk_Tab: 	event.character = '\t'; break;
+		/* These keys have no mapping in SWT yet */
+		case OS.Pk_Clear:
+		case OS.Pk_Menu:
+		case OS.Pk_Hyper_L:
+		case OS.Pk_Hyper_R:
+			break;
+		default: {
+			if (event.keyCode == 0) {
+				if ((ke.key_flags & OS.Pk_KF_Cap_Valid) != 0) {
+					event.keyCode = ke.key_cap;
+				}
+			}
+			/*
+			* Fetuare in Photon.  The key_sym value is not valid when Ctrl
+			* or Alt is pressed. The fix is to detect this case and try to
+			* use the key_cap value.
+			*/
+			if ((ke.key_mods & (OS.Pk_KM_Alt | OS.Pk_KM_Ctrl)) != 0) {
+				if (0 <= key && key <= 0x7F) {
+					if ((ke.key_mods & OS.Pk_KM_Ctrl) != 0) {
+						isNull = key == '@';
+						if ('a' <= key && key <= 'z') key -= 'a' - 'A';
+						if (64 <= key && key <= 95) key -= 64;
+						event.character = (char) key;
+						isNull &= key == 0;
+					} else {
+						if ((ke.key_flags & OS.Pk_KF_Sym_Valid) != 0) {
+							event.character = (char) ke.key_sym;
+						}
+					}
+				}
+			} else {
+				byte [] buffer = new byte [6];
+				int length = OS.PhKeyToMb (buffer, ke);
+				if (length > 0) {
+					char [] unicode = Converter.mbcsToWcs (null, buffer);
+					if (unicode.length > 0) event.character = unicode [0];
+				}
+			}
+		}
+	}
+	if (event.keyCode == 0 && event.character == 0) {
+		if (!isNull) return false;
+	}
+	return setInputState (event, type, ke.key_mods, ke.button_state);
+}
+
+boolean setMouseState(Event event, int type, PhPointerEvent_t pe, PhEvent_t ev) {
 	int buttons = pe.buttons;
 	event.x = pe.pos_x + ev.translation_x;
 	event.y = pe.pos_y + ev.translation_y;
@@ -1061,20 +1145,7 @@ void setMouseState(Event event, PhPointerEvent_t pe, PhEvent_t ev) {
 			case OS.Ph_BUTTON_MENU:		event.button = 3; break;
 		}
 	}
-	setInputState(event, pe.key_mods, pe.button_state);
-	switch (ev.type) {
-		case OS.Ph_EV_BUT_PRESS:
-			if (buttons == OS.Ph_BUTTON_SELECT) event.stateMask &= ~SWT.BUTTON1;
-			if (buttons == OS.Ph_BUTTON_ADJUST) event.stateMask &= ~SWT.BUTTON2;
-			if (buttons == OS.Ph_BUTTON_MENU) event.stateMask &= ~SWT.BUTTON3;
-			break;
-		case OS.Ph_EV_BUT_RELEASE:
-		case OS.Ph_EV_DRAG:
-			if (buttons == OS.Ph_BUTTON_SELECT) event.stateMask |= SWT.BUTTON1;
-			if (buttons == OS.Ph_BUTTON_ADJUST) event.stateMask |= SWT.BUTTON2;
-			if (buttons == OS.Ph_BUTTON_MENU) event.stateMask |= SWT.BUTTON3;
-			break;
-	}
+	return setInputState(event, type, pe.key_mods, pe.button_state);
 }
 
 int topHandle () {
@@ -1109,6 +1180,7 @@ int windowProc (int handle, int data, int info) {
 		case OS.Pt_CB_GOT_FOCUS:			return Pt_CB_GOT_FOCUS (handle, info);
 		case OS.Pt_CB_LOST_FOCUS:			return Pt_CB_LOST_FOCUS (handle, info);
 		case OS.Pt_CB_MODIFY_VERIFY:			return Pt_CB_MODIFY_VERIFY (handle, info);
+		case OS.Pt_CB_NUMERIC_CHANGED:			return Pt_CB_NUMERIC_CHANGED (handle, info);
 		case OS.Pt_CB_OUTBOUND:			return Pt_CB_OUTBOUND (handle, info);
 		case OS.Pt_CB_PG_PANEL_SWITCHING:			return Pt_CB_PG_PANEL_SWITCHING (handle, info);
 		case OS.Pt_CB_REALIZED:			return Pt_CB_REALIZED (handle, info);
