@@ -1,16 +1,17 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.swt.graphics;
 
 
+import org.eclipse.swt.internal.gdip.*;
 import org.eclipse.swt.internal.win32.*;
 import org.eclipse.swt.*;
 import java.io.*;
@@ -53,7 +54,7 @@ import java.io.*;
  * loading process should use the support provided in class
  * <code>ImageLoader</code>.
  * </p><p>
- * Application code must explicitely invoke the <code>Image.dispose()</code> 
+ * Application code must explicitly invoke the <code>Image.dispose()</code> 
  * method to release the operating system resources managed by each instance
  * when those instances are no longer required.
  * </p>
@@ -63,7 +64,7 @@ import java.io.*;
  * @see ImageLoader
  */
 
-public final class Image implements Drawable {
+public final class Image extends Resource implements Drawable {
 	
 	/**
 	 * specifies whether the receiver is a bitmap or an icon
@@ -72,50 +73,45 @@ public final class Image implements Drawable {
 	public int type;
 	
 	/**
-	 * the OS resource of the image
+	 * the handle to the OS image resource
 	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
 	 */
 	public int handle;
 	
-	/**
-	 * the device where this image was created
-	 */
-	Device device;
-	
 	/**
 	 * specifies the transparent pixel
-	 * (Warning: This field is platform dependent)
 	 */
 	int transparentPixel = -1;
 	
 	/**
 	 * the GC which is drawing on the image
-	 * (Warning: This field is platform dependent)
 	 */
 	GC memGC;
 	
 	/**
 	 * the alpha data for the image
-	 * (Warning: This field is platform dependent)
 	 */
 	byte[] alphaData;
 	
 	/**
 	 * the global alpha value to be used for every pixel
-	 * (Warning: This field is platform dependent)
 	 */
 	int alpha = -1;
 	
 	/**
 	 * the image data used to create this image if it is a
 	 * icon. Used only in WinCE
-	 * (Warning: This field is platform dependent)
 	 */
 	ImageData data;
 	
 	/**
 	 * specifies the default scanline padding
-	 * (Warning: This field is platform dependent)
 	 */
 	static final int DEFAULT_SCANLINE_PAD = 4;
 
@@ -188,6 +184,7 @@ public Image(Device device, int width, int height) {
  * @exception SWTException <ul>
  *    <li>ERROR_INVALID_IMAGE - if the image is not a bitmap or an icon, or
  *          is otherwise in an invalid state</li>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the depth of the Image is not supported</li>
  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
@@ -458,6 +455,7 @@ public Image(Device device, Image srcImage, int flag) {
 				newData = new ImageData(r.width, r.height, 8, new PaletteData(rgbs));
 				newData.maskData = data.maskData;
 				newData.maskPad = data.maskPad;
+				if (data.transparentPixel != -1) newData.transparentPixel = 254; 
 
 				/* Convert the pixels. */
 				int[] scanline = new int[r.width];
@@ -472,14 +470,20 @@ public Image(Device device, Image srcImage, int flag) {
 					data.getPixels(0, y, r.width, scanline, 0);
 					for (int x=0; x<r.width; x++) {
 						int pixel = scanline[x];
-						int red = pixel & redMask;
-						red = (redShift < 0) ? red >>> -redShift : red << redShift;
-						int green = pixel & greenMask;
-						green = (greenShift < 0) ? green >>> -greenShift : green << greenShift;
-						int blue = pixel & blueMask;
-						blue = (blueShift < 0) ? blue >>> -blueShift : blue << blueShift;
-						newData.data[offset++] =
-							(byte)((red+red+green+green+green+green+green+blue) >> 3);
+						if (pixel != data.transparentPixel) {
+							int red = pixel & redMask;
+							red = (redShift < 0) ? red >>> -redShift : red << redShift;
+							int green = pixel & greenMask;
+							green = (greenShift < 0) ? green >>> -greenShift : green << greenShift;
+							int blue = pixel & blueMask;
+							blue = (blueShift < 0) ? blue >>> -blueShift : blue << blueShift;
+							int intensity = (red+red+green+green+green+green+green+blue) >> 3;
+							if (newData.transparentPixel == intensity) intensity = 255;
+							newData.data[offset] = (byte)intensity;
+						} else {
+							newData.data[offset] = (byte)254;
+						}
+						offset++;
 					}
 				}
 			}
@@ -541,6 +545,9 @@ public Image(Device device, Rectangle bounds) {
  *    <li>ERROR_NULL_ARGUMENT - if device is null and there is no current device</li>
  *    <li>ERROR_NULL_ARGUMENT - if the image data is null</li>
  * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the depth of the ImageData is not supported</li>
+ * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
  * </ul>
@@ -555,10 +562,8 @@ public Image(Device device, ImageData data) {
 /**
  * Constructs an instance of this class, whose type is 
  * <code>SWT.ICON</code>, from the two given <code>ImageData</code>
- * objects. The two images must be the same size, and the mask image
- * must have a color depth of 1. Pixel transparency in either image
- * will be ignored. If either image is an icon to begin with, an
- * exception is thrown.
+ * objects. The two images must be the same size. Pixel transparency
+ * in either image will be ignored.
  * <p>
  * The mask image should contain white wherever the icon is to be visible,
  * and black wherever the icon is to be transparent. In addition,
@@ -573,9 +578,7 @@ public Image(Device device, ImageData data) {
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if device is null and there is no current device</li>
  *    <li>ERROR_NULL_ARGUMENT - if either the source or mask is null </li>
- *    <li>ERROR_INVALID_ARGUMENT - if source and mask are different sizes or
- *          if the mask is not monochrome, or if either the source or mask
- *          is already an icon</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if source and mask are different sizes</li>
  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
@@ -589,122 +592,8 @@ public Image(Device device, ImageData source, ImageData mask) {
 	if (source.width != mask.width || source.height != mask.height) {
 		SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	}
-	if (mask.depth != 1) {
-		/*
-		 * Feature in Windows. 1-bit DIB sections are buggy on Win98, so we
-		 * create 4-bit DIBs when given a 1-bit ImageData. In order to allow
-		 * users to draw on the masks, we must also support 4-bit masks in
-		 * icon creation by converting them into 1-bit masks.
-		 */
-		if (mask.depth != 4) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
-		PaletteData palette = new PaletteData(new RGB[] {new RGB(0, 0, 0), new RGB(255,255,255)});
-		ImageData tempMask = new ImageData(mask.width, mask.height, 1, palette);
-		/* Find index of black in mask palette */
-		RGB[] rgbs = mask.getRGBs();
-		int blackIndex = 0;
-		while (blackIndex < rgbs.length) {
-			if (rgbs[blackIndex].equals(palette.colors[0])) break;
-			blackIndex++;
-		}
-		if (blackIndex == rgbs.length) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
-		int[] pixels = new int[mask.width];
-		for (int y = 0; y < mask.height; y++) {
-			mask.getPixels(0, y, mask.width, pixels, 0);
-			for (int i = 0; i < pixels.length; i++) {
-				if (pixels[i] == blackIndex) {
-					pixels[i] = 0;
-				} else {
-					pixels[i] = 1;
-				}
-			}
-			tempMask.setPixels(0, y, mask.width, pixels, 0);
-		}
-		mask = tempMask;
-	}
-	/* Create a temporary image and locate the black pixel */
-	ImageData image;
-	int blackIndex = 0;
-	if (source.palette.isDirect) {
-		image = new ImageData(source.width, source.height, source.depth, source.palette);
-	} else {
-		RGB black = new RGB(0, 0, 0);
-		RGB[] rgbs = source.getRGBs();
-		if (source.transparentPixel != -1) {
-			/*
-			 * The source had transparency, so we can use the transparent pixel
-			 * for black.
-			 */
-			RGB[] newRGBs = new RGB[rgbs.length];
-			System.arraycopy(rgbs, 0, newRGBs, 0, rgbs.length);
-			if (source.transparentPixel >= newRGBs.length) {
-				/* Grow the palette with black */
-				rgbs = new RGB[source.transparentPixel + 1];
-				System.arraycopy(newRGBs, 0, rgbs, 0, newRGBs.length);
-				for (int i = newRGBs.length; i <= source.transparentPixel; i++) {
-					rgbs[i] = new RGB(0, 0, 0);
-				}
-			} else {
-				newRGBs[source.transparentPixel] = black;
-				rgbs = newRGBs;
-			}
-			blackIndex = source.transparentPixel;
-			image = new ImageData(source.width, source.height, source.depth, new PaletteData(rgbs));
-		} else {
-			while (blackIndex < rgbs.length) {
-				if (rgbs[blackIndex].equals(black)) break;
-				blackIndex++;
-			}
-			if (blackIndex == rgbs.length) {
-				/*
-				 * We didn't find black in the palette, and there is no transparent
-				 * pixel we can use.
-				 */
-				if ((1 << source.depth) > rgbs.length) {
-					/* We can grow the palette and add black */
-					RGB[] newRGBs = new RGB[rgbs.length + 1];
-					System.arraycopy(rgbs, 0, newRGBs, 0, rgbs.length);
-					newRGBs[rgbs.length] = black;
-					rgbs = newRGBs;
-				} else {
-					/* No room to grow the palette */
-					blackIndex = -1;
-				}
-			}
-			image = new ImageData(source.width, source.height, source.depth, new PaletteData(rgbs));
-		}
-	}
-	if (blackIndex == -1) {
-		/* There was no black in the palette, so just copy the data over */
-		System.arraycopy(source.data, 0, image.data, 0, image.data.length);
-	} else {
-		/* Modify the source image to contain black wherever the mask is 0 */
-		int[] imagePixels = new int[image.width];
-		int[] maskPixels = new int[mask.width];
-		for (int y = 0; y < image.height; y++) {
-			source.getPixels(0, y, image.width, imagePixels, 0);
-			mask.getPixels(0, y, mask.width, maskPixels, 0);
-			for (int i = 0; i < imagePixels.length; i++) {
-				if (maskPixels[i] == 0) imagePixels[i] = blackIndex;
-			}
-			image.setPixels(0, y, source.width, imagePixels, 0);
-		}
-	}
-	/*
-	 * Make sure the mask is padded properly. Windows requires icon masks
-	 * to have a scanline pad of 2.
-	 */
-	int bytesPerLine = (((mask.width + 7) / 8) + 1) / 2 * 2;
-	byte[] newMaskData = new byte[bytesPerLine * mask.height];
-	ImageData newMask = new ImageData(mask.width, mask.height, 1, mask.palette, 2, newMaskData);
-	int[] maskPixels = new int[mask.width];
-	for (int y = 0; y < mask.height; y++) {
-		mask.getPixels(0, y, mask.width, maskPixels, 0);
-		newMask.setPixels(0, y, newMask.width, maskPixels, 0);
-	}
-	/* Set the fields and create the icon */
-	image.maskPad = newMask.scanlinePad;
-	image.maskData = newMask.data;
-	init(device, image);
+	mask = ImageData.convertMask(mask);
+	init(device, this, source, mask);
 	if (device.tracking) device.new_Object(this);	
 }
 
@@ -712,7 +601,8 @@ public Image(Device device, ImageData source, ImageData mask) {
  * Constructs an instance of this class by loading its representation
  * from the specified input stream. Throws an error if an error
  * occurs while loading the image, or if the result is an image
- * of an unsupported type.
+ * of an unsupported type.  Application code is still responsible
+ * for closing the input stream.
  * <p>
  * This constructor is provided for convenience when loading a single
  * image only. If the stream contains multiple images, only the first
@@ -722,7 +612,20 @@ public Image(Device device, ImageData source, ImageData mask) {
  * This constructor may be used to load a resource as follows:
  * </p>
  * <pre>
- *     new Image(device, clazz.getResourceAsStream("file.gif"));
+ *     static Image loadImage (Display display, Class clazz, String string) {
+ *          InputStream stream = clazz.getResourceAsStream (string);
+ *          if (stream == null) return null;
+ *          Image image = null;
+ *          try {
+ *               image = new Image (display, stream);
+ *          } catch (SWTException ex) {
+ *          } finally {
+ *               try {
+ *                    stream.close ();
+ *               } catch (IOException ex) {}
+ *          }
+ *          return image;
+ *     }
  * </pre>
  *
  * @param device the device on which to create the image
@@ -735,7 +638,9 @@ public Image(Device device, ImageData source, ImageData mask) {
  * @exception SWTException <ul>
  *    <li>ERROR_INVALID_IMAGE - if the image file contains invalid data </li>
  *    <li>ERROR_IO - if an IO error occurs while reading data</li>
- * </ul>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the InputStream describes an image with an unsupported depth</li>
+ *    <li>ERROR_UNSUPPORTED_FORMAT - if the image file contains an unrecognized format</li>
+ *  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
  * </ul>
@@ -767,6 +672,8 @@ public Image (Device device, InputStream stream) {
  * @exception SWTException <ul>
  *    <li>ERROR_INVALID_IMAGE - if the image file contains invalid data </li>
  *    <li>ERROR_IO - if an IO error occurs while reading data</li>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the image file has an unsupported depth</li>
+ *    <li>ERROR_UNSUPPORTED_FORMAT - if the image file contains an unrecognized format</li>
  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
@@ -892,6 +799,159 @@ int createDIBFromDDB(int hDC, int hBitmap, int width, int height) {
 	return hDib;
 }
 
+int[] createGdipImage() {
+	switch (type) {
+		case SWT.BITMAP: {
+			if (alpha != -1 || alphaData != null || transparentPixel != -1) {
+				BITMAP bm = new BITMAP();
+				OS.GetObject(handle, BITMAP.sizeof, bm);
+				int imgWidth = bm.bmWidth;
+				int imgHeight = bm.bmHeight;
+				int hDC = device.internal_new_GC(null);
+				int srcHdc = OS.CreateCompatibleDC(hDC);
+				int oldSrcBitmap = OS.SelectObject(srcHdc, handle);
+				int memHdc = OS.CreateCompatibleDC(hDC);
+				int memDib = createDIB(imgWidth, imgHeight, 32);
+				if (memDib == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+				int oldMemBitmap = OS.SelectObject(memHdc, memDib);	
+				BITMAP dibBM = new BITMAP();
+				OS.GetObject(memDib, BITMAP.sizeof, dibBM);
+				int sizeInBytes = dibBM.bmWidthBytes * dibBM.bmHeight;	
+			 	OS.BitBlt(memHdc, 0, 0, imgWidth, imgHeight, srcHdc, 0, 0, OS.SRCCOPY);
+			 	byte red = 0, green = 0, blue = 0;
+			 	if (transparentPixel != -1) {
+					if (bm.bmBitsPixel <= 8)  {
+						byte[] color = new byte[4];
+						OS.GetDIBColorTable(srcHdc, transparentPixel, 1, color);
+						blue = color[0];
+						green = color[1];
+						red = color[2];
+					} else {
+						switch (bm.bmBitsPixel) {
+							case 16:
+								blue = (byte)((transparentPixel & 0x1F) << 3);
+								green = (byte)((transparentPixel & 0x3E0) >> 2);
+								red = (byte)((transparentPixel & 0x7C00) >> 7);
+								break;
+							case 24:
+								blue = (byte)((transparentPixel & 0xFF0000) >> 16);
+								green = (byte)((transparentPixel & 0xFF00) >> 8);
+								red = (byte)(transparentPixel & 0xFF);
+								break;
+							case 32:
+								blue = (byte)((transparentPixel & 0xFF000000) >>> 24);
+								green = (byte)((transparentPixel & 0xFF0000) >> 16);
+								red = (byte)((transparentPixel & 0xFF00) >> 8);
+								break;
+						}
+					}
+			 	}
+				OS.SelectObject(srcHdc, oldSrcBitmap);
+				OS.SelectObject(memHdc, oldMemBitmap);
+				OS.DeleteObject(srcHdc);
+				OS.DeleteObject(memHdc);
+			 	byte[] srcData = new byte[sizeInBytes];
+				OS.MoveMemory(srcData, dibBM.bmBits, sizeInBytes);
+				OS.DeleteObject(memDib);
+				device.internal_dispose_GC(hDC, null);
+				if (alpha != -1) {
+					for (int y = 0, dp = 0; y < imgHeight; ++y) {
+						for (int x = 0; x < imgWidth; ++x) {
+							srcData[dp + 3] = (byte)alpha;
+							dp += 4;
+						}
+					}
+				} else if (alphaData != null) {
+					for (int y = 0, dp = 0, ap = 0; y < imgHeight; ++y) {
+						for (int x = 0; x < imgWidth; ++x) {
+							srcData[dp + 3] = alphaData[ap++];
+							dp += 4;
+						}
+					}
+				} else if (transparentPixel != -1) {
+					for (int y = 0, dp = 0; y < imgHeight; ++y) {
+						for (int x = 0; x < imgWidth; ++x) {
+							if (srcData[dp] == blue && srcData[dp + 1] == green && srcData[dp + 2] == red) {
+								srcData[dp + 3] = (byte)0;
+							} else {
+								srcData[dp + 3] = (byte)0xFF;
+							}
+							dp += 4;
+						}
+					}
+				}
+				int hHeap = OS.GetProcessHeap();
+				int pixels = OS.HeapAlloc(hHeap, OS.HEAP_ZERO_MEMORY, srcData.length);
+				OS.MoveMemory(pixels, srcData, sizeInBytes);
+				return new int[]{Gdip.Bitmap_new(imgWidth, imgHeight, dibBM.bmWidthBytes, Gdip.PixelFormat32bppARGB, pixels), pixels};
+			}
+			return new int[]{Gdip.Bitmap_new(handle, 0), 0};
+		}
+		case SWT.ICON: {
+			/*
+			* Bug in GDI+. Creating a new GDI+ Bitmap from a HICON segment faults
+			* when the icon width is bigger than the icon height.  The fix is to
+			* detect this and create a PixelFormat32bppARGB image instead.
+			*/
+			ICONINFO iconInfo = new ICONINFO();
+			if (OS.IsWinCE) {
+				GetIconInfo(this, iconInfo);
+			} else {
+				OS.GetIconInfo(handle, iconInfo);
+			}
+			int hBitmap = iconInfo.hbmColor;
+			if (hBitmap == 0) hBitmap = iconInfo.hbmMask;
+			BITMAP bm = new BITMAP();
+			OS.GetObject(hBitmap, BITMAP.sizeof, bm);
+			int imgWidth = bm.bmWidth;
+			int imgHeight = hBitmap == iconInfo.hbmMask ? bm.bmHeight / 2 : bm.bmHeight;
+			int img = 0, pixels = 0;
+			if (imgWidth > imgHeight) {
+				int hDC = device.internal_new_GC(null);
+				int srcHdc = OS.CreateCompatibleDC(hDC);
+				int oldSrcBitmap = OS.SelectObject(srcHdc, hBitmap);
+				int memHdc = OS.CreateCompatibleDC(hDC);
+				int memDib = createDIB(imgWidth, imgHeight, 32);
+				if (memDib == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+				int oldMemBitmap = OS.SelectObject(memHdc, memDib);	
+				BITMAP dibBM = new BITMAP();
+				OS.GetObject(memDib, BITMAP.sizeof, dibBM);
+			 	OS.BitBlt(memHdc, 0, 0, imgWidth, imgHeight, srcHdc, 0, hBitmap == iconInfo.hbmMask ? imgHeight : 0, OS.SRCCOPY);
+				OS.SelectObject(memHdc, oldMemBitmap);
+				OS.DeleteObject(memHdc);
+			 	byte[] srcData = new byte[dibBM.bmWidthBytes * dibBM.bmHeight];
+				OS.MoveMemory(srcData, dibBM.bmBits, srcData.length);
+				OS.DeleteObject(memDib);
+				OS.SelectObject(srcHdc, iconInfo.hbmMask);
+				for (int y = 0, dp = 0; y < imgHeight; ++y) {
+					for (int x = 0; x < imgWidth; ++x) {
+						if (OS.GetPixel(srcHdc, x, y) != 0) {
+							srcData[dp + 3] = (byte)0;
+						} else {
+							srcData[dp + 3] = (byte)0xFF;
+						}
+						dp += 4;
+					}
+				}
+				OS.SelectObject(srcHdc, oldSrcBitmap);
+				OS.DeleteObject(srcHdc);
+				device.internal_dispose_GC(hDC, null);
+				int hHeap = OS.GetProcessHeap();
+				pixels = OS.HeapAlloc(hHeap, OS.HEAP_ZERO_MEMORY, srcData.length);
+				OS.MoveMemory(pixels, srcData, srcData.length);
+				img = Gdip.Bitmap_new(imgWidth, imgHeight, dibBM.bmWidthBytes, Gdip.PixelFormat32bppARGB, pixels);
+			} else {
+				img = Gdip.Bitmap_new(handle);
+			}
+			if (iconInfo.hbmColor != 0) OS.DeleteObject(iconInfo.hbmColor);
+			if (iconInfo.hbmMask != 0) OS.DeleteObject(iconInfo.hbmMask);
+			return new int[]{img, pixels};
+		}
+		default: SWT.error(SWT.ERROR_UNSUPPORTED_FORMAT);
+	}
+	return null;
+}
+
 /**
  * Disposes of the operating system resources associated with
  * the image. Applications must dispose of all images which
@@ -900,6 +960,7 @@ int createDIBFromDDB(int hDC, int hBitmap, int width, int height) {
 public void dispose () {
 	if (handle == 0) return;
 	if (device.isDisposed()) return;
+	if (memGC != null) memGC.dispose();
 	if (type == SWT.ICON) {
 		if (OS.IsWinCE) data = null;
 		OS.DestroyIcon (handle);
@@ -1187,20 +1248,13 @@ public ImageData getImageData() {
 					maskData[i] ^= -1;
 				}
 				/* Make sure mask scanlinePad is 2 */
-				int desiredScanline = (width + 7) / 8;
-				desiredScanline = desiredScanline + (desiredScanline % 2);
-				int realScanline = imageSize / height;
-				if (realScanline != desiredScanline) {
-					byte[] newData = new byte[desiredScanline * height];
-					int srcIndex = 0;
-					int destIndex = 0;
-					for (int i = 0; i < height; i++) {
-						System.arraycopy(maskData, srcIndex, newData, destIndex, desiredScanline);
-						destIndex += desiredScanline;
-						srcIndex += realScanline;
-					}
-					maskData = newData;
+				int maskPad;
+				int bpl = imageSize / height;
+				for (maskPad = 1; maskPad < 128; maskPad++) {
+					int calcBpl = (((width + 7) / 8) + (maskPad - 1)) / maskPad * maskPad;
+					if (calcBpl == bpl) break;
 				}
+				maskData = ImageData.convertPad(maskData, width, height, 1, maskPad, 2);
 			}
 			/* Clean up */
 			OS.HeapFree(hHeap, 0, lpvBits);
@@ -1219,7 +1273,6 @@ public ImageData getImageData() {
 			/* Construct and return the ImageData */
 			ImageData imageData = new ImageData(width, height, depth, palette, 4, data);
 			imageData.maskData = maskData;
-//			imageData.maskPad = 4;
 			imageData.maskPad = 2;
 			return imageData;
 		}
@@ -1245,6 +1298,7 @@ public ImageData getImageData() {
 				if (!isDib) {
 					boolean mustRestore = false;
 					if (memGC != null && !memGC.isDisposed()) {
+						memGC.flush ();
 						mustRestore = true;
 						GCData data = memGC.data;
 						if (data.hNullBitmap != 0) {
@@ -1418,7 +1472,7 @@ public ImageData getImageData() {
 
 /**
  * Returns an integer hash code for the receiver. Any two 
- * objects which return <code>true</code> when passed to 
+ * objects that return <code>true</code> when passed to 
  * <code>equals</code> must return the same value for this
  * method.
  *
@@ -1436,21 +1490,67 @@ void init(Device device, int width, int height) {
 	}
 	this.device = device;
 	type = SWT.BITMAP;
-	
-	/* Get the HDC for the device */
 	int hDC = device.internal_new_GC(null);
-			
-	/* Fill the bitmap with the current background color */
 	handle = OS.CreateCompatibleBitmap(hDC, width, height);
-	if (handle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-	int memDC = OS.CreateCompatibleDC(hDC);
-	int hOldBitmap = OS.SelectObject(memDC, handle);
-	OS.PatBlt(memDC, 0, 0, width, height, OS.PATCOPY);
-	OS.SelectObject(memDC, hOldBitmap);
-	OS.DeleteDC(memDC);
-	
-	/* Release the HDC for the device */	
+	/*
+	* Feature in Windows.  CreateCompatibleBitmap() may fail
+	* for large images.  The fix is to create a DIB section
+	* in that case.
+	*/
+	if (handle == 0) {
+		int bits = OS.GetDeviceCaps(hDC, OS.BITSPIXEL);
+		int planes = OS.GetDeviceCaps(hDC, OS.PLANES);
+		int depth = bits * planes;
+		if (depth < 16) depth = 16;
+		handle = createDIB(width, height, depth);
+	}
+	if (handle != 0) {
+		int memDC = OS.CreateCompatibleDC(hDC);
+		int hOldBitmap = OS.SelectObject(memDC, handle);
+		OS.PatBlt(memDC, 0, 0, width, height, OS.PATCOPY);
+		OS.SelectObject(memDC, hOldBitmap);
+		OS.DeleteDC(memDC);
+	}
 	device.internal_dispose_GC(hDC, null);
+	if (handle == 0) {
+		SWT.error(SWT.ERROR_NO_HANDLES, null, device.getLastError());
+	}
+}
+
+static int createDIB(int width, int height, int depth) {
+	BITMAPINFOHEADER bmiHeader = new BITMAPINFOHEADER();
+	bmiHeader.biSize = BITMAPINFOHEADER.sizeof;
+	bmiHeader.biWidth = width;
+	bmiHeader.biHeight = -height;
+	bmiHeader.biPlanes = 1;
+	bmiHeader.biBitCount = (short)depth;
+	if (OS.IsWinCE) bmiHeader.biCompression = OS.BI_BITFIELDS;
+	else bmiHeader.biCompression = OS.BI_RGB;
+	byte[] bmi = new byte[BITMAPINFOHEADER.sizeof + (OS.IsWinCE ? 12 : 0)];
+	OS.MoveMemory(bmi, bmiHeader, BITMAPINFOHEADER.sizeof);
+	/* Set the rgb colors into the bitmap info */
+	if (OS.IsWinCE) {
+		int redMask = 0xFF00;
+		int greenMask = 0xFF0000;
+		int blueMask = 0xFF000000;
+		/* big endian */
+		int offset = BITMAPINFOHEADER.sizeof;
+		bmi[offset] = (byte)((redMask & 0xFF000000) >> 24);
+		bmi[offset + 1] = (byte)((redMask & 0xFF0000) >> 16);
+		bmi[offset + 2] = (byte)((redMask & 0xFF00) >> 8);
+		bmi[offset + 3] = (byte)((redMask & 0xFF) >> 0);
+		bmi[offset + 4] = (byte)((greenMask & 0xFF000000) >> 24);
+		bmi[offset + 5] = (byte)((greenMask & 0xFF0000) >> 16);
+		bmi[offset + 6] = (byte)((greenMask & 0xFF00) >> 8);
+		bmi[offset + 7] = (byte)((greenMask & 0xFF) >> 0);
+		bmi[offset + 8] = (byte)((blueMask & 0xFF000000) >> 24);
+		bmi[offset + 9] = (byte)((blueMask & 0xFF0000) >> 16);
+		bmi[offset + 10] = (byte)((blueMask & 0xFF00) >> 8);
+		bmi[offset + 11] = (byte)((blueMask & 0xFF) >> 0);
+	}
+
+	int[] pBits = new int[1];
+	return OS.CreateDIBSection(0, bmi, OS.DIB_RGB_COLORS, pBits, 0, 0);
 }
 
 /**
@@ -1481,7 +1581,7 @@ static int[] init(Device device, Image image, ImageData i) {
 	 * into 4-bit ImageData before creating the image.
 	 */
 	/* Windows does not support 2-bit images. Convert to 4-bit image. */
-	if ((i.depth == 1 && i.getTransparencyType() != SWT.TRANSPARENCY_MASK) || i.depth == 2) {
+	if ((OS.IsWin95 && i.depth == 1 && i.getTransparencyType() != SWT.TRANSPARENCY_MASK) || i.depth == 2) {
 		ImageData img = new ImageData(i.width, i.height, 4, i.palette);
 		ImageData.blit(ImageData.BLIT_SRC, 
 			i.data, i.depth, i.bytesPerLine, i.getByteOrder(), 0, 0, i.width, i.height, null, null, null,
@@ -1634,16 +1734,7 @@ static int[] init(Device device, Image image, ImageData i) {
 	/* In case of a scanline pad other than 4, do the work to convert it */
 	byte[] data = i.data;
 	if (i.scanlinePad != 4 && (i.bytesPerLine % 4 != 0)) {
-		int newBpl = i.bytesPerLine + (4 - (i.bytesPerLine % 4));
-		byte[] newData = new byte[i.height * newBpl];
-		int srcPtr = 0;
-		int destPtr = 0;
-		for (int y = 0; y < i.height; y++) {
-			System.arraycopy(data, srcPtr, newData, destPtr, i.bytesPerLine);
-			srcPtr += i.bytesPerLine;
-			destPtr += newBpl;
-		}
-		data = newData;
+		data = ImageData.convertPad(data, i.width, i.height, i.depth, i.scanlinePad, 4);
 	}
 	OS.MoveMemory(pBits[0], data, data.length);
 	
@@ -1664,20 +1755,9 @@ static int[] init(Device device, Image image, ImageData i) {
 		/* Release the HDC for the device */	
 		device.internal_dispose_GC(hDC, null);
 			
-		/* Create the mask */
-//		int hHeap = OS.GetProcessHeap();
-//		int bmBits = OS.HeapAlloc(hHeap, OS.HEAP_ZERO_MEMORY, i.maskData.length);
-//		OS.MoveMemory(bmBits, i.maskData, i.maskData.length);
-//		BITMAP bm = new BITMAP();
-//		bm.bmWidth = i.width;
-//		bm.bmHeight = i.height;
-//		bm.bmWidthBytes = (((i.width + 7) / 8) + 3) / 4 * 4;
-//		bm.bmPlanes = 1;
-//		bm.bmBitsPixel = 1;
-//		bm.bmBits = bmBits;
-//		int hMask = OS.CreateBitmapIndirect(bm);
-//		OS.HeapFree(hHeap, 0, bmBits);
-		int hMask = OS.CreateBitmap(i.width, i.height, 1, 1, i.maskData);
+		/* Create the mask. Windows requires icon masks to have a scanline pad of 2. */
+		byte[] maskData = ImageData.convertPad(i.maskData, i.width, i.height, 1, i.maskPad, 2);
+		int hMask = OS.CreateBitmap(i.width, i.height, 1, 1, maskData);
 		if (hMask == 0) SWT.error(SWT.ERROR_NO_HANDLES);	
 		OS.SelectObject(hdcSrc, hMask);
 		OS.PatBlt(hdcSrc, 0, 0, i.width, i.height, OS.DSTINVERT);
@@ -1721,6 +1801,79 @@ static int[] init(Device device, Image image, ImageData i) {
 	return result;
 }
 
+static int[] init(Device device, Image image, ImageData source, ImageData mask) {
+	/* Create a temporary image and locate the black pixel */
+	ImageData imageData;
+	int blackIndex = 0;
+	if (source.palette.isDirect) {
+		imageData = new ImageData(source.width, source.height, source.depth, source.palette);
+	} else {
+		RGB black = new RGB(0, 0, 0);
+		RGB[] rgbs = source.getRGBs();
+		if (source.transparentPixel != -1) {
+			/*
+			 * The source had transparency, so we can use the transparent pixel
+			 * for black.
+			 */
+			RGB[] newRGBs = new RGB[rgbs.length];
+			System.arraycopy(rgbs, 0, newRGBs, 0, rgbs.length);
+			if (source.transparentPixel >= newRGBs.length) {
+				/* Grow the palette with black */
+				rgbs = new RGB[source.transparentPixel + 1];
+				System.arraycopy(newRGBs, 0, rgbs, 0, newRGBs.length);
+				for (int i = newRGBs.length; i <= source.transparentPixel; i++) {
+					rgbs[i] = new RGB(0, 0, 0);
+				}
+			} else {
+				newRGBs[source.transparentPixel] = black;
+				rgbs = newRGBs;
+			}
+			blackIndex = source.transparentPixel;
+			imageData = new ImageData(source.width, source.height, source.depth, new PaletteData(rgbs));
+		} else {
+			while (blackIndex < rgbs.length) {
+				if (rgbs[blackIndex].equals(black)) break;
+				blackIndex++;
+			}
+			if (blackIndex == rgbs.length) {
+				/*
+				 * We didn't find black in the palette, and there is no transparent
+				 * pixel we can use.
+				 */
+				if ((1 << source.depth) > rgbs.length) {
+					/* We can grow the palette and add black */
+					RGB[] newRGBs = new RGB[rgbs.length + 1];
+					System.arraycopy(rgbs, 0, newRGBs, 0, rgbs.length);
+					newRGBs[rgbs.length] = black;
+					rgbs = newRGBs;
+				} else {
+					/* No room to grow the palette */
+					blackIndex = -1;
+				}
+			}
+			imageData = new ImageData(source.width, source.height, source.depth, new PaletteData(rgbs));
+		}
+	}
+	if (blackIndex == -1) {
+		/* There was no black in the palette, so just copy the data over */
+		System.arraycopy(source.data, 0, imageData.data, 0, imageData.data.length);
+	} else {
+		/* Modify the source image to contain black wherever the mask is 0 */
+		int[] imagePixels = new int[imageData.width];
+		int[] maskPixels = new int[mask.width];
+		for (int y = 0; y < imageData.height; y++) {
+			source.getPixels(0, y, imageData.width, imagePixels, 0);
+			mask.getPixels(0, y, mask.width, maskPixels, 0);
+			for (int i = 0; i < imagePixels.length; i++) {
+				if (maskPixels[i] == 0) imagePixels[i] = blackIndex;
+			}
+			imageData.setPixels(0, y, source.width, imagePixels, 0);
+		}
+	}
+	imageData.maskPad = mask.scanlinePad;
+	imageData.maskData = mask.data;	
+	return init(device, image, imageData);
+}
 void init(Device device, ImageData i) {
 	if (i == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
 	init(device, this, i);
@@ -1780,7 +1933,7 @@ public int internal_new_GC (GCData data) {
  * application code.
  * </p>
  *
- * @param handle the platform specific GC handle
+ * @param hDC the platform specific GC handle
  * @param data the platform specific GC data 
  */
 public void internal_dispose_GC (int hDC, GCData data) {
@@ -1812,7 +1965,7 @@ public boolean isDisposed() {
  * these cases. For example:
  * <pre>
  *    Button b = new Button();
- *    image.setBackground(b.getBackground());>
+ *    image.setBackground(b.getBackground());
  *    b.setImage(image);
  * </pre>
  * </p><p>
@@ -1893,7 +2046,7 @@ public String toString () {
  * @param device the device on which to allocate the color
  * @param type the type of the image (<code>SWT.BITMAP</code> or <code>SWT.ICON</code>)
  * @param handle the OS handle for the image
- * @param hPalette the OS handle for the palette, or 0
+ * @return a new image object containing the specified device, type and handle
  */
 public static Image win32_new(Device device, int type, int handle) {
 	if (device == null) device = Device.getDevice();
