@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -38,10 +38,12 @@ import org.eclipse.swt.graphics.*;
  * </p>
  */
 public class Label extends Control {
-	Image image;
-	int font;
+	String text = "";
+	Image image, image2;
+	int font, hCopiedBitmap;
 	static final int LabelProc;
 	static final TCHAR LabelClass = new TCHAR (0, "STATIC", true);
+	static final int ICON_WIDTH = 128, ICON_HEIGHT = 128;
 	static {
 		WNDCLASS lpWndClass = new WNDCLASS ();
 		OS.GetClassInfo (0, LabelClass, lpWndClass);
@@ -89,12 +91,126 @@ public Label (Composite parent, int style) {
 	super (parent, checkStyle (style));
 }
 
-int callWindowProc (int msg, int wParam, int lParam) {
+void _setImage (Image image) {
+	if (image2 != null) image2.dispose ();
+	image2 = null;
+	if (hCopiedBitmap != 0) OS.DeleteObject (hCopiedBitmap);
+	hCopiedBitmap = 0;
+	boolean hasAlpha = false;
+	int hImage = 0, imageBits = 0, fImageType = 0;
+	if (image != null) {
+		switch (image.type) {
+			case SWT.BITMAP: {
+				ImageData data = image.getImageData ();
+				if (OS.COMCTL32_MAJOR < 6) {
+					Rectangle rect = image.getBounds ();
+					switch (data.getTransparencyType ()) {
+						case SWT.TRANSPARENCY_PIXEL: 
+							if (rect.width <= ICON_WIDTH && rect.height <= ICON_HEIGHT) {
+								image2 = new Image (display, data, data.getTransparencyMask ());
+								hImage = image2.handle;
+								imageBits = OS.SS_ICON;
+								fImageType = OS.IMAGE_ICON;
+								break;
+							}
+							//FALL THROUGH
+						case SWT.TRANSPARENCY_ALPHA:
+							image2 = new Image (display, rect.width, rect.height);
+							GC gc = new GC (image2);
+							gc.setBackground (getBackground ());
+							gc.fillRectangle (rect);
+							gc.drawImage (image, 0, 0);
+							gc.dispose ();
+							hImage = image2.handle;
+							imageBits = OS.SS_BITMAP;
+							fImageType = OS.IMAGE_BITMAP;
+							break;
+						case SWT.TRANSPARENCY_NONE:
+							hImage = image.handle;
+							imageBits = OS.SS_BITMAP;
+							fImageType = OS.IMAGE_BITMAP;
+							break;
+					}
+				} else {
+					if (data.alpha != -1 || data.alphaData != null || data.transparentPixel != -1) {
+						hasAlpha = true;
+						hImage = Display.create32bitDIB (image.handle, data.alpha, data.alphaData, data.transparentPixel);
+					} else {
+						hImage = image.handle;
+						/*
+						* Bug in Windows.  For some reason in Windows XP only, indexed palette
+						* bitmaps are not drawn properly even though the screen depth can
+						* handle all colors in the palette.  The fix is to use a higher depth
+						* bitmap instead.
+						*/
+						if (data.depth <= 8 && display.getDepth () > 8) {
+							image2 = new Image (display, data.width, data.height);
+							GC gc = new GC (image2);
+							gc.drawImage (image, 0, 0);
+							gc.dispose ();
+							hImage = image2.handle;
+						}
+					}
+					imageBits = OS.SS_BITMAP;
+					fImageType = OS.IMAGE_BITMAP;
+				}
+				break;
+			}
+			case SWT.ICON: {
+				hImage = image.handle;
+				imageBits = OS.SS_ICON;
+				fImageType = OS.IMAGE_ICON;
+				break;
+			}
+		}
+	}
+	RECT rect = new RECT ();
+	OS.GetWindowRect (handle, rect);
+	int newBits = OS.GetWindowLong (handle, OS.GWL_STYLE);
+	int oldBits = newBits;
+	newBits &= ~(OS.SS_BITMAP | OS.SS_ICON);
+	newBits |= imageBits | OS.SS_REALSIZEIMAGE | OS.SS_CENTERIMAGE;
+	if (newBits != oldBits) {
+		OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);
+	}
+	OS.SendMessage (handle, OS.STM_SETIMAGE, fImageType, hImage);
+	
+	/*
+	* When STM_SETIMAGE encounters a bitmap with alpha information,
+	* it makes a copy of the bitmap.  Therefore the bitmap that was
+	* created to preserve transparency can be deleted right away.
+	* 
+	* Note: The client code also needs to delete the copied image
+	* created by Windows when the image changed but does not need
+	* to delete the copied image when the control is disposed.
+	*/
+	if (hasAlpha && hImage != 0) {
+		OS.DeleteObject (hImage);
+		hCopiedBitmap = OS.SendMessage (handle, OS.STM_GETIMAGE, OS.IMAGE_BITMAP, 0);
+	}
+
+	/*
+	* Feature in Windows.  When STM_SETIMAGE is used to set the
+	* image for a static control, Windows either streches the image
+	* to fit the control or shrinks the control to fit the image.
+	* While not stricly wrong, neither of these is desirable.
+	* The fix is to stop Windows from stretching the image by
+	* using SS_REALSIZEIMAGE and SS_CENTERIMAGE, allow Windows
+	* to shrink the control, and then restore the control to the
+	* original size.
+	*/
+	int flags = OS.SWP_NOZORDER | OS.SWP_DRAWFRAME | OS.SWP_NOACTIVATE | OS.SWP_NOMOVE;
+	SetWindowPos (handle, 0, 0, 0, rect.right - rect.left, rect.bottom - rect.top, flags);
+	OS.InvalidateRect (handle, null, true);
+}
+
+int callWindowProc (int hwnd, int msg, int wParam, int lParam) {
 	if (handle == 0) return 0;
-	return OS.CallWindowProc (LabelProc, handle, msg, wParam, lParam);
+	return OS.CallWindowProc (LabelProc, hwnd, msg, wParam, lParam);
 }
 
 static int checkStyle (int style) {
+	style |= SWT.NO_FOCUS;
 	if ((style & SWT.SEPARATOR) != 0) {
 		style = checkBits (style, SWT.VERTICAL, SWT.HORIZONTAL, 0, 0, 0, 0);
 		return checkBits (style, SWT.SHADOW_OUT, SWT.SHADOW_IN, SWT.SHADOW_NONE, 0, 0, 0);
@@ -149,7 +265,7 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 		width = rect.right - rect.left;
 		height = rect.bottom - rect.top;
 		if (height == 0) {
-			TEXTMETRIC tm = new TEXTMETRIC ();
+			TEXTMETRIC tm = OS.IsUnicode ? (TEXTMETRIC) new TEXTMETRICW () : new TEXTMETRICA ();
 			OS.GetTextMetrics (hDC, tm);
 			height = tm.tmHeight;
 		}
@@ -228,11 +344,7 @@ String getNameText () {
 public String getText () {
 	checkWidget ();
 	if ((style & SWT.SEPARATOR) != 0) return "";
-	int length = OS.GetWindowTextLength (handle);
-	if (length == 0) return "";
-	TCHAR buffer = new TCHAR (getCodePage (), length + 1);
-	OS.GetWindowText (handle, buffer, length + 1);
-	return buffer.toString (0, length);
+	return text;
 }
 
 /*
@@ -271,7 +383,13 @@ boolean mnemonicMatch (char key) {
 
 void releaseWidget () {
 	super.releaseWidget ();
+	if (image2 != null) image2.dispose ();
+	image2 = null;
+	text = null;
 	image = null;
+
+	/* Windows deletes the copied image when the control is disposed */
+	hCopiedBitmap = 0;
 }
 
 /**
@@ -316,11 +434,6 @@ public void setAlignment (int alignment) {
 	OS.InvalidateRect (handle, null, true);
 }
 
-public boolean setFocus () {
-	checkWidget();
-	return false;
-}
-
 /**
  * Sets the receiver's image to the argument, which may be
  * null indicating that no image should be displayed.
@@ -338,47 +451,8 @@ public boolean setFocus () {
 public void setImage (Image image) {
 	checkWidget ();
 	if ((style & SWT.SEPARATOR) != 0) return;
-	int hImage = 0, imageBits = 0, fImageType = 0;
-	if (image != null) {
-		if (image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
-		hImage = image.handle;
-		switch (image.type) {
-			case SWT.BITMAP:
-				imageBits = OS.SS_BITMAP;
-				fImageType = OS.IMAGE_BITMAP;
-				break;
-			case SWT.ICON:
-				imageBits = OS.SS_ICON;
-				fImageType = OS.IMAGE_ICON;
-				break;
-			default:
-				return;
-		}
-	}
-	this.image = image;
-	RECT rect = new RECT ();
-	OS.GetWindowRect (handle, rect);
-	int newBits = OS.GetWindowLong (handle, OS.GWL_STYLE);
-	int oldBits = newBits;
-	newBits &= ~(OS.SS_BITMAP | OS.SS_ICON);
-	newBits |= imageBits | OS.SS_REALSIZEIMAGE | OS.SS_CENTERIMAGE;
-	if (newBits != oldBits) {
-		OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);
-	}
-	OS.SendMessage (handle, OS.STM_SETIMAGE, fImageType, hImage);
-	/*
-	* Feature in Windows.  When STM_SETIMAGE is used to set the
-	* image for a static control, Windows either streches the image
-	* to fit the control or shrinks the control to fit the image.
-	* While not stricly wrong, neither of these is desirable.
-	* The fix is to stop Windows from stretching the image by
-	* using SS_REALSIZEIMAGE and SS_CENTERIMAGE, allow Windows
-	* to shrink the control, and then restore the control to the
-	* original size.
-	*/
-	int flags = OS.SWP_NOZORDER | OS.SWP_DRAWFRAME | OS.SWP_NOACTIVATE | OS.SWP_NOMOVE;
-	OS.SetWindowPos (handle, 0, 0, 0, rect.right - rect.left, rect.bottom - rect.top, flags);
-	OS.InvalidateRect (handle, null, true);
+	if (image != null && image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
+	_setImage (this.image = image);
 }
 
 /**
@@ -412,6 +486,14 @@ public void setText (String string) {
 	checkWidget ();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if ((style & SWT.SEPARATOR) != 0) return;
+	/*
+	* Feature in Windows.  For some reason, SetWindowText() for
+	* static controls redraws the control, even when the text has
+	* has not changed.  The fix is to check for this case and do
+	* nothing.
+	*/
+	if (string.equals (text)) return;
+	text = string;
 	int newBits = OS.GetWindowLong (handle, OS.GWL_STYLE), oldBits = newBits;
 	newBits &= ~(OS.SS_BITMAP | OS.SS_ICON | OS.SS_REALSIZEIMAGE | OS.SS_CENTERIMAGE);
 	if ((style & SWT.LEFT) != 0 && (style & SWT.WRAP) == 0) newBits |= OS.SS_LEFTNOWORDWRAP;
@@ -566,6 +648,29 @@ LRESULT WM_SIZE (int wParam, int lParam) {
 	return result;
 }
 
+LRESULT wmColorChild (int wParam, int lParam) {
+	LRESULT result = super.wmColorChild (wParam, lParam);
+	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+		Control control = findThemeControl ();
+		if (control != null) {
+			OS.SetBkMode (wParam, OS.TRANSPARENT);
+			RECT rect = new RECT ();
+			OS.GetClientRect (control.handle, rect);
+			OS.MapWindowPoints (control.handle, handle, rect, 2);
+			control.drawThemeBackground (wParam, rect);
+			return new LRESULT (OS.GetStockObject (OS.NULL_BRUSH));
+		}
+	}
+	return result;
+}
+
+LRESULT WM_SYSCOLORCHANGE (int wParam, int lParam) {
+	LRESULT result = super.WM_SYSCOLORCHANGE (wParam, lParam);
+	if (result != null) return result;
+	if (image2 != null) _setImage (image);
+	return result;
+}
+
 LRESULT wmDrawChild (int wParam, int lParam) {
 	DRAWITEMSTRUCT struct = new DRAWITEMSTRUCT ();
 	OS.MoveMemory (struct, lParam, DRAWITEMSTRUCT.sizeof);
