@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -21,7 +21,7 @@ import org.eclipse.swt.events.*;
  * objects that represent a range of positive, numeric values. 
  * <p>
  * At any given moment, a given slider will have a 
- * single <em>selection</em> that is considered to be its
+ * single 'selection' that is considered to be its
  * value, which is constrained to be within the range of
  * values the slider represents (that is, between its
  * <em>minimum</em> and <em>maximum</em> values).
@@ -114,7 +114,7 @@ public Slider (Composite parent, int style) {
  * interface.
  * <p>
  * When <code>widgetSelected</code> is called, the event object detail field contains one of the following values:
- * <code>0</code> - for the end of a drag.
+ * <code>SWT.NONE</code> - for the end of a drag.
  * <code>SWT.DRAG</code>.
  * <code>SWT.HOME</code>.
  * <code>SWT.END</code>.
@@ -147,9 +147,22 @@ public void addSelectionListener (SelectionListener listener) {
 	addListener (SWT.DefaultSelection,typedListener);
 }
 
-int callWindowProc (int msg, int wParam, int lParam) {
+int callWindowProc (int hwnd, int msg, int wParam, int lParam) {
 	if (handle == 0) return 0;
-	return OS.CallWindowProc (ScrollBarProc, handle, msg, wParam, lParam);
+	/*
+	* Feature in Windows.  Windows runs a modal message
+	* loop when the user drags a scroll bar.  This means
+	* that mouse down events won't get delivered until
+	* after the loop finishes.  The fix is to run any
+	* deferred messages, including mouse down messages
+	* before calling the scroll bar window proc.
+	*/
+	switch (msg) {
+		case OS.WM_LBUTTONDOWN:
+		case OS.WM_LBUTTONDBLCLK: 
+			display.runDeferredEvents ();
+	}
+	return OS.CallWindowProc (ScrollBarProc, hwnd, msg, wParam, lParam);
 }
 
 static int checkStyle (int style) {
@@ -198,6 +211,19 @@ int defaultForeground () {
 	return OS.GetSysColor (OS.COLOR_BTNFACE);
 }
 
+void enableWidget (boolean enabled) {
+	super.enableWidget (enabled);
+	if (!OS.IsWinCE) {
+		int flags = enabled ? OS.ESB_ENABLE_BOTH : OS.ESB_DISABLE_BOTH;
+		OS.EnableScrollBar (handle, OS.SB_CTL, flags);
+	}
+	if (enabled) {
+		state &= ~DISABLED;
+	} else {
+		state |= DISABLED;
+	}
+}
+
 public boolean getEnabled () {
 	checkWidget ();
 	return (state & DISABLED) == 0;
@@ -276,7 +302,7 @@ public int getPageIncrement () {
 }
 
 /**
- * Returns the single <em>selection</em> that is the receiver's value.
+ * Returns the 'selection', which is the receiver's value.
  *
  * @return the selection
  *
@@ -353,18 +379,6 @@ void setBounds (int x, int y, int width, int height, int flags) {
 	}
 }
 
-public void setEnabled (boolean enabled) {
-	checkWidget ();
-	if (OS.IsWinCE) {
-		super.setEnabled (enabled);
-	} else {
-		int flags = enabled ? OS.ESB_ENABLE_BOTH : OS.ESB_DISABLE_BOTH;
-		OS.EnableScrollBar (handle, OS.SB_CTL, flags);
-	}
-	state &= ~DISABLED;
-	if (!enabled) state |= DISABLED;
-}
-
 /**
  * Sets the amount that the receiver's value will be
  * modified by when the up/down (or right/left) arrows
@@ -385,11 +399,12 @@ public void setIncrement (int value) {
 }
 
 /**
- * Sets the maximum value which the receiver will allow
- * to be the argument which must be greater than or
- * equal to zero.
+ * Sets the maximum. If this value is negative or less than or
+ * equal to the minimum, the value is ignored. If necessary, first
+ * the thumb and then the selection are adjusted to fit within the
+ * new range.
  *
- * @param value the new maximum (must be zero or greater)
+ * @param value the new maximum, which must be greater than the current minimum
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -405,41 +420,16 @@ public void setMaximum (int value) {
 	OS.GetScrollInfo (handle, OS.SB_CTL, info);
 	if (value - info.nMin - info.nPage < 1) return;
 	info.nMax = value;
-	OS.SetScrollInfo (handle, OS.SB_CTL, info, (state & DISABLED) == 0);
-		
-	/*
-	* Feature in Windows.  Using SIF_DISABLENOSCROLL,
-	* SetScrollInfo () can change enabled and disabled
-	* state of the scroll bar causing a scroll bar that
-	* was disabled by the application to become enabled.
-	* The fix is to disable the scroll bar (again) when
-	* the application has disabled the scroll bar.
-	*/
-	if ((state & DISABLED) != 0) {
-		if (OS.IsWinCE) {
-			OS.EnableWindow (handle, false);
-		} else {
-			OS.EnableScrollBar (handle, OS.SB_CTL, OS.ESB_DISABLE_BOTH);
-		}
-	}
-	
-	/*
-	* Bug in Windows.  If the thumb is resized when it has focus,
-	* the flashing cursor that is used to show that the scroll bar has
-	* focus is not moved.  The fix is to post a fake WM_SETFOCUS to
-	* get the scroll bar to recompute the size of the flashing cursor.
-	*/
-	if (OS.GetFocus () == handle) {
-		OS.PostMessage (handle, OS.WM_SETFOCUS, 0, 0);
-	}
+	SetScrollInfo (handle, OS.SB_CTL, info, true);
 }
 
 /**
- * Sets the minimum value which the receiver will allow
- * to be the argument which must be greater than or
- * equal to zero.
+ * Sets the minimum value. If this value is negative or greater
+ * than or equal to the maximum, the value is ignored. If necessary,
+ * first the thumb and then the selection are adjusted to fit within
+ * the new range.
  *
- * @param value the new minimum (must be zero or greater)
+ * @param value the new minimum
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -455,33 +445,7 @@ public void setMinimum (int value) {
 	OS.GetScrollInfo (handle, OS.SB_CTL, info);
 	if (info.nMax - value - info.nPage < 1) return;
 	info.nMin = value;
-	OS.SetScrollInfo (handle, OS.SB_CTL, info, true);
-
-	/*
-	* Feature in Windows.  Using SIF_DISABLENOSCROLL,
-	* SetScrollInfo () can change enabled and disabled
-	* state of the scroll bar causing a scroll bar that
-	* was disabled by the application to become enabled.
-	* The fix is to disable the scroll bar (again) when
-	* the application has disabled the scroll bar.
-	*/
-	if ((state & DISABLED) != 0) {
-		if (OS.IsWinCE) {
-			OS.EnableWindow (handle, false);
-		} else {
-			OS.EnableScrollBar (handle, OS.SB_CTL, OS.ESB_DISABLE_BOTH);
-		}
-	}
-	
-	/*
-	* Bug in Windows.  If the thumb is resized when it has focus,
-	* the flashing cursor that is used to show that the scroll bar has
-	* focus is not moved.  The fix is to post a fake WM_SETFOCUS to
-	* get the scroll bar to recompute the size of the flashing cursor.
-	*/
-	if (OS.GetFocus () == handle) {
-		OS.PostMessage (handle, OS.WM_SETFOCUS, 0, 0);
-	}
+	SetScrollInfo (handle, OS.SB_CTL, info, true);
 }
 
 /**
@@ -490,7 +454,7 @@ public void setMinimum (int value) {
  * are selected to the argument, which must be at least
  * one.
  *
- * @return the page increment (must be greater than zero)
+ * @param value the page increment (must be greater than zero)
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -503,9 +467,40 @@ public void setPageIncrement (int value) {
 	pageIncrement = value;
 }
 
+boolean SetScrollInfo (int hwnd, int flags, SCROLLINFO info, boolean fRedraw) {
+	/*
+	* Feature in Windows.  Using SIF_DISABLENOSCROLL,
+	* SetScrollInfo () can change enabled and disabled
+	* state of the scroll bar causing a scroll bar that
+	* was disabled by the application to become enabled.
+	* The fix is to disable the scroll bar (again) when
+	* the application has disabled the scroll bar.
+	*/
+	if ((state & DISABLED) != 0) fRedraw = false;
+	boolean result = OS.SetScrollInfo (hwnd, flags, info, fRedraw);
+	if ((state & DISABLED) != 0) {
+		OS.EnableWindow (handle, false);
+		if (!OS.IsWinCE) {
+			OS.EnableScrollBar (handle, OS.SB_CTL, OS.ESB_DISABLE_BOTH);
+		}
+	}
+	
+	/*
+	* Bug in Windows.  If the thumb is resized when it has focus,
+	* the flashing cursor that is used to show that the scroll bar
+	* has focus is not moved.  The fix is to post a fake WM_SETFOCUS
+	* to get the scroll bar to recompute the size of the flashing
+	* cursor.
+	*/
+	if (OS.GetFocus () == handle) {
+		OS.PostMessage (handle, OS.WM_SETFOCUS, 0, 0);
+	}
+	return result;
+}
+
 /**
- * Sets the single <em>selection</em> that is the receiver's
- * value to the argument which must be greater than or equal
+ * Sets the 'selection', which is the receiver's
+ * value, to the argument which must be greater than or equal
  * to zero.
  *
  * @param value the new selection (must be zero or greater)
@@ -521,62 +516,33 @@ public void setSelection (int value) {
 	info.cbSize = SCROLLINFO.sizeof;
 	info.fMask = OS.SIF_POS;
 	info.nPos = value;
-	OS.SetScrollInfo (handle, OS.SB_CTL, info, true);
+	SetScrollInfo (handle, OS.SB_CTL, info, true);
 }
 
 /**
  * Sets the size of the receiver's thumb relative to the
- * difference between its maximum and minimum values to the
- * argument which must be at least one.
+ * difference between its maximum and minimum values.  This new
+ * value will be ignored if it is less than one, and will be
+ * clamped if it exceeds the receiver's current range.
  *
- * @param value the new thumb value (must be at least one)
+ * @param value the new thumb value, which must be at least one and not
+ * larger than the size of the current range
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- *
- * @see ScrollBar
  */
 public void setThumb (int value) {
 	checkWidget ();
-
-	/* Position the thumb */
 	if (value < 1) return;
 	SCROLLINFO info = new SCROLLINFO ();
 	info.cbSize = SCROLLINFO.sizeof;
 	info.fMask = OS.SIF_PAGE | OS.SIF_RANGE | OS.SIF_DISABLENOSCROLL;
 	OS.GetScrollInfo (handle, OS.SB_CTL, info);
-	if (info.nMax - info.nMin - value < 0) return;
 	info.nPage = value;
 	if (info.nPage != 0) info.nPage++;
-	OS.SetScrollInfo (handle, OS.SB_CTL, info, true);
-		
-	/*
-	* Feature in Windows.  Using SIF_DISABLENOSCROLL,
-	* SetScrollInfo () can change enabled and disabled
-	* state of the scroll bar causing a scroll bar that
-	* was disabled by the application to become enabled.
-	* The fix is to disable the scroll bar (again) when
-	* the application has disabled the scroll bar.
-	*/
-	if ((state & DISABLED) != 0) {
-		if (OS.IsWinCE) {
-			OS.EnableWindow (handle, false);
-		} else {
-			OS.EnableScrollBar (handle, OS.SB_CTL, OS.ESB_DISABLE_BOTH);
-		}
-	}
-	
-	/*
-	* Bug in Windows.  If the thumb is resized when it has focus,
-	* the flashing cursor that is used to show that the scroll bar has
-	* focus is not moved.  The fix is to post a fake WM_SETFOCUS to
-	* get the scroll bar to recompute the size of the flashing cursor.
-	*/
-	if (OS.GetFocus () == handle) {
-		OS.PostMessage (handle, OS.WM_SETFOCUS, 0, 0);
-	}
+	SetScrollInfo (handle, OS.SB_CTL, info, true);
 }
 
 /**
@@ -605,7 +571,6 @@ public void setValues (int selection, int minimum, int maximum, int thumb, int i
 	if (minimum < 0) return;
 	if (maximum < 0) return;
 	if (thumb < 1) return;
-	if (maximum - minimum - thumb < 0) return;
 	if (increment < 1) return;
 	if (pageIncrement < 1) return;
 	this.increment = increment;
@@ -618,33 +583,7 @@ public void setValues (int selection, int minimum, int maximum, int thumb, int i
 	info.nMax = maximum;
 	info.nPage = thumb;
 	if (info.nPage != 0) info.nPage++;
-	OS.SetScrollInfo (handle, OS.SB_CTL, info, true);
-		
-	/*
-	* Feature in Windows.  Using SIF_DISABLENOSCROLL,
-	* SetScrollInfo () can change enabled and disabled
-	* state of the scroll bar causing a scroll bar that
-	* was disabled by the application to become enabled.
-	* The fix is to disable the scroll bar (again) when
-	* the application has disabled the scroll bar.
-	*/
-	if ((state & DISABLED) != 0) {
-		if (OS.IsWinCE) {
-			OS.EnableWindow (handle, false);
-		} else {
-			OS.EnableScrollBar (handle, OS.SB_CTL, OS.ESB_DISABLE_BOTH);
-		}
-	}
-	
-	/*
-	* Bug in Windows.  If the thumb is resized when it has focus,
-	* the flashing cursor that is used to show that the scroll bar has
-	* focus is not moved.  The fix is to post a fake WM_SETFOCUS to
-	* get the scroll bar to recompute the size of the flashing cursor.
-	*/
-	if (OS.GetFocus () == handle) {
-		OS.PostMessage (handle, OS.WM_SETFOCUS, 0, 0);
-	}
+	SetScrollInfo (handle, OS.SB_CTL, info, true);
 }
 
 int widgetExtStyle () {
@@ -683,7 +622,6 @@ LRESULT WM_KEYDOWN (int wParam, int lParam) {
  	LRESULT result = super.WM_KEYDOWN (wParam, lParam);
  	if (result != null) return result;
  	if ((style & SWT.VERTICAL) != 0) return result;
- 	
  	/*
  	* Bug in Windows.  When a horizontal scroll bar is mirrored,
  	* the native control does not correctly swap the arrow keys.
@@ -698,7 +636,7 @@ LRESULT WM_KEYDOWN (int wParam, int lParam) {
 	 		case OS.VK_LEFT: 
 			case OS.VK_RIGHT: {
 				int key = wParam == OS.VK_LEFT ? OS.VK_RIGHT : OS.VK_LEFT;
-				int code = callWindowProc (OS.WM_KEYDOWN, key, lParam);
+				int code = callWindowProc (handle, OS.WM_KEYDOWN, key, lParam);
 	 			return new LRESULT (code);
 	 		}
 	 	}
@@ -708,14 +646,6 @@ LRESULT WM_KEYDOWN (int wParam, int lParam) {
  
 LRESULT WM_LBUTTONDBLCLK (int wParam, int lParam) {
 	
-	/*
-	* Feature in Windows.  For some reason, capturing
-	* the mouse after processing WM_LBUTTONDBLCLK for the
-	* widget interferes with the normal mouse processing
-	* for the widget.  The fix is to avoid the automatic
-	* mouse capture.
-	*/
-
 	/*
 	* Feature in Windows.  Windows uses the WS_TABSTOP
 	* style for the scroll bar to decide that focus
@@ -724,27 +654,27 @@ LRESULT WM_LBUTTONDBLCLK (int wParam, int lParam) {
 	* and restore WS_TABSTOP so that Windows will not
 	* assign focus.
 	*/
-
-	int hwndCapture = OS.GetCapture ();
 	int oldBits = OS.GetWindowLong (handle, OS.GWL_STYLE);
 	int newBits = oldBits & ~OS.WS_TABSTOP;
 	OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);	
 	LRESULT result = super.WM_LBUTTONDBLCLK (wParam, lParam);
 	OS.SetWindowLong (handle, OS.GWL_STYLE, oldBits);
-	if (OS.GetCapture () != hwndCapture) OS.SetCapture (hwndCapture);
+	
+	/*
+	* Feature in Windows.  Windows runs a modal message loop
+	* when the user drags a scroll bar that terminates when
+	* it sees an WM_LBUTTONUP.  Unfortunately the WM_LBUTTONUP
+	* is consumed.  The fix is to send a fake mouse up and
+	* release the automatic capture.
+	*/
+	if (!OS.IsWinCE) {
+		sendMouseEvent (SWT.MouseUp, 1, handle, OS.WM_LBUTTONUP, wParam, lParam);
+		if (OS.GetCapture () == handle) OS.ReleaseCapture ();
+	}
 	return result;
 }
 
 LRESULT WM_LBUTTONDOWN (int wParam, int lParam) {
-	
-	/*
-	* Feature in Windows.  For some reason, capturing
-	* the mouse after processing WM_LBUTTONDOWN for the
-	* widget interferes with the normal mouse processing
-	* for the widget.  The fix is to avoid the automatic
-	* mouse capture.
-	*/
-
 	/*
 	* Feature in Windows.  Windows uses the WS_TABSTOP
 	* style for the scroll bar to decide that focus
@@ -753,14 +683,23 @@ LRESULT WM_LBUTTONDOWN (int wParam, int lParam) {
 	* and restore WS_TABSTOP so that Windows will not
 	* assign focus.
 	*/
-
-	int hwndCapture = OS.GetCapture ();
 	int oldBits = OS.GetWindowLong (handle, OS.GWL_STYLE);
 	int newBits = oldBits & ~OS.WS_TABSTOP;
-	OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);	
+	OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);
 	LRESULT result = super.WM_LBUTTONDOWN (wParam, lParam);
 	OS.SetWindowLong (handle, OS.GWL_STYLE, oldBits);
-	if (OS.GetCapture () != hwndCapture) OS.SetCapture (hwndCapture);
+
+	/*
+	* Feature in Windows.  Windows runs a modal message loop
+	* when the user drags a scroll bar that terminates when
+	* it sees an WM_LBUTTONUP.  Unfortunately the WM_LBUTTONUP
+	* is consumed.  The fix is to send a fake mouse up and
+	* release the automatic capture.
+	*/	
+	if (!OS.IsWinCE) {
+		sendMouseEvent (SWT.MouseUp, 1, handle, OS.WM_LBUTTONUP, wParam, lParam);
+		if (OS.GetCapture () == handle) OS.ReleaseCapture ();
+	}
 	return result;
 }
 
@@ -779,10 +718,7 @@ LRESULT wmScrollChild (int wParam, int lParam) {
 	info.fMask = OS.SIF_POS;
 	switch (code) {
 		case OS.SB_THUMBPOSITION:
-			/*
-			* Do not set the detail field to DRAG to
-			* indicate that the dragging has ended.
-			*/
+			event.detail = SWT.NONE;
 			info.nPos = info.nTrackPos;
 			break;
 		case OS.SB_THUMBTRACK:
@@ -817,10 +753,11 @@ LRESULT wmScrollChild (int wParam, int lParam) {
 	OS.SetScrollInfo (handle, OS.SB_CTL, info, true);
 	
 	/*
-	* Send the event because WM_HSCROLL and
-	* WM_VSCROLL are sent from a modal message
-	* loop in Windows that is active when the
-	* user is scrolling.
+	* Feature in Windows.  Windows runs a modal message
+	* loop when the user drags a scroll bar.  This means
+	* that selection event must be sent because WM_HSCROLL
+	* and WM_VSCROLL are sent from the modal message loop
+	* so that they are delivered during inside the loop.
 	*/
 	sendEvent (SWT.Selection, event);
 	// the widget could be destroyed at this point
