@@ -1,42 +1,28 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Red Hat Incorporated - is/setExecutable() code
  *******************************************************************************/
 package org.eclipse.team.internal.ccvs.core.resources;
 
 import java.io.File;
 import java.io.InputStream;
-import java.util.ArrayList;
-import java.util.Date;
-import java.util.List;
+import java.util.*;
 
-import org.eclipse.core.resources.IFile;
-import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.IResourceStatus;
-import org.eclipse.core.runtime.CoreException;
-import org.eclipse.core.runtime.IPath;
-import org.eclipse.core.runtime.IProgressMonitor;
-import org.eclipse.core.runtime.Path;
+import org.eclipse.core.resources.*;
+import org.eclipse.core.runtime.*;
+import org.eclipse.osgi.util.NLS;
+import org.eclipse.team.core.RepositoryProvider;
 import org.eclipse.team.core.TeamException;
-import org.eclipse.team.internal.ccvs.core.CVSException;
-import org.eclipse.team.internal.ccvs.core.ICVSFile;
-import org.eclipse.team.internal.ccvs.core.ICVSFolder;
-import org.eclipse.team.internal.ccvs.core.ICVSRemoteFile;
-import org.eclipse.team.internal.ccvs.core.ICVSRemoteResource;
-import org.eclipse.team.internal.ccvs.core.ICVSResourceVisitor;
-import org.eclipse.team.internal.ccvs.core.ICVSRunnable;
-import org.eclipse.team.internal.ccvs.core.ILogEntry;
-import org.eclipse.team.internal.ccvs.core.Policy;
-import org.eclipse.team.internal.ccvs.core.syncinfo.BaserevInfo;
-import org.eclipse.team.internal.ccvs.core.syncinfo.MutableResourceSyncInfo;
-import org.eclipse.team.internal.ccvs.core.syncinfo.NotifyInfo;
-import org.eclipse.team.internal.ccvs.core.syncinfo.ResourceSyncInfo;
+import org.eclipse.team.internal.ccvs.core.*;
+import org.eclipse.team.internal.ccvs.core.client.Session;
+import org.eclipse.team.internal.ccvs.core.syncinfo.*;
 import org.eclipse.team.internal.ccvs.core.util.Assert;
 
 /**
@@ -62,7 +48,7 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 		try {
 			((IFile)resource).delete(false /*force*/, true /*keepHistory*/, null);
 		} catch(CoreException e) {
-			throw CVSException.wrapException(resource, Policy.bind("EclipseFile_Problem_deleting_resource", resource.getFullPath().toString(), e.getStatus().getMessage()), e); //$NON-NLS-1$ //$NON-NLS-2$
+			throw CVSException.wrapException(resource, NLS.bind(CVSMessages.EclipseFile_Problem_deleting_resource, new String[] { resource.getFullPath().toString(), e.getStatus().getMessage() }), e); //$NON-NLS-1$ //$NON-NLS-2$
 		}
 	}
 	
@@ -74,7 +60,7 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
  		try {
 			return getIFile().getContents();
 		} catch (CoreException e) {
- 			throw CVSException.wrapException(resource, Policy.bind("EclipseFile_Problem_accessing_resource", resource.getFullPath().toString(), e.getStatus().getMessage()), e); //$NON-NLS-1$ //$NON-NLS-2$
+ 			throw CVSException.wrapException(resource, NLS.bind(CVSMessages.EclipseFile_Problem_accessing_resource, new String[] { resource.getFullPath().toString(), e.getStatus().getMessage() }), e); //$NON-NLS-1$ //$NON-NLS-2$
  		}
  	}
 	
@@ -82,16 +68,16 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 	 * @see ICVSFile#getTimeStamp()
 	 */
 	public Date getTimeStamp() {
-		File ioFile = getIOFile();
-		if (ioFile == null) {
+		long timestamp = getIFile().getLocalTimeStamp();
+		if( timestamp == IResource.NULL_STAMP) {
 			// If there is no file, return the same timestamp as ioFile.lastModified() would
 			return new Date(0L);
 		}			
-		return new Date((ioFile.lastModified()/1000)*1000);
+		return new Date((timestamp/1000)*1000);
 	}
  
 	/*
-	 * @see ICVSFile#setTimeStamp(String)
+	 * @see ICVSFile#setTimeStamp(Date)
 	 */
 	public void setTimeStamp(Date date) throws CVSException {
 		long time;
@@ -100,13 +86,7 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 		} else {
 			time = date.getTime();
 		}
-		getIOFile().setLastModified(time);
-		try {
-			// Needed for workaround to Platform Core Bug #
-			resource.refreshLocal(IResource.DEPTH_ZERO, null);
-		} catch (CoreException e) {
-			throw CVSException.wrapException(e);
-		}		
+		EclipseSynchronizer.getInstance().setTimeStamp(this, time);
 	}
 
 	/*
@@ -124,34 +104,25 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 		// ignore the monitor, there is no valuable progress to be shown when
 		// calculating the dirty state for files. It is relatively fast.
 		
-		if (!exists()) return true;
+		if (!exists()) {
+			return getSyncBytes() != null;
+		}
 		int state = EclipseSynchronizer.getInstance().getModificationState(getIFile());
 
 		if (state != UNKNOWN) {
-			return state != CLEAN;
+			boolean dirty = state != CLEAN;
+			// Check to make sure that cached state is the real state.
+			// They can be different if deltas happen in the wrong order.
+			if (dirty == isDirty()) {
+				return dirty;
+			}
 		}
 		
 		// nothing cached, need to manually check (and record)
-		ResourceSyncInfo info = getSyncInfo();
-		if (info == null && isIgnored()) return false;
+		byte[] syncBytes = getSyncBytes();
+		if (syncBytes == null && isIgnored()) return false;
 		// unmanaged files are reported as modified
-		boolean dirty = computeModified(info);
-		setModified(dirty);
-		return dirty;
-	}
-	
-	/*
-	 * Deteremine if the receiver is modified when compared with the given sync
-	 * info.
-	 */
-	private boolean computeModified(ResourceSyncInfo info) throws CVSException {
-		if (info == null) return true;
-		
-		// isMerged() must be called because when a file is updated and merged by the cvs server the timestamps
-		// are equal. Merged files should however be reported as dirty because the user should take action and commit
-		// or review the merged contents.
-		if(info.isMerged() || !exists()) return true;
-		return !getTimeStamp().equals(info.getTimeStamp());
+		return EclipseSynchronizer.getInstance().setModified(this, UNKNOWN);
 	}
 	
 	/*
@@ -173,7 +144,14 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 	 */
 	public void copyTo(String filename) throws CVSException {
 		try {
-			getIFile().copy(new Path(filename), true /*force*/, null);
+			IPath targetPath = new Path(null, filename);
+			IFile targetFile = getIFile().getParent().getFile(targetPath);
+			if (targetFile.exists()) {
+				// There is a file in the target location. 
+				// Delete it and keep the history just in case
+				targetFile.delete(false /* force */, true /* keep history */, null);
+			}
+			getIFile().copy(targetPath, true /*force*/, null);
 		} catch(CoreException e) {
 			throw new CVSException(e.getStatus());
 		}
@@ -198,7 +176,7 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 			switch (responseType) {
 				case UPDATED:
 					if (resource.exists()) {
-						file.setContents(stream, true /*force*/, true /*keep history*/, monitor);
+						file.setContents(stream, false /*force*/, true /*keep history*/, monitor);
 						break;
 					}
 				case CREATED: // creating a new file so it should not exist locally
@@ -206,17 +184,17 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 					break;
 				case MERGED: // merging contents into a file that exists locally
 					// Ensure we don't leave the file in a partially written state
-					IFile tempFile = file.getParent().getFile(new Path(file.getName() + TEMP_FILE_EXTENSION));
+					IFile tempFile = file.getParent().getFile(new Path(null, file.getName() + TEMP_FILE_EXTENSION));
 					monitor.beginTask(null, 100);
 					if (tempFile.exists()) 
-						tempFile.delete(true, Policy.subMonitorFor(monitor, 25));
+						tempFile.delete(true /* force */, Policy.subMonitorFor(monitor, 25));
 					tempFile.create(stream, true /*force*/, Policy.subMonitorFor(monitor, 25));
-					file.delete(false, true, Policy.subMonitorFor(monitor, 25));
-					tempFile.move(new Path(file.getName()), true /*force*/, false /*history*/, Policy.subMonitorFor(monitor, 25));
+					file.delete(false /* force */, true /* keep history */, Policy.subMonitorFor(monitor, 25));
+					tempFile.move(new Path(null, file.getName()), false /*force*/, true /*history*/, Policy.subMonitorFor(monitor, 25));
 					monitor.done();
 					break;
 				case UPDATE_EXISTING: // creating a new file so it should exist locally
-					file.setContents(stream, true /*force*/, true /*keep history*/, monitor);
+					file.setContents(stream, false /*force*/, true /*keep history*/, monitor);
 					break;
 			}
 		} catch(CoreException e) {
@@ -228,7 +206,7 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 				if (t != null) message = t.getMessage();
 			}
 			if (message == null) message = e.getMessage();
-			throw CVSException.wrapException(resource, Policy.bind("EclipseFile_Problem_writing_resource", resource.getFullPath().toString(), message), e); //$NON-NLS-1$
+			throw CVSException.wrapException(resource, NLS.bind(CVSMessages.EclipseFile_Problem_writing_resource, new String[] { resource.getFullPath().toString(), message }), e); //$NON-NLS-1$
 		}
 	}
 			
@@ -236,7 +214,15 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 	 * @see ICVSFile#setReadOnly()
 	 */
 	public void setReadOnly(boolean readOnly) throws CVSException {
-		getIFile().setReadOnly(readOnly);
+		ResourceAttributes attributes = resource.getResourceAttributes();
+		if (attributes != null) {
+			attributes.setReadOnly(readOnly);
+			try {
+                resource.setResourceAttributes(attributes);
+            } catch (CoreException e) {
+                throw CVSException.wrapException(e);
+            }
+		}
 	}
 
 	/*
@@ -246,6 +232,33 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 		return getIFile().isReadOnly();
 	}
 	
+	/*
+	 * @see ICVSFile#setExecutable()
+	 */
+	public void setExecutable(boolean executable) throws CVSException {
+		ResourceAttributes attributes = resource.getResourceAttributes();
+		if (attributes != null) {
+			attributes.setExecutable(executable);
+			try {
+                resource.setResourceAttributes(attributes);
+            } catch (CoreException e) {
+                throw CVSException.wrapException(e);
+            }
+		}
+	}
+
+	/*
+	 * @see ICVSFile#isExectuable()
+	 */
+	public boolean isExecutable() throws CVSException {
+		ResourceAttributes attributes = resource.getResourceAttributes();
+		if (attributes != null) {
+			return attributes.isExecutable();
+		} else {
+			return false;
+		}
+	}
+	
 	/*
 	 * Typecasting helper
 	 */
@@ -321,8 +334,8 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 	/**
 	 * @see org.eclipse.team.internal.ccvs.core.ICVSFile#checkout(int)
 	 */
-	public void edit(final int notifications, IProgressMonitor monitor) throws CVSException {
-		if (!isReadOnly()) return;
+	public void edit(final int notifications, boolean notifyForWritable, IProgressMonitor monitor) throws CVSException {
+		if (!notifyForWritable && !isReadOnly()) return;
 		run(new ICVSRunnable() {
 			public void run(IProgressMonitor monitor) throws CVSException {
 				byte[] syncBytes = getSyncBytes();
@@ -358,8 +371,13 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 					setBaserevInfo(new BaserevInfo(getName(), ResourceSyncInfo.getRevision(syncBytes)));
 				}
 				
-				// allow editing
-				setReadOnly(false);
+				try {
+                    // allow editing
+                    setReadOnly(false);
+                } catch (CVSException e) {
+                    // Just log and keep going
+                    CVSProviderPlugin.log(e);
+                }
 			}
 		}, monitor);
 		
@@ -394,7 +412,7 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 						setSyncInfo(newInfo, ICVSFile.CLEAN);
 					} else {
 						// an unedited file is no longer modified
-						setModified(false);
+						EclipseSynchronizer.getInstance().setModified(EclipseFile.this, CLEAN);
 					}
 				} else {
 					// We still need to report a state change
@@ -402,8 +420,13 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 				}
 				setBaserevInfo(null);
 					
-				// prevent editing
-				setReadOnly(true);
+				try {
+                    // prevent editing
+                    setReadOnly(true);
+                } catch (CVSException e) {
+                    // Just log and keep going
+                    CVSProviderPlugin.log(e);
+                }
 			}
 		}, monitor);
 	}
@@ -422,37 +445,70 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 		return getNotifyInfo();
 	}
 	
-	/**
-	 * @see org.eclipse.team.internal.ccvs.core.ICVSFile#committed(String)
+	/* (non-Javadoc)
+	 * @see org.eclipse.team.internal.ccvs.core.ICVSFile#checkedIn(java.lang.String)
 	 */
-	public void checkedIn(String entryLine) throws CVSException {
+	public void checkedIn(String entryLine, boolean commit) throws CVSException {
 		ResourceSyncInfo oldInfo = getSyncInfo();
 		ResourceSyncInfo newInfo = null;
 		int modificationState = ICVSFile.CLEAN;
 		if (entryLine == null) {
-			// The file contents matched the server contents so no entry line was sent
+			// cvs commit: the file contents matched the server contents so no entry line was sent
 			if (oldInfo == null) return;
 			Date timeStamp = oldInfo.getTimeStamp();
 			if (timeStamp == null || oldInfo.isMergedWithConflicts()) {
 				// If the entry line has no timestamp, put the file timestamp in the entry line
-				MutableResourceSyncInfo mutable = oldInfo.cloneMutable();
-				mutable.setTimeStamp(getTimeStamp(), true /* clear merged */);
-				newInfo = mutable;
+				if(! oldInfo.isAdded()) {
+					MutableResourceSyncInfo mutable = oldInfo.cloneMutable();
+					mutable.setTimeStamp(getTimeStamp(), true /* clear merged */);
+					newInfo = mutable;
+				}
 			} else {
 				// reset the file timestamp to the one from the entry line
+			    boolean reset = false;
+			    if (isReadOnly()) {
+			        // We can only set the timestamp if the file is not read-only.
+			        // We will get a situation like this if the timestamp changes due to a time change (e.g. daylight savings)
+			        setReadOnly(false);
+			        reset = true;
+			    }
 				setTimeStamp(timeStamp);
+				if (reset) {
+				    // Make sure we set the file back to read-only
+				    setReadOnly(true);
+				}
 				// (newInfo = null) No need to set the newInfo as there is no sync info change
 			}
 			// (modified = false) the file will be no longer modified
 		} else if (oldInfo == null) {
-			// cvs add of a file
-			newInfo = new ResourceSyncInfo(entryLine, null, null);
+			// cvs add: addition of a file
+			newInfo = new ResourceSyncInfo(entryLine, null);
 			// an added file should show up as modified
 			modificationState = ICVSFile.DIRTY;
 		} else {
-			// commit of a changed file
-			newInfo = new ResourceSyncInfo(entryLine, oldInfo.getPermissions(), getTimeStamp());
-			// (modified = false) a committed file is no longer modified
+			// cvs commit: commit of a changed file
+		    // cvs update: update of a file whose contents match the server contents
+		    Date timeStamp;
+		    if (commit) {
+		        // This is a commit. Put the file timestamp in the entry
+		        timeStamp = getTimeStamp();
+		    } else {
+		        // This is an update. We need to change the tiemstamp in the
+                // entry file to match the file timestamp returned by Java
+		        timeStamp = oldInfo.getTimeStamp();
+		        if (timeStamp == null) {
+		            timeStamp = getTimeStamp();
+		        } else {
+                    // First, set the timestamp of the file to the timestamp from the entry
+                    // There is a chance this will do nothing as the call to Java on some
+                    // file systems munges the timestamps
+		            setTimeStamp(timeStamp);
+                    // To compensate for the above, reset the timestamp in the entry
+                    // to match the timestamp in the file
+                    timeStamp = getTimeStamp();
+		        }
+		    }
+	        newInfo = new ResourceSyncInfo(entryLine, timeStamp);
 			
 		}
 		if (newInfo != null) setSyncInfo(newInfo, modificationState);
@@ -463,8 +519,24 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 		BaserevInfo base = getBaserevInfo();
 		if (base != null) {
 			setBaserevInfo(null);
-			setReadOnly(true);
-		}
+			try {
+                setReadOnly(true);
+            } catch (CVSException e) {
+                // Just log and keep going
+                CVSProviderPlugin.log(e);
+            }
+		} else {
+            // Check to see if watch-edit is enabled for the project
+            CVSTeamProvider provider = (CVSTeamProvider)RepositoryProvider.getProvider(resource.getProject(), CVSProviderPlugin.getTypeId());
+            if (provider != null && provider.isWatchEditEnabled()) {
+                try {
+                    setReadOnly(true);
+                } catch (CVSException e) {
+                    // Just log and keep going
+                    CVSProviderPlugin.log(e);
+                }
+            }
+        }
 	}
 
 	/**
@@ -506,16 +578,7 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 	private void setSyncBytes(byte[] syncBytes, ResourceSyncInfo info, int modificationState) throws CVSException {
 		Assert.isNotNull(syncBytes);
 		setSyncBytes(syncBytes);
-		boolean modified;
-		if (modificationState == UNKNOWN) {
-			if (info == null) {
-				info = new ResourceSyncInfo(syncBytes);
-			}
-			modified = computeModified(info);
-		} else {
-			modified = modificationState == DIRTY;
-		}
-		setModified(modified);
+		EclipseSynchronizer.getInstance().setModified(this, modificationState);
 	}
 	
 	public void handleModification(boolean forAddition) throws CVSException {
@@ -527,19 +590,42 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 			// There may be a better was of handling resources that transition from un-managed to
 			// ignored but for now this seems like the safest change. 
 			if(! resource.isDerived()) {
-				setModified(false);
+				EclipseSynchronizer.getInstance().setModified(this, CLEAN);
 			}
 			return;
 		} 
 		// set the modification state to what it really is and return true if the modification state changed
-		setModified(computeModified(getSyncInfo()));
+		EclipseSynchronizer.getInstance().setModified(this, UNKNOWN);
+	}
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.team.internal.ccvs.core.ICVSResource#getRepositoryRelativePath()
+	 */
+	public String getRepositoryRelativePath() throws CVSException {
+		if (!isManaged()) return null;
+		String parentPath = getParent().getRepositoryRelativePath();
+		if (parentPath == null) return null;
+		return parentPath + Session.SERVER_SEPARATOR + getName();
 	}
 	
-	/**
-	 * @see org.eclipse.team.internal.ccvs.core.resources.EclipseResource#run(org.eclipse.team.internal.ccvs.core.ICVSRunnable, org.eclipse.core.runtime.IProgressMonitor)
-	 */
-	protected void run(ICVSRunnable job, IProgressMonitor monitor) throws CVSException {
-		getParent().run(job, monitor);
+	protected boolean isDirty() throws CVSException {
+		boolean dirty;
+		byte[] syncBytes = getSyncBytes();
+		if (syncBytes == null) {
+			dirty = exists();
+		} else {
+			// isMerged() must be called because when a file is updated and merged by the cvs server the timestamps
+			// are equal. Merged files should however be reported as dirty because the user should take action and commit
+			// or review the merged contents.
+			if(ResourceSyncInfo.isAddition(syncBytes) || ResourceSyncInfo.isMerge(syncBytes) || !exists()) {
+				dirty = true;
+			} else {
+				// TODO: non-optimal as ResourceSyncInfo is created each time
+				ResourceSyncInfo info = new ResourceSyncInfo(syncBytes);
+				dirty = !getTimeStamp().equals(info.getTimeStamp());
+			}
+		}
+		return dirty;
 	}
 
 }
