@@ -1,35 +1,24 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.team.internal.ccvs.core.resources;
 
-import org.eclipse.core.resources.IContainer;
-import org.eclipse.core.resources.IFile;
-import org.eclipse.core.resources.IProject;
-import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.IResourceStatus;
-import org.eclipse.core.resources.IResourceVisitor;
-import org.eclipse.core.resources.ISaveContext;
-import org.eclipse.core.resources.ISaveParticipant;
-import org.eclipse.core.resources.ISynchronizer;
-import org.eclipse.core.resources.ResourcesPlugin;
-import org.eclipse.core.runtime.CoreException;
-import org.eclipse.core.runtime.IStatus;
-import org.eclipse.core.runtime.Path;
-import org.eclipse.core.runtime.QualifiedName;
+import java.util.*;
+
+import org.eclipse.core.resources.*;
+import org.eclipse.core.runtime.*;
+import org.eclipse.osgi.util.NLS;
 import org.eclipse.team.core.RepositoryProvider;
-import org.eclipse.team.internal.ccvs.core.CVSException;
-import org.eclipse.team.internal.ccvs.core.CVSProviderPlugin;
-import org.eclipse.team.internal.ccvs.core.ICVSDecoratorEnablementListener;
-import org.eclipse.team.internal.ccvs.core.Policy;
+import org.eclipse.team.internal.ccvs.core.*;
 import org.eclipse.team.internal.ccvs.core.syncinfo.FolderSyncInfo;
+import org.eclipse.team.internal.ccvs.core.syncinfo.ResourceSyncInfo;
 import org.eclipse.team.internal.ccvs.core.util.FileNameMatcher;
 import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 
@@ -39,33 +28,33 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
  * state for all cvs managed folders are persisted using the resource's plugin
  * synchronizer.
  */
-/*package*/ class SessionPropertySyncInfoCache extends SyncInfoCache 
-																		  implements ISaveParticipant, ICVSDecoratorEnablementListener {
+/*package*/ class SessionPropertySyncInfoCache extends SyncInfoCache implements ISaveParticipant {
 	
 	// key used on a folder to indicate that the resource sync has been cahced for it's children
 	private static final QualifiedName RESOURCE_SYNC_CACHED_KEY = new QualifiedName(CVSProviderPlugin.ID, "resource-sync-cached"); //$NON-NLS-1$
 	private static final Object RESOURCE_SYNC_CACHED = new Object();
 	
 	/*package*/ static final FileNameMatcher NULL_IGNORES = new FileNameMatcher();
-	private static final FolderSyncInfo NULL_FOLDER_SYNC_INFO = new FolderSyncInfo("", "", null, false); //$NON-NLS-1$ //$NON-NLS-2$
+	private static final FolderSyncInfo NULL_FOLDER_SYNC_INFO = new FolderSyncInfo("dummy-repo", "dummy-root", null, false); //$NON-NLS-1$ //$NON-NLS-2$
 	
 	private QualifiedName FOLDER_DIRTY_STATE_KEY = new QualifiedName(CVSProviderPlugin.ID, "folder-dirty-state-cached"); //$NON-NLS-1$
-	private boolean isDecoratorEnabled = true;
-	private boolean hasBeenSaved = false;
 	
-	/*package*/ SessionPropertySyncInfoCache() {
+	// defer to the sychronizer if there is no sync info
+	// (i.e. for those cases where a deleted resource is recreated)
+	private SynchronizerSyncInfoCache synchronizerCache;
+	
+	/*package*/ SessionPropertySyncInfoCache(SynchronizerSyncInfoCache synchronizerCache) {
+		this.synchronizerCache = synchronizerCache;
 		try {
 			// this save participant is removed when the plugin is shutdown.			
 			ResourcesPlugin.getWorkspace().addSaveParticipant(CVSProviderPlugin.getPlugin(), this);
-			CVSProviderPlugin.getPlugin().addDecoratorEnablementListener(this);
-
-			final ISynchronizer synchronizer = ResourcesPlugin.getWorkspace().getSynchronizer();
+			ISynchronizer synchronizer = ResourcesPlugin.getWorkspace().getSynchronizer();
 			synchronizer.add(FOLDER_DIRTY_STATE_KEY);
 		} catch (CoreException e) {
-			CVSProviderPlugin.log(e.getStatus());
+			CVSProviderPlugin.log(e);
 		}
 	}
-	
+
 	/**
 	 * If not already cached, loads and caches the folder ignores sync for the container.
 	 * Folder must exist and must not be the workspace root.
@@ -73,10 +62,10 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 	 * @param container the container
 	 * @return the folder ignore patterns, or an empty array if none
 	 */
-	/*package*/ FileNameMatcher cacheFolderIgnores(IContainer container) throws CVSException {
+	/*package*/ FileNameMatcher getFolderIgnores(IContainer container, boolean threadSafeAccess) throws CVSException {
 		// don't try to load if the information is already cached
 		FileNameMatcher matcher = (FileNameMatcher)safeGetSessionProperty(container, IGNORE_SYNC_KEY);
-		if (matcher == null) {
+		if (threadSafeAccess && matcher == null) {
 			// read folder ignores and remember it
 			String[] ignores = SyncFileWriter.readCVSIgnoreEntries(container);
 			if (ignores == null) {
@@ -88,9 +77,18 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 		}
 		return matcher;
 	}
+    
+    /* package */ boolean isIgnoresCached(IContainer container) throws CVSException {
+        return safeGetSessionProperty(container, IGNORE_SYNC_KEY) != null;
+    }
 
 	/*package*/ boolean isFolderSyncInfoCached(IContainer container) throws CVSException {
-		return safeGetSessionProperty(container, FOLDER_SYNC_KEY) != null;
+		Object info = safeGetSessionProperty(container, FOLDER_SYNC_KEY);
+		if (info == null){
+			// Defer to the synchronizer in case the folder was recreated
+			info = synchronizerCache.getCachedFolderSync(container, true);
+		}
+		return info != null;
 	}
 
 	/*package*/ boolean isResourceSyncInfoCached(IContainer container) throws CVSException {
@@ -105,16 +103,26 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 	 * Returns the folder sync info for the container; null if none.
 	 * Folder must exist and must not be the workspace root.
 	 * The folder sync info for the container MUST ALREADY BE CACHED.
-	 *
 	 * @param container the container
+	 * @param threadSafeAccess if false, the return value can only be used if not null
 	 * @return the folder sync info for the folder, or null if none.
 	 * @see #cacheFolderSync
 	 */
-	/*package*/ FolderSyncInfo getCachedFolderSync(IContainer container) throws CVSException {
+	FolderSyncInfo getCachedFolderSync(IContainer container, boolean threadSafeAccess) throws CVSException {
 		FolderSyncInfo info = (FolderSyncInfo)safeGetSessionProperty(container, FOLDER_SYNC_KEY);
+        // If we are not thread safe, just return whatever was found in the session property
+        if (!threadSafeAccess)
+            return info == NULL_FOLDER_SYNC_INFO ? null : info;
+		if (info == null) {
+			// Defer to the synchronizer in case the folder was recreated
+			info = synchronizerCache.getCachedFolderSync(container, true);
+			if (info != null) {
+				safeSetSessionProperty(container, FOLDER_SYNC_KEY, info);
+			}
+		}
 		if (info == null) {
 			// There should be sync info but it was missing. Report the error
-			throw new CVSException(Policy.bind("EclipseSynchronizer.folderSyncInfoMissing", container.getFullPath().toString())); //$NON-NLS-1$
+			throw new CVSException(NLS.bind(CVSMessages.EclipseSynchronizer_folderSyncInfoMissing, new String[] { container.getFullPath().toString() })); //$NON-NLS-1$
 		}
 		if (info == NULL_FOLDER_SYNC_INFO) return null;
 		return info;
@@ -123,23 +131,30 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 	/**
 	 * Purges the cache recursively for all resources beneath the container.
 	 * There must not be any pending uncommitted changes.
+	 * @return the resources whose sync info was flushed
 	 */
-	/*package*/ void purgeCache(IContainer container, boolean deep) throws CVSException {
-		if (! container.exists()) return;
+	/*package*/ IResource[] purgeCache(IContainer container, boolean deep) throws CVSException {
+		if (! container.exists()) return new IResource[0];
 		try {
+			Set flushed = new HashSet();
 			if (container.getType() != IResource.ROOT) {
 				safeSetSessionProperty(container, IGNORE_SYNC_KEY, null);
 				safeSetSessionProperty(container, FOLDER_SYNC_KEY, null);
 				safeSetSessionProperty(container, RESOURCE_SYNC_CACHED_KEY, null);
+				flushed.add(container);
+				EclipseSynchronizer.getInstance().adjustDirtyStateRecursively(container, RECOMPUTE_INDICATOR);
 			}
 			IResource[] members = container.members();
 			for (int i = 0; i < members.length; i++) {
 				IResource resource = members[i];
 				purgeResourceSyncCache(resource);
+				flushed.add(resource);
 				if (deep && resource.getType() != IResource.FILE) {
-					purgeCache((IContainer) resource, deep);
+					IResource[] flushedChildren = purgeCache((IContainer) resource, deep);
+					flushed.addAll(Arrays.asList(flushedChildren));
 				}
 			}
+			return (IResource[]) flushed.toArray(new IResource[flushed.size()]);
 		} catch (CoreException e) {
 			throw CVSException.wrapException(e);
 		}
@@ -147,6 +162,7 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 	
 	/* package*/ void purgeResourceSyncCache(IResource resource) throws CVSException {
 		safeSetSessionProperty(resource, RESOURCE_SYNC_KEY, null);
+		EclipseSynchronizer.getInstance().adjustDirtyStateRecursively(resource, RECOMPUTE_INDICATOR);
 	}
 	
 	/**
@@ -169,12 +185,16 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 	 * @param container the container
 	 * @param info the new folder sync info
 	 */
-	/*package*/ void setCachedFolderSync(IContainer container, FolderSyncInfo info) throws CVSException {
+	void setCachedFolderSync(IContainer container, FolderSyncInfo info, boolean canModifyWorkspace) throws CVSException {
 		if (!container.exists()) return;
 		if (info == null) {
 			info = NULL_FOLDER_SYNC_INFO;
 		} 
 		safeSetSessionProperty(container, FOLDER_SYNC_KEY, info);
+		// Ensure the synchronizer is clear for exiting resources
+		if (canModifyWorkspace && synchronizerCache.getCachedFolderSync(container, true) != null) {
+			synchronizerCache.setCachedFolderSync(container, null, true);
+		}
 	}
 
 	/*package*/ void setDirtyIndicator(IResource resource, String indicator) throws CVSException {
@@ -184,11 +204,11 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 			internalSetDirtyIndicator((IContainer)resource, indicator);
 		}
 	}
-	/*package*/ String getDirtyIndicator(IResource resource) throws CVSException {
+	/*package*/ String getDirtyIndicator(IResource resource, boolean threadSafeAccess) throws CVSException {
 		if (resource.getType() == IResource.FILE) {
-			return internalGetDirtyIndicator((IFile)resource);
+			return internalGetDirtyIndicator((IFile)resource, threadSafeAccess);
 		} else {
-			return internalGetDirtyIndicator((IContainer)resource);
+			return internalGetDirtyIndicator((IContainer)resource, threadSafeAccess);
 		}
 	}
 	
@@ -196,7 +216,7 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 		safeSetSessionProperty(file, IS_DIRTY, indicator);
 	}
 	
-	private String internalGetDirtyIndicator(IFile file) throws CVSException {
+	private String internalGetDirtyIndicator(IFile file, boolean threadSafeAccess) throws CVSException {
 		String di = (String)safeGetSessionProperty(file, IS_DIRTY);
 		if(di == null) {
 			di = RECOMPUTE_INDICATOR;
@@ -208,7 +228,7 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 		safeSetSessionProperty(container, IS_DIRTY, indicator);
 	}
 	
-	private String internalGetDirtyIndicator(IContainer container) throws CVSException {
+	private String internalGetDirtyIndicator(IContainer container, boolean threadSafeAccess) throws CVSException {
 		try {
 			String di = (String)safeGetSessionProperty(container, IS_DIRTY);
 			
@@ -217,7 +237,7 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 			// a delta.			
 			if(di == null) {
 				byte [] diBytes = ResourcesPlugin.getWorkspace().getSynchronizer().getSyncInfo(FOLDER_DIRTY_STATE_KEY, container);
-				if(diBytes != null) {
+				if(diBytes != null && !CVSProviderPlugin.getPlugin().crashOnLastRun()) {
 					di = new String(diBytes);
 					if(di.equals(NOT_DIRTY_INDICATOR)) {
 						di = NOT_DIRTY_INDICATOR;
@@ -229,7 +249,10 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 				} else {
 					di = RECOMPUTE_INDICATOR;
 				}
-				setDirtyIndicator(container, di);
+                // Only set the session property if we are thread safe
+                if (threadSafeAccess) {
+                    setDirtyIndicator(container, di);
+                }
 			}
 			return di;
 		} catch (CoreException e) {
@@ -272,10 +295,34 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 	}
 	
 	/**
-	 * @see org.eclipse.team.internal.ccvs.core.resources.SyncInfoCache#getCachedSyncBytes(org.eclipse.core.resources.IResource)
+	 * @see org.eclipse.team.internal.ccvs.core.resources.SyncInfoCache#getCachedSyncBytes(org.eclipse.core.resources.IResource, boolean)
 	 */
-	/*package*/ byte[] getCachedSyncBytes(IResource resource) throws CVSException {
-		return (byte[])safeGetSessionProperty(resource, RESOURCE_SYNC_KEY);
+	byte[] getCachedSyncBytes(IResource resource, boolean threadSafeAccess) throws CVSException {
+		byte[] bytes = (byte[])safeGetSessionProperty(resource, RESOURCE_SYNC_KEY);
+		// If we are not thread safe, just return whatever was found in the session property
+        if (!threadSafeAccess)
+            return bytes;
+		if (bytes == null) {
+			// Defer to the synchronizer in case the file was recreated
+			bytes = synchronizerCache.getCachedSyncBytes(resource, true);
+			if (bytes != null) {
+				boolean genderChange = false;
+				if (resource.getType() == IResource.FILE) {
+					if (ResourceSyncInfo.isFolder(bytes)) {
+						genderChange = true;
+					}
+				} else if (!ResourceSyncInfo.isFolder(bytes)) {
+					genderChange = true;
+				}
+				if (genderChange) {
+					// Return null if it is a gender change
+					bytes = null;
+				} else {
+					safeSetSessionProperty(resource, RESOURCE_SYNC_KEY, ResourceSyncInfo.convertFromDeletion(bytes));
+				}
+			}
+		}
+		return bytes;
 	}
 
 	Object safeGetSessionProperty(IResource resource, QualifiedName key) throws CVSException {
@@ -301,35 +348,24 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 			try {
 				resource.setSessionProperty(key, value);
 			} catch (CoreException e) {
-				IStatus status = e.getStatus();
-				if(status != null) {
-					int code = e.getStatus().getCode();
-					if(code == IResourceStatus.RESOURCE_NOT_LOCAL ||
-					    code == IResourceStatus.RESOURCE_NOT_FOUND) {
-					    	// ignore error since a phantom would of been created
-					    	// and we can safely ignore these cases
-					}
-					// some other error we did not expect
-					throw CVSException.wrapException(e);
-				}
+				throw CVSException.wrapException(e);
 			}
 		}
 	
 	/**
 	 * @see org.eclipse.team.internal.ccvs.core.resources.SyncInfoCache#setCachedSyncBytes(org.eclipse.core.resources.IResource, byte[])
 	 */
-	/*package*/ void setCachedSyncBytes(IResource resource, byte[] syncBytes) throws CVSException {
+	void setCachedSyncBytes(IResource resource, byte[] syncBytes, boolean canModifyWorkspace) throws CVSException {
+		// Ensure that the sync bytes do not indicate a deletion
+		if (syncBytes != null && ResourceSyncInfo.isDeletion(syncBytes)) {
+			syncBytes = ResourceSyncInfo.convertFromDeletion(syncBytes);
+		}
+		// Put the sync bytes into the cache
 		safeSetSessionProperty(resource, RESOURCE_SYNC_KEY, syncBytes);
-	}
-
-	/* (non-Javadoc)
-	 * @see org.eclipse.team.internal.ccvs.core.resources.SyncInfoCache#isDirtyCacheFlushed(org.eclipse.core.resources.IContainer)
-	 */
-	boolean isDirtyCacheFlushed(IContainer resource) throws CVSException {
-		if (resource.exists()) {
-			return getDirtyIndicator(resource) == RECOMPUTE_INDICATOR;					
+		// Ensure the synchronizer is clear
+		if (canModifyWorkspace && synchronizerCache.getCachedSyncBytes(resource, true) != null) {
+			synchronizerCache.setCachedSyncBytes(resource, null, canModifyWorkspace);
 		}
-		return false;
 	}
 	
 	/* (non-Javadoc)
@@ -360,7 +396,7 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 		boolean fullSave = (context.getKind() == ISaveContext.FULL_SAVE);
 		boolean projectSave = (context.getKind() == ISaveContext.PROJECT_SAVE);
 		
-		if(isDecoratorEnabled && (projectSave || fullSave)) {
+		if((projectSave || fullSave)) {
 			// persist all session properties for folders into sync info.
 			final ISynchronizer synchronizer = ResourcesPlugin.getWorkspace().getSynchronizer();
 		
@@ -387,7 +423,7 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 							if(resource.getType() != IResource.FILE) {
 								String di = null;
 								try {
-									di = getDirtyIndicator(resource);
+									di = getDirtyIndicator(resource, true);
 								} catch (CVSException e) {
 									// continue traversal
 									CVSProviderPlugin.log(e);
@@ -401,41 +437,6 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 					});
 				}
 			}
-			hasBeenSaved = true;
-		}
-	}
-
-	/* (non-Javadoc)
-	 * @see ICVSDecoratorEnablementListener#decoratorEnablementChanged(boolean)
-	 */
-	public void decoratorEnablementChanged(boolean enabled) {
-		isDecoratorEnabled = enabled;
-		if(!enabled && !hasBeenSaved) {
-			// flush the dirty state cache for all managed resources
-			IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
-			for (int i = 0; i < projects.length; i++) {
-				IProject project = projects[i];
-				RepositoryProvider provider = RepositoryProvider.getProvider(
-														project,
-														CVSProviderPlugin.getTypeId());
-														
-				if (provider != null) {
-					try {
-						project.accept(new IResourceVisitor() {
-							public boolean visit(IResource resource) throws CoreException {
-								try {
-									flushDirtyCache(resource);
-								} catch(CVSException e) {
-									throw new CoreException(e.getStatus());
-								}
-								return true;
-							}
-						});
-					} catch (CoreException e) {
-						CVSProviderPlugin.log(e.getStatus());
-					}
-				}
-			}		
 		}
 	}
 		
@@ -448,7 +449,35 @@ import org.eclipse.team.internal.ccvs.core.util.SyncFileWriter;
 		try {
 			synchronizer.flushSyncInfo(FOLDER_DIRTY_STATE_KEY, container, IResource.DEPTH_INFINITE);
 		} catch (CoreException e) {
-			CVSProviderPlugin.log(e.getStatus());
+			CVSProviderPlugin.log(e);
 		}
 	}
+
+	/*
+	 * Flush all the cahced dirty state for the resource and its members.
+	 */
+	/* package*/ void purgeDirtyCache(IResource resource) throws CVSException {
+		if (! resource.exists()) return;
+		try {
+			if (resource.getType() != IResource.ROOT) {
+				safeSetSessionProperty(resource, IS_DIRTY, null);
+			}
+			if (resource.getType() != IResource.FILE) {
+				ResourcesPlugin.getWorkspace().getSynchronizer().flushSyncInfo(FOLDER_DIRTY_STATE_KEY, resource, IResource.DEPTH_INFINITE);
+				IResource[] members = ((IContainer)resource).members();
+				for (int i = 0; i < members.length; i++) {
+					purgeDirtyCache(members[i]);
+				}
+			}
+		} catch (CoreException e) {
+			throw CVSException.wrapException(e);
+		}
+	}
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.team.internal.ccvs.core.resources.SyncInfoCache#cachesDirtyState()
+	 */
+	public boolean cachesDirtyState() {
+		return true;
+	}
 }
