@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -23,6 +23,7 @@ import java.math.BigInteger;
 import java.net.Socket;
 
 import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.osgi.util.NLS;
 import org.eclipse.team.internal.ccvs.core.connection.CVSAuthenticationException;
 import org.eclipse.team.internal.ccvs.core.util.Util;
 import org.eclipse.team.internal.core.streams.PollingInputStream;
@@ -61,23 +62,11 @@ public class Client {
 	private static String[] cipherNames = { "None", "IDEA", "DES", "3DES", "TSS", "RC4", "Blowfish" }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$
 
 	// cipher types
-	private static int SSH_CIPHER_NONE = 0;
-	private static int SSH_CIPHER_IDEA = 1;
-	private static int SSH_CIPHER_DES = 2;
-	private static int SSH_CIPHER_3DES = 3;
-	private static int SSH_CIPHER_TSS = 4;
-	private static int SSH_CIPHER_RC4 = 5;
 	private static int SSH_CIPHER_BLOWFISH = 6;
 
 	// preferred cipher types
 	private int[] preferredCipherTypes = { SSH_CIPHER_BLOWFISH };
 
-	// authentication methods 
-	private final int SSH_AUTH_RHOSTS = 1;
-	private final int SSH_AUTH_RSA = 2;
-	private final int SSH_AUTH_PASSWORD = 3;
-	private final int SSH_AUTH_RHOSTS_RSA = 4;
-
 	private String host;
 	private int port;
 	private String username;
@@ -86,7 +75,7 @@ public class Client {
 
 	private Socket socket;
 	private InputStream socketIn;
-	private OutputStream socketOut;
+	private PollingOutputStream socketOut;
 	private InputStream is;
 	private OutputStream os;
 	private boolean connected = false;
@@ -97,14 +86,12 @@ public class Client {
 	private class StandardInputStream extends InputStream {
 		private ServerPacket packet = null;
 		private InputStream buffer = null;
-		private int buflen = 0;
 		private boolean atEnd = false;
 		private boolean closed = false;
-		private int exitStatus = 0;
 
 		public int available() throws IOException {
 			if (closed) {
-				throw new IOException(Policy.bind("closed")); //$NON-NLS-1$
+				throw new IOException(CVSSSHMessages.closed); //$NON-NLS-1$
 			}
 
 			int available = buffer == null ? 0 : buffer.available();
@@ -127,14 +114,13 @@ public class Client {
 					packet.close(false);
 					buffer = null;
 					packet = null;
-					buflen = 0;
 				}
 			}
 		}
 
 		public int read() throws IOException {
 			if (closed) {
-				throw new IOException(Policy.bind("closed"));//$NON-NLS-1$
+				throw new IOException(CVSSSHMessages.closed);//$NON-NLS-1$
 			}
 
 			if (atEnd) {
@@ -153,7 +139,7 @@ public class Client {
 
 		public int read(byte b[], int off, int len) throws IOException {
 			if (closed) {
-				throw new IOException(Policy.bind("closed"));//$NON-NLS-1$
+				throw new IOException(CVSSSHMessages.closed);//$NON-NLS-1$
 			}
 
 			if (atEnd) {
@@ -183,25 +169,23 @@ public class Client {
 				case SSH_SMSG_STDERR_DATA :
 				case SSH_MSG_DEBUG :
 					buffer = packet.getInputStream();
-					buflen = Misc.readInt(buffer);
+					Misc.readInt(buffer);
 					break;
 				case SSH_SMSG_EXITSTATUS :
 					buffer = null;
-					buflen = 0;
 					atEnd = true;
 					InputStream pis = packet.getInputStream();
-					exitStatus = Misc.readInt(pis);
+					Misc.readInt(pis);
 					pis.close();
 					send(SSH_CMSG_EXIT_CONFIRMATION, null);
 					break;
 				case SSH_MSG_DISCONNECT :
 					buffer = null;
-					buflen = 0;
 					atEnd = true;
 					handleDisconnect(packet.getInputStream());
 					break;
 				default :
-					throw new IOException(Policy.bind("Client.packetType", new Object[] {new Integer(packetType)} ));//$NON-NLS-1$
+					throw new IOException(NLS.bind(CVSSSHMessages.Client_packetType, (new Object[] {new Integer(packetType)})));//$NON-NLS-1$
 			}
 		}
 		
@@ -216,11 +200,11 @@ public class Client {
 			
 			// Log the description provided by the server
 			if (description == null) {
-				description = Policy.bind("Client.noDisconnectDescription"); //$NON-NLS-1$
+				description = CVSSSHMessages.Client_noDisconnectDescription; //$NON-NLS-1$
 			}
 			
 			// Throw an IOException with the proper text
-			throw new IOException(Policy.bind("Client.disconnectDescription", new Object[] {description}));//$NON-NLS-1$
+			throw new IOException(NLS.bind(CVSSSHMessages.Client_disconnectDescription, (new Object[] {description})));//$NON-NLS-1$
 		}
 	}
 
@@ -242,7 +226,7 @@ public class Client {
 
 		public void flush() throws IOException {
 			if (closed) {
-				throw new IOException(Policy.bind("closed"));//$NON-NLS-1$
+				throw new IOException(CVSSSHMessages.closed);//$NON-NLS-1$
 			}
 
 			if (bufpos > 0) {
@@ -253,7 +237,7 @@ public class Client {
 
 		public void write(int b) throws IOException {
 			if (closed) {
-				throw new IOException(Policy.bind("closed"));//$NON-NLS-1$
+				throw new IOException(CVSSSHMessages.closed);//$NON-NLS-1$
 			}
 
 			buffer[bufpos++] = (byte) b;
@@ -265,7 +249,7 @@ public class Client {
 
 		public void write(byte b[], int off, int len) throws IOException {
 			if (closed) {
-				throw new IOException(Policy.bind("closed")); //$NON-NLS-1$
+				throw new IOException(CVSSSHMessages.closed); //$NON-NLS-1$
 			}
 
 			int bytesWritten = 0;
@@ -294,16 +278,6 @@ public class Client {
 			}
 		}
 	}
-public Client(InputStream socketIn, OutputStream socketOut, String username, String password) {
-	this.socketIn = socketIn;
-	this.socketOut = socketOut;
-	this.username = username;
-	this.password = password;
-}
-public Client(InputStream socketIn, OutputStream socketOut, String username, String password, String command) {
-	this(socketIn, socketOut, username, password);
-	this.command = command;
-}
 public Client(String host, int port, String username, String password) {
 	this.host = host;
 	this.port = port;
@@ -355,12 +329,12 @@ private void cleanup() throws IOException {
  */
 public void connect(IProgressMonitor monitor) throws IOException, CVSAuthenticationException {
 	// If we're already connected, just ignore the invokation
-	if (connected)
+	if (connected || monitor.isCanceled())
 		return;
 		
 	// Otherwise, set up the connection
 	try {
-		
+		PollingInputStream pollingInputStream = null;
 		// Create the socket (the socket should always be null here)
 		if (socket == null) {
 			try {
@@ -369,14 +343,16 @@ public void connect(IProgressMonitor monitor) throws IOException, CVSAuthenticat
 				socket.setTcpNoDelay(true);
 			} catch (InterruptedIOException e) {
 				// If we get this exception, chances are the host is not responding
-				throw new InterruptedIOException(Policy.bind("Client.socket", new Object[] {host}));//$NON-NLS-1$
+				throw new InterruptedIOException(NLS.bind(CVSSSHMessages.Client_socket, (new Object[] {host})));//$NON-NLS-1$
 
 			}
 			if (timeout >= 0) {
 				socket.setSoTimeout(1000);
 			}
-			socketIn = new BufferedInputStream(new PollingInputStream(socket.getInputStream(),
-				timeout > 0 ? timeout : 1, monitor));
+			pollingInputStream = new PollingInputStream(socket.getInputStream(),
+				timeout > 0 ? timeout : 1, monitor);
+			socketIn = new BufferedInputStream(pollingInputStream);
+			
 			socketOut = new PollingOutputStream(new TimeoutOutputStream(
 				socket.getOutputStream(), 8192 /*bufferSize*/, 1000 /*writeTimeout*/, 1000 /*closeTimeout*/),
 				timeout > 0 ? timeout : 1, monitor);
@@ -385,11 +361,16 @@ public void connect(IProgressMonitor monitor) throws IOException, CVSAuthenticat
 		// read the ssh server id. The socket creation may of failed if the
 		// server cannot accept our connection request. We don't expect the
 		// socket to be closed at this point.
+		// Don't allow cancellation during the initial handshake and login since this
+		// can potentially cause the SSH server to think that it is being hacked and
+		// disable the account.		
+		socketOut.setIsCancellable(false /* don't allow cancellation */);
+		pollingInputStream.setIsCancellable(false);
 		StringBuffer buf = new StringBuffer();
 		int c;
 		while ((c = socketIn.read()) != '\n') {
 			if (c == -1)
-				throw new IOException(Policy.bind("Client.socketClosed"));//$NON-NLS-1$
+				throw new IOException(CVSSSHMessages.Client_socketClosed);//$NON-NLS-1$
 			buf.append((char) c);
 		}
 		serverId = buf.toString();
@@ -401,7 +382,7 @@ public void connect(IProgressMonitor monitor) throws IOException, CVSAuthenticat
 		
 		if (!serverId.startsWith("SSH-1.")) { //$NON-NLS-1$
 			String sshVersion = (serverId.startsWith("SSH-")? serverId:""); //$NON-NLS-1$ //$NON-NLS-2$
-			throw new IOException(Policy.bind("Client.sshProtocolVersion", sshVersion));//$NON-NLS-1$
+			throw new IOException(NLS.bind(CVSSSHMessages.Client_sshProtocolVersion, new String[] { sshVersion }));//$NON-NLS-1$
 		} 
 		
 		// send our id.
@@ -409,6 +390,9 @@ public void connect(IProgressMonitor monitor) throws IOException, CVSAuthenticat
 		socketOut.flush();
 
 		login();
+
+		socketOut.setIsCancellable(true /* allow cancellation */);
+		pollingInputStream.setIsCancellable(true);
 		
 		// start a shell and enter interactive session or start by
 		// executing the given command.
@@ -420,7 +404,6 @@ public void connect(IProgressMonitor monitor) throws IOException, CVSAuthenticat
 
 		is = new StandardInputStream();
 		os = new StandardOutputStream();
-
 		connected = true;
 	// If an exception occurs while connected, make sure we disconnect before passing the exception on
 	} finally {
@@ -436,20 +419,23 @@ public void disconnect() throws IOException {
 	}	
 	if (connected) {
 		connected = false;
-		send(SSH_MSG_DISCONNECT, null);
-		cleanup();
+		try {
+			send(SSH_MSG_DISCONNECT, null);
+		} finally {
+			cleanup();
+		}
 	}
 }
 public InputStream getInputStream() throws IOException {
 	if (!connected) {
-		throw new IOException(Policy.bind("Client.notConnected"));//$NON-NLS-1$
+		throw new IOException(CVSSSHMessages.Client_notConnected);//$NON-NLS-1$
 	}
 
 	return is;
 }
 public OutputStream getOutputStream() throws IOException {
 	if (!connected) {
-		throw new IOException(Policy.bind("Client.notConnected"));//$NON-NLS-1$
+		throw new IOException(CVSSSHMessages.Client_notConnected);//$NON-NLS-1$
 	}
 
 	return os;
@@ -466,7 +452,7 @@ private void startShell() throws IOException {
 		packetType = packet.getType();
 
 		if (packetType != SSH_SMSG_SUCCESS) {
-			throw new IOException(Policy.bind("Client.packetType", new Object[] {new Integer(packetType)} ));//$NON-NLS-1$
+			throw new IOException(NLS.bind(CVSSSHMessages.Client_packetType, (new Object[] {new Integer(packetType)})));//$NON-NLS-1$
 		}
 	} finally {
 		if (packet != null) {
@@ -490,7 +476,7 @@ private void login() throws IOException, CVSAuthenticationException {
 		packetType = packet.getType();
 
 		if (packetType != SSH_SMSG_PUBLIC_KEY) {
-			throw new IOException(Policy.bind("Client.packetType", new Object[] {new Integer(packetType)} ));//$NON-NLS-1$
+			throw new IOException(NLS.bind(CVSSSHMessages.Client_packetType, (new Object[] {new Integer(packetType)})));//$NON-NLS-1$
 		}
 
 		receive_SSH_SMSG_PUBLIC_KEY(packet);
@@ -505,7 +491,7 @@ private void login() throws IOException, CVSAuthenticationException {
 		packetType = packet.getType();
 
 		if (packetType != SSH_SMSG_SUCCESS) {
-			throw new IOException(Policy.bind("Client.packetType", new Object[] {new Integer(packetType)} ));//$NON-NLS-1$
+			throw new IOException(NLS.bind(CVSSSHMessages.Client_packetType, (new Object[] {new Integer(packetType)})));//$NON-NLS-1$
 		}
 	} finally {
 		if (packet != null) {
@@ -520,7 +506,7 @@ private void login() throws IOException, CVSAuthenticationException {
 		packetType = packet.getType();
 
 		if (packetType != SSH_SMSG_FAILURE) {
-			throw new IOException(Policy.bind("Client.packetType", new Object[] {new Integer(packetType)} ));//$NON-NLS-1$
+			throw new IOException(NLS.bind(CVSSSHMessages.Client_packetType, (new Object[] {new Integer(packetType)})));//$NON-NLS-1$
 		}
 	} finally {
 		if (packet != null) {
@@ -535,11 +521,11 @@ private void login() throws IOException, CVSAuthenticationException {
 		packetType = packet.getType();
 
 		if (packetType == SSH_SMSG_FAILURE) {
-			throw new CVSAuthenticationException(Policy.bind("Client.authenticationFailed"));//$NON-NLS-1$
+			throw new CVSAuthenticationException(CVSSSHMessages.Client_authenticationFailed, CVSAuthenticationException.RETRY);//$NON-NLS-1$
 		}
 
 		if (packetType != SSH_SMSG_SUCCESS) {
-			throw new IOException(Policy.bind("Client.packetType", new Object[] {new Integer(packetType)} ));//$NON-NLS-1$
+			throw new IOException(NLS.bind(CVSSSHMessages.Client_packetType, (new Object[] {new Integer(packetType)})));//$NON-NLS-1$
 		}
 	} finally {
 		if (packet != null) {
@@ -649,7 +635,7 @@ private void send_SSH_CMSG_SESSION_KEY(byte[] anti_spoofing_cookie, byte[] host_
 	}
 
 	if (!foundSupportedCipher) {
-		throw new IOException(Policy.bind("Client.cipher"));//$NON-NLS-1$
+		throw new IOException(CVSSSHMessages.Client_cipher);//$NON-NLS-1$
 	}
 
 	// session_key
@@ -664,7 +650,7 @@ private void send_SSH_CMSG_SESSION_KEY(byte[] anti_spoofing_cookie, byte[] host_
 	BigInteger host_e = new BigInteger(1, host_key_public_exponent);
 	BigInteger host_n = new BigInteger(1, host_key_public_modulus);
 	if (!new KnownHosts().verifyKey(host, host_key_bits, host_e, host_n)) {
-		throw new CVSAuthenticationException(Policy.bind("Client.hostIdChanged")); //$NON-NLS-1$
+		throw new CVSAuthenticationException(CVSSSHMessages.Client_hostIdChanged, CVSAuthenticationException.NO_RETRY); //$NON-NLS-1$
 	};
 	byte[] result;
 	if (new BigInteger(1,server_key_public_modulus).compareTo(host_n) == -1) {
@@ -690,7 +676,7 @@ private void send_SSH_CMSG_SESSION_KEY(byte[] anti_spoofing_cookie, byte[] host_
 	byte[] data = new byte[1 + anti_spoofing_cookie.length + session_key_encrypted.length + protocol_flags.length];
 
 	offset = 0;
-	data[offset++] = (byte) cipher_type;
+	data[offset++] = cipher_type;
 
 	System.arraycopy(anti_spoofing_cookie, 0, data, offset, anti_spoofing_cookie.length);
 
