@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -11,44 +11,29 @@
 package org.eclipse.team.internal.ccvs.ui.actions;
 
 import java.lang.reflect.InvocationTargetException;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collections;
-import java.util.Comparator;
-import java.util.HashSet;
-import java.util.Hashtable;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Set;
+import java.util.*;
 
-import org.eclipse.core.resources.IProject;
-import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.IResourceStatus;
-import org.eclipse.core.runtime.CoreException;
-import org.eclipse.core.runtime.IPath;
-import org.eclipse.core.runtime.IProgressMonitor;
-import org.eclipse.core.runtime.IStatus;
-import org.eclipse.core.runtime.SubProgressMonitor;
+import org.eclipse.core.resources.*;
+import org.eclipse.core.runtime.*;
 import org.eclipse.jface.action.IAction;
 import org.eclipse.jface.dialogs.MessageDialog;
+import org.eclipse.jface.operation.IRunnableWithProgress;
+import org.eclipse.osgi.util.NLS;
 import org.eclipse.swt.widgets.Display;
 import org.eclipse.swt.widgets.Shell;
 import org.eclipse.team.core.RepositoryProvider;
 import org.eclipse.team.core.TeamException;
-import org.eclipse.team.internal.ccvs.core.CVSException;
-import org.eclipse.team.internal.ccvs.core.CVSProviderPlugin;
-import org.eclipse.team.internal.ccvs.core.CVSTag;
-import org.eclipse.team.internal.ccvs.core.CVSTeamProvider;
-import org.eclipse.team.internal.ccvs.core.ICVSFolder;
-import org.eclipse.team.internal.ccvs.core.ICVSResource;
+import org.eclipse.team.internal.ccvs.core.*;
 import org.eclipse.team.internal.ccvs.core.resources.CVSWorkspaceRoot;
 import org.eclipse.team.internal.ccvs.core.resources.EclipseSynchronizer;
 import org.eclipse.team.internal.ccvs.core.syncinfo.FolderSyncInfo;
 import org.eclipse.team.internal.ccvs.core.syncinfo.ResourceSyncInfo;
 import org.eclipse.team.internal.ccvs.core.util.Util;
+import org.eclipse.team.internal.ccvs.ui.CVSUIMessages;
 import org.eclipse.team.internal.ccvs.ui.Policy;
-import org.eclipse.team.internal.ui.IPromptCondition;
-import org.eclipse.team.internal.ui.PromptingDialog;
+import org.eclipse.team.internal.ui.actions.TeamAction;
+import org.eclipse.team.internal.ui.dialogs.IPromptCondition;
+import org.eclipse.team.internal.ui.dialogs.PromptingDialog;
 
 /**
  * This class represents an action performed on a local CVS workspace
@@ -66,9 +51,13 @@ public abstract class WorkspaceAction extends CVSAction {
 		if (super.beginExecution(action)) {
 			// Ensure that the required sync info is loaded
 			if (requiresLocalSyncInfo()) {
+				// There is a possibility of the selection containing an orphaned subtree.
+				// If it does, they will be purged and enablement rechecked before the 
+				// operation is performed.
+				handleOrphanedSubtrees();
 				// Check enablement just in case the sync info wasn't loaded
 				if (!isEnabled()) {
-					MessageDialog.openInformation(getShell(), Policy.bind("CVSAction.disabledTitle"), Policy.bind("CVSAction.disabledMessage")); //$NON-NLS-1$ //$NON-NLS-2$
+					MessageDialog.openInformation(getShell(), CVSUIMessages.CVSAction_disabledTitle, CVSUIMessages.CVSAction_disabledMessage); //$NON-NLS-1$ //$NON-NLS-2$
 					return false;
 				}
 			}
@@ -78,6 +67,64 @@ public abstract class WorkspaceAction extends CVSAction {
 		}
 	}
 
+	/*
+	 * Determine if any of the selected resources are deascendants of
+	 * an orphaned CVS subtree and if they are, purge the CVS folders.
+	 */
+	private boolean handleOrphanedSubtrees() {
+		// invoke the inherited method so that overlaps are maintained
+		IResource[] resources = getSelectedResources();
+		for (int i = 0; i < resources.length; i++) {
+			IResource resource = resources[i];
+			handleOrphanedSubtree(resource);
+		}
+		return false;
+	}
+
+	/*
+	 * Determine if the resource is a descendant of an orphaned subtree.
+	 * If it is, purge the CVS folders of the subtree.
+	 */
+	private void handleOrphanedSubtree(IResource resource) {
+		try {
+			if (!CVSWorkspaceRoot.isSharedWithCVS(resource)) return ;
+			ICVSFolder folder;
+			if (resource.getType() == IResource.FILE) {
+				folder = CVSWorkspaceRoot.getCVSFolderFor(resource.getParent());
+			} else {
+				folder = CVSWorkspaceRoot.getCVSFolderFor((IContainer)resource);
+			}
+			handleOrphanedSubtree(folder);
+		} catch (CVSException e) {
+			CVSProviderPlugin.log(e);
+		}
+	}
+	
+	/*
+	 * Recursively check for and handle orphaned CVS folders
+	 */
+	private void handleOrphanedSubtree(final ICVSFolder folder) throws CVSException {
+		if (folder.getIResource().getType() == IResource.PROJECT) return;
+		if (CVSWorkspaceRoot.isOrphanedSubtree((IContainer)folder.getIResource())) {
+			try {
+				run(new IRunnableWithProgress() {
+					public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
+						try {
+							folder.unmanage(null);
+						} catch (CVSException e) {
+							CVSProviderPlugin.log(e);
+						}
+					}
+				}, true, PROGRESS_BUSYCURSOR);
+			} catch (InvocationTargetException e) {
+				// Ignore this since we logged the one we care about above
+			} catch (InterruptedException e) {
+				throw new OperationCanceledException();
+			}
+		}
+		handleOrphanedSubtree(folder.getParent());
+	}
+	
 	/**
 	 * Return true if the sync info is loaded for all selected resources.
 	 * The purpose of this method is to allow enablement code to be as fast
@@ -197,11 +244,11 @@ public abstract class WorkspaceAction extends CVSAction {
 				}
 				String question;
 				if (resources.length == 1) {
-					question = Policy.bind("CVSAction.refreshQuestion", status.getMessage(), resources[0].getFullPath().toString()); //$NON-NLS-1$
+					question = NLS.bind(CVSUIMessages.CVSAction_refreshQuestion, new String[] { status.getMessage(), resources[0].getFullPath().toString() }); //$NON-NLS-1$
 				} else {
-					question = Policy.bind("CVSAction.refreshMultipleQuestion", status.getMessage()); //$NON-NLS-1$
+					question = NLS.bind(CVSUIMessages.CVSAction_refreshMultipleQuestion, new String[] { status.getMessage() }); //$NON-NLS-1$
 				}
-				result[0] = MessageDialog.openQuestion(shellToUse, Policy.bind("CVSAction.refreshTitle"), question); //$NON-NLS-1$
+				result[0] = MessageDialog.openQuestion(shellToUse, CVSUIMessages.CVSAction_refreshTitle, question); //$NON-NLS-1$
 			}
 		};
 		Display.getDefault().syncExec(runnable);
@@ -235,8 +282,12 @@ public abstract class WorkspaceAction extends CVSAction {
 	 */
 	protected boolean isEnabled() throws TeamException {
 		
+		// allow the super to decide enablement. if the super doesn't know it will return false.
+		boolean enabled = super.isEnabled();
+		if(enabled) return true;
+		
 		// invoke the inherited method so that overlaps are maintained
-		IResource[] resources = super.getSelectedResources();
+		IResource[] resources = getSelectedResourcesWithOverlap();
 		
 		// disable if no resources are selected
 		if(resources.length==0) return false;
@@ -251,7 +302,9 @@ public abstract class WorkspaceAction extends CVSAction {
 			IResource resource = resources[i];
 			
 			// only enable for accessible resources
-			if (! resource.isAccessible()) return false;
+			if(resource.getType() == IResource.PROJECT) {
+				if (! resource.isAccessible()) return false;
+			}
 			
 			// no CVS actions are enabled if the selection contains a linked resource
 			if (CVSWorkspaceRoot.isLinkedResource(resource)) return false;
@@ -270,7 +323,7 @@ public abstract class WorkspaceAction extends CVSAction {
 			}
 			
 			// ensure that resource management state matches what the action requires
-			ICVSResource cvsResource = CVSWorkspaceRoot.getCVSResourceFor(resource);
+			ICVSResource cvsResource = getCVSResourceFor(resource);
 			if (!isEnabledForCVSResource(cvsResource)) {
 				return false;
 			}
@@ -290,8 +343,8 @@ public abstract class WorkspaceAction extends CVSAction {
 		}
 		return true;
 	}
-	
-	/**
+
+    /**
 	 * Method isEnabledForCVSResource.
 	 * @param cvsResource
 	 * @return boolean
@@ -313,6 +366,7 @@ public abstract class WorkspaceAction extends CVSAction {
 		if ( ! managed && ! isEnabledForUnmanagedResources()) return false;
 		if ( ignored && ! isEnabledForIgnoredResources()) return false;
 		if (added && ! isEnabledForAddedResources()) return false;
+		if ( ! cvsResource.exists() && ! isEnabledForNonExistantResources()) return false;
 		return true;
 	}
 	
@@ -357,57 +411,11 @@ public abstract class WorkspaceAction extends CVSAction {
 	}
 	
 	/**
-	 * Method getNonOverlapping ensures that a resource is not covered more than once.
-	 * @param resources
-	 * @return IResource[]
+	 * Method isEnabledForNonExistantResources.
+	 * @return boolean
 	 */
-	public static IResource[] getNonOverlapping(IResource[] resources) {
-		// Sort the resources so the shortest paths are first
-		List sorted = new ArrayList();
-		sorted.addAll(Arrays.asList(resources));
-		Collections.sort(sorted, new Comparator() {
-			public int compare(Object arg0, Object arg1) {
-				IResource resource0 = (IResource) arg0;
-				IResource resource1 = (IResource) arg1;
-				return resource0.getFullPath().segmentCount() - resource1.getFullPath().segmentCount();
-			}
-			public boolean equals(Object arg0) {
-				return false;
-			}
-		});
-		// Collect all non-overlapping resources
-		List coveredPaths = new ArrayList();
-		for (Iterator iter = sorted.iterator(); iter.hasNext();) {
-			IResource resource = (IResource) iter.next();
-			IPath resourceFullPath = resource.getFullPath();
-			boolean covered = false;
-			for (Iterator it = coveredPaths.iterator(); it.hasNext();) {
-				IPath path = (IPath) it.next();
-				if(path.isPrefixOf(resourceFullPath)) {
-					covered = true;
-				}
-			}
-			if (covered) {
-				// if the resource is covered by a parent, remove it
-				iter.remove();
-			} else {
-				// if the resource is a non-covered folder, add it to the covered paths
-				if (resource.getType() == IResource.FOLDER) {
-					coveredPaths.add(resource.getFullPath());
-				}
-			}
-		}
-		return (IResource[]) sorted.toArray(new IResource[sorted.size()]);
-	}
-	
-	/**
-	 * Override to ensure that the selected resources so not overlap.
-	 * This method assumes that all actions are deep.
-	 * 
-	 * @see org.eclipse.team.internal.ui.actions.TeamAction#getSelectedResources()
-	 */
-	protected IResource[] getSelectedResources() {
-		return getNonOverlapping(super.getSelectedResources());
+	protected boolean isEnabledForNonExistantResources() {
+		return false;
 	}
 
 	protected void executeProviderAction(IProviderAction action, IResource[] resources, IProgressMonitor monitor) throws InvocationTargetException {
@@ -447,7 +455,7 @@ public abstract class WorkspaceAction extends CVSAction {
 			boolean multipleSameNames = true;
 			
 			for (int i = 0; i < resources.length; i++) {
-				ICVSResource cvsResource = CVSWorkspaceRoot.getCVSResourceFor(resources[i]);
+				ICVSResource cvsResource = getCVSResourceFor(resources[i]);
 				CVSTag tag = null;
 				if(cvsResource.isFolder()) {
 					FolderSyncInfo info = ((ICVSFolder)cvsResource).getFolderSyncInfo();
@@ -476,25 +484,25 @@ public abstract class WorkspaceAction extends CVSAction {
 			}
 			
 			// set text to default
-			String actionText = Policy.bind("ReplaceWithLatestAction.multipleTags"); //$NON-NLS-1$
+			String actionText = CVSUIMessages.ReplaceWithLatestAction_multipleTags; //$NON-NLS-1$
 			if(commonTag != null) {
 				int tagType = commonTag.getType();
 				String tagName = commonTag.getName();
 				// multiple tag names but of the same type
 				if(sameTagType && !multipleSameNames) {
 					if(tagType == CVSTag.BRANCH) {
-						actionText = Policy.bind("ReplaceWithLatestAction.multipleBranches"); //$NON-NLS-1$					
+						actionText = CVSUIMessages.ReplaceWithLatestAction_multipleBranches; //$NON-NLS-1$					
 					} else {
-						actionText = Policy.bind("ReplaceWithLatestAction.multipleVersions"); //$NON-NLS-1$
+						actionText = CVSUIMessages.ReplaceWithLatestAction_multipleVersions; //$NON-NLS-1$
 					}
 				// same tag names and types
 				} else if(sameTagType && multipleSameNames) {
 					if(tagType == CVSTag.BRANCH) {
-						actionText = Policy.bind("ReplaceWithLatestAction.singleBranch", tagName); //$NON-NLS-1$					
+						actionText = NLS.bind(CVSUIMessages.ReplaceWithLatestAction_singleBranch, new String[] { tagName }); //$NON-NLS-1$					
 					} else if(tagType == CVSTag.VERSION){
-						actionText = Policy.bind("ReplaceWithLatestAction.singleVersion", tagName); //$NON-NLS-1$
+						actionText = NLS.bind(CVSUIMessages.ReplaceWithLatestAction_singleVersion, new String[] { tagName }); //$NON-NLS-1$
 					} else if(tagType == CVSTag.HEAD) {
-						actionText = Policy.bind("ReplaceWithLatestAction.singleHEAD", tagName); //$NON-NLS-1$
+						actionText = NLS.bind(CVSUIMessages.ReplaceWithLatestAction_singleHEAD, new String[] { tagName }); //$NON-NLS-1$
 					}
 				}
 			}
@@ -502,7 +510,7 @@ public abstract class WorkspaceAction extends CVSAction {
 			return actionText;
 		} catch (CVSException e) {
 			// silently ignore
-			return Policy.bind("ReplaceWithLatestAction.multipleTags"); //$NON-NLS-1$ 
+			return CVSUIMessages.ReplaceWithLatestAction_multipleTags; //$NON-NLS-1$ 
 		}
 	}
 
@@ -513,10 +521,10 @@ public abstract class WorkspaceAction extends CVSAction {
 		try {
 			monitor = Policy.monitorFor(monitor);
 			monitor.beginTask(null, selectedResources.length * 100);
-			monitor.setTaskName(Policy.bind("ReplaceWithAction.calculatingDirtyResources")); //$NON-NLS-1$
+			monitor.setTaskName(CVSUIMessages.ReplaceWithAction_calculatingDirtyResources); //$NON-NLS-1$
 			for (int i = 0; i < selectedResources.length; i++) {
 				IResource resource = selectedResources[i];
-				ICVSResource cvsResource = CVSWorkspaceRoot.getCVSResourceFor(resource);
+				ICVSResource cvsResource = getCVSResourceFor(resource);
 				if(cvsResource.isModified(Policy.subMonitorFor(monitor, 100))) {
 					dirtyResources.add(resource);
 				}			
@@ -526,7 +534,7 @@ public abstract class WorkspaceAction extends CVSAction {
 		}
 		
 		PromptingDialog dialog = new PromptingDialog(getShell(), selectedResources, 
-				getPromptCondition((IResource[]) dirtyResources.toArray(new IResource[dirtyResources.size()])), Policy.bind("ReplaceWithAction.confirmOverwrite"));//$NON-NLS-1$
+				getPromptCondition((IResource[]) dirtyResources.toArray(new IResource[dirtyResources.size()])), CVSUIMessages.ReplaceWithAction_confirmOverwrite);//$NON-NLS-1$
 		return dialog.promptForMultiple();
 	}
 
