@@ -1,12 +1,13 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Felix Pahl (fpahl@web.de) - fixed https://bugs.eclipse.org/bugs/show_bug.cgi?id=51820
  *******************************************************************************/
 
 package org.eclipse.ui.texteditor;
@@ -25,20 +26,22 @@ import org.eclipse.swt.events.VerifyEvent;
 import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.graphics.Point;
 
+import org.eclipse.jface.action.IStatusLineManager;
+import org.eclipse.jface.viewers.ISelection;
+import org.eclipse.jface.viewers.ISelectionChangedListener;
+import org.eclipse.jface.viewers.ISelectionProvider;
+import org.eclipse.jface.viewers.SelectionChangedEvent;
+
 import org.eclipse.jface.text.Assert;
 import org.eclipse.jface.text.IFindReplaceTarget;
 import org.eclipse.jface.text.IFindReplaceTargetExtension;
 import org.eclipse.jface.text.IRegion;
 import org.eclipse.jface.text.ITextListener;
+import org.eclipse.jface.text.ITextSelection;
 import org.eclipse.jface.text.ITextViewer;
 import org.eclipse.jface.text.ITextViewerExtension;
 import org.eclipse.jface.text.TextEvent;
 
-import org.eclipse.jface.action.IStatusLineManager;
-import org.eclipse.jface.viewers.ISelectionChangedListener;
-import org.eclipse.jface.viewers.ISelectionProvider;
-import org.eclipse.jface.viewers.SelectionChangedEvent;
-
 /**
  * An incremental find target. Replace is always disabled.
  * @since 2.0
@@ -46,26 +49,33 @@ import org.eclipse.jface.viewers.SelectionChangedEvent;
 class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExtension, VerifyKeyListener, MouseListener, FocusListener, ISelectionChangedListener, ITextListener {
 
 	/** The string representing rendered tab */
-	private final static String TAB= EditorMessages.getString("Editor.FindIncremental.render.tab"); //$NON-NLS-1$
-
+	private final static String TAB= EditorMessages.Editor_FindIncremental_render_tab;
+	/**
+	 * The string representing "Reverse Incremental Find"
+	 * @since 3.0
+	 */
+	private final static String FIELD_NAME= EditorMessages.Editor_FindIncremental_name;
+	/**
+	 * The string representing "Incremental Find"
+	 * @since 3.0
+	 */
+	private final static String REVERSE_FIELD_NAME= EditorMessages.Editor_FindIncremental_reverse_name;
 	/**
 	 * The string representing reverse
 	 * @since 2.1
 	 */
-	private final static String REVERSE= EditorMessages.getString("Editor.FindIncremental.reverse"); //$NON-NLS-1$
+	private final static String REVERSE= EditorMessages.Editor_FindIncremental_reverse;
 	/**
 	 * The string representing wrapped
 	 * @since 2.1
 	 */
-	private final static String WRAPPED= EditorMessages.getString("Editor.FindIncremental.wrapped"); //$NON-NLS-1$
-
+	private final static String WRAPPED= EditorMessages.Editor_FindIncremental_wrapped;
 	/** The text viewer to operate on */
 	private final ITextViewer fTextViewer;
 	/** The status line manager for output */
 	private final IStatusLineManager fStatusLine;
 	/** The find replace target to delegate find requests */
 	private final IFindReplaceTarget fTarget;
-
 	/** The current find string */
 	private StringBuffer fFindString= new StringBuffer();
 	/** The position of the first upper case character, -1 if none */
@@ -76,9 +86,9 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 	 */
 	private int fWrapPosition;
 	/** The position of the last successful find */
-	private int fCurrentIndex;	
+	private int fCurrentIndex;
 	/** A flag indicating if last find was successful */
-	private boolean fFound;	
+	private boolean fFound;
 	/**
 	 * A flag indicating if the last search was forward
 	 * @since 2.1
@@ -99,6 +109,23 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 	 * @since 2.1
 	 */
 	private String fPrevFindString= ""; //$NON-NLS-1$
+	/**
+	 * The previous position of the first upper case character, -1 if none
+	 * @since 3.0
+	 */
+	private int fPrevCasePosition= -1;
+	/**
+	 * The find status field.
+	 * @since 3.0
+	 */
+	private IStatusField fStatusField;
+	/**
+	 * Tells whether the status field implements
+	 * <code>IStatusFieldExtension</code>.
+	 * @see IStatusFieldExtension
+	 * @since 3.0
+	 */
+	private boolean fIsStatusFieldExtension;
 
 	/**
 	 * Data structure for a search result.
@@ -133,7 +160,7 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 
 	/**
 	 * Restores the search result.
-	 *  
+	 *
 	 * @since 2.1
 	 */
 	private void restoreState() {
@@ -171,7 +198,7 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 	 * Sets the direction for the next search.
 	 * This can be called before <code>beginSession</code> to set the initial search direction.
 	 * @param forward <code>true</code> if the next search should be forward
-	 * @see #beginSession
+	 * @see #beginSession()
 	 * @since 2.1
 	 */
 	public void setDirection(boolean forward) {
@@ -249,7 +276,7 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 
 		fFindString.setLength(0);
 		fSessionStack= new Stack();
-		fCasePosition= -1;		
+		fCasePosition= -1;
 		fWrapPosition= -1;
 		fFound= true;
 
@@ -261,7 +288,7 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 		} else {
 			fCurrentIndex= 0;
 		}
-		
+
 		install();
 
 		// Set the mark
@@ -269,10 +296,10 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 			((ITextViewerExtension) fTextViewer).setMark(fCurrentIndex);
 
 		updateStatus();
-		
+
 		if (fTarget instanceof IFindReplaceTargetExtension)
 			((IFindReplaceTargetExtension) fTarget).beginSession();
-			
+
 		fSearching= false;
 	}
 
@@ -304,13 +331,13 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 	 */
 	public void setScope(IRegion scope) {
 	}
-	
+
 	/*
 	 * @see IFindReplaceTargetExtension#setReplaceAllMode(boolean)
 	 */
 	public void setReplaceAllMode(boolean replaceAll) {
 	}
-	
+
 	/**
 	 * Installs this target. I.e. adds all required listeners.
 	 */
@@ -322,7 +349,7 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 		StyledText text= fTextViewer.getTextWidget();
 		if (text == null)
 			return;
-		
+
 		text.addMouseListener(this);
 		text.addFocusListener(this);
 		fTextViewer.addTextListener(this);
@@ -330,15 +357,15 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 		ISelectionProvider selectionProvider= fTextViewer.getSelectionProvider();
 		if (selectionProvider != null)
 			selectionProvider.addSelectionChangedListener(this);
-					
+
 		if (fTextViewer instanceof ITextViewerExtension)
 			((ITextViewerExtension) fTextViewer).prependVerifyKeyListener(this);
 		else
 			text.addVerifyKeyListener(this);
-		
+
 		fInstalled= true;
 	}
-	
+
 	/**
 	 * Uninstalls itself. I.e. removes all listeners installed in <code>install</code>.
 	 */
@@ -355,7 +382,7 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 			text.removeMouseListener(this);
 			text.removeFocusListener(this);
 		}
-				
+
 		if (fTextViewer instanceof ITextViewerExtension) {
 			((ITextViewerExtension) fTextViewer).removeVerifyKeyListener(this);
 
@@ -363,7 +390,7 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 			if (text != null)
 				text.removeVerifyKeyListener(this);
 		}
-		
+
 		fInstalled= false;
 	}
 
@@ -381,12 +408,19 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 		String reversePrefix= fForward ? "" : REVERSE; //$NON-NLS-1$
 
 		if (!fFound) {
-			String pattern= EditorMessages.getString("Editor.FindIncremental.not_found.pattern"); //$NON-NLS-1$
+			String pattern= EditorMessages.Editor_FindIncremental_not_found_pattern;
 			statusError(MessageFormat.format(pattern, new Object[] { reversePrefix, wrapPrefix, string }));
 
-		} else {
-			String pattern= EditorMessages.getString("Editor.FindIncremental.found.pattern"); //$NON-NLS-1$
+		} else if (string.length() == 0) {
+			if (fForward)
+				statusMessage(FIELD_NAME);
+			else
+				statusMessage(REVERSE_FIELD_NAME);
+		} else if (!fForward || fWrapPosition > -1) {
+			String pattern= EditorMessages.Editor_FindIncremental_found_pattern;
 			statusMessage(MessageFormat.format(pattern, new Object[] { reversePrefix, wrapPrefix, string }));
+		} else {
+			statusMessage(string);
 		}
 	}
 
@@ -400,10 +434,10 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 
 		fSearching= true;
 		if (event.character == 0) {
-	
+
 			switch (event.keyCode) {
-			
-			// ALT, CTRL, ARROW_LEFT, ARROW_RIGHT == leave	
+
+			// ALT, CTRL, ARROW_LEFT, ARROW_RIGHT == leave
 			case SWT.ARROW_LEFT:
 			case SWT.ARROW_RIGHT:
 			case SWT.HOME:
@@ -417,57 +451,61 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 				saveState();
 				setDirection(true);
 				repeatSearch(fForward);
-				event.doit= false;				
+				event.doit= false;
 				break;
 
 			case SWT.ARROW_UP:
 				saveState();
 				setDirection(false);
 				repeatSearch(fForward);
-				event.doit= false;				
-				break;			
+				event.doit= false;
+				break;
 			}
-	
+
 		// event.character != 0
 		} else {
-			
+
 			switch (event.character) {
-			
+
 			// ESC, CR = quit
 			case 0x1B:
 			case 0x0D:
 				leave();
 				event.doit= false;
 				break;
-				
+
 			// backspace	and delete
 			case 0x08:
 			case 0x7F:
 				restoreState();
 				event.doit= false;
-				break;		
-			
+				break;
+
 			default:
-				if (event.stateMask == 0 || event.stateMask == SWT.SHIFT) {
+				if (event.stateMask == 0 || event.stateMask == SWT.SHIFT || event.stateMask == (SWT.ALT | SWT.CTRL)) { // SWT.ALT | SWT.CTRL covers AltGr (see bug 43049)
 					saveState();
 					addCharSearch(event.character);
 					event.doit= false;
 				}
 				break;
-			}		
+			}
 		}
 		updateStatus();
 		fSearching= false;
 	}
 
 	/**
-	 * Repeats the last search while possibly changing the direciton.
+	 * Repeats the last search while possibly changing the direction.
+	 *
 	 * @param forward <code>true</code> iff the next search should be forward
+	 * @return if the search was successful
 	 * @since 2.1
 	 */
 	private boolean repeatSearch(boolean forward) {
-		if (fFindString.length() == 0)
+		if (fFindString.length() == 0) {
 			fFindString= new StringBuffer(fPrevFindString);
+			fCasePosition= fPrevCasePosition;
+		}
 
 		String string= fFindString.toString();
 		if (string.length() == 0) {
@@ -519,7 +557,9 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 
 	/**
 	 * Adds the given character to the search string and repeats the search with the last parameters.
+	 *
 	 * @param c the character to append to the search pattern
+	 * @return <code>true</code> the search found a match
 	 * @since 2.1
 	 */
 	private boolean addCharSearch(char c) {
@@ -557,10 +597,12 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 	 * Leaves this incremental search session.
 	 */
 	private void leave() {
-		if (fFindString.length() != 0) 
+		if (fFindString.length() != 0) {
 			fPrevFindString= fFindString.toString();
+			fPrevCasePosition= fCasePosition;
+		}
 		statusClear();
-		uninstall();				
+		uninstall();
 		fSessionStack = null;
 	}
 
@@ -568,14 +610,15 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 	 * @see ITextListener#textChanged(TextEvent)
 	 */
 	public void textChanged(TextEvent event) {
-		leave();
+		if (event.getDocumentEvent() != null)
+			leave();
 	}
 
 	/*
 	 * @see MouseListener##mouseDoubleClick(MouseEvent)
 	 */
 	public void mouseDoubleClick(MouseEvent e) {
-		leave();		
+		leave();
 	}
 
 	/*
@@ -611,8 +654,20 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 	 * @param string the status message
 	 */
 	private void statusMessage(String string) {
-		fStatusLine.setErrorMessage(""); //$NON-NLS-1$
-		fStatusLine.setMessage(escapeTabs(string));
+		if (fStatusField != null) {
+			if (fIsStatusFieldExtension) {
+				((IStatusFieldExtension)fStatusField).setErrorText(null);
+				fStatusField.setText(escapeTabs(string));
+				((IStatusFieldExtension)fStatusField).setVisible(true);
+				fStatusLine.update(true);
+			} else {
+				fStatusLine.setErrorMessage(null);
+				fStatusField.setText(escapeTabs(string));
+			}
+		} else {
+			fStatusLine.setErrorMessage(null);
+			fStatusLine.setMessage(escapeTabs(string));
+		}
 	}
 
 	/**
@@ -620,18 +675,42 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 	 * @param string the status error message
 	 */
 	private void statusError(String string) {
-		fStatusLine.setErrorMessage(escapeTabs(string));
-		fStatusLine.setMessage(""); //$NON-NLS-1$
+		if (fStatusField != null) {
+			if (fIsStatusFieldExtension) {
+				((IStatusFieldExtension)fStatusField).setErrorText(escapeTabs(string));
+				fStatusField.setText(""); //$NON-NLS-1$
+				((IStatusFieldExtension)fStatusField).setVisible(true);
+				fStatusLine.update(true);
+			} else {
+				fStatusLine.setErrorMessage(escapeTabs(string));
+				fStatusField.setText(""); //$NON-NLS-1$
+			}
+		} else {
+			fStatusLine.setErrorMessage(escapeTabs(string));
+			fStatusLine.setMessage(null);
+		}
 	}
 
 	/**
 	 * Clears the status message and the status error message.
 	 */
 	private void statusClear() {
-		fStatusLine.setErrorMessage(""); //$NON-NLS-1$
-		fStatusLine.setMessage(""); //$NON-NLS-1$
+		if (fStatusField != null) {
+			if (fIsStatusFieldExtension) {
+				fStatusField.setText(""); //$NON-NLS-1$
+				((IStatusFieldExtension)fStatusField).setErrorText(null);
+				((IStatusFieldExtension)fStatusField).setVisible(false);
+				fStatusLine.update(true);
+			} else {
+				fStatusField.setText(""); //$NON-NLS-1$
+				fStatusLine.setErrorMessage(null);
+			}
+		} else {
+			fStatusLine.setErrorMessage(null);
+			fStatusLine.setMessage(null);
+		}
 	}
-	
+
 	/**
 	 * Translates all tab characters into a proper status line presentation.
 	 * @param string the string in which to translate the tabs
@@ -642,7 +721,7 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 
 		int begin= 0;
 		int end= string.indexOf('\t', begin);
-		
+
 		while (end >= 0) {
 			buffer.append(string.substring(begin, end));
 			buffer.append(TAB);
@@ -650,17 +729,17 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 			end= string.indexOf('\t', begin);
 		}
 		buffer.append(string.substring(begin));
-		
+
 		return buffer.toString();
 	}
-	
+
 	/*
 	 * @see IFindReplaceTargetExtension#getLineSelection()
 	 */
 	public Point getLineSelection() {
 		if (fTarget instanceof IFindReplaceTargetExtension)
 			return ((IFindReplaceTargetExtension) fTarget).getLineSelection();
-		
+
 		return null; // XXX should not return null
 	}
 
@@ -683,7 +762,25 @@ class IncrementalFindTarget implements IFindReplaceTarget, IFindReplaceTargetExt
 	 * @since 2.1
 	 */
 	public void selectionChanged(SelectionChangedEvent e) {
-		if (!fSearching)
+		boolean ignore= false;
+		ISelection selection= e.getSelection();
+		if (selection instanceof ITextSelection) {
+			ITextSelection textSelection= (ITextSelection)selection;
+			Point range= getSelection();
+			ignore= textSelection.getOffset() + textSelection.getLength() == range.x + range.y;
+		}
+		if (!fSearching && !ignore)
 			leave();
 	}
+
+	/**
+	 * Sets the find status field for this incremental find target.
+	 *
+	 * @param statusField the status field
+	 * @since 3.0
+	 */
+	void setStatusField(IStatusField statusField) {
+		fStatusField= statusField;
+		fIsStatusFieldExtension= fStatusField instanceof IStatusFieldExtension;
+	}
 }
