@@ -1,20 +1,20 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.update.core;
 
-
 import java.net.*;
 import java.util.*;
 
 import org.eclipse.core.runtime.*;
+import org.eclipse.osgi.util.NLS;
 import org.eclipse.update.core.model.*;
 import org.eclipse.update.internal.core.*;
 
@@ -22,6 +22,12 @@ import org.eclipse.update.internal.core.*;
  * Convenience implementation of a feature.
  * <p>
  * This class may be instantiated or subclassed by clients.
+ * </p>
+ * <p>
+ * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
+ * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
+ * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
+ * (repeatedly) as the API evolves.
  * </p> 
  * @see org.eclipse.update.core.IFeature
  * @see org.eclipse.update.core.model.FeatureModel
@@ -45,10 +51,12 @@ public class Feature extends FeatureModel implements IFeature {
 	private IFeatureContentProvider featureContentProvider; // content provider
 	private List /*of IFeatureReference*/
 	includedFeatureReferences;
-	
+
 	//PERF: new instance variable
 	private VersionedIdentifier versionId;
 
+	private InstallAbortedException abortedException = null;
+
 	/**
 	 * Feature default constructor
 	 * 
@@ -89,11 +97,12 @@ public class Feature extends FeatureModel implements IFeature {
 				versionId = new VersionedIdentifier(id, ver);
 				return versionId;
 			} catch (Exception e) {
-				UpdateCore.warn("Unable to create versioned identifier:" + id + ":" + ver);
+				UpdateCore.warn(
+					"Unable to create versioned identifier:" + id + ":" + ver); //$NON-NLS-1$ //$NON-NLS-2$
 			}
 		}
 
-		versionId = new VersionedIdentifier(getURL().toExternalForm(),null);
+		versionId = new VersionedIdentifier(getURL().toExternalForm(), null);
 		return versionId;
 	}
 
@@ -118,7 +127,7 @@ public class Feature extends FeatureModel implements IFeature {
 		try {
 			contentProvider = getFeatureContentProvider();
 		} catch (CoreException e) {
-			UpdateCore.warn("No content Provider", e);
+			UpdateCore.warn("No content Provider", e); //$NON-NLS-1$
 		}
 		return (contentProvider != null) ? contentProvider.getURL() : null;
 	}
@@ -222,7 +231,11 @@ public class Feature extends FeatureModel implements IFeature {
 	 * @see IFeature#install(IFeature, IVerificationListener, IProgressMonitor)
 	 * @since 2.0
 	 */
-	public IFeatureReference install(IFeature targetFeature, IVerificationListener verificationListener, IProgressMonitor progress) throws InstallAbortedException, CoreException {
+	public IFeatureReference install(
+		IFeature targetFeature,
+		IVerificationListener verificationListener,
+		IProgressMonitor progress)
+		throws InstallAbortedException, CoreException {
 		// call other API with all optional features, or setup variable meaning install all
 		return install(targetFeature, null, verificationListener, progress);
 	}
@@ -236,10 +249,15 @@ public class Feature extends FeatureModel implements IFeature {
 	 * @see IFeature#install(IFeature, IVerificationListener, IProgressMonitor)
 	 * @since 2.0
 	 */
-	public IFeatureReference install(IFeature targetFeature, IFeatureReference[] optionalfeatures, IVerificationListener verificationListener, IProgressMonitor progress) throws InstallAbortedException, CoreException {
+	public IFeatureReference install(
+		IFeature targetFeature,
+		IFeatureReference[] optionalfeatures,
+		IVerificationListener verificationListener,
+		IProgressMonitor progress)
+		throws InstallAbortedException, CoreException {
 
 		//DEBUG
-		debug("Installing...:" + getURL().toExternalForm());
+		debug("Installing...:" + getURL().toExternalForm()); //$NON-NLS-1$
 		ErrorRecoveryLog recoveryLog = ErrorRecoveryLog.getLog();
 
 		// make sure we have an InstallMonitor		
@@ -252,10 +270,15 @@ public class Feature extends FeatureModel implements IFeature {
 			monitor = new InstallMonitor(progress);
 
 		// Setup optional install handler
-		InstallHandlerProxy handler = new InstallHandlerProxy(IInstallHandler.HANDLER_ACTION_INSTALL, this, this.getInstallHandlerEntry(), monitor);
+		InstallHandlerProxy handler =
+			new InstallHandlerProxy(
+				IInstallHandler.HANDLER_ACTION_INSTALL,
+				this,
+				this.getInstallHandlerEntry(),
+				monitor);
 		boolean success = false;
 		Throwable originalException = null;
-		InstallAbortedException abortedException = null;
+		abortedException = null;
 
 		// Get source feature provider and verifier.
 		// Initialize target variables.
@@ -265,6 +288,7 @@ public class Feature extends FeatureModel implements IFeature {
 		IFeatureReference alreadyInstalledFeature = null;
 		IFeatureContentConsumer consumer = null;
 		IPluginEntry[] targetSitePluginEntries = null;
+		ArrayList justInstalledPlugins = new ArrayList();
 
 		try {
 			// determine list of plugins to install
@@ -273,17 +297,23 @@ public class Feature extends FeatureModel implements IFeature {
 			IPluginEntry[] sourceFeaturePluginEntries = getPluginEntries();
 			ISite targetSite = targetFeature.getSite();
 			if (targetSite == null) {
-				debug("The site to install in is null");
+				debug("The site to install in is null"); //$NON-NLS-1$
 				targetSitePluginEntries = new IPluginEntry[0];
 			} else {
 				targetSitePluginEntries = targetSite.getPluginEntries();
 			}
-			IPluginEntry[] pluginsToInstall = UpdateManagerUtils.diff(sourceFeaturePluginEntries, targetSitePluginEntries);
+			IPluginEntry[] pluginsToInstall =
+				UpdateManagerUtils.diff(
+					sourceFeaturePluginEntries,
+					targetSitePluginEntries);
 			INonPluginEntry[] nonPluginsToInstall = getNonPluginEntries();
 
 			IFeatureReference[] children = getIncludedFeatureReferences();
 			if (optionalfeatures != null) {
-				children = UpdateManagerUtils.optionalChildrenToInstall(children, optionalfeatures);
+				children =
+					UpdateManagerUtils.optionalChildrenToInstall(
+						children,
+						optionalfeatures);
 			}
 
 			// determine number of monitor tasks
@@ -292,8 +322,13 @@ public class Feature extends FeatureModel implements IFeature {
 			// + 1*m tasks per non-plugin data entry (download for each)
 			// + 1 task for custom non-plugin entry handling (1 for all combined)
 			// + 5*x tasks for children features (5 subtasks per install)
-			int taskCount = 2 + 2 * pluginsToInstall.length + nonPluginsToInstall.length + 1 + 5 * children.length;
-			monitor.beginTask("", taskCount);
+			int taskCount =
+				2
+					+ 2 * pluginsToInstall.length
+					+ nonPluginsToInstall.length
+					+ 1
+					+ 5 * children.length;
+			monitor.beginTask("", taskCount); //$NON-NLS-1$
 			SubProgressMonitor subMonitor = null;
 
 			// start log
@@ -303,24 +338,42 @@ public class Feature extends FeatureModel implements IFeature {
 			handler.installInitiated();
 
 			// Download and verify feature archive(s)
-			ContentReference[] references = provider.getFeatureEntryArchiveReferences(monitor);
-			verifyReferences(verifier, references, monitor, verificationListener, true);
+			ContentReference[] references =
+				provider.getFeatureEntryArchiveReferences(monitor);
+			verifyReferences(
+				verifier,
+				references,
+				monitor,
+				verificationListener,
+				true);
 			monitorWork(monitor, 1);
-
+			
 			// Download and verify plugin archives
 			for (int i = 0; i < pluginsToInstall.length; i++) {
-				references = provider.getPluginEntryArchiveReferences(pluginsToInstall[i], monitor);
-				verifyReferences(verifier, references, monitor, verificationListener, false);
+				references = provider.getPluginEntryArchiveReferences(
+						pluginsToInstall[i], monitor);
+				verifyReferences(verifier, references, monitor,
+								verificationListener, false);
 				monitorWork(monitor, 1);
 			}
+			
 			handler.pluginsDownloaded(pluginsToInstall);
 
-			// Download non-plugin archives. Verification handled by optional install handler
-			for (int i = 0; i < nonPluginsToInstall.length; i++) {
-				references = provider.getNonPluginEntryArchiveReferences(nonPluginsToInstall[i], monitor);
-				monitorWork(monitor, 1);
-			}
-			handler.nonPluginDataDownloaded(nonPluginsToInstall, verificationListener);
+			Vector filteredPlugins = new Vector();
+            // Download non-plugin archives. Verification handled by optional
+            // install handler
+            for (int i = 0; i < nonPluginsToInstall.length; i++) {
+                if (handler.acceptNonPluginData(nonPluginsToInstall[i])) {
+                    references = provider.getNonPluginEntryArchiveReferences(
+                            nonPluginsToInstall[i], monitor);
+                    monitorWork(monitor, 1);
+                    filteredPlugins.add(nonPluginsToInstall[i]);
+                }
+            }
+            nonPluginsToInstall = (INonPluginEntry[]) filteredPlugins
+                    .toArray(new INonPluginEntry[0]);
+            handler.nonPluginDataDownloaded(nonPluginsToInstall,
+                    verificationListener);
 
 			// All archives are downloaded and verified. Get ready to install
 			consumer = targetFeature.getFeatureContentConsumer();
@@ -337,53 +390,96 @@ public class Feature extends FeatureModel implements IFeature {
 				if (childFeature != null) {
 					subMonitor = new SubProgressMonitor(monitor, 5);
 					((Site) targetSite).install(// need to cast
-					childFeature, optionalfeatures, consumer, verifier, verificationListener, subMonitor);
+					childFeature,
+						optionalfeatures,
+						consumer,
+						verifier,
+						verificationListener,
+						subMonitor);
 				}
 			}
 
 			// Install plugin files
 			for (int i = 0; i < pluginsToInstall.length; i++) {
-				references = provider.getPluginEntryContentReferences(pluginsToInstall[i], monitor);
-				IContentConsumer pluginConsumer = consumer.open(pluginsToInstall[i]);
+				// if another feature has already installed this plugin, skip it
+				if (InstallRegistry.getInstance().isPluginJustInstalled(pluginsToInstall[i])) {
+					monitor.worked(1);
+					continue;
+				}
+				IContentConsumer pluginConsumer =
+					consumer.open(pluginsToInstall[i]);
+				// TODO consumer.open returns either
+				// SiteFilePackedPluginContentConsumer or SiteFilePluginContentConsumer
+				// and they are fed either
+				// PluginEntryArchiveReference or PluginEntryContentReferences
+				// it would be better to have one that is given PluginEntryArchiveReference
+				// but it would break external IContentConsumers
 
-				String msg = "";
+				if(pluginsToInstall[i] instanceof PluginEntryModel && !((PluginEntryModel)pluginsToInstall[i]).isUnpack()){
+					// plugin can run from a jar
+					references = provider.getPluginEntryArchiveReferences(
+							pluginsToInstall[i], monitor);
+				} else{
+					// plugin must be unpacked
+					references =
+						provider.getPluginEntryContentReferences(
+							pluginsToInstall[i],
+							monitor);
+				}
+
+				String msg = ""; //$NON-NLS-1$
 				subMonitor = new SubProgressMonitor(monitor, 1);
-				VersionedIdentifier pluginVerId = pluginsToInstall[i].getVersionedIdentifier();
-				String pluginID = (pluginVerId == null) ? "" : pluginVerId.getIdentifier();
-				msg = Policy.bind("Feature.TaskInstallPluginFiles", pluginID); //$NON-NLS-1$
-				
+				VersionedIdentifier pluginVerId =
+					pluginsToInstall[i].getVersionedIdentifier();
+				String pluginID =
+					(pluginVerId == null) ? "" : pluginVerId.getIdentifier(); //$NON-NLS-1$
+				msg = NLS.bind(Messages.Feature_TaskInstallPluginFiles, (new String[] { pluginID }));
+
 				for (int j = 0; j < references.length; j++) {
-					setMonitorTaskName(subMonitor, msg + references[j].getIdentifier());
+					setMonitorTaskName(
+						subMonitor,
+						msg + references[j].getIdentifier());
 					pluginConsumer.store(references[j], subMonitor);
 				}
 
 				if (monitor.isCanceled())
-						abort();
+					abort();
+				else {
+					justInstalledPlugins.add(pluginsToInstall[i]);
+					InstallRegistry.registerPlugin(pluginsToInstall[i]);
+				}
 			}
 
 			// check if we need to install feature files [16718]	
 			// store will throw CoreException if another feature is already
 			// installed in the same place
-			alreadyInstalledFeature = featureAlreadyInstalled(targetSite); // 18867
+			alreadyInstalledFeature = featureAlreadyInstalled(targetSite);
+			// 18867
 			if (alreadyInstalledFeature == null) {
 				//Install feature files
 				references = provider.getFeatureEntryContentReferences(monitor);
 
-				String msg = "";
+				String msg = ""; //$NON-NLS-1$
 				subMonitor = new SubProgressMonitor(monitor, 1);
-				msg = Policy.bind("Feature.TaskInstallFeatureFiles"); //$NON-NLS-1$
-				
+				msg = Messages.Feature_TaskInstallFeatureFiles; 
+
 				for (int i = 0; i < references.length; i++) {
-					setMonitorTaskName(subMonitor, msg + " " + references[i].getIdentifier());
+					setMonitorTaskName(
+						subMonitor,
+						msg + " " + references[i].getIdentifier()); //$NON-NLS-1$
 					consumer.store(references[i], subMonitor);
 				}
-			} else {
-				monitor.worked(1);
-			}
 
-			if (monitor.isCanceled())
-				abort();
-			
+				if (monitor.isCanceled())
+					abort();
+				else
+					InstallRegistry.registerFeature(this);
+			} else {
+				if (monitor.isCanceled())
+					abort();
+				else
+					monitor.worked(1);
+			}
 
 			// call handler to complete installation (eg. handle non-plugin entries)
 			handler.completeInstall(consumer);
@@ -404,7 +500,9 @@ public class Feature extends FeatureModel implements IFeature {
 						result = consumer.close();
 						if (result == null) {
 							result = alreadyInstalledFeature; // 18867
-							if (result != null && optionalfeatures != null && optionalfeatures.length > 0) {
+							if (result != null
+								&& optionalfeatures != null
+								&& optionalfeatures.length > 0) {
 								// reinitialize as new optional children may have been installed
 								reinitializeFeature(result);
 							}
@@ -412,6 +510,9 @@ public class Feature extends FeatureModel implements IFeature {
 						// close the log
 						recoveryLog.close(ErrorRecoveryLog.END_INSTALL_LOG);
 					} else {
+						// unregister the just installed plugins
+						for (int i=0; i<justInstalledPlugins.size(); i++)
+							InstallRegistry.unregisterPlugin(((IPluginEntry)justInstalledPlugins.get(i)));
 						consumer.abort();
 					}
 				}
@@ -425,11 +526,15 @@ public class Feature extends FeatureModel implements IFeature {
 			// original exception wins unless it is InstallAbortedException
 			// and an error occured during abort
 			if (originalException != null) {
-				throw Utilities.newCoreException(Policy.bind("InstallHandler.error", this.getLabel()), originalException);
+				throw Utilities.newCoreException(
+					NLS.bind(Messages.InstallHandler_error, (new String[] { this.getLabel() })),
+					originalException);
 			}
 
 			if (newException != null)
-				throw Utilities.newCoreException(Policy.bind("InstallHandler.error", this.getLabel()), newException);
+				throw Utilities.newCoreException(
+					NLS.bind(Messages.InstallHandler_error, (new String[] { this.getLabel() })),
+					newException);
 
 			if (abortedException != null) {
 				throw abortedException;
@@ -449,8 +554,8 @@ public class Feature extends FeatureModel implements IFeature {
 		PluginEntryModel[] result = getPluginEntryModels();
 		if (result.length == 0)
 			return new IPluginEntry[0];
-		else 
-			return (IPluginEntry[])result;
+		else
+			return (IPluginEntry[]) result;
 	}
 
 	/*
@@ -460,22 +565,21 @@ public class Feature extends FeatureModel implements IFeature {
 	 */
 	private IPluginEntry[] filterPluginEntry(IPluginEntry[] all) {
 		List list = new ArrayList();
-		if (all!=null){
+		if (all != null) {
 			for (int i = 0; i < all.length; i++) {
 				if (UpdateManagerUtils.isValidEnvironment(all[i]))
-					list.add((IPluginEntry)all[i]);
+					list.add(all[i]);
 			}
 		}
-		
+
 		IPluginEntry[] result = new IPluginEntry[list.size()];
-		if (!list.isEmpty()){
+		if (!list.isEmpty()) {
 			list.toArray(result);
 		}
-		
+
 		return result;
 	}
 
-
 	/**
 	 * Returns the count of referenced plug-in entries.
 	 * 
@@ -516,14 +620,16 @@ public class Feature extends FeatureModel implements IFeature {
 	 * @return an erray of feature references, or an empty array.
 	 * @since 2.0
 	 */
-	public IIncludedFeatureReference[] getRawIncludedFeatureReferences() throws CoreException {
+	public IIncludedFeatureReference[] getRawIncludedFeatureReferences()
+		throws CoreException {
 		if (includedFeatureReferences == null)
 			initializeIncludedReferences();
 
 		if (includedFeatureReferences.size() == 0)
 			return new IncludedFeatureReference[0];
 
-		return (IIncludedFeatureReference[]) includedFeatureReferences.toArray(arrayTypeFor(includedFeatureReferences));
+		return (IIncludedFeatureReference[]) includedFeatureReferences.toArray(
+				new IIncludedFeatureReference[includedFeatureReferences.size()]);
 	}
 	/**
 	 * Returns the download size of the feature, if it can be determined.
@@ -549,8 +655,10 @@ public class Feature extends FeatureModel implements IFeature {
 				allNonPluginEntries.addAll(Arrays.asList(nonPlugins));
 			}
 
-			IPluginEntry[] totalPlugins = new IPluginEntry[allPluginEntries.size()];
-			INonPluginEntry[] totalNonPlugins = new INonPluginEntry[allNonPluginEntries.size()];
+			IPluginEntry[] totalPlugins =
+				new IPluginEntry[allPluginEntries.size()];
+			INonPluginEntry[] totalNonPlugins =
+				new INonPluginEntry[allNonPluginEntries.size()];
 			if (allPluginEntries.size() != 0) {
 				allPluginEntries.toArray(totalPlugins);
 			}
@@ -558,7 +666,9 @@ public class Feature extends FeatureModel implements IFeature {
 				allNonPluginEntries.toArray(totalNonPlugins);
 			}
 
-			return getFeatureContentProvider().getDownloadSizeFor(totalPlugins, totalNonPlugins);
+			return getFeatureContentProvider().getDownloadSizeFor(
+				totalPlugins,
+				totalNonPlugins);
 
 		} catch (CoreException e) {
 			UpdateCore.warn(null, e);
@@ -590,8 +700,10 @@ public class Feature extends FeatureModel implements IFeature {
 				allNonPluginEntries.addAll(Arrays.asList(nonPlugins));
 			}
 
-			IPluginEntry[] totalPlugins = new IPluginEntry[allPluginEntries.size()];
-			INonPluginEntry[] totalNonPlugins = new INonPluginEntry[allNonPluginEntries.size()];
+			IPluginEntry[] totalPlugins =
+				new IPluginEntry[allPluginEntries.size()];
+			INonPluginEntry[] totalNonPlugins =
+				new INonPluginEntry[allNonPluginEntries.size()];
 			if (allPluginEntries.size() != 0) {
 				allPluginEntries.toArray(totalPlugins);
 			}
@@ -599,7 +711,9 @@ public class Feature extends FeatureModel implements IFeature {
 				allNonPluginEntries.toArray(totalNonPlugins);
 			}
 
-			return getFeatureContentProvider().getInstallSizeFor(totalPlugins, totalNonPlugins);
+			return getFeatureContentProvider().getInstallSizeFor(
+				totalPlugins,
+				totalNonPlugins);
 
 		} catch (CoreException e) {
 			UpdateCore.warn(null, e);
@@ -613,10 +727,12 @@ public class Feature extends FeatureModel implements IFeature {
 	 * @see IFeature#getFeatureContentProvider()
 	 * @since 2.0
 	 */
-	public IFeatureContentProvider getFeatureContentProvider() throws CoreException {
+	public IFeatureContentProvider getFeatureContentProvider()
+		throws CoreException {
 		if (featureContentProvider == null) {
-			throw Utilities.newCoreException(Policy.bind("Feature.NoContentProvider", getVersionedIdentifier().toString()), null);
-			//$NON-NLS-1$
+			throw Utilities.newCoreException(
+				NLS.bind(Messages.Feature_NoContentProvider, (new String[] { getVersionedIdentifier().toString() })),
+				null);	
 		}
 		return this.featureContentProvider;
 	}
@@ -627,7 +743,8 @@ public class Feature extends FeatureModel implements IFeature {
 	 * @see IFeature#getFeatureContentConsumer()
 	 * @since 2.0
 	 */
-	public IFeatureContentConsumer getFeatureContentConsumer() throws CoreException {
+	public IFeatureContentConsumer getFeatureContentConsumer()
+		throws CoreException {
 		throw new UnsupportedOperationException();
 	}
 
@@ -639,9 +756,11 @@ public class Feature extends FeatureModel implements IFeature {
 	 */
 	public void setSite(ISite site) throws CoreException {
 		if (this.site != null) {
-			String featureURLString = (getURL() != null) ? getURL().toExternalForm() : "";
-			throw Utilities.newCoreException(Policy.bind("Feature.SiteAlreadySet", featureURLString), null);
-			//$NON-NLS-1$
+			String featureURLString =
+				(getURL() != null) ? getURL().toExternalForm() : ""; //$NON-NLS-1$
+			throw Utilities.newCoreException(
+				NLS.bind(Messages.Feature_SiteAlreadySet, (new String[] { featureURLString })),
+				null);
 		}
 		this.site = site;
 	}
@@ -664,19 +783,22 @@ public class Feature extends FeatureModel implements IFeature {
 	 * @since 2.0
 	 */
 	public String toString() {
-		String URLString = (getURL() == null) ? Policy.bind("Feature.NoURL") : getURL().toExternalForm();
-		//$NON-NLS-1$
-		String verString = Policy.bind("Feature.FeatureVersionToString", URLString, getVersionedIdentifier().toString());
-		//$NON-NLS-1$
-		String label = getLabel() == null ? "" : getLabel();
-		return verString + " [" + label + "]";
+		String URLString =
+			(getURL() == null)
+				? Messages.Feature_NoURL
+				: getURL().toExternalForm();
+
+		String verString =
+			NLS.bind(Messages.Feature_FeatureVersionToString, (new String[] { URLString, getVersionedIdentifier().toString() }));
+		String label = getLabel() == null ? "" : getLabel(); //$NON-NLS-1$
+		return verString + " [" + label + "]"; //$NON-NLS-1$ //$NON-NLS-2$
 	}
 
 	/*
 	 * Installation has been cancelled, abort and revert
 	 */
 	private void abort() throws CoreException {
-		String msg = Policy.bind("Feature.InstallationCancelled"); //$NON-NLS-1$
+		String msg = Messages.Feature_InstallationCancelled; 
 		throw new InstallAbortedException(msg, null);
 	}
 
@@ -690,7 +812,7 @@ public class Feature extends FeatureModel implements IFeature {
 		includedFeatureReferences = new ArrayList();
 
 		IIncludedFeatureReference[] nestedFeatures = getFeatureIncluded();
-		if (nestedFeatures.length==0)
+		if (nestedFeatures.length == 0)
 			return;
 
 		ISite site = getSite();
@@ -699,7 +821,8 @@ public class Feature extends FeatureModel implements IFeature {
 
 		for (int i = 0; i < nestedFeatures.length; i++) {
 			IIncludedFeatureReference include = nestedFeatures[i];
-			IIncludedFeatureReference newRef = getPerfectIncludeFeature(site, include);
+			IIncludedFeatureReference newRef =
+				getPerfectIncludeFeature(site, include);
 			includedFeatureReferences.add(newRef);
 		}
 	}
@@ -707,54 +830,69 @@ public class Feature extends FeatureModel implements IFeature {
 	/*
 	 * 
 	 */
-	private IIncludedFeatureReference getPerfectIncludeFeature(ISite site, IIncludedFeatureReference include) throws CoreException {
-	
+	private IIncludedFeatureReference getPerfectIncludeFeature(
+		ISite site,
+		IIncludedFeatureReference include)
+		throws CoreException {
+
 		// [20367] no site, cannot initialize nested references
 		ISiteFeatureReference[] refs = site.getFeatureReferences();
 		VersionedIdentifier identifier = include.getVersionedIdentifier();
-		
+
 		// too long to compute if not a file system
 		// other solution would be to parse feature.xml
 		// when parsing file system to create archive features/FeatureId_Ver.jar
-		if ("file".equals(site.getURL().getProtocol())) {
+		if ("file".equals(site.getURL().getProtocol())) { //$NON-NLS-1$
 			// check if declared on the Site
 			if (refs != null) {
 				for (int ref = 0; ref < refs.length; ref++) {
 					if (refs[ref] != null) {
-						VersionedIdentifier id = refs[ref].getVersionedIdentifier();
+						VersionedIdentifier id =
+							refs[ref].getVersionedIdentifier();
 						if (identifier.equals(id)) {
 							// found a ISiteFeatureReference that matches our IIncludedFeatureReference
-							IncludedFeatureReference newRef = new IncludedFeatureReference(refs[ref]);
+							IncludedFeatureReference newRef =
+								new IncludedFeatureReference(refs[ref]);
 							newRef.isOptional(include.isOptional());
-							if (include instanceof FeatureReferenceModel)	
-								newRef.setLabel(((FeatureReferenceModel)include).getLabel());
-							newRef.setMatchingRule(include.getMatch());
-							newRef.setSearchLocation(include.getSearchLocation());
+							if (include instanceof FeatureReferenceModel)
+								newRef.setLabel(
+									((FeatureReferenceModel) include)
+										.getLabel());
+							newRef.setSearchLocation(
+								include.getSearchLocation());
 							return newRef;
 						}
 					}
 				}
 			}
 		}
-	
-		// instanciate by mapping it based on the site.xml
-		// in future we may ask for a factory to create the feature ref
-		IncludedFeatureReference newRef = new IncludedFeatureReference(include);
-		newRef.setSite(getSite());
-		IFeatureReference parentRef = getSite().getFeatureReference(this);
-		if (parentRef instanceof FeatureReference) {
-			newRef.setType(((FeatureReference) parentRef).getType());
-		}
-		String featureID = Site.DEFAULT_FEATURE_PATH + identifier.toString() + ".jar";
-		URL featureURL = getSite().getSiteContentProvider().getArchiveReference(featureID);
+
+        // instantiate by mapping it based on the site.xml
+        // in future we may ask for a factory to create the feature ref
+        IncludedFeatureReference newRef = new UpdateSiteIncludedFeatureReference(include);
+        newRef.setSite(getSite());
+        IFeatureReference parentRef = getSite().getFeatureReference(this);
+        if (parentRef instanceof FeatureReference) {
+         newRef.setType(((FeatureReference) parentRef).getType());
+        }
+        String featureID = Site.DEFAULT_FEATURE_PATH + identifier.toString();
+        if(this instanceof TargetFeature)
+         featureID = featureID + "/"; //$NON-NLS-1$
+        else
+         featureID = featureID + ".jar"; //$NON-NLS-1$
+		URL featureURL =
+			getSite().getSiteContentProvider().getArchiveReference(featureID);
 		newRef.setURL(featureURL);
 		newRef.setFeatureIdentifier(identifier.getIdentifier());
 		newRef.setFeatureVersion(identifier.getVersion().toString());
 		try {
-			newRef.resolve(getSite().getURL(), null); // no need to get the bundle
+			newRef.resolve(getSite().getURL(), null);
+			// no need to get the bundle
 			return newRef;
 		} catch (Exception e) {
-			throw Utilities.newCoreException(Policy.bind("Feature.UnableToInitializeFeatureReference", identifier.toString()), e);
+			throw Utilities.newCoreException(
+				NLS.bind(Messages.Feature_UnableToInitializeFeatureReference, (new String[] { identifier.toString() })),
+				e);
 		}
 	}
 
@@ -771,7 +909,9 @@ public class Feature extends FeatureModel implements IFeature {
 	/*
 	 * 
 	 */
-	private void setMonitorTaskName(IProgressMonitor monitor, String taskName) {
+	private void setMonitorTaskName(
+		IProgressMonitor monitor,
+		String taskName) {
 		if (monitor != null)
 			monitor.setTaskName(taskName);
 	}
@@ -779,7 +919,8 @@ public class Feature extends FeatureModel implements IFeature {
 	/*
 	 *
 	 */
-	private void monitorWork(IProgressMonitor monitor, int tick) throws CoreException {
+	private void monitorWork(IProgressMonitor monitor, int tick)
+		throws CoreException {
 		if (monitor != null) {
 			monitor.worked(tick);
 			if (monitor.isCanceled()) {
@@ -791,7 +932,13 @@ public class Feature extends FeatureModel implements IFeature {
 	/*
 	 * 
 	 */
-	private void verifyReferences(IVerifier verifier, ContentReference[] references, InstallMonitor monitor, IVerificationListener verificationListener, boolean isFeature) throws CoreException {
+	private void verifyReferences(
+		IVerifier verifier,
+		ContentReference[] references,
+		InstallMonitor monitor,
+		IVerificationListener verificationListener,
+		boolean isFeature)
+		throws CoreException {
 		IVerificationResult vr = null;
 		if (verifier != null) {
 			for (int j = 0; j < references.length; j++) {
@@ -803,13 +950,14 @@ public class Feature extends FeatureModel implements IFeature {
 					int result = verificationListener.prompt(vr);
 
 					if (result == IVerificationListener.CHOICE_ABORT) {
-						String msg = Policy.bind("JarVerificationService.CancelInstall"); //$NON-NLS-1$
+						String msg = Messages.JarVerificationService_CancelInstall; 
 						Exception e = vr.getVerificationException();
 						throw new InstallAbortedException(msg, e);
 					}
 					if (result == IVerificationListener.CHOICE_ERROR) {
-						throw Utilities.newCoreException(Policy.bind("JarVerificationService.UnsucessfulVerification"),
-						//$NON-NLS-1$
+						throw Utilities
+							.newCoreException(
+								Messages.JarVerificationService_UnsucessfulVerification,	//$NON-NLS-1$
 						vr.getVerificationException());
 					}
 				}
@@ -836,7 +984,11 @@ public class Feature extends FeatureModel implements IFeature {
 			}
 		}
 
-		UpdateCore.warn("ValidateAlreadyInstalled:Feature " + this +" not found on site:" + this.getURL());
+		UpdateCore.warn(
+			"ValidateAlreadyInstalled:Feature " //$NON-NLS-1$
+				+ this
+				+ " not found on site:" //$NON-NLS-1$
+				+ this.getURL());
 		return null;
 	}
 
@@ -846,10 +998,12 @@ public class Feature extends FeatureModel implements IFeature {
 	 */
 	private void reinitializeFeature(IFeatureReference referenceToReinitialize) {
 
-		if (referenceToReinitialize==null) return;
+		if (referenceToReinitialize == null)
+			return;
 
 		if (UpdateCore.DEBUG && UpdateCore.DEBUG_SHOW_CONFIGURATION)
-			UpdateCore.debug("Re initialize feature reference:" + referenceToReinitialize);
+			UpdateCore.debug(
+				"Re initialize feature reference:" + referenceToReinitialize); //$NON-NLS-1$
 
 		IFeature feature = null;
 		try {
@@ -860,24 +1014,28 @@ public class Feature extends FeatureModel implements IFeature {
 			// bug 24981 - recursively go into hierarchy
 			// only if site if file 
 			ISite site = referenceToReinitialize.getSite();
-			if (site==null) return;
+			if (site == null)
+				return;
 			URL url = site.getURL();
-			if (url==null) return;
-			if ("file".equals(url.getProtocol())){
-				IFeatureReference[] included = feature.getIncludedFeatureReferences();
+			if (url == null)
+				return;
+			if ("file".equals(url.getProtocol())) { //$NON-NLS-1$
+				IFeatureReference[] included =
+					feature.getIncludedFeatureReferences();
 				for (int i = 0; i < included.length; i++) {
 					reinitializeFeature(included[i]);
 				}
 			}
 		} catch (CoreException e) {
-			UpdateCore.warn("", e);
+			UpdateCore.warn("", e); //$NON-NLS-1$
 		}
 	}
-	
+
 	/**
 	 * @see org.eclipse.update.core.IFeature#getRawIncludedFeatureReferences()
 	 */
-	public IIncludedFeatureReference[] getIncludedFeatureReferences() throws CoreException {
+	public IIncludedFeatureReference[] getIncludedFeatureReferences()
+		throws CoreException {
 		return filterFeatures(getRawIncludedFeatureReferences());
 	}
 
@@ -890,25 +1048,27 @@ public class Feature extends FeatureModel implements IFeature {
 	 */
 	private IIncludedFeatureReference[] filterFeatures(IIncludedFeatureReference[] allIncluded) {
 		List list = new ArrayList();
-		if (allIncluded!=null){
+		if (allIncluded != null) {
 			for (int i = 0; i < allIncluded.length; i++) {
 				IIncludedFeatureReference included = allIncluded[i];
 				if (UpdateManagerUtils.isValidEnvironment(included))
 					list.add(included);
-				else{
-					if (UpdateCore.DEBUG && UpdateCore.DEBUG_SHOW_WARNINGS){
-						UpdateCore.warn("Filtered out feature reference:"+included);
+				else {
+					if (UpdateCore.DEBUG && UpdateCore.DEBUG_SHOW_WARNINGS) {
+						UpdateCore.warn(
+							"Filtered out feature reference:" + included); //$NON-NLS-1$
 					}
 				}
 			}
 		}
-		
-		IIncludedFeatureReference[] result = new IIncludedFeatureReference[list.size()];
-		if (!list.isEmpty()){
+
+		IIncludedFeatureReference[] result =
+			new IIncludedFeatureReference[list.size()];
+		if (!list.isEmpty()) {
 			list.toArray(result);
 		}
-		
-		return result;	
+
+		return result;
 	}
 
 	/**
@@ -920,27 +1080,26 @@ public class Feature extends FeatureModel implements IFeature {
 
 	/**
 	 * Method filterPluginEntry.
-	 * @param iNonPluginEntrys
+	 * @param all
 	 * @return INonPluginEntry[]
 	 */
 	private INonPluginEntry[] filterNonPluginEntry(INonPluginEntry[] all) {
 		List list = new ArrayList();
-		if (all!=null){
+		if (all != null) {
 			for (int i = 0; i < all.length; i++) {
 				if (UpdateManagerUtils.isValidEnvironment(all[i]))
-					list.add((INonPluginEntry)all[i]);
+					list.add(all[i]);
 			}
 		}
-		
+
 		INonPluginEntry[] result = new INonPluginEntry[list.size()];
-		if (!list.isEmpty()){
+		if (!list.isEmpty()) {
 			list.toArray(result);
 		}
-		
+
 		return result;
 	}
 
-
 	/**
 	 * @see org.eclipse.update.core.IFeature#getRawPluginEntries()
 	 */
@@ -954,26 +1113,26 @@ public class Feature extends FeatureModel implements IFeature {
 	public IImport[] getImports() {
 		return filterImports(getRawImports());
 	}
-	
+
 	/**
 	 * Method filterImports.
-	 * @param iImports
+	 * @param all
 	 * @return IImport[]
 	 */
 	private IImport[] filterImports(IImport[] all) {
 		List list = new ArrayList();
-		if (all!=null){
+		if (all != null) {
 			for (int i = 0; i < all.length; i++) {
 				if (UpdateManagerUtils.isValidEnvironment(all[i]))
-					list.add((IImport)all[i]);
+					list.add(all[i]);
 			}
 		}
-		
+
 		IImport[] result = new IImport[list.size()];
-		if (!list.isEmpty()){
+		if (!list.isEmpty()) {
 			list.toArray(result);
 		}
-		
+
 		return result;
 	}
 
